require("ui/node");
require("ui/image");
require("ui/text");
require("core/anim");
require("core/system");

-- 提示信息动画：游戏中任何地方都可能调用，
-- 接口：play：播放动画
--       load: 加载资源
--       stop: 停止动画，删除anim和prop
--       relsease: 释放资源
-- 说明：调用动画会返回该动画中所有绘制对象的根节点，可以将其加入场景中的任一节点作为其子节点，也可忽视；
--       动画播放完毕的时候会自行调用release接口释放资源。
-- 调用示例：AlarmTip.play("message",nil,50,nil,-50);

AlarmTip = {};
AlarmTip.ms = 3000; -- 动画总时常
AlarmTip.loaded = false; -- 加载状态
AlarmTip.spaceX = 50;
AlarmTip.translateX = 0;
AlarmTip.translateY = 80;
AlarmTip.textsNodes = {}
AlarmTip.is_play = false
AlarmTip.is_play2 = false

--msg :需要显示的提示信息     type：string
--translateX,translateY:分别表示该动画在x和y方向上移动的距离
--x,y :动画的位置，默认值都为屏幕上方
AlarmTip.play = function(msg, ...)
    if not msg or msg == "" then
      return;
    end
    msg = string.format(msg, ...)
    local needTranslate, diffX = AlarmTip.load(msg);
    --当提示框在未播放完动画的时候再次调用play,则先强制清除prop , anim;(重入)
    AlarmTip.stop();
    AlarmTip.is_play = true
    AlarmTip.root:setVisible(true);

    local timeStamp = needTranslate and diffX * 20 or AlarmTip.ms;
    if not needTranslate then
        AlarmTip.onTimer(AlarmTip.ms);
    else
        AlarmTip.animIndex = new(AnimInt,kAnimNormal,0,1, AlarmTip.ms/8,0);
        AlarmTip.animIndex:setDebugName("AlarmTip | AlarmTip.animIndex2");
        AlarmTip.animIndex:setEvent(diffX , AlarmTip.startTranslate);
    end
    return AlarmTip.root;
end

AlarmTip.add_textnode = function(msg)
    if not AlarmTip.is_play2 and not AlarmTip.is_play and #AlarmTip.textsNodes == 0 then
        if msg[2] == 1 then
            AlarmTip.play_texts_btn(msg[1])
        elseif msg[2] == 2 then
            AlarmTip.play_texts(msg[1])
        end
        AlarmTip.is_play2 = true
    end
    AlarmTip.textsNodes[#AlarmTip.textsNodes+1] = msg
end

AlarmTip.play_texts = function(texts)
    
    local needTranslate, diffX = AlarmTip.load_texts(texts);
    --当提示框在未播放完动画的时候再次调用play,则先强制清除prop , anim;(重入)
    --AlarmTip.stop();

    AlarmTip.root:setVisible(true);

    local timeStamp = needTranslate and diffX * 20 or AlarmTip.ms;
    if not needTranslate then
        AlarmTip.onTimer(AlarmTip.ms);
    else
        AlarmTip.animIndex = new(AnimInt,kAnimNormal,0,1, AlarmTip.ms/8,0);
        AlarmTip.animIndex:setDebugName("AlarmTip | AlarmTip.animIndex2");
        AlarmTip.animIndex:setEvent(diffX , AlarmTip.startTranslate);
    end
    return AlarmTip.root;
end

AlarmTip.play_texts_btn = function(texts)
    
    local needTranslate, diffX = AlarmTip.load_texts_btn(texts);
    --当提示框在未播放完动画的时候再次调用play,则先强制清除prop , anim;(重入)
    --AlarmTip.stop();

    AlarmTip.root:setVisible(true);

    local timeStamp = needTranslate and diffX * 20 or AlarmTip.ms;
    if not needTranslate then
        AlarmTip.onTimer(AlarmTip.ms);
    else
        AlarmTip.animIndex = new(AnimInt,kAnimNormal,0,1, AlarmTip.ms/8,0);
        AlarmTip.animIndex:setDebugName("AlarmTip | AlarmTip.animIndex2");
        AlarmTip.animIndex:setEvent(diffX , AlarmTip.startTranslate);
    end
    return AlarmTip.root;
end

AlarmTip.play2Btn = function(msg, btnFile1, btnFile2, obj, func)
    if not msg or msg == "" then
      return;
    end

    local needTranslate, diffX = AlarmTip.load(msg, btnFile1, btnFile2, obj, func);
    --当提示框在未播放完动画的时候再次调用play,则先强制清除prop , anim;(重入)
    AlarmTip.stop();

    AlarmTip.root:setVisible(true);

    local timeStamp = needTranslate and diffX * 20 or AlarmTip.ms;
    if not needTranslate then
        AlarmTip.onTimer(AlarmTip.ms);
    else
        AlarmTip.animIndex = new(AnimInt,kAnimNormal,0,1, AlarmTip.ms/8,0);
        AlarmTip.animIndex:setDebugName("AlarmTip | AlarmTip.animIndex2");
        AlarmTip.animIndex:setEvent(diffX , AlarmTip.startTranslate);
    end
    return AlarmTip.root;
end

AlarmTip.load_texts_btn = function(texts)
    if not AlarmTip.loaded then
        AlarmTip.loaded=true;

        AlarmTip.root = new(Node);
        AlarmTip.root:addToRoot();
        AlarmTip.root:setLevel(100);
        
        AlarmTip.bg = new(Image,"animation/alarmNoticeBg.png");
        
        local w,h = AlarmTip.bg:getSize();

        local screenW = System.getScreenScaleWidth();
        local screenH = System.getScreenScaleHeight();
        AlarmTip.root:setSize(w,h);

        AlarmTip.root:addChild(AlarmTip.bg);
        AlarmTip.root:setAlign(kAlignTop);

        -- 隐藏drawing
        AlarmTip.root:setVisible(false);
    end

    print_string("load_texts_btn called!!")
    AlarmTip.bg:removeAllChildren()

    local w,h = AlarmTip.bg:getSize();

    --laba在文字左边
    local laba_dx = 50
    local texts_w,texts_h = texts:getSize()
    AlarmTip.laba = new(Image,"animation/laba.png");
    AlarmTip.bg:addChild(AlarmTip.laba)
    AlarmTip.laba:setAlign(kAlignCenter);
    AlarmTip.laba:setPos(-texts_w/2-laba_dx,0)

    local laba_w, laba_h = AlarmTip.laba:getSize()
    AlarmTip.text = new(Text, "", 0, 0, kAlignCenter,"", 32, 0xff, 0xff, 0xff);
    AlarmTip.text:setAlign(kAlignCenter);
    texts:setAlign(kAlignCenter);
    AlarmTip.text:addChild(texts)
    AlarmTip.text:addChild(AlarmTip.laba)
    AlarmTip.text:setSize(texts_w+laba_w/2+laba_dx,texts_h)

    --AlarmTip.text = texts
    AlarmTip.bg:addChild(AlarmTip.text);

    AlarmTip.text_btn = new(Button,"ui/blank.png")
    -- AlarmTip.btn_text = new(Text, "รีบมาดูสิ", 0, 0, kAlignCenter, "", 45, 0xff, 0xff, 0xff);
    -- AlarmTip.bg:addChild(AlarmTip.btn_text)
    -- local btn_text_w,btn_text_h = AlarmTip.btn_text:getSize()
    -- AlarmTip.text_btn:setSize(btn_text_w,btn_text_h)
    -- AlarmTip.text_btn:setAlign(kAlignCenter);
    -- AlarmTip.btn_text:setAlign(kAlignCenter);
    -- AlarmTip.text_btn:setPos(texts_w/2+btn_text_w/2+10,0)
    -- AlarmTip.btn_text:setPos(texts_w/2+btn_text_w/2+10,0)
    AlarmTip.text_btn:setSize(texts_w,texts_h)
    AlarmTip.bg:addChild(AlarmTip.text_btn)
    AlarmTip.text_btn:setOnClick(self, function (self)
                print_string("AlarmTip.text_btn OnClick")
            end)

    -- 滚动适配
    local m_width, m_height = AlarmTip.text:getSize();
    local widthDiff = w - AlarmTip.spaceX*2;
    if m_width <= widthDiff then
        AlarmTip.text:setAlign(kAlignCenter);
        return false
    elseif m_width > widthDiff then
        AlarmTip.text:setAlign(kAlignLeft)
        --local diffX = m_width - w + AlarmTip.spaceX*3;
        local diffX = m_width - widthDiff
        AlarmTip.text:setPos(AlarmTip.spaceX+laba_w/2, 0);
        AlarmTip.text:setClip(AlarmTip.spaceX, 0, w - AlarmTip.spaceX*2 , h);
        return true, diffX;
    end
end

AlarmTip.load_texts = function(texts)
    if not AlarmTip.loaded then
        AlarmTip.loaded=true;

        AlarmTip.root = new(Node);
        AlarmTip.root:addToRoot();
        AlarmTip.root:setLevel(100);
        
        AlarmTip.bg = new(Image,"animation/alarmNoticeBg.png");
        
        local w,h = AlarmTip.bg:getSize();

        local screenW = System.getScreenScaleWidth();
        local screenH = System.getScreenScaleHeight();
        AlarmTip.root:setSize(w,h);

        AlarmTip.root:addChild(AlarmTip.bg);
        AlarmTip.root:setAlign(kAlignTop);

        -- 隐藏drawing
        AlarmTip.root:setVisible(false);
    end

    AlarmTip.bg:removeAllChildren()

    local w,h = AlarmTip.bg:getSize();

    --laba在文字左边
    -- local texts_w = 0
    -- local texts_childs = texts:getChildren()
    -- for i = 1,#texts_childs do
    --     texts_w = texts_w + texts_childs[i]:getSize()
    -- end
    local laba_dx = 50
    local texts_w,texts_h = texts:getSize()
    AlarmTip.laba = new(Image,"animation/laba.png");
    --AlarmTip.bg:addChild(AlarmTip.laba)
    AlarmTip.laba:setAlign(kAlignCenter);
    AlarmTip.laba:setPos(-texts_w/2-laba_dx,0)

    local laba_w, laba_h = AlarmTip.laba:getSize()
    AlarmTip.text = new(Text, "", 0, 0, kAlignCenter,"", 32, 0xff, 0xff, 0xff);
    AlarmTip.text:setAlign(kAlignCenter);
    texts:setAlign(kAlignCenter);
    AlarmTip.text:addChild(texts)
    AlarmTip.text:addChild(AlarmTip.laba)
    AlarmTip.text:setSize(texts_w+laba_w/2+laba_dx,texts_h)
    --AlarmTip.text = texts
    AlarmTip.bg:addChild(AlarmTip.text);

    -- 滚动适配
    local m_width, m_height = AlarmTip.text:getSize();
    
    --m_width = m_width + laba_w + 50
    local widthDiff = w - AlarmTip.spaceX*2;
    if m_width <= widthDiff then
        AlarmTip.text:setAlign(kAlignCenter);
        return false
    elseif m_width > widthDiff then
        AlarmTip.text:setAlign(kAlignLeft);
        --local diffX = m_width - w + AlarmTip.spaceX*3;
        local diffX = m_width - widthDiff
        AlarmTip.text:setPos(AlarmTip.spaceX+laba_w/2, 0);
        AlarmTip.text:setClip(AlarmTip.spaceX, 0, w - AlarmTip.spaceX*2 , h);
        return true, diffX;
    end
end

-- 加载资源
AlarmTip.load = function(msg, btnFile1, btnFile2, obj, func)
    if not AlarmTip.loaded then
        AlarmTip.loaded=true;

        AlarmTip.root = new(Node);
        AlarmTip.root:addToRoot();
        AlarmTip.root:setLevel(100);
        
        AlarmTip.bg = new(Image,"animation/alarmNoticeBg.png");
        
        local w,h = AlarmTip.bg:getSize();

        local screenW = System.getScreenScaleWidth();
        local screenH = System.getScreenScaleHeight();
        AlarmTip.root:setSize(w,h);

        AlarmTip.root:addChild(AlarmTip.bg);
        AlarmTip.root:setAlign(kAlignTop);

        -- 隐藏drawing
        AlarmTip.root:setVisible(false);
    end

    AlarmTip.bg:removeAllChildren()

    AlarmTip.text = new(Text, msg, 0, h, kAlignCenter,"", 32, 0xf2, 0xc9, 0x17);       
    AlarmTip.bg:addChild(AlarmTip.text);


    -- 滚动适配
    local btnPos = AlarmTip.spaceX
    if btnFile2 then
        AlarmTip.btn2 = new(Button, btnFile2)
        AlarmTip.bg:addChild(AlarmTip.btn2);
        AlarmTip.btn2:setAlign(kAlignRight)
        AlarmTip.btn2:setPos(btnPos, 0)

        AlarmTip.btn2:setOnClick(self, function ()
            -- body
            if func then
                func(obj, 2)
            end
        end)
        btnPos = btnPos + AlarmTip.btn2.m_res.m_width + 10
    end

    if btnFile1 then
        AlarmTip.btn1 = new(Button, btnFile1)
        AlarmTip.bg:addChild(AlarmTip.btn1);
        AlarmTip.btn1:setAlign(kAlignRight)
        AlarmTip.btn1:setPos(btnPos, 0)
        btnPos = btnPos + AlarmTip.btn1.m_res.m_width + 10
        AlarmTip.btn1:setOnClick(self, function ()
            -- body
            if func then
                func(obj, 1)
            end
        end)
    end

    local m_width, m_height = AlarmTip.text:getSize();
    local w,h = AlarmTip.bg:getSize();
    --左右两边预留50个像素，再去掉按钮的空间
    local widthDiff = w - btnPos - AlarmTip.spaceX;
    printInfo("w_width "..m_width.." widthDiff "..widthDiff)
    if m_width <= widthDiff then
        AlarmTip.text:setPos(AlarmTip.spaceX + (widthDiff - m_width) / 2)
        -- AlarmTip.text:setAlign(kAlignCenter);
        -- local x, y  = AlarmTip.text:getAbsolutePos()
        -- local btnPos = x + AlarmTip.text.m_res.m_width + 10
        return false;
    elseif m_width > widthDiff then
        -- local diffX = m_width - w + AlarmTip.spaceX*3;
        local diffX = m_width - widthDiff
        AlarmTip.text:setPos(AlarmTip.spaceX, 0);
        AlarmTip.text:setClip(AlarmTip.spaceX, 0, widthDiff , h);
        return true, diffX;
    end
end

AlarmTip.startTranslate = function(diffX)
    delete(AlarmTip.animIndex);
    AlarmTip.animIndex = nil;
    AlarmTip.animText = AlarmTip.text:addPropTranslate(1, kAnimNormal, diffX * 15, 0, 0, -diffX, 0, 0);
    if AlarmTip.animText then
        AlarmTip.animText:setDebugName("AlarmTip | AlarmTip.animText");
        AlarmTip.animText:setEvent(AlarmTip.ms/2, AlarmTip.onTimer);
    end
end

-- 定时器
AlarmTip.onTimer = function(time)
    --删除anim
    local anim = AlarmTip.root:addPropTranslate(0, kAnimNormal, AlarmTip.ms/8, time or AlarmTip.ms, 0, -AlarmTip.translateX, 0,-AlarmTip.translateY);
    if anim then
        anim:setDebugName("AlarmTip | anim")
        anim:setEvent(nil, AlarmTip.stop2)
    end
end

-- 暂停动画
AlarmTip.stop =function()
    if  AlarmTip.loaded then   
        AlarmTip.root:setVisible(false);  
        if not AlarmTip.root:checkAddProp(0) then
            AlarmTip.root:removeProp(0);  -- 移除属性
        end
    end

    --删除anim
    if AlarmTip.animIndex then
        delete(AlarmTip.animIndex);
        AlarmTip.animIndex = nil;   
    end

    if AlarmTip.is_play2 then
        table.remove(AlarmTip.textsNodes,1)
    end
end

AlarmTip.stop2 =function()
print_string("onStop!!!!!!!!!!")
    if  AlarmTip.loaded then   
        AlarmTip.root:setVisible(false);  
        if not AlarmTip.root:checkAddProp(0) then
            AlarmTip.root:removeProp(0);  -- 移除属性
        end
    end

    --删除anim
    if AlarmTip.animIndex then
        delete(AlarmTip.animIndex);
        AlarmTip.animIndex = nil;   
    end

    print_string("before removing, #AlarmTip.textsNodes = "..#AlarmTip.textsNodes)

    if not AlarmTip.is_play then
        table.remove(AlarmTip.textsNodes,1)
    end
    AlarmTip.is_play = false

    if #AlarmTip.textsNodes > 0 then
        if AlarmTip.textsNodes[1][2] == 1 then
            AlarmTip.play_texts_btn(AlarmTip.textsNodes[1][1])
        elseif AlarmTip.textsNodes[1][2] == 2 then
            AlarmTip.play_texts(AlarmTip.textsNodes[1][1])
        end
    else
        AlarmTip.is_play2 = false
    end
end

-- 释放资源
AlarmTip.release = function()
    AlarmTip.stop(); 

    if AlarmTip.root then
        local parent = AlarmTip.root:getParent();
        if parent then
            parent:removeChild(AlarmTip.root);
        end
        delete(AlarmTip.root);
        AlarmTip.root = nil;
    end

    AlarmTip.loaded = false;
end