--[[
用户播放帧动画，
imgName，拼图名称，
mapFile，拼图脚本
deltaTime，帧动画时间间隔
repeatTime，重复次数
completeFunc，动画结束后的回调
autoRelease，自动释放资源
--]]

local animFrame = class(Node)

function animFrame:ctor(imgName, mapFile, deltaTime, repeatTime, completeFunc, autoRelease)
	if type(mapFile) ~= type({}) then
		printInfo("packFile is not a table ")
		return
	end
	self.imgName = imgName
	self.deltaTime = deltaTime
	self.repeatTime = repeatTime
    self.completeFunc = completeFunc
    self.autoRelease = (autoRelease == nil or autoRelease)

	self.dirs = {}
	for k, v in pairs(mapFile) do
		local i = #self.dirs + 1
		table.insert(self.dirs, mapFile[string.format("%s_%s.png", imgName, i)])
	end

	self.frames = new(Images, self.dirs)
	self.frames:setImageIndex(0)
	self.frames:setVisible(false)
	self:addChild(self.frames)
	self:setSize(self.frames:getSize())
end

function animFrame:dtor()

end

function animFrame:play()
    --先停掉，避免两动作重叠
    self:stop()
	self.frames:setVisible(true)
	local imgIndex = 0
	local repeatCount = 0
	self.anim = self.frames:addPropTranslate(1, kAnimRepeat, self.deltaTime, 0, 0, 0, 0, 0)
	self.anim:setEvent(nil, function()
        imgIndex = imgIndex + 1
		if imgIndex >= #self.dirs then
			imgIndex = 0
			repeatCount = repeatCount + 1
			if repeatCount >= self.repeatTime then
				self:stop()
                if self.autoRelease then
                    self:removeSelf()
                end
				return
			end
		end
		self.frames:setImageIndex(imgIndex)
	end)
end

function animFrame:stop()
	printInfo("anim stop")
	self.frames:removeProp(1)
	self.frames:setVisible(false)
    if self.completeFunc then
        self.completeFunc()
    end
end

return animFrame