
Batch2d = class(Node);

Batch2d.ctor = function (self,x,y,w,h,pinLua,pinFileTex,...)
    --local argsNum = select("#",...)

    local args = {...};
    local chipsNum = {};
    local chipsTex = {};
    self.m_pinLua = pinLua;
    local numChipsAll = 0;
    if args ~= nil and math.fmod(#args,2) == 0 then
        for i = 1,#args/2 do 
            chipsNum[i] = args[i];
            chipsTex[i] = args[#args/2+i];
            numChipsAll = numChipsAll+ chipsNum[i];
        end
    end
    
    self.m_vertexId = res_alloc_id();
    self.m_indexId  = res_alloc_id();
    self.m_textureId  = res_alloc_id();
    self.m_texcoordId = res_alloc_id();

    res_create_image(0,self.m_textureId,pinFileTex,0,1);

    self:createVertexData(x,y,w,h,numChipsAll,chipsNum,chipsTex);
   
    res_create_double_array(0,self.m_vertexId,self.m_vertex);
    res_create_ushort_array(0,self.m_indexId,self.m_index);   
    res_create_double_array(0,self.m_texcoordId,self.m_texcoord);
    

    drawing_set_node_renderable(self.m_drawingID,0x0007,0x40)
    drawing_set_node_vertex(self.m_drawingID,self.m_vertexId,self.m_indexId);
    drawing_set_node_texture(self.m_drawingID,self.m_textureId,self.m_texcoordId);
end

Batch2d.createVertexData = function (self,x,y,w,h,numChipsAll,chipsNum,chipsTex)
    self.m_vertex = {};
    self.m_index = {};
    self.m_texcoord = {};
    self.m_posOffsetX = {};
    self.m_posOffsetY = {};
    self.m_posX = x;
    self.m_posY = y;
    self.m_width = w;
    self.m_height = h;

    self.m_numChips = numChipsAll;

    self.m_mainTexW = res_get_image_width(self.m_textureId);
    self.m_mainTexH = res_get_image_height(self.m_textureId);

    for i = 1, self.m_numChips do    
        local ran = math.random();
        self.m_vertex[(i-1)*12+1] = x;
        self.m_vertex[(i-1)*12+2] = y-i*0.2;
        self.m_vertex[(i-1)*12+3] = 0.0;
        self.m_vertex[(i-1)*12+4] = x+w;
        self.m_vertex[(i-1)*12+5] = y-i*0.2;
        self.m_vertex[(i-1)*12+6] = 0.0;
        self.m_vertex[(i-1)*12+7] = x+w;
        self.m_vertex[(i-1)*12+8] = y+h-i*0.2;
        self.m_vertex[(i-1)*12+9] = 0.0;
        self.m_vertex[(i-1)*12+10] = x;
        self.m_vertex[(i-1)*12+11] = y+h-i*0.2;
        self.m_vertex[(i-1)*12+12] = 0.0;

        self.m_index[(i-1)*6+1] = (i-1)*4;
        self.m_index[(i-1)*6+2] = (i-1)*4+1;
        self.m_index[(i-1)*6+3] = (i-1)*4+2;
        self.m_index[(i-1)*6+4] = (i-1)*4;
        self.m_index[(i-1)*6+5] = (i-1)*4+2;
        self.m_index[(i-1)*6+6] = (i-1)*4+3; 

    end

    local startIndex = 1;
    for i = 1,#chipsNum do
        for j = 1,chipsNum[i] do
            self.m_texcoord[(startIndex-1)*8+1] = self.m_pinLua[chipsTex[i]].x/self.m_mainTexW;
            self.m_texcoord[(startIndex-1)*8+2] = self.m_pinLua[chipsTex[i]].y/self.m_mainTexH;
            self.m_texcoord[(startIndex-1)*8+3] = (self.m_pinLua[chipsTex[i]].x+self.m_pinLua[chipsTex[i]].width)/self.m_mainTexW;
            self.m_texcoord[(startIndex-1)*8+4] = self.m_pinLua[chipsTex[i]].y/self.m_mainTexH;
            self.m_texcoord[(startIndex-1)*8+5] = (self.m_pinLua[chipsTex[i]].x+self.m_pinLua[chipsTex[i]].width)/self.m_mainTexW;
            self.m_texcoord[(startIndex-1)*8+6] = (self.m_pinLua[chipsTex[i]].y+self.m_pinLua[chipsTex[i]].height)/self.m_mainTexH;
            self.m_texcoord[(startIndex-1)*8+7] = self.m_pinLua[chipsTex[i]].x/self.m_mainTexW;
            self.m_texcoord[(startIndex-1)*8+8] = (self.m_pinLua[chipsTex[i]].y+self.m_pinLua[chipsTex[i]].height)/self.m_mainTexH;
            startIndex = startIndex+1;
        end
    end
end

Batch2d.start = function (self,duration)
    self.m_animFrame = new(AnimInt,kAnimRepeat,0,1,16,0); 
    self.m_animFrame:setEvent(self,self.update);
    self.m_animTimer = new(AnimInt,kAnimNormal,1,60,duration,0);
    self.m_animTimer:setEvent(self,self.updatePos);
end

Batch2d.update = function (self)
    local step = self.m_animTimer:getCurValue();
    self:updateVertex(step);     
end

Batch2d.updateVertex = function (self, step)

    self.m_currentVertex = {};
    for i = 1, self.m_numChips do
        self.m_currentVertex[(i-1)*12+1] = self.m_vertex[(i-1)*12+1]+self.m_posOffsetX[i]*step;
        self.m_currentVertex[(i-1)*12+2] = self.m_vertex[(i-1)*12+2]+self.m_posOffsetY[i]*step;
        self.m_currentVertex[(i-1)*12+3] = 0.0;
        self.m_currentVertex[(i-1)*12+4] = self.m_vertex[(i-1)*12+4]+self.m_posOffsetX[i]*step;
        self.m_currentVertex[(i-1)*12+5] = self.m_vertex[(i-1)*12+5]+self.m_posOffsetY[i]*step;
        self.m_currentVertex[(i-1)*12+6] = 0.0;
        self.m_currentVertex[(i-1)*12+7] = self.m_vertex[(i-1)*12+7]+self.m_posOffsetX[i]*step;
        self.m_currentVertex[(i-1)*12+8] = self.m_vertex[(i-1)*12+8]+self.m_posOffsetY[i]*step;
        self.m_currentVertex[(i-1)*12+9] = 0.0;
        self.m_currentVertex[(i-1)*12+10] = self.m_vertex[(i-1)*12+10]+self.m_posOffsetX[i]*step;
        self.m_currentVertex[(i-1)*12+11] = self.m_vertex[(i-1)*12+11]+self.m_posOffsetY[i]*step;
        self.m_currentVertex[(i-1)*12+12] = 0.0;
    end

    res_set_double_array(self.m_vertexId,self.m_currentVertex);
end


Batch2d.setScatterPos = function (self,posX,posY,scatterRadius) 
    self.m_destPosX = posX
    self.m_destPosY = posY
    
    local offsetX = self.m_destPosX-self.m_posX;
    local offsetY = self.m_destPosY-self.m_posY;

    for i = 1,self.m_numChips do
        self.m_posOffsetX[i] = (offsetX + math.random()*scatterRadius*math.sin(math.random()*6.28))/60;
        self.m_posOffsetY[i] = (offsetY + math.random()*scatterRadius*math.cos(math.random()*6.28))/60;  
    end  
end

Batch2d.setAssemblePos = function (self,posX,posY)
    for i = 1,self.m_numChips do 
        self.m_posOffsetX[i] = (posX - self.m_vertex[(i-1)*12+1])/60; 
        self.m_posOffsetY[i] = (posY - self.m_vertex[(i-1)*12+2])/60;
    end

    self.m_destPosX = posX
    self.m_destPosY = posY
end

Batch2d.updatePos = function (self)
     self.m_animFrame:dtor();
     self.m_animTimer:dtor();
     self.m_posX = self.m_destPosX;
     self.m_posY = self.m_destPosY;
     self.m_vertex = self.m_currentVertex;
end 

Batch2d.addChips = function (self,texture,x,y)
    self.m_numChips = self.m_numChips + 1; 
    
    table.insert(self.m_vertex,self.m_posX+x);
    table.insert(self.m_vertex,self.m_posY+y);
    table.insert(self.m_vertex,0);
    table.insert(self.m_vertex,self.m_posX+self.m_width+x);
    table.insert(self.m_vertex,self.m_posY+y);
    table.insert(self.m_vertex,0);
    table.insert(self.m_vertex,self.m_posX+self.m_width+x);
    table.insert(self.m_vertex,self.m_posY+self.m_height+y);
    table.insert(self.m_vertex,0);
    table.insert(self.m_vertex,self.m_posX+x);
    table.insert(self.m_vertex,self.m_posY+self.m_height+y);
    table.insert(self.m_vertex,0);

    table.insert(self.m_index,(self.m_numChips-1)*4);
    table.insert(self.m_index,(self.m_numChips-1)*4+1);
    table.insert(self.m_index,(self.m_numChips-1)*4+2);
    table.insert(self.m_index,(self.m_numChips-1)*4);
    table.insert(self.m_index,(self.m_numChips-1)*4+2);
    table.insert(self.m_index,(self.m_numChips-1)*4+3);

    table.insert(self.m_texcoord,self.m_pinLua[texture].x/self.m_mainTexW);
    table.insert(self.m_texcoord,self.m_pinLua[texture].y/self.m_mainTexH);
    table.insert(self.m_texcoord,(self.m_pinLua[texture].x+self.m_pinLua[texture].width)/self.m_mainTexW);
    table.insert(self.m_texcoord,self.m_pinLua[texture].y/self.m_mainTexH);
    table.insert(self.m_texcoord,(self.m_pinLua[texture].x+self.m_pinLua[texture].width)/self.m_mainTexW);
    table.insert(self.m_texcoord,(self.m_pinLua[texture].y+self.m_pinLua[texture].height)/self.m_mainTexH);
    table.insert(self.m_texcoord,self.m_pinLua[texture].x/self.m_mainTexW);
    table.insert(self.m_texcoord,(self.m_pinLua[texture].y+self.m_pinLua[texture].height)/self.m_mainTexH);

    res_set_double_array(self.m_vertexId,self.m_vertex);
    res_set_ushort_array(self.m_indexId,self.m_index);
    res_set_double_array(self.m_texcoordId,self.m_texcoord);
    --table.insert(self.m_posOffsetX,(self.m_destPosX-self.m_posX + math.random()*100*math.sin(math.random()*6.28))/60);
    --table.insert(self.m_posOffsetY,(self.m_destPosY-self.m_posY + math.random()*100*math.sin(math.random()*6.28))/60);
    
end

--[[Batch2d.removeChips = function (self,chipIndex)
    for i = 1, 12 do 
        table.remove(self.m_vertex,#self.m_vertex);
    end

    res_set_double_array(self.m_vertexId,self.m_vertex);

    for i = 1, 6 do
        table.remove(self.m_index,#self.m_index);
    end

    res_set_ushort_array(self.m_indexId,self.m_index);

    for i = 1, 8 do 
        table.remove(self.m_texcoord,#self.m_texcoord);
    end

    res_set_double_array(self.m_texcoordId,self.m_texcoord);
end]]--

Batch2d.release = function (self)
 
    res_delete(self.m_vertexId);
    self.m_vertexId = nil;
    self.m_vertex = nil;
    res_delete(self.m_indexId);
    self.m_indexId = nil;
    self.m_index = nil;
    res_delete(self.m_texcoordId);
    self.m_texcoordId = nil;
    self.m_texcoord = nil
    res_delete(self.m_textureId);

end

