
-- NativeEvent.lua
-- 本地事件方法
NativeEvent = class();
local printInfo, printError = overridePrint("NativeEvent")

NativeEvent.s_platform = System.getPlatform();

NativeEvent.getInstance = function()
	if not NativeEvent.s_instance then 
		NativeEvent.s_instance = new(NativeEvent);
	end
	return NativeEvent.s_instance;
end

NativeEvent.onEventCall = function()
	local callParam, json_data, resultStr = NativeEvent.getInstance():getNativeCallResult();
	-- -- 全局有效的本地方法 在此处拦截
	printInfo("java 调用 lua 方法 callParam = %s, status = %s", callParam or "null", status or -1)
	NativeEvent.getInstance():onNativeCallDone(callParam, json_data, resultStr);
end

NativeEvent.onEventPause = function()
	audio_music_pause()
	EventDispatcher.getInstance():dispatch(Event.onEventPause);--jaywillou-20160126-add-抛出pause事件
end

-- application come to foreground 
-- 注意容错 
NativeEvent.onEventResume = function()
	if GameSetting then
    	GameSetting:setIsSecondScene(false)
	end
	audio_music_resume()
	EventDispatcher.getInstance():dispatch(Event.onEventResume);--jaywillou-20160126-add-抛出resume事件
	-- 发送心跳包测试
	if GameSocketMgr then
		GameSocketMgr:onHeartBeat()
	end
end

NativeEvent.onNativeCallDone = function(self, param, ...)
	if NativeEvent.callEventFuncMap[param] then
		NativeEvent.callEventFuncMap[param](self, ...);
	else
		EventDispatcher.getInstance():dispatch(Event.Call, param,...);
	end
end

-- 解析 call_native 返回值
NativeEvent.getNativeCallResult = function(self)
	local callParam 	= dict_get_string(kLuaCallEvent, kLuaCallEvent);
	local callResult 	= dict_get_int(callParam, kCallResult, -1);
    if callResult ~= 0 then -- 获取数值失败
        return callParam , nil, false;
    end
    local result = dict_get_string(callParam, callParam .. kResultPostfix);
    dict_delete(callParam);
    local json_data = json.decode_node(result);
    return callParam, json_data, result;
end

NativeEvent.onWinKeyDown = function(key)
	printInfo("onWinKeyDown" .. key)
	if key == 60 or key == 81 then --Q 键返回
		EventDispatcher.getInstance():dispatch(Event.Back);
	elseif key == 55 or key == 76 then -- L键 debug
		dofile("../Resource/scripts/debug.lua")
	else
		EventDispatcher.getInstance():dispatch(Event.KeyDown,key);
	end
end

NativeEvent.onTimeOut = function(self)
	self:stopTimeOutAnim();
	if kCurrentState == States.Room and kNetConnect ~= 1 then
		SingleSocketMgr.getInstance():onTimeout();
	end
end

NativeEvent.stopTimeOutAnim = function(self)
	delete(self.timeOutAnim)
	self.timeOutAnim = nil;
end

NativeEvent.startTimeOutAnim = function(self)
	-- 10s后网络没连接
	if self.timeOutAnim then
		delete(self.timeOutAnim);
	end
	self.timeOutAnim = new(AnimInt, kAnimNormal, 0, 1, 5000, 0);
	self.timeOutAnim:setDebugName("AnimInt | self.timeOutAnim")
	self.timeOutAnim:setEvent(self, self.onTimeOut);
end

--/////////////////////////////// android //////////////////////////////////

if NativeEvent.s_platform == kPlatformAndroid then
	-- 公共call_native 方法
	NativeEvent.callNativeEvent = function(self, keyParm, data)
		if data then
			dict_set_string(keyParm, keyParm .. kparmPostfix, data);
		end
		dict_set_string(kLuaCallEvent, kLuaCallEvent, keyParm);
		call_native("OnLuaCall");
	end

	-- 立即返回结果的的操作 
	NativeEvent.encodeStr = function(self, str)
		-- local key = "encodeStr"
		-- self:callNativeEvent(key, str)
	 --   	return dict_get_string(key, key .. kResultPostfix);
	 	return str
	end

	NativeEvent.compressString = function(self, str)
		local key = "compressString"
		self:callNativeEvent(key, str)
		return dict_get_string(key, key .. kResultPostfix)
	end

	NativeEvent.unCompressString = function(self, str)
		local key = "unCompressString"
		self:callNativeEvent(key, str)
		return dict_get_string(key, key .. kResultPostfix)
	end

	NativeEvent.isFileExist = function(self, filePath, fileFolder)
		local key = "isFileExist"
		local tab = {}
		tab.path = filePath
		tab.folder = fileFolder
		self:callNativeEvent(key, json.encode(tab))
		return dict_get_int(key, key .. kResultPostfix, 0)
	end

	-- 涉及ui的操作 放在消息队列中执行
	--延时释放 defualt png , 解决 Android 游戏启动时屏幕黑一下的问题
	NativeEvent.CloseStartScreen = function(self)
		local key = "CloseStartScreen"
		self:callNativeEvent(key)
	end

    NativeEvent.GetInitValue =function(self)
    	local key = "GetInitValue"
    	self:callNativeEvent(key)
	    return dict_get_string(key, key.."_result");
    end

    NativeEvent.GetNetAvaliable = function(self)
    	local key = "GetNetAvaliable"
    	self:callNativeEvent(key)
	    return dict_get_int(key, key..kResultPostfix, 0);
   	end

    NativeEvent.PreloadAllSound = function(self, musicsMap, effectsMap)
	    local key = "LoadSoundRes"
		local tab={};
	    tab.bgMusic = musicsMap;
	    tab.soundRes = effectsMap;
	    self:callNativeEvent(key, json.encode(tab))
	end

	NativeEvent.ReportLuaError = function(self, faultInfo)
		local key = "ReportLuaError"
		local tab = {};
		tab.faultInfo = faultInfo;
	    local json_data = json.encode(tab);
		self:callNativeEvent(key, json_data);
	end

	NativeEvent.Exit = function(self)
		local key = "Exit"
		self:callNativeEvent(key)
 	end

	NativeEvent.StartUnitePay = function(self, param_data)
		local key = "unitePay"
		local json_data = json.encode(param_data)
		self:callNativeEvent(key, json_data)
	end

	-- 拍照 
	NativeEvent.takePhoto = function(self, userId)
		self:callNativeEvent(kTakePhoto, json.encode({name = tostring(userId)}));
	end

	--选择图片
	NativeEvent.pickPhoto = function(self, userId)
		self:callNativeEvent(kPickPhoto, json.encode({name = tostring(userId)}));
	end

	--上传图片
	NativeEvent.uploadImage = function(self, url, uploadImageName, doType)
        local api = PhpManager:getAPI();
		api['do'] = doType;
		api['sid'] = PhpManager:getGame();
		api['time'] = os.time()
		api['sig'] = md5_string(MyUserData:getId() .. '|' .. api['sid'] .. '|' .. api['time'] .. '~#kevin&^$xie$&boyaa')
		api['upload'] = uploadImageName
		local sid = api['sid']
		local time = api['time']
		local sig = api['sig']
		local upload = api['upload']
		self:callNativeEvent(kUploadImage, json.encode({url = url, sid = sid, mid = MyUserData:getId(), time = time, sig = sig, upload = upload, uploadImageName = uploadImageName, api = api, ['type'] = 0}));
	end

	--上传图片
	NativeEvent.uploadFeedbackImage = function(self, url, uploadImageName, api)
		self:callNativeEvent(kUploadFeedbackImage, json.encode({url = url, uploadImageName = uploadImageName, api = api, ['type'] = 1}));
	end

	--打开网页
	NativeEvent.openWeb = function(self, url)

	    local api=PhpManager:getAPI();
		self:callNativeEvent(kOpenWeb, json.encode({url = url, api = api}));
	end

	--打开网页
	NativeEvent.closeWeb = function(self)
		self:callNativeEvent(kCloseWeb, json.encode({}));
	end

	--打开条款
	NativeEvent.openRule = function(self, ruleUrl)
		self:callNativeEvent(kOpenRule, json.encode({url = ruleUrl,api = PhpManager:getAPI()}));
	end

	--关闭条款
	NativeEvent.closeRule = function(self)
		self:callNativeEvent(kCloseRule, json.encode({}));
	end
	--打开粉丝页
	NativeEvent.openLink = function(self, url)
		self:callNativeEvent(kOpenLink, json.encode({url = url}));
	end


	--打开第三方登录
	NativeEvent.login = function(self, param)
		-- 包装登录方式
		param = param or {}
		table.merge(param, { loginType = MyUserData:getUserType() })
		self:callNativeEvent(kLogin, json.encode(param));
		--
		-- NativeEvent.getInstance():login();
	end

	--分享
	NativeEvent.share = function(self, param)
		self:callNativeEvent(kShare, json.encode(param))
	end

	--获取clientId
	NativeEvent.getClientId = function(self, param)
		self:callNativeEvent(kClientId, json.encode(param))
	end
	

	--评分
	NativeEvent.score = function(self, param)
		self:callNativeEvent(kScore, json.encode(param))
	end

	NativeEvent.boyaaAd = function(self, param)
		self:callNativeEvent('boyaaAd', json.encode(param))
	end

	--注销
	NativeEvent.logout = function(self, param)
		-- 包装登录方式
		param = param or {}
		table.merge(param, { loginType = MyUserData:getUserType() })
		self:callNativeEvent(kLogout, json.encode(param));
	end

	--获取Facebook好友
	NativeEvent.getFbFriend = function(self, param)
		self:callNativeEvent(kGetFbFriend, "");
	end
	--获取Facebook 信息
	NativeEvent.getFbAppInfo = function(self, param)
		self:callNativeEvent('getFbAppInfo', "");
	end

	--邀请好友
	NativeEvent.inviteFbFriend = function(self, param)
		self:callNativeEvent(kInviteFbFriend, json.encode(param));
	end
	--邀请短信好友
	NativeEvent.inviteSmsFriend = function(self, param)
		self:callNativeEvent(kInviteSmsFriend, json.encode(param));
	end

	--本地更新
	NativeEvent.UpdateByLocal = function(self, url, force)
		local param = {
						url 	= url,
						force 	= force or 0,
					};

		self:callNativeEvent(kUpdateByLocal , json.encode(param));
	end
	--友盟更新
	NativeEvent.UpdateByUmeng = function(self, isActUpdata, isDeltaUpdate, isForceUpdate, isCheckForProcess)
		local param = {
						isActUpdata 		= isActUpdata,
						isDeltaUpdate 		= isDeltaUpdate,
						isForceUpdate 		= isForceUpdate,
						isCheckForProcess 	= isCheckForProcess,
					};

		self:callNativeEvent(kUpdateByUmeng , json.encode(param));
	end

	--检查本地更新
	NativeEvent.CheckPackByLocal = function(self, url)
		local param = {
						url 	= url,
					};

		self:callNativeEvent(kCheckPackByLocal , json.encode(param));
	end
	

	NativeEvent.downloadImage = function(self, param)
		self:callNativeEvent(kDownloadImage , json.encode(param));
	end

	--重命名图片
	NativeEvent.renameImage = function(self, oldName, newName, url)

		local param = {
						oldName = oldName,
						newName = newName,
						url 	= url;
					};

		self:callNativeEvent(kRenameImage , json.encode(param));
	end

	--下载文件
	--type: 'image' or 'audio'
	NativeEvent.downloadFile = function(self, type, url, file, tag)
		printInfo("开始下载文件")
		local param = {
						file 	= file,
						type 	= type,
						url 	= url,
						tag 	= tag,
					};

		self:callNativeEvent(kDownloadFile , json.encode(param));
	end

	--解压文件
	--type: 'image' or 'audio'
	NativeEvent.unzip = function(self, type, path, file, tag)

		local param = {
						file 	= file,
						type 	= type,
						path 	= path,
						tag 	= tag,
					};

		self:callNativeEvent(kUnzip , json.encode(param));
	end

	--解压游戏
	NativeEvent.unzipGame = function(self, file)

		local param = {
						file 	= file
					};

		self:callNativeEvent(kUnzipGame , json.encode(param));
	end

	 --传支付配置
    NativeEvent.payConfig = function(self, param)
        local key = "payConfig";
        self:callNativeEvent(key, json.encode(param));
    end

    NativeEvent.startPay = function(self, param)
    	local key = "startPay"
    	self:callNativeEvent(key, json.encode(param))
   	end

   	 --支付
    NativeEvent.pay = function(self, param)
        self:callNativeEvent(kPay, json.encode(param));
    end
    --友盟统计
    NativeEvent.collectByUmeng = function(self, param)
        self:callNativeEvent(kCollectByUmeng, param);
    end
    --推广
    NativeEvent.loadAdData = function(self, userId, isShowAd)
        self:callNativeEvent(kLoadAdData, json.encode({userId = userId, isShowAd = isShowAd}))
    end
    NativeEvent.showAd = function(self, isFloat)
        self:callNativeEvent(kShowAd, json.encode({isFloat = isFloat}))
    end

    --获取Facebook好友
    --打开活动中心
	NativeEvent.openActivity = function(self, param)
        local activityParam = {
        	mid 	= MyUserData:getId(),
			version = PhpManager:getVersionName(),
			sid 	= PhpManager:getGame(),
			appId 	= ACTIVITY_APPID,
			sitemid = string.format(PhpManager:getDevice_id()..'_gamehallboyaa'),
			userType= MyUserData:getUserType(),
	        secret_key = ACTIVITY_SECRET_KEY,
	        url = LOCAL_NET and "http://192.168.204.68/operating/web/index.php" or ACTIVITY_URL,
			debug = LOCAL_NET and 1 or 0,
	        language = 6,   --0中文 1泰语 2印尼
	        isReleated = param and param.isReleated or -1, -- -1是默认的，打开活动中心，0、1、2是打开强推界面，对应小中大窗口
            channelID = "", --目前我们不需要这个参数来区分，但是sdk里必须要传

        }
		self:callNativeEvent(kActivity, json.encode(activityParam));
	end


	--录制及回放
	NativeEvent.startRecord = function(self)
		self:callNativeEvent(kStartRecord , "");
	end
	NativeEvent.stopRecord = function(self)
		self:callNativeEvent(kStopRecord , "");
	end
	NativeEvent.playBack = function(self)
		self:callNativeEvent(kPlayBack , "");
	end
	NativeEvent.stopPlayBack = function(self)
		self:callNativeEvent(kStopPlayBack , "");
	end

	NativeEvent.hideEditTextView = function(self)
		self:callNativeEvent("hideEditTextView");
	end

    NativeEvent.shareToMessenger = function(self, param)
        self:callNativeEvent(kShareToMessenger, json.encode(param));
    end
    
    NativeEvent.isMessengerExist = function(self)
        local key = "isMessengerExist"
        self:callNativeEvent(key);
        return dict_get_int(key, key .. kResultPostfix, 0)
    end
    --获取启动方式，默认0是点击图标启动，1是从fb messenger启动，用于统计启动方式
    NativeEvent.getStartWay = function(self)
        local key = "getStartWay"
        self:callNativeEvent(key)
        return dict_get_string(key, key..kResultPostfix)
    end

    NativeEvent.getAllUpdateFile = function(self, param)
        local key = "getAllUpdateFile"
        self:callNativeEvent(key)
        return dict_get_string(key, key..kResultPostfix)
    end

    NativeEvent.getNetworkType = function(self)
        local key = "getNetworkType"
        self:callNativeEvent(key)
        return dict_get_int(key, key..kResultPostfix, 0)
    end

    NativeEvent.backupScripts = function(self)
        local key = "backupScripts"
        self:callNativeEvent(key)
        return dict_get_int(key, key..kResultPostfix, 0)
    end
    NativeEvent.updateGameFinish = function(self, param)
        local key = "updateGameFinish"
        self:callNativeEvent(key, json.encode(param))
        return dict_get_int(key, key..kResultPostfix, 0)
    end
    NativeEvent.deleteFile = function(self, param)
        local key = "deleteFile"
        self:callNativeEvent(key, json.encode(param))
    end
    NativeEvent.getLocation = function(self)
        local key = "getLocation"
        self:callNativeEvent(key)
        return dict_get_string(key, key..kResultPostfix)
    end
    NativeEvent.checkUnfinishIAP = function(self)
        self:callNativeEvent("checkUnfinishIAP")
    end
     NativeEvent.consumeProduct = function(self, productId)
   	  	local key = "consumeProduct"
        self:callNativeEvent(key, json.encode({productId = productId}))
    end
    NativeEvent.getFileMD5 = function(self, param)
        local key = "getFileMD5"
        self:callNativeEvent(key, json.encode(param))
        return dict_get_string(key, key..kResultPostfix)
    end
    NativeEvent.getPermission = function(self, param)
        local key = "getPermission"
        self:callNativeEvent(key, json.encode(param))
        return dict_get_string(key, key..kResultPostfix)
    end
    NativeEvent.downloadUpdateApk = function(self, param)
        print_string("NativeEvent.downloadUpdateApk ")
        self:callNativeEvent("downloadUpdateApk", json.encode(param))
    end
    NativeEvent.pauseDownloadUpdateApk = function(self, param)
        self:callNativeEvent("pauseDownloadUpdateApk",json.encode(param));
    end
    NativeEvent.installUpdateApk = function(self, param)
        self:callNativeEvent("installUpdateApk",json.encode(param));
    end
    NativeEvent.queryDownloadProgress = function(self, param)
        local key = "queryDownloadProgress"
        self:callNativeEvent(key,json.encode(param))
        return dict_get_int(key, key..kResultPostfix, 0)
    end
    NativeEvent.queryDownloadStatus = function(self, param)
        local key = "queryDownloadStatus"
        self:callNativeEvent(key,json.encode(param))
        return dict_get_int(key, key..kResultPostfix, 0)
    end
    NativeEvent.deleteDownloadTask = function(self, param)
        local key = "deleteDownloadTask"
        self:callNativeEvent(key,json.encode(param))
        return dict_get_int(key, key..kResultPostfix, 1)
    end
end



if NativeEvent.s_platform == kPlatformIOS then
	-- 公共call_native 方法
	NativeEvent.callNativeEvent = function(self, keyParm, data)
		if data then
			dict_set_string(keyParm, keyParm .. kparmPostfix, data);
		end
		dict_set_string(kLuaCallEvent, kLuaCallEvent, keyParm);
		call_native("OnLuaCall");
	end

	-- 立即返回结果的的操作 
	NativeEvent.encodeStr = function(self, str)
		-- local key = "encodeStr"
		-- self:callNativeEvent(key, str)
	 --    return dict_get_string(key, key .. kResultPostfix);
	 	return str
	end

	NativeEvent.compressString = function(self, str)
		local key = "compressString"
		self:callNativeEvent(key, str)
		return dict_get_string(key, key .. kResultPostfix)
	end

	NativeEvent.unCompressString = function(self, str)
		local key = "unCompressString"
		self:callNativeEvent(key, str)
		return dict_get_string(key, key .. kResultPostfix)
	end

	NativeEvent.isFileExist = function(self, filePath, fileFolder)
        --ios
		local key = "isFileExist"
		local tab = {}
		tab.path = filePath
		tab.folder = fileFolder
		self:callNativeEvent(key, json.encode(tab))
		return dict_get_int(key, key .. kResultPostfix, 0)
	end

	-- 涉及ui的操作 放在消息队列中执行
	--延时释放 defualt png , 解决 Android 游戏启动时屏幕黑一下的问题
	NativeEvent.CloseStartScreen = function(self)
		local key = "CloseStartScreen"
		self:callNativeEvent(key)
	end

    NativeEvent.GetInitValue =function(self)
    	local key = "GetInitValue"
    	self:callNativeEvent(key)
	    return dict_get_string(key, key.."_result");
    end

    NativeEvent.GetNetAvaliable = function(self)
    	local key = "GetNetAvaliable"
    	self:callNativeEvent(key)
	    return dict_get_int(key, key..kResultPostfix, 0);
   	end

    NativeEvent.PreloadAllSound = function(self, musicsMap, effectsMap)
	    local key = "LoadSoundRes"
		local tab={};
	    tab.bgMusic = musicsMap;
	    tab.soundRes = effectsMap;
	    self:callNativeEvent(key, json.encode(tab))
	end

	NativeEvent.ReportLuaError = function(self, faultInfo)
		local key = "ReportLuaError"
		local tab = {};
		tab.faultInfo = faultInfo;
	    local json_data = json.encode(tab);
		self:callNativeEvent(key, json_data);
	end

	NativeEvent.Exit = function(self)
		local key = "Exit"
		self:callNativeEvent(key)
 	end

	NativeEvent.StartUnitePay = function(self, param_data)
		local key = "unitePay"
		local json_data = json.encode(param_data)
		self:callNativeEvent(key, json_data)
	end

	-- 拍照 
	NativeEvent.takePhoto = function(self, name,folder,basePath,imgName,tag)
		self:callNativeEvent(kTakePhoto, json.encode({name = name,folder = folder,basePath = basePath,tag = tag,imgName = imgName}));
	end

	--选择图片
	NativeEvent.pickPhoto = function(self, name,folder,basePath,imgName,tag)

		self:callNativeEvent(kPickPhoto, json.encode({name = name,folder = folder,basePath = basePath,tag = tag,imgName = imgName}));
	end

	--上传图片
	NativeEvent.uploadImage = function(self, url, uploadImageName, doType)
		local api = PhpManager:getAPI();
		api['do'] = doType;
		api['sid'] = PhpManager:getGame();
		api['time'] = os.time()
		api['sig'] = md5_string(MyUserData:getId() .. '|' .. api['sid'] .. '|' .. api['time'] .. '~#kevin&^$xie$&boyaa')
		api['upload'] = uploadImageName
		local sid = api['sid']
		local time = api['time']
		local sig = api['sig']
		local upload = api['upload']
		self:callNativeEvent(kUploadImage, json.encode({url = url, sid = sid, mid = MyUserData:getId(), time = time, sig = sig, upload = upload, uploadImageName = uploadImageName, api = api, ['type'] = 0}));
	end

	-- 上传头像
	NativeEvent.uploadHeadImage = function(self,url,imageName,doType,basePath,folder)
		basePath = basePath or ""
		folder = folder or ""

		local api = PhpManager:getAPI();
		api['do'] = doType;
		api['sid'] = 102;
		api['time'] = os.time()
		api['sig'] = md5_string(MyUserData:getId() .. '|' .. api['sid'] .. '|' .. api['time'] .. '~#kevin&^$xie$&boyaa')
		api['upload'] = imageName

		local sid = api['sid']
		local time = api['time']
		local sig = api['sig']
		local upload = api['upload']

		self:callNativeEvent("uploadHeadImage", json.encode({url = url, sid = sid, mid = MyUserData:getId(), time = time, sig = sig, upload = upload, imgName = imageName, api = api, ['type'] = 0,basePath = basePath,folder = folder,fileKey = "upload"}));

	end

	--上传图片
	NativeEvent.uploadFeedbackImage = function(self, url, imgName, api,basePath,folder)
		api = json.encode(api)
		self:callNativeEvent(kUploadFeedbackImage, json.encode({url = url, imgName = imgName, api = api,basePath = basePath,folder = folder,fileKey = "pfile"}));
	end

	--打开网页
	NativeEvent.openWeb = function(self, url)

	    local api=PhpManager:getAPI();
		self:callNativeEvent(kOpenWeb, json.encode({url = url, api = api}));
	end

	--打开网页
	NativeEvent.closeWeb = function(self)
		self:callNativeEvent(kCloseWeb, json.encode({}));
	end

	--打开条款
	NativeEvent.openRule = function(self, ruleUrl)
		self:callNativeEvent(kOpenRule, json.encode({url = ruleUrl,api = PhpManager:getAPI()}));
	end

	--关闭条款
	NativeEvent.closeRule = function(self)
		self:callNativeEvent(kCloseRule, json.encode({}));
	end
	--打开粉丝页
	NativeEvent.openLink = function(self, url)
		self:callNativeEvent(kOpenLink, json.encode({url = url}));
	end

	--打开第三方登录
	NativeEvent.login = function(self, param)
		-- 包装登录方式
		param = param or {}
		table.merge(param, { loginType = MyUserData:getUserType() })
		self:callNativeEvent(kLogin, json.encode(param));
		--
		-- NativeEvent.getInstance():login();
	end

	--分享
	NativeEvent.share = function(self, param)
		self:callNativeEvent(kShare, json.encode(param))
	end

	--获取clientId
	NativeEvent.getClientId = function(self, param)
		self:callNativeEvent(kClientId, json.encode(param))
	end
	

	--评分
	NativeEvent.score = function(self, param)
        local appid = {
            ["501"] = "1091505360",
            ["505"] = "670832349",
            ["506"] = "1116009488",
            ["508"] = "1125314410",
        }
--		local appid = "1091505360"  -- 骰子大厅appstore应用ID
		local url = string.format("itms-apps://itunes.apple.com/us/app/id%s?mt=8",appid[PhpManager:getGame()] or "1091505360")
		self:callNativeEvent(kOpenLink, json.encode({url = url}));
	end

	NativeEvent.boyaaAd = function(self, param)
		self:callNativeEvent('boyaaAd', json.encode(param))
	end

	--注销
	NativeEvent.logout = function(self, param)
		-- 包装登录方式
        if MyUserData:getUserType() == UserType.Facebook then
		    param = param or {}
		    table.merge(param, { loginType = MyUserData:getUserType() })
		    self:callNativeEvent(kLogout, json.encode(param));
        end
	end

	--获取Facebook好友
	NativeEvent.getFbFriend = function(self, param)
		self:callNativeEvent(kGetFbFriend, "");
	end
	--获取Facebook 信息
	NativeEvent.getFbAppInfo = function(self, param)
		self:callNativeEvent('getFbAppInfo', "");
	end

	--邀请好友
	NativeEvent.inviteFbFriend = function(self, param)
		self:callNativeEvent(kInviteFbFriend, json.encode(param));
	end
	--邀请短信好友
	NativeEvent.inviteSmsFriend = function(self, param)
		self:callNativeEvent(kInviteSmsFriend, json.encode(param));
	end

	--本地更新
	NativeEvent.UpdateByLocal = function(self, url, force)
		local param = {
						url 	= url,
						force 	= force or 0,
					};

		self:callNativeEvent(kUpdateByLocal , json.encode(param));
	end
	--友盟更新
	NativeEvent.UpdateByUmeng = function(self, isActUpdata, isDeltaUpdate, isForceUpdate, isCheckForProcess)
		local param = {
						isActUpdata 		= isActUpdata,
						isDeltaUpdate 		= isDeltaUpdate,
						isForceUpdate 		= isForceUpdate,
						isCheckForProcess 	= isCheckForProcess,
					};

		self:callNativeEvent(kUpdateByUmeng , json.encode(param));
	end

	--检查本地更新
	NativeEvent.CheckPackByLocal = function(self, url)
		local param = {
						url 	= url,
					};

		self:callNativeEvent(kCheckPackByLocal , json.encode(param));
	end
	

    NativeEvent.downloadImage = function(self, param)

		self:callNativeEvent(kDownloadImage , json.encode(param));
	end

	--重命名图片
	NativeEvent.renameImage = function(self, oldName, newName, url)

		local param = {
						oldName = oldName,
						newName = newName,
						url 	= url;
					};

		self:callNativeEvent(kRenameImage , json.encode(param));
	end

	--下载文件
	--type: 'image' or 'audio'
	NativeEvent.downloadFile = function(self, type, url, file, tag,basePath,folder)
		printInfo("开始下载文件")
		-- print(debug.traceback("", 2))
		local param = {
						file 	= file,
						type 	= type,
						url 	= url,
						tag 	= tag,
					};
		if basePath then
			param.basePath = basePath
		end

		if folder then
			param.folder = folder
		end
		self:callNativeEvent(kDownloadFile , json.encode(param));
	end

	--解压文件
	--type: 'image' or 'audio'
	NativeEvent.unzip = function(self, type, path, file, tag)
		-- print(debug.traceback("", 2))
		local param = {
						file 	= file,
						type 	= type,
						path 	= path,
						tag 	= tag,
					};

		self:callNativeEvent(kUnzip , json.encode(param));
	end

	--解压游戏
	NativeEvent.unzipGame = function(self, file)

		local param = {
						file 	= file
					};

		self:callNativeEvent(kUnzipGame , json.encode(param));
	end

	 --传支付配置
    NativeEvent.payConfig = function(self, param)
        local key = "payConfig";
        self:callNativeEvent(key, json.encode(param));
    end

    NativeEvent.startPay = function(self, param)
    	local key = "startPay"
    	self:callNativeEvent(key, json.encode(param))
   	end

   	 --支付
    NativeEvent.pay = function(self, param)
        self:callNativeEvent(kPay, json.encode(param));
    end

    NativeEvent.IosPayResultCallback = function(self,orderId)
    	self:callNativeEvent("IosPayResultCallback",orderId)
    end

    NativeEvent.checkUnfinishIAP = function(self)
    	dump("checkUnfinishIAP=====")
    	self:callNativeEvent("checkUnfinishIAP","")
    end



    --友盟统计
    NativeEvent.collectByUmeng = function(self, param)
        self:callNativeEvent(kCollectByUmeng, param);
    end
    --推广
    NativeEvent.loadAdData = function(self, userId, isShowAd)
        self:callNativeEvent(kLoadAdData, json.encode({userId = userId, isShowAd = isShowAd}))
    end
    NativeEvent.showAd = function(self, isFloat)
        self:callNativeEvent(kShowAd, json.encode({isFloat = isFloat}))
    end

    --IOS打开活动中心
	NativeEvent.openActivity = function(self, param)
        local activityParam = {
            mid 	= MyUserData:getId(),
			version = PhpManager:getVersionName(),
            sid     = PhpManager:getGame(),
			appId 	= ACTIVITY_APPID,
			sitemid = string.format(PhpManager:getDevice_id()..'_gamehallboyaa'),
			userType= MyUserData:getUserType(),
	        secret_key = ACTIVITY_SECRET_KEY,
	        url = LOCAL_NET and "http://192.168.204.68/operating/web/index.php" or ACTIVITY_URL,
            language = 1,   --0中文 1泰语 2印尼
            isReleated = -1, -- -1是默认的，打开活动中心，0、1、2是打开强推界面，对应小中大窗口
            channelID = "", --目前我们不需要这个参数来区分，但是sdk里必须要传
        }
		self:callNativeEvent(kActivity, json.encode(activityParam));
	end


	--录制及回放
	NativeEvent.startRecord = function(self)
		self:callNativeEvent(kStartRecord , "");
	end
	NativeEvent.stopRecord = function(self)
		self:callNativeEvent(kStopRecord , "");
	end
	NativeEvent.playBack = function(self)
		self:callNativeEvent(kPlayBack , "");
	end
	NativeEvent.stopPlayBack = function(self)
		self:callNativeEvent(kStopPlayBack , "");
	end

	NativeEvent.hideEditTextView = function(self)
	self:callNativeEvent("hideEditTextView");
	end

    NativeEvent.shareToMessenger = function(self, param)
    --    self:callNativeEvent(kShareToMessenger, json.encode(param));
    end
    
    NativeEvent.isMessengerExist = function(self)
    --    self:callNativeEvent("isMessengerExist");
        return 0
    end
        --获取启动方式，默认0是点击图标启动，1是从fb messenger启动，用于统计启动方式
    NativeEvent.getStartWay = function(self)
        return ""
    end
    NativeEvent.getAllUpdateFile = function(self, param)
       return ""
    end
    NativeEvent.getNetworkType = function(self)
        return 0;
    end

    NativeEvent.backupScripts = function(self)
        return 1
    end
    NativeEvent.updateGameFinish = function(self)
        return 0
    end
    NativeEvent.deleteFile = function(self, param)
    end
    NativeEvent.getLocation = function(self)
        return ""
    end
    NativeEvent.getFileMD5 = function(self, param)
        return ""
    end
    NativeEvent.getPermission = function(self, param)
        return "success"
    end
end



--///////////////////////////////// Win32 ////////////////////////////////
if NativeEvent.s_platform ~= kPlatformAndroid and NativeEvent.s_platform ~= kPlatformIOS  then

	NativeEvent.callNativeEvent = function(self, keyParm , data)
		
	end

	-- 拍照 
	NativeEvent.takePhoto = function(self, str)
	end

	--选择图片
	NativeEvent.pickPhoto = function(self, str)
	end

	--上传图片
	NativeEvent.uploadImage = function(self, str)
	end

	--打开网页
	NativeEvent.openWeb = function(self, url)
	end

	--打开网页
	NativeEvent.closeWeb = function(self)
	end

	--打开条款
	NativeEvent.openRule = function(self, ruleUrl)
	end

	--关闭条款
	NativeEvent.closeRule = function(self)
	end
	--打开粉丝页
	NativeEvent.openLink = function(self, url)
	end
	--打开第三方登录
	NativeEvent.login = function(self, param)
	end
	--注销
	NativeEvent.logout = function(self, param)
	end
	--获取Facebook好友
	NativeEvent.getFbFriend = function(self, param)
	end
	--邀请好友
	NativeEvent.inviteFbFriend = function(self, param)
	end
	--邀请短信好友
	NativeEvent.inviteSmsFriend = function(self, param)
		
	end
	--本地更新
	NativeEvent.UpdateByLocal = function(self, url, force)
	end
	--友盟更新
	NativeEvent.UpdateByUmeng = function(self, isActUpdata, isDeltaUpdate, isForceUpdate, isCheckForProcess)
	end
	--检测更新
	NativeEvent.CheckPackByLocal = function(self, url)
	end
	--头像下载
	NativeEvent.downloadImage = function(self, param)
	end
	--重命令头像
	NativeEvent.renameImage = function(self, oldName, newName, url)
	end
	--下载文件
	NativeEvent.downloadFile = function(self, type, url, file, tag)
		printInfo("开始下载文件")
	end

	
	--解压文件
	NativeEvent.unzip = function(self, type, path, file, tag)
	end
	--解压游戏
	NativeEvent.unzipGame = function(self, file)
	end
	-- 立即返回结果的的操作 
	NativeEvent.encodeStr = function(self, str)
		return str
	end

	NativeEvent.compressString = function(self, str)
	end

	NativeEvent.unCompressString = function(self, str)
	end

	NativeEvent.isFileExist = function(self, str)
		return 1
	end
	NativeEvent.collectByUmeng = function(self, param)
    end
    --推广
    NativeEvent.loadAdData = function(self, param)
    end
    NativeEvent.showAd = function(self, param)
    end
    NativeEvent.openActivity = function(self, param)
	end
	--录制及回放
	NativeEvent.startRecord = function(self)
	end
	NativeEvent.stopRecord = function(self)
	end
	NativeEvent.playBack = function(self)
	end
	NativeEvent.stopPlayBack = function(self)
	end
	-- 涉及ui的操作 放在消息队列中执行
	--延时释放 defualt png , 解决 Android 游戏启动时屏幕黑一下的问题
	NativeEvent.CloseStartScreen = function(self)
	end

    NativeEvent.GetInitValue =function(self)
    end
    
    NativeEvent.GetNetAvaliable = function(self)
    	return 1
    end

    NativeEvent.PreloadAllSound = function(self, musicsMap, effectsMap)
	end

	NativeEvent.ReportLuaError = function(self, faultInfo)
	end

	NativeEvent.Exit = function(self)
	end

	NativeEvent.StartUnitePay = function(self, param_data)
	end

	--传支付配置
    NativeEvent.payConfig = function(self,param)
    end

    --下单支付
    NativeEvent.startPay = function(self,param)
    end

     --支付
    NativeEvent.pay = function(self, param)
    end

    --分享
    NativeEvent.share = function(self, param)
	end

	--cid
	NativeEvent.getClientId = function(self, param)
	end

	--评分
    NativeEvent.score = function(self, param)
	end	

	NativeEvent.boyaaAd = function(self, param)
	end

    NativeEvent.shareToMessenger = function(self, param)
    end
    NativeEvent.shareToMessenger = function(self, param)
    end
    
    NativeEvent.isMessengerExist = function(self)
        return 0
    end

        --获取启动方式，默认0是点击图标启动，1是从fb messenger启动，用于统计启动方式
    NativeEvent.getStartWay = function(self)
        return ""
    end
    NativeEvent.getAllUpdateFile = function(self, param)
        return ""
    end
    NativeEvent.getNetworkType = function(self)
        return 0
    end

    NativeEvent.backupScripts = function(self)
         return 1
    end
    NativeEvent.updateGameFinish = function(self)
         return 0
    end
    NativeEvent.deleteFile = function(self, param)
    end
    NativeEvent.getLocation = function(self)
        return ""
    end
    NativeEvent.checkUnfinishIAP = function(self)
    end
    NativeEvent.consumeProduct = function(self, param)
    end
    NativeEvent.getFileMD5 = function(self, param)
        return ""
    end
    NativeEvent.getPermission = function(self, param)
        return "success"
    end
    NativeEvent.downloadUpdateApk = function(self, param)
    end
    NativeEvent.pauseDownloadUpdateApk = function(self, param)
    end
    NativeEvent.installUpdateApk = function(self, param)
    end
    NativeEvent.queryDownloadProgress = function(self, param)
        return 0
    end
    NativeEvent.queryDownloadStatus = function(self, param)
        return 0
    end
    NativeEvent.deleteDownloadTask = function(self, param)
        return 1
    end
end

function NativeEvent:onLoadSoundResCallback(jsonData, resultStr)
	-- AlarmTip.play("resultStr =" .. resultStr)
end
function NativeEvent:onActiviyCloseCallback(jsonData, resultStr)
	WindowManager:onKeyBack();
end 

--function NativeEvent:onDownloadImageCallback(jsonData)
--	local stat 		= jsonData.stat:get_value();
--	local folder 	= jsonData.folder:get_value();
--	local name 		= jsonData.name:get_value();
--	local id 		= tostring(jsonData.id:get_value());
--	local callback = NativeEvent.callEventFuncMap[tostring(id)];

--	if callback then
--		callback(stat, folder, name);
--	end

--	NativeEvent.callEventFuncMap[id] = nil;
--end

--邀请FACEBOOK好友回调
function NativeEvent:onSendInvitesCallback(jsonData)
    if jsonData and jsonData.toIds and jsonData.toIds:get_value() then
        NativeEvent.getInstance():boyaaAd({type = kADInvite or 10, value = MyUserData:getId()})
        local hasInviteIds = jsonData.toIds:get_value();
        local ids = {}
        for id in string.gmatch(hasInviteIds, '%d+') do
            table.insert(ids, id);
        end
        HttpModule.getInstance():execute(HttpModule.s_cmds.GET_FB_INVITE_AWARD, {arrFriends=ids}, false, true);

        local toStr = jsonData.to:get_value();

        local to = {}
        for id in string.gmatch(toStr, '[^,]+') do

            table.insert(to, id);
        end  
        local inviteFriend = MyUserData:getInviteFriend();
        local invitableFriend = {}
        invitableFriend.status = 0
        local strInvites = ""
        local hasInvite = {}
        if inviteFriend then
            for i = 1, #inviteFriend do
                local flag = false
                for j = 1, #to do
                    if tostring(inviteFriend[i].id) == tostring(to[j]) then
                        table.insert(hasInvite, inviteFriend[i].urlMd5)

                        strInvites = strInvites..(inviteFriend[i].urlMd5)..","
                        flag = true
                        break;
                    end
                end
                if not flag then
                    table.insert(invitableFriend, inviteFriend[i])
                end
            end
        end
        GameSetting:setInviteIdGuest(GameSetting:getInviteIdGuest()..strInvites)
        GameSetting:save()
        MyUserData:setInviteFriend(invitableFriend)
		MyUserData:setLeftInviteNum(MyUserData:getLeftInviteNum()-#hasInvite)
        EventDispatcher.getInstance():dispatch(Event.Call, kGetFbFriend);
    end
	
end

function NativeEvent:onSendRecallsCallback(jsonData) 
    --召回消息发送成功，抛出Event.call
    EventDispatcher.getInstance():dispatch(Event.Call, "sendFBRecallSuccess")
end
--分享回调
function NativeEvent:onShareCallback(jsonData)
	if jsonData and jsonData.shareData then
		local result = jsonData.shareData:get_value()
		if result == "-1" then
			AlarmTip.play(STR_SHARE_FAILED)
		elseif result == "0" then
			AlarmTip.play(STR_SHARE_CANCEL)
		elseif result == "1" then
            NativeEvent.getInstance():boyaaAd({type = kADShare or 9, value = MyUserData:getId()})
			HttpModule.getInstance():execute(HttpModule.s_cmds.FRONTSTATISTICS, {reportData =  {{id= 'shareSucc'}}}, false, true)
			AlarmTip.play(STR_SHARE_SUCCESS)
            local from = jsonData.from and jsonData.from:get_value() or ""
            --分享是来自活动中心
            if from == "activity" then
                HttpModule.getInstance():execute(HttpModule.s_cmds.SEND_ACTIVITY_SHARE_STATISTIC, {}, false, false)
            elseif from == "honourEmblem" then
             --    local ids = jsonData.expand and json.decode(jsonData.expand:get_value() or "") or {}
            	-- local gifts = {}
            	-- gifts["1"] = 13
            	-- local params = {gift = gifts, broadcastId = ids}
            	-- HttpModule.getInstance():execute(HttpModule.s_cmds.GIFT_UPDATE_MINE, params, false, false)
            	-- GameSetting:setCurrentUIDShareHonourEmblemInfo(MyUserData:getId(), tostring(os.date("%Y%m%d")))
            	-- printInfo("honourEmblem::::"..from..":::"..json.encode(params))
            end
		end
	end
end

--个推回调
function NativeEvent:onPostClientId(jsonData)
    if jsonData and jsonData.clientId then
	    local result = jsonData.clientId:get_value()
	    if result ~= nil then
		    if MyUserData and MyUserData:getId() then
                if result ~= GameSetting:getGcmToken() then
                    GameSetting:setGcmToken(result)
                    GameSetting:save()
				    HttpModule.getInstance():execute(HttpModule.s_cmds.POST_CLIENTID, {mid = MyUserData:getId(), token = result}, false, true)
                end
		    end
	    end
    end
end

--评分
function NativeEvent:onScoreCallback(jsonData)

end

function NativeEvent:onGetFbAppCallback(jsonData)
	printInfo("onGetFbAppCallback")
	if jsonData and jsonData.length and jsonData.length:get_value() then
		local len  = jsonData.length:get_value() or 0;
		local jdata= jsonData.dataArrStr:get_value();
		local data = json.decode(jdata);
        --zyh 只给第一个用户发奖励
        if data[1] and data[1].data and MyUserData then
            HttpModule.getInstance():execute(HttpModule.s_cmds.GET_FB_INVITE_SUC_AWARD, {fid=data[1].data or 0}, false, true);
        end
	end

end

function NativeEvent:onDownloadFileCallback(jsonData)
	printInfo('kDownloadFile finish');
	-- printInfo("status = " .. jsonData.status:get_value() or "");
	-- printInfo("path = " .. jsonData.path:get_value() or "");
	-- printInfo("url = " .. jsonData.url:get_value() or "");
	-- printInfo("tab = " .. jsonData.tag:get_value() or "");
	-- local status = tonumber(jsonData.status:get_value()) or 0
	-- local tag = jsonData.tag:get_value() or ""
	-- local path = jsonData.path:get_value() or ""
	-- local type = jsonData.type:get_value() or ""
	-- local progress = jsonData.progress:get_value() or ""

	--if tag == 'verconf' or tag == 'update_lua' then
	EventDispatcher.getInstance():dispatch(Event.Call, kDownloadUpdate, jsonData);
	-- 	return 
	-- end

	-- if status == 1 then  --下载成功
	-- 	AlarmTip.play(string.format("下载%s完成，正在解压", tag))
	-- 	local folder
	-- 	if tag == "表情" then
	-- 		folder = "expression/"
	-- 	elseif tag == "道具" then
	-- 		folder = "friendsAnim/"
	-- 	elseif tag == "广东声音" then
	-- 		folder = "ogg/gdmj/"
	-- 	elseif tag == "上海声音" then
	-- 		folder = "ogg/shmj/"
	-- 	end
	-- 	NativeEvent.getInstance():unzip(type, folder, path, tag);
	-- elseif status == 2 then  --正在下载
	-- 	local progress = jsonData.progress:get_value() or 0
	-- 	AlarmTip.play(string.format("正在下载%s %s", tag, progress) .. "%s", "\%")
	-- else --下载异常失败
	-- 	if tag == "表情" then
	-- 		PhpManager:setFaceCanUse(0)
	-- 	elseif tag == "道具" then
	-- 		PhpManager:setPropCanUse(0)
	-- 	elseif tag == "广东声音" then
	-- 		PhpManager:setGdmjCanUse(0)
	-- 	elseif tag == "上海声音" then
	-- 		PhpManager:setShmjCanUse(0)
	-- 	end
	-- 	AlarmTip.play(string.format("下载%s失败", tag))
	-- end
	
end

function NativeEvent:onUnzipCallback(jsonData)
	EventDispatcher.getInstance():dispatch(Event.Call, kUnzip, jsonData);

end
function NativeEvent:onUnzipGameCallback(jsonData)
	printInfo('kUnzipGame finish');
	printInfo("status = " .. jsonData.status:get_value() or "");
	printInfo("file = " .. jsonData.file:get_value() or "");

	local status = tonumber(jsonData.status:get_value()) or 0
	if status == 1 then
		AlarmTip.play(string.format("解压成功"));
	else
		AlarmTip.play(string.format("解压失败"));
	end
end


function NativeEvent:onUploadImageCallBack(result)
	if result then
		if result.iconname then
			HttpModule.getInstance():execute(HttpModule.s_cmds.UPDATE_USER_ICON,{iconname = result.iconname:get_value()}, false, false)
		end
		if result.code:get_value() == 1 then
			AlarmTip.play(STR_UPLOAD_SUCC);
		else
			AlarmTip.play(STR_UPLOAD_HEADER_ERROR)
		end
		
		--local url = result.data.bimg:get_value();
		--local md5Name = ToolKit.getMd5ImageName(kHeadImagePrefix, MyUserData:getId(), url);
		--NativeEvent.getInstance():renameImage(MyUserData:getUploadTemp(), string.format("%s%s",kHeadImageFolder, md5Name), url);
	else
		AlarmTip.play(STR_SVAE_HEARD_ERROR);
	end
end

function NativeEvent:onUploadHeadImageCallBack(result)

	local sid = tonumber(PhpManager:getGame())

	if result then
		local isSucc = result.isSucc and result.isSucc:get_value() or "false"
		local dResult = result.result and result.result:get_value() or ""
		local dResultJsonObj = json.decode(dResult)

		if dResultJsonObj and dResultJsonObj.iconname then
			if result.iconname then
				HttpModule.getInstance():execute(HttpModule.s_cmds.UPDATE_USER_ICON,{iconname = (result.iconname:get_value() or dResultJsonObj.iconname), sid = sid}, false, false)
			end

		end

		
		if (isSucc == "true") and dResultJsonObj and dResultJsonObj.code == 1 then
			AlarmTip.play(STR_UPLOAD_SUCC);
		else
			AlarmTip.play(STR_UPLOAD_HEADER_ERROR)
		end
		
		--local url = result.data.bimg:get_value();
		--local md5Name = ToolKit.getMd5ImageName(kHeadImagePrefix, MyUserData:getId(), url);
		--NativeEvent.getInstance():renameImage(MyUserData:getUploadTemp(), string.format("%s%s",kHeadImageFolder, md5Name), url);
	else
		AlarmTip.play(STR_SVAE_HEARD_ERROR);
	end
end

function NativeEvent:onRenameImageCallBack(result)
	
	if result and tonumber(result.status:get_value() or 0) == 1 and MyUserData:getId() > 0 then
		url = result.url:get_value();
		MyUserData:setHeadUrl(url);
	end
	
end

function NativeEvent:onGetFbFriendCallback(result)
	if MyUserData then
	    local invitableFriend = MyUserData:getInviteFriend() or {}
	    invitableFriend.status = 1
	    local status = tonumber(result.status and result.status:get_value() or 0);
	    if status == 0 and MyUserData then
	        invitableFriend.status = 0
	        local str = GameSetting:getInviteIdGuest()
	        local hasInviteIds = {}
	        local hasInviteNum = 0
	        for id in string.gmatch(str, "(.-),") do
	            hasInviteIds[id] = true
	            hasInviteNum = hasInviteNum + 1
	        end
	        local data = result
	        for i = 1, #data do
	            local url = data[i].picture.data.url:get_value() or ""
	            local strUrlMd5 = md5_string(url)
	            if not hasInviteIds[strUrlMd5] then
	                local facebookFriend = setProxy(new(require("data.facebookFriend")));
	                facebookFriend:setId(data[i].id:get_value());
	                facebookFriend:setNickname(data[i].first_name:get_value() or data[i].name:get_value());
	                facebookFriend:setMoney(MyUserData:getInvitemoney() or 1000);
	                facebookFriend:setHeadUrl(data[i].picture.data.url:get_value());
	                facebookFriend:setUrlMd5(strUrlMd5)
	                facebookFriend:setSex(1)
	                table.insert(invitableFriend, facebookFriend)
	            end
	        end
	        MyUserData:setInviteFriend(invitableFriend)
		
	    end
	    EventDispatcher.getInstance():dispatch(Event.Call, kGetFbFriend);
	end
end

function NativeEvent:onPayCallback(result)
	MyPay:paycb(result);
	
end

function NativeEvent:onActivityCallback(result)
	printInfo('onActivityCallback')
end
function NativeEvent:onAndroidE2PayBack(result)

	 local code = tonumber(result.code:get_value()) 
	 local content = ""
	 if code == 404 then
		content = STR_PAY_JMTSMS_NOCARD
		--content = "没SIM卡"
	elseif code == 100 then
		content = STR_PAY_SUC
	else
		content = STR_PAY_FAILED
        --上报支付失败记录
		local errorLog = "E2PpayFail "..tostring(code)
        HttpModule.getInstance():execute(HttpModule.s_cmds.REPORT_DEBUG_INFO, {log = errorLog})
	end

    WindowManager:showWindow(WindowTag.LobbyConfirmPopu, {
    	title = (content == STR_PAY_FAILED) and "ERROR: "..tostring(code) or nil,
        content = content,
        confirm = STR_EXIT_GAME_CONFIRM,
    }, WindowStyle.POPUP)


end
function NativeEvent:onGetOneTwoCallPinNoCallBack(result)

	local pmode = tonumber(result.pmode:get_value())
	if  pmode == 621 then
		local params = {

		       	    pmode = tostring(result.pmode:get_value()),
                    mid = tostring(result.mid:get_value()),
                    channel = result.channel:get_value(),
                    id =  tostring(result.id:get_value()),--商品ID
                    sitemid =  result.sitemid:get_value(),
                    pin_no = result.pinNo:get_value(),
                    sid =  PhpManager:getGame(),
                    lid = '',
    	 }
		HttpModule.getInstance():execute(HttpModule.s_cmds.GET_PAY_ORDER,params,true, true);
	elseif pmode == 623 then
		local params = {

		       	    pmode = tostring(result.pmode:get_value()),
                    username = tostring(result.username:get_value()),
                    channel = result.channel:get_value(),
                    id =  tostring(result.id:get_value()),--商品ID
                    sitemid =  result.sitemid:get_value(),
                    pin_no = result.pinNo:get_value(),
                    sid =  PhpManager:getGame(),
                    lid = '',
    	 }
		HttpModule.getInstance():execute(HttpModule.s_cmds.GET_PAY_ORDER,params,true, true);
	end

		print('onGetOneTwoCallPinNoCallBack')
end



function NativeEvent:onDownloadApkCallBack(result)
    local status = tonumber(result.status:get_value())
    local url = result.url:get_value()
print_string("zyh onDownloadApkCallBack "..tostring(status).." url  "..tostring(url))
    --开始下载
    if status == 0 then
    --下载中
    elseif status == 1 then
        local progress = tonumber(result.progress:get_value())
        print_string("zyh downloadApk progress is "..tostring(progress))
        if progress then
            MyApkUpdateInfo:setUpdateProgress(progress)
        end
    --暂停
    elseif status == 2 then

    --下载完成
    elseif status == 3 then
        local fileName = result.fileName:get_value()
        print_string("zyh onDownloadApkCallBack fileName is "..tostring(fileName))
        local version = MyApkUpdateInfo.updateVer or ""
        WindowManager:showWindow(WindowTag.LobbyConfirmPopu, {
            content = string.format(STR_UPDATE_FINISH_INSTALL, version),
            confirm = STR_UPDATE_INSTALL,
            confirmFunc = function()
                NativeEvent.getInstance():installUpdateApk({fileName = fileName})
            end
        }, WindowStyle.POPUP)
    --下载失败
    elseif status == 4 then
        WindowManager:showWindow(WindowTag.LobbyConfirmPopu, {
            content = STR_UPDATE_FAIL,
            confirm = STR_EXIT_GAME_CONFIRM,
        }, WindowStyle.POPUP)
    end
end


NativeEvent.callEventFuncMap = {
	["LoadSoundRes"]  	= NativeEvent.onLoadSoundResCallback,
	["ActivityClose"] 	= NativeEvent.onActiviyCloseCallback,
	["sendInvites"] 	= NativeEvent.onSendInvitesCallback,
    ["sendRecalls"]     = NativeEvent.onSendRecallsCallback,
	["getFbAppInfo"] 	= NativeEvent.onGetFbAppCallback,
	["shareResult"]		= NativeEvent.onShareCallback,
	["postClientId"]	= NativeEvent.onPostClientId,
	[kDownloadFile] 	= NativeEvent.onDownloadFileCallback,
	[kUnzip] 			= NativeEvent.onUnzipCallback,
	[kUnzipGame] 		= NativeEvent.onUnzipGameCallback,
	[kUploadImage] 		= NativeEvent.onUploadImageCallBack,
	[kRenameImage] 		= NativeEvent.onRenameImageCallBack,
	[kGetFbFriend]  	= NativeEvent.onGetFbFriendCallback,
	[kPay] 				= NativeEvent.onPayCallback,
	[kScore]			= NativeEvent.onScoreCallback,
	[kActivity] 		= NativeEvent.onActivityCallback,
	["uploadHeadImage"] = NativeEvent.onUploadHeadImageCallBack,
	["oneTwoCallOrder"] = NativeEvent.onGetOneTwoCallPinNoCallBack,
	["Android_E2PayBack"] = NativeEvent.onAndroidE2PayBack,
    ["downloadApk"]     = NativeEvent.onDownloadApkCallBack,
}
