if kPlatformIOS == System.getPlatform() then
	if DrawingBase then
		-------------DrawingImage---------------------------
		DrawingImage.ctor = function(self, res, leftWidth, rightWidth, topWidth, bottomWidth)
		    self.m_resID = res:getID();
		    self.m_width = res:getWidth();
		    self.m_height = res:getHeight();
		    
		    self.m_isGrid9 = (leftWidth and rightWidth and bottomWidth and topWidth) and true or false;
			
			local realWidth,realHeight = DrawingImage.getRealSize(self);

			local platform = System.getPlatform();

		    local scale = System:getLayoutScale();
			    leftWidth = leftWidth and leftWidth or 0;
			    rightWidth = rightWidth and rightWidth or 0;
			    bottomWidth = bottomWidth and bottomWidth or 0;
			    topWidth = topWidth and topWidth or 0;
			    drawing_create_image2(0, self.m_drawingID, self.m_resID,
										self.m_x, self.m_y, realWidth, realHeight, 
										self.m_isGrid9  and kTrue or kFalse, leftWidth, rightWidth, bottomWidth,topWidth, 
			                            leftWidth * scale, rightWidth * scale, bottomWidth * scale,topWidth * scale, 
										self.m_level);


			DrawingImage.setResRect(self,0,res);
			self.m_originWidth = self.m_width
			self.m_originHeight = self.m_height
		end


		function DrawingImage:getResSize()
			return self.m_res:getWidth(), self.m_res:getHeight()
		end

		function DrawingBase:getOriginSize()
			return self.m_originWidth or 1, self.m_originHeight or 1
		end

		function DrawingBase:getScale()
			return self.m_scaleX or 1, self.m_scaleY or 1
		end

		local setSize = DrawingBase.setSize
		function DrawingBase:setSize(width, height)
			setSize(self, width, height)
			return self
		end

		function DrawingBase:setScale(scaleX, scaleY)
			scaleY = scaleY or scaleX
			local originWidth, originHeight = self:getOriginSize()
			self:setSize(scaleX * originWidth, scaleY * originHeight)

			self.m_scaleX = scaleX
			self.m_scaleY = scaleY
			return self
		end

		function DrawingBase:addTo(parent, level)
			parent:addChild(self)
			if level then self:setLevel(level) end
			return self
		end

		function DrawingBase:pos(x, y)
			self:setPos(x, y)
			return self
		end

		function DrawingBase:align(align, x, y)
			self:setAlign(align)
			if x and y then
				self:setPos(x, y)
			end
			return self
		end

		function DrawingBase:removeSelf()
			local parent = self:getParent()
			if parent then
				return parent:removeChild(self, true)
			else
				delete(self)
			end
		end

		function DrawingBase:show()
			self:setVisible(true)
			return self
		end

		function DrawingBase:hide()
			self:setVisible(false)
			return self
		end

		function DrawingBase:_addAnim(animLoop, animTime, delayTime)
			self._anims = self._anims or {}
			local anim = AnimFactory.createAnimDouble(animLoop, 0, 1, animTime, delayTime or 0)
			local animId = anim:getID()
			self._anims[animId] = anim
			return anim
		end

		function DrawingBase:schedule(func, perTime, delayTime)
			local anim = self:_addAnim(kAnimRepeat, perTime, delayTime)
			anim:setEvent(nil, func)
			return anim
		end

		function DrawingBase:performWithDelay(func, delayTime)
			local anim = self:_addAnim(kAnimNormal, delayTime)
			anim:setEvent(nil, func)
			return anim
		end

		function DrawingBase:getActionByTag(tag)
			return self._anims and self._anims[tag]
		end

		function DrawingBase:stopActionByTag(tag)
			if not self._anims then return end
			for k,v in pairs(self._anims) do
				if v:getTag() == tag then
					delete(v)
					self._anims[k] = nil
					break
				end
			end
		end

		function DrawingBase:stopAllActions()
			if type(self.m_props) == "table" then
				for sequence, v in pairs(self.m_props) do 
					drawing_prop_remove(self.m_drawingID, sequence)
					delete(v["prop"]);
					for _,anim in pairs(v["anim"]) do 
						delete(anim);
					end
					for _,res in pairs(v["res"]) do 
						delete(res);
					end
				end
			end
			self.m_props = {};

			if type(self._anims) == "table" then
				for k,v in pairs(self._anims) do
					delete(v)
				end
			end
			self._anims = {}
		end

		DrawingBase.dtor = function(self)
			--This is safe, only because the drawing is going to release;
			--Otherwise it should remove the prop first then release it.
			delete(self.m_doubleClickAnim);
			self.m_doubleClickAnim = nil;
			
			for k,v in pairs(self.m_props) do 
				delete(v["prop"]);
				for _,anim in pairs(v["anim"]) do 
					delete(anim);
					anim = nil;
				end
			end
			self.m_props = {};
			
			if self.m_parent then
				DrawingBase.removeChild(self.m_parent,self);
			else
		        self:removeChildFromRoot(self);   
			end
			
		    for _,v in pairs(self.m_children) do 
		        delete(v);
		    end
		    self.m_children = {};
		    self.m_rchildren  = {};
		    
		    self.m_touchEventCallbacks = {
				touch = {};
				drag = {};
				doubleClick = {};
				focus = {};
			};

		    --This drawing_delete should actually be in derided class dtor,
		    --It is here because the children release.
		    --It is ugly but useful,so keep it for now.
		    drawing_delete(self.m_drawingID);
		    drawing_free_id(self.m_drawingID);
		    self:stopAllActions()
		end


		function DrawingBase:setDebugNameByPropAndAnim(sequence , name)
			name = name or "";
			if self.m_props[sequence] then
				local prop = self.m_props[sequence]["prop"];
				if prop then
					prop:setDebugName(name);
					for _,v in pairs(self.m_props[sequence]["anim"]) do 
						if v then
							v:setDebugName(name);
						end
					end
				end
			end
		end


		function DrawingImage:playAnimation(param)
			local frames = param.frames;				-- 动画帧表，file or ResImage;
			local inteval = param.inteval or 500;		-- 隔多少时间更新一帧
			local repeatCount = param.repeatCount or 1;	-- 动画重复次数 如果要永久动画，设为 < 1 的值
			local delay = param.delay or 0;				-- 多长时间后开始播放动画。
			local onComplete = param.onComplete;		-- 动画全播放完后回调

			local currentIndex = 0;						-- 当前帧
			self.m_reses = self.m_reses or {};
			local reses = self.m_reses;					-- 帧资源

			-- 动画播放完后调用，永久性动画进不来这里
			local function playFinished()
				self:stopAnimation();
				if onComplete then
					onComplete();
				end
			end

			-- 更新帧
			local function updateFrame()
				currentIndex = currentIndex + 1;
				if currentIndex > #frames then
					if repeatCount > 0 then
						repeatCount = repeatCount - 1;
						if repeatCount <= 0 then
							playFinished();
							return;
						end
					end
					currentIndex = 1;
				end
			
				if currentIndex > 0 and not reses[currentIndex] then
					local frame = frames[currentIndex];
					if typeof(frame, ResImage) then
						reses[currentIndex] = frame;
					else
						reses[currentIndex] = new(ResImage, frame);
					end
					self:addImage(reses[currentIndex], currentIndex);
				end
				-- 设置当前帧
				self:setImageIndex(currentIndex);
			end

			-- 开始播放帧动画
			local function playAnim()
				self._frameAnimation = new(AnimInt, kAnimLoop, 0, 1, inteval, 0);
				self._frameAnimation:setDebugName(param.DebugName or "FrameAnimation");
				self._frameAnimation:setEvent(self, updateFrame);
			end

			-- 动画开始前先stop掉 前动画
			self:stopAnimation();
			self:setImageIndex(0);

			-- 如果有 延迟播放动画，先延迟
			if delay and delay > 0 then
				self._delayAnimation = new(AnimInt, kAnimNormal, 0, 1, delay, 0);
				self._delayAnimation:setDebugName("delayAnimation");
				self._delayAnimation:setEvent(self, playAnim);
			else
				playAnim();
			end
		end


		function DrawingImage:stopAnimation()
			if self._frameAnimation then delete(self._frameAnimation); self._frameAnimation = nil; end
			if self._delayAnimation then delete(self._delayAnimation); self._delayAnimation = nil; end
		end

		function DrawingImage:dtor()
			self:stopAnimation();
			if self.m_reses then
				for _,v in pairs(self.m_reses) do
					delete(v);
				end
			end
			self.m_reses = nil;
		end

		-------------Blend---------------------------
		function DrawingBase:setBlend(blendSrc, blendDst)
			drawing_set_blend_mode(self.m_drawingID, blendSrc, blendDst);
		end


		DrawingBase.s_root = nil;

		DrawingBase.s_rootNodes = {

		};

		local drawing_delete_all_ex = drawing_delete_all;
		drawing_delete_all = function()
		    drawing_delete_all_ex();
		    DrawingBase.s_root = nil;
		    DrawingBase.s_rootNodes = {};
		end

		DrawingBase.__createRootNode = function()
		    if not DrawingBase.s_root then
		        DrawingBase.s_root = new(Node);
		        DrawingBase.s_root:setSize(System.getScreenScaleWidth(), System.getScreenScaleHeight());
		        table.insert(DrawingBase.s_rootNodes, DrawingBase.s_root);
		        DrawingBase.s_root:setParent();
		    end
		end

		DrawingBase.getRootNode = function()
		    DrawingBase.__createRootNode();
		    return DrawingBase.s_root;
		end

		DrawingBase.refreshRootNodeSize = function(isVerticalScreen)
			DrawingBase.__createRootNode();
			if isVerticalScreen == 1 then
				DrawingBase.s_root:setSize(System.getScreenScaleHeight(), System.getScreenScaleWidth());
				local sw = System.getScreenScaleWidth();
				local sh = System.getScreenScaleHeight();
				local lx = (sw - sh) / 2;
				DrawingBase.s_root:setPos(lx,-lx);
				DrawingBase.s_root:addPropRotateSolid(0,-90,kCenterDrawing);

				DrawingBase.setClip = function(self, x, y, w, h)
				end
			else
				DrawingBase.s_root:setSize(System.getScreenScaleWidth(), System.getScreenScaleHeight());
				DrawingBase.s_root:setPos(0,0);
				if not DrawingBase.s_root:checkAddProp(0) then
		     		DrawingBase.s_root:removeProp(0);

				DrawingBase.setClip = function(self, x, y, w, h)
					local layoutScale = System.getLayoutScale();
					_drawing_set_clip_rect(self.m_drawingID, x*layoutScale,y*layoutScale,w*layoutScale,h*layoutScale);
				end

		     	end
			end
			
		end


		DrawingBase.setMirror = function(self, isMirrorX, isMirrorY)
			drawing_set_mirror(self.m_drawingID,
					isMirrorX and kTrue or kFalse,
					isMirrorY and kTrue or kFalse);
		end

		DrawingBase.checkAddProp = function(self, sequence)
		    if self.m_props[sequence] then
		        return false;
		    end
		    return true;
		end

		DrawingBase.getAlign = function(self)
			return self.m_align or kAlignTopLeft;
		end

		DrawingBase.addToRoot = function(self)
			if self.m_parent then
				self.m_parent:removeChild(self);
			end

		    DrawingBase.__createRootNode();
			DrawingBase.addChild(DrawingBase.s_root,self)
		end

		DrawingBase.addChild = function(self, child)
		    if not child then
		        return;
		    end
		    
		    if child.m_parent then
		        child.m_parent:removeChild(child);
		    else
		        self:removeChildFromRoot(child);   
		    end


		    local ret = child:setParent(self); 
		    --local ret = drawing_set_parent(child.m_drawingID,self.m_drawingID);
		    
		    local index = #self.m_children+1;
		    self.m_children[index] = child;
		    self.m_rchildren[child] = index;
		    --child.m_parent = self;

		    --child:revisePos();

		    return ret;
		end

		DrawingBase.removeChild = function(self, child, doCleanup)
			local ret = child:setParent();
			local index = self.m_rchildren[child];
			self.m_rchildren[child] = nil;
			if index then
				self.m_children[index] = nil;
			end
			if doCleanup then
				delete(child);
				child = nil;
			end

			return ret == 0;
		end

		DrawingBase.removeChildFromRoot = function(self, child)
		    local keys = {};
		    for k, v in pairs(DrawingBase.s_rootNodes) do 
		        if v == child then
		            keys[1+#keys] = k;
		        end
		    end
		    for i, key in pairs(keys) do 
		        DrawingBase.s_rootNodes[key] = nil;
		    end
		end

		DrawingImage.addPropImageIndex = function(self, sequence, animType, duration, delay, startValue, endValue)
		    return DrawingBase.addAnimProp(self, sequence, PropImageIndex, nil, nil, nil, animType, duration, delay, startValue, endValue);
		end

		DrawingImage.addPropClip = function(self, sequence, animType, duration, delay, startX, endX, startY, endY, startW, endW, startH, endH)
		    return DrawingBase.addAnimProp(self, sequence, PropClip, nil, nil, nil, animType, duration, delay, startX, endX, startY, endY, startW, endW, startH, endH);
		end

		DrawingBase.setVisible = function(self, visible)
		    if visible then
		        local x,y = DrawingBase.getPos(self);
		        DrawingBase.setPos(self,x,y);
		    end
		    self.m_visible = visible and true or false;
		    drawing_set_visible(self.m_drawingID,self.m_visible and kTrue or kFalse);
		end





		end
else

	if DrawingBase then

	local ctor = DrawingImage.ctor
	function DrawingImage:ctor(...)
		ctor(self, ...)
		self.m_originWidth = self.m_width
		self.m_originHeight = self.m_height
	end

	function DrawingImage:getResSize()
		return self.m_res:getWidth(), self.m_res:getHeight()
	end

	function DrawingBase:getOriginSize()
		return self.m_originWidth or 1, self.m_originHeight or 1
	end

	function DrawingBase:getScale()
		return self.m_scaleX or 1, self.m_scaleY or 1
	end

	local setSize = DrawingBase.setSize
	function DrawingBase:setSize(width, height)
		setSize(self, width, height)
		return self
	end

	function DrawingBase:setScale(scaleX, scaleY)
		scaleY = scaleY or scaleX
		local originWidth, originHeight = self:getOriginSize()
		self:setSize(scaleX * originWidth, scaleY * originHeight)

		self.m_scaleX = scaleX
		self.m_scaleY = scaleY
		return self
	end

	function DrawingBase:addTo(parent, level)
		parent:addChild(self)
		if level then self:setLevel(level) end
		return self
	end

	function DrawingBase:pos(x, y)
		self:setPos(x, y)
		return self
	end

	function DrawingBase:align(align, x, y)
		self:setAlign(align)
		if x and y then
			self:setPos(x, y)
		end
		return self
	end

	function DrawingBase:removeSelf()
		local parent = self:getParent()
		if parent then
			return parent:removeChild(self, true)
		else
			delete(self)
		end
	end

	function DrawingBase:show()
		self:setVisible(true)
		return self
	end

	function DrawingBase:hide()
		self:setVisible(false)
		return self
	end

	function DrawingBase:_addAnim(animLoop, animTime, delayTime)
		self._anims = self._anims or {}
		local anim = AnimFactory.createAnimDouble(animLoop, 0, 1, animTime, delayTime or 0)
		local animId = anim:getID()
		self._anims[animId] = anim
		return anim
	end

	function DrawingBase:schedule(func, perTime, delayTime)
		local anim = self:_addAnim(kAnimRepeat, perTime, delayTime)
		anim:setEvent(nil, func)
		return anim
	end

	function DrawingBase:performWithDelay(func, delayTime)
		local anim = self:_addAnim(kAnimNormal, delayTime)
		anim:setEvent(nil, func)
		return anim
	end

	function DrawingBase:getActionByTag(tag)
		return self._anims and self._anims[tag]
	end

	function DrawingBase:stopActionByTag(tag)
		if not self._anims then return end
		for k,v in pairs(self._anims) do
			if v:getTag() == tag then
				delete(v)
				self._anims[k] = nil
				break
			end
		end
	end

	function DrawingBase:stopAllActions()
		if type(self.m_props) == "table" then
			for sequence, v in pairs(self.m_props) do 
				drawing_prop_remove(self.m_drawingID, sequence)
				delete(v["prop"]);
				for _,anim in pairs(v["anim"]) do 
					delete(anim);
				end
				for _,res in pairs(v["res"]) do 
					delete(res);
				end
			end
		end
		self.m_props = {};

		if type(self._anims) == "table" then
			for k,v in pairs(self._anims) do
				delete(v)
			end
		end
		self._anims = {}
	end

	local dtor = DrawingBase.dtor
	function DrawingBase:dtor()
		self:stopAllActions()
		dtor(self)
	end

	function DrawingBase:setDebugNameByPropAndAnim(sequence , name)
		name = name or "";
		if self.m_props[sequence] then
			local prop = self.m_props[sequence]["prop"];
			if prop then
				prop:setDebugName(name);
				for _,v in pairs(self.m_props[sequence]["anim"]) do 
					if v then
						v:setDebugName(name);
					end
				end
			end
		end
	end


	function DrawingImage:playAnimation(param)
		local frames = param.frames;				-- 动画帧表，file or ResImage;
		local inteval = param.inteval or 500;		-- 隔多少时间更新一帧
		local repeatCount = param.repeatCount or 1;	-- 动画重复次数 如果要永久动画，设为 < 1 的值
		local delay = param.delay or 0;				-- 多长时间后开始播放动画。
		local onComplete = param.onComplete;		-- 动画全播放完后回调

		local currentIndex = 0;						-- 当前帧
		self.m_reses = self.m_reses or {};
		local reses = self.m_reses;					-- 帧资源

		-- 动画播放完后调用，永久性动画进不来这里
		local function playFinished()
			self:stopAnimation();
			if onComplete then
				onComplete();
			end
		end

		-- 更新帧
		local function updateFrame()
			currentIndex = currentIndex + 1;
			if currentIndex > #frames then
				if repeatCount > 0 then
					repeatCount = repeatCount - 1;
					if repeatCount <= 0 then
						playFinished();
						return;
					end
				end
				currentIndex = 1;
			end
		
			if currentIndex > 0 and not reses[currentIndex] then
				local frame = frames[currentIndex];
				if typeof(frame, ResImage) then
					reses[currentIndex] = frame;
				else
					reses[currentIndex] = new(ResImage, frame);
				end
				self:addImage(reses[currentIndex], currentIndex);
			end
			-- 设置当前帧
			self:setImageIndex(currentIndex);
		end

		-- 开始播放帧动画
		local function playAnim()
			self._frameAnimation = new(AnimInt, kAnimLoop, 0, 1, inteval, 0);
			self._frameAnimation:setDebugName(param.DebugName or "FrameAnimation");
			self._frameAnimation:setEvent(self, updateFrame);
		end

		-- 动画开始前先stop掉 前动画
		self:stopAnimation();
		self:setImageIndex(0);

		-- 如果有 延迟播放动画，先延迟
		if delay and delay > 0 then
			self._delayAnimation = new(AnimInt, kAnimNormal, 0, 1, delay, 0);
			self._delayAnimation:setDebugName("delayAnimation");
			self._delayAnimation:setEvent(self, playAnim);
		else
			playAnim();
		end
	end


	function DrawingImage:stopAnimation()
		if self._frameAnimation then delete(self._frameAnimation); self._frameAnimation = nil; end
		if self._delayAnimation then delete(self._delayAnimation); self._delayAnimation = nil; end
	end

	function DrawingImage:dtor()
		self:stopAnimation();
		if self.m_reses then
			for _,v in pairs(self.m_reses) do
				delete(v);
			end
		end
		self.m_reses = nil;
	end

	-------------Blend---------------------------
	function DrawingBase:setBlend(blendSrc, blendDst)
		drawing_set_blend_mode(self.m_drawingID, blendSrc, blendDst);
	end

	end
end