--[[
	骰子socket接收协议  2015-03-03
]]

local BaseReader = require("mjSocket.game.baseReader")
local RoomReader = class(BaseReader)
local printInfo, printError = overridePrint("RoomReader")
function RoomReader:ctor()
end

function RoomReader:onGetTableId(packetId)
	local data = {};
	data.tableId 	= self.m_socket:readInt(packetId, -1);
	data.isInTable	= self.m_socket:readInt(packetId, -1);
	data.gameLevel	= self.m_socket:readInt(packetId, -1);
	data.gameId		= self.m_socket:readInt(packetId, -1);
	return data;
end
function RoomReader:onFollow(packetId)
	local data = {};
	data.code = self.m_socket:readInt(packetId, -1);
	return data;
end
function RoomReader:onFollowed(packetId)
	local data = {};
	data.code = self.m_socket:readInt(packetId, -1);
	data.uid = self.m_socket:readInt(packetId, -1);
	data.nick= self.m_socket:readString(packetId);
	return data;
end


-- game_level		: int (桌子所在游戏场次)
-- 	table_tatus 	: int (桌子状态 0 已结束 1 正在摇骰子 2 正在下注 3 结算中)
-- 	user_count 		: int (桌子上的玩家数)
-- 	for (int i = 0; i < user_count; ++ i)
-- 	{
-- 		seat_id : int (玩家座位号)
-- 		uid 	: int (用户id)
-- 		bet_count : int (玩家下了几种类型的注)
-- 		for (int j = 0; j < bet_count; ++ j)
-- 		{
-- 			bet_type : int (见上面枚举值)
-- 			param    : int (参数 有的类型需要参数,比如猜几点猜哪个3条,有的类型不需要参数时为0)
-- 			bet_money: int (在该类型上下注的金额)
-- 		}
-- 	}
function RoomReader:onLoginGame(packetId)
	printInfo("RoomReader:onLoginGame");
	local data = {};
	data.gameLevel = self.m_socket:readInt(packetId, -1);
	data.gameStatus= self.m_socket:readInt(packetId, -1);
	data.userCount = self.m_socket:readInt(packetId, -1);

	data.user = {}
	for i = 1, data.userCount do
		local user = {};
		user.seatId 	= self.m_socket:readInt(packetId, -1) + 1;
		user.uid 	 	= self.m_socket:readInt(packetId, -1);
		user.chip 		= self.m_socket:readInt64(packetId, -1);
		user.userInfo 	= self.m_socket:readString(packetId);
		user.betCount 	= self.m_socket:readInt(packetId, -1);
		user.bet 		= {};
		for j = 1, user.betCount do
			local bet = {}
			bet.betType = self.m_socket:readInt(packetId, -1);
			bet.param  	= self.m_socket:readInt(packetId, -1);
			bet.money 	= self.m_socket:readInt64(packetId, -1);
			table.insert(user.bet, bet);
		end
		table.insert(data.user, user)
	end

	data.left_time = self.m_socket:readInt(packetId, 0);
    data.pokerLogCount = self.m_socket:readInt(packetId, 0)
    data.pokerLog = {}
    for i = 1, data.pokerLogCount do
        local log = {}
        log.point1 = self.m_socket:readByte(packetId, 1)
        log.point2 = self.m_socket:readByte(packetId, 1)
        log.point3 = self.m_socket:readByte(packetId, 1)
        table.insert(data.pokerLog, log)
    end
	
	return data;
end
function RoomReader:onLoginErrorGame(packetId)
	local data = {};
	data.retcode = self.m_socket:readByte(packetId, -1);
	return data;
end


function RoomReader:onReLoginGame(packetId)
	local data = {};
	data.gameLevel = self.m_socket:readInt(packetId, -1);
	data.gameStatus= self.m_socket:readInt(packetId, -1);
	data.userCount = self.m_socket:readInt(packetId, -1);

	data.user = {}
	for i = 1, data.userCount do
		local user = {};
		user.seatId 	= self.m_socket:readInt(packetId, -1) + 1;
		user.uid 	 	= self.m_socket:readInt(packetId, -1);
		user.chip 		= self.m_socket:readInt64(packetId, -1);
		user.userInfo 	= self.m_socket:readString(packetId);
		user.betCount 	= self.m_socket:readInt(packetId, -1);
		user.bet 		= {};
		for j = 1, user.betCount do
			local bet = {}
			bet.betType = self.m_socket:readInt(packetId, -1);
			bet.param  	= self.m_socket:readInt(packetId, -1);
			bet.money 	= self.m_socket:readInt64(packetId, -1);
			table.insert(user.bet, bet);
		end
		table.insert(data.user, user)
	end
	data.left_time = self.m_socket:readInt(packetId, 0);
	return data;
end

function RoomReader:onSitDownInGame(packetId)
	printInfo("RoomReader:onSitDownInGame");
	local data = {};
	data.retcode = self.m_socket:readInt(packetId, -1);
	data.money	= self.m_socket:readInt64(packetId, -1);
	if data.retcode == 0 then
		data.seatId = self.m_socket:readInt(packetId, -1) + 1;
		data.chip 	= self.m_socket:readInt64(packetId, -1);
	end
	return data;
end


function RoomReader:onBroadcastSitdown(packetId)
	printInfo("RoomReader:onBroadcastSitdown");
	local data = {};
	data.uid 		= self.m_socket:readInt(packetId, -1);
	data.seatId 	= self.m_socket:readInt(packetId, -1) + 1;
	data.chip 		= self.m_socket:readInt64(packetId, -1);
	data.money 		= self.m_socket:readInt64(packetId, -1);
	data.userInfo	= self.m_socket:readString(packetId);
	return data;
end

function RoomReader:onBet(packetId)
	printInfo("RoomReader:onBet");
	local data = {};
	data.retcode = self.m_socket:readInt(packetId, -1);
	if data.retcode == 0 then
		data.type   = self.m_socket:readInt(packetId, -1);
		data.param 	= self.m_socket:readInt(packetId, -1);
		data.money	= self.m_socket:readInt64(packetId, -1);
	end
	return data;
end
function RoomReader:onOtherBet(packetId)
	printInfo("RoomReader:onOtherBet");
	local data = {};
	data.seatId = self.m_socket:readInt(packetId, -1) + 1;
	data.type   = self.m_socket:readInt(packetId, -1);
	data.param 	= self.m_socket:readInt(packetId, -1);
	data.money	= self.m_socket:readInt64(packetId, -1);
	return data;
end


function RoomReader:onGameStart(packetId)
	printInfo("RoomReader:onGameStart");
	local data = {};
	return data;
end
function RoomReader:onStartBet(packetId)
	printInfo("RoomReader:onStartBet");
	local data = {};
	return data;
end

--  point1 		: byte(骰子1点数)
-- 	point2 		: byte(骰子2点数)
-- 	point3 		: byte(骰子3点数)
-- 	user_count 	: int (桌子上的玩家数)
-- 	for (int i = 0; i < user_count; ++ i)
-- 	{
-- 		seat_id : int (玩家座位号)
-- 		uid 	: int (用户id)
-- 		win_money: int64 (赢的钱,即桌子上需要返回给该玩家的的筹码)
-- 	}	
function RoomReader:onGameEnd(packetId)
	printInfo("RoomReader:onGameEnd");
	local data = {};
	data.p1 = self.m_socket:readByte(packetId, -1);
	data.p2 = self.m_socket:readByte(packetId, -1);
	data.p3 = self.m_socket:readByte(packetId, -1);
	data.userCount = self.m_socket:readInt(packetId, -1);
	data.user = {};
	for i = 1, data.userCount do
		local user 		= {};
		user.seatId 	= self.m_socket:readInt(packetId, -1) + 1;
		user.uid 		= self.m_socket:readInt(packetId, -1);
		user.betCount 	= self.m_socket:readInt(packetId, -1);
		user.bet = {};
		for j = 1, user.betCount do
			local bet = {};
			bet.type 	 = self.m_socket:readInt(packetId, -1);
			bet.param 	 = self.m_socket:readInt(packetId, -1);
			bet.multiple = self.m_socket:readInt(packetId, -1);
			bet.chip 	 = self.m_socket:readInt64(packetId, -1);
			bet.win 	 = self.m_socket:readInt64(packetId, -1);
			table.insert(user.bet, bet);
		end

		table.insert(data.user, user)
	end
	data.userCountUp = self.m_socket:readInt(packetId, -1)
	if data.userCountUp then
		data.userUp = {}
		for k = 1, data.userCountUp do
			local userUp = {}
			userUp.uid = self.m_socket:readInt(packetId, -1)
			userUp.exp = self.m_socket:readInt(packetId, -1)
			userUp.WinTimes = self.m_socket:readInt(packetId, -1)
			userUp.LoseTimes = self.m_socket:readInt(packetId, -1)
			table.insert(data.userUp, userUp)
		end
        data.userIncExp = {}
        for k = 1, data.userCountUp do
            local userIncExp = {}
            userIncExp.uid = self.m_socket:readInt(packetId, -1)
            userIncExp.turn_exp = self.m_socket:readInt(packetId, -1)
            table.insert(data.userIncExp, userIncExp)
        end
	end
	return data;
	
end

-- ret_code : int (返回码 0成功)
-- seat_id : int (座位号)
-- reason  : int (1玩家主动站起 2携带不够最下买入 3连续两局没下注)

function RoomReader:onStandUp(packetId)
	printInfo("RoomReader:onStandUp");
	local data = {};
	data.code = self.m_socket:readInt(packetId, -1);
	if data.code == 0 then
		data.seatId = self.m_socket:readInt(packetId, -1) + 1;
		data.reason = self.m_socket:readInt(packetId, -1);
        data.money = self.m_socket:readInt64(packetId, 0)
	end
	return data;
end
-- seat_id : int (座位号)
function RoomReader:onBroadcastStandUp(packetId)
	printInfo("RoomReader:onBroadcastStandUp");
	local data = {};
	data.seatId = self.m_socket:readInt(packetId, -1) + 1;
	return data;
end
-- money: int64 (同步money)
function RoomReader:onExitRoom(packetId)
	printInfo("RoomReader:onExitRoom");
	local data = {};
	data.money = self.m_socket:readInt64(packetId, -1);
	return data;
end

--   type 	: int (类型)
--	mid 	: int (触发用户id)
--	dest_mid: int (目标用户id)
--	msg_info: string (发给客户端的json串)
--	default : string (预留字段,默认传空字符串即可)字段,默认传空字符串即可)
    -- 发送互动道具
function RoomReader:onSendProp(packetId)

	printInfo("RoomReader:onSendProp"..packetId);

	local data 		= {};
    data.type 		= self.m_socket:readInt(packetId, -1);
	data.mid 		= self.m_socket:readInt(packetId, -1);
    data.dest_mid 	= self.m_socket:readInt(packetId, -1);
    data.msg_info 	= self.m_socket:readString(packetId);
    data.default 	= self.m_socket:readString(packetId);
    
	return data;
end

function RoomReader:onSendFace(packetId)
	printInfo("RoomReader:onSendFace"..packetId);
	local data = {};
	data.mid = self.m_socket:readInt(packetId, -1);
	data.type = self.m_socket:readInt(packetId, -1);
	data.isVipFace	= self.m_socket:readInt(packetId, -1);
	return data;
end

function RoomReader:onSendChat(packetId)
	printInfo("RoomReader:onSendChat"..packetId);
	local data = {};
	data.mid = self.m_socket:readInt(packetId, -1);
	data.msg = self.m_socket:readString(packetId);
	return data;
end

function RoomReader:initCommandFuncMap()
	RoomReader.super.initCommandFuncMap(self)
	require('dice.protocol.protocol')
	--[[
		骰子接收协议
	]]
	local s_severCmdFunMap = {
		[DICE_GET_TABLEID_RSP] 		= self.onGetTableId,	--
		[DICE_FOLLOW_RSP] 			= self.onFollow,	--
		[DICE_FOLLOWED_RSP] 		= self.onFollowed,	--
		[DICE_LOGIN_GAME_RSP] 		= self.onLoginGame,
		[DICE_LOGIN_GAME_ERROR_RSP] = self.onLoginErrorGame,
		[DICE_RELOGIN_GAME_RSP] 	= self.onReLoginGame,
		[DICE_SITDOWN_IN_GAME_RSP] 	= self.onSitDownInGame,
		[DICE_BROADCAST_SITDOWN_RSP]= self.onBroadcastSitdown,
		[DICE_START_BET_RSP] 		= self.onStartBet,
		[DICE_BET_RSP] 				= self.onBet,
		[DICE_OTHER_BET_RSP] 		= self.onOtherBet,
		[DICE_GAME_START_RSP] 		= self.onGameStart,
		[DICE_GAME_END_RSP] 		= self.onGameEnd,
		[DICE_STANDUP_RSP] 			= self.onStandUp,
		[DICE_BROADCAST_STANDUP_RSP]= self.onBroadcastStandUp,
		[DICE_EXIT_ROOM_RSP] 		= self.onExitRoom,
        [DICE_SEND_PROP_RSP] 		= self.onSendProp,
        [DICE_SEND_FACE]			= self.onSendFace,
        [DICE_SEND_CHAT]			= self.onSendChat,
	}
	-- 避免覆盖baseReader的方法
	table.merge(self.s_severCmdFunMap, s_severCmdFunMap)
end

return RoomReader
