local playerLayout = requireview("view.dice.playerLayout")
local diceCupLayout= requireview("view.dice.diceCupLayout")
local roomPlayerInfoLayout= requireview("view.dice.roomPlayerInfoLayout")
local historyItemLayout = requireview("view.dice.historyItemLayout")
local roomStartTip = requireview("view.dice.roomStartTip")
local roomClock = requireview("view.dice.roomClock")

require('dice.protocol.protocol')
require('dice.popu.config')

require("pintu.dice.bet")
require("pintu.dice.resultnumber")
require("pintu.dice.dicePoint")

require("pintu.gift")

local AnimationFriend 		= require('animation.friendsAnim.animationFriend')
local DiceRoomController 	= class(BaseController)
local printInfo, printError = overridePrint("DiceRoomController")
local AnimChat = require("animation/animChat")
local AnimFace = require("animation.animFace")
local FaceConfig = require("room.chat.faceConfig")
local STRING = require("dice.res.config")

local MY_LOCAL_SEAT_ID = 5

local BET_TYPE_BIG				= 1 -- 大
local BET_TYPE_SMALL			= 2 -- 小
local BET_TYPE_THREE			= 3 -- 3条
local BET_TYPE_SPECIAL_THREE	= 4 -- 特定3条
local BET_TYPE_TOTAL_POINTS		= 5 -- 猜总点数
local BET_TYPE_SINGLE_POINTS	= 6 -- 猜单个骰子点数

local ERROR_TABLE_MAX_COUNT    	= 2          --桌子人满
local ERROR_NOT_ENOUGH_MONEY   	= 3          --钱太少
local ERROR_TOO_MUCH_MONEY	 	= 4          --钱太多
local ERROR_WRONG_UID			= 5          --无效的用户ID
local ERROR_TABLE_NOT_EXIST	 	= 6          --桌子不存在
local ERROR_ANTE_MONEY         	= 7		   --携带金额错误
local ERROR_SEAT_HAS_PLAYER	 	= 8		   --座位上有人

function DiceRoomController:ctor(viewConfig, param1, param2, param3)
	dicePopuInit()
    self:loadPicture()

	self.mGameId  	= 13002
	self.mRoomInfo  = param3[2]
	self.mStateFrom = param3[1]
	self.mFollowId  = param3[3]
	self.mFollowNick= param3[4] or ''
	--连胜局数
	self.winTimes = 0
	--送筹码配置
	self.mPresent = self.mRoomInfo:getPresent()
	-- 
	--骰子游戏所需要的数据结构
	self.mTableId = setProxy(new(require('dice.data.tableId')))
	self.mPlayer  = {};
	for i = 1, 9 do
		self.mPlayer[i] = setProxy(new(require('dice.data.player')))
	end
	--下注
	self.mBet = {};
	for i = 1, 4 do
		self.mBet[i] = setProxy(new(require('dice.data.bet')))
	end

	--玩家输赢缓存
	self.mGameResultTemp = nil;

	self.mLastBetted = {}; 	--上场下注
	self.mBetted 	= {}; 	--已下注

	--加入骰子SERVER协议
	self.m_reader		= new(require('dice.protocol.room.roomReader'))
	self.m_writer		= new(require('dice.protocol.room.roomWriter'))
	self.m_processer 	= new(require('dice.protocol.room.roomProcesser'))

	GameSocketMgr:addCommonSocketReader(self.m_reader);
	GameSocketMgr:addCommonSocketWriter(self.m_writer);
	GameSocketMgr:addCommonSocketProcesser(self.m_processer);
	--HTTP
	EventDispatcher.getInstance():register(HttpModule.s_event, self, self.onHttpRequestsCallBack);
	EventDispatcher.getInstance():register(Event.onEventResume,self,self.onEventResume);
	EventDispatcher.getInstance():register(Event.onEventPause,self,self.onEventPause);

    --记录最近玩的游戏
    GameSetting:addGameRecord(self.mGameId)
    --发送游戏注册统计，以前没有注册过
    local registerGames = GameSetting:getRegisterGame()
    if not string.find(registerGames, tostring(self.mGameId)) then
        HttpModule.getInstance():execute(HttpModule.s_cmds.SEND_REGISTER_GAME_STA, {gameId = self.mGameId}, false, false)
        registerGames = registerGames..","..self.mGameId
        GameSetting:setRegisterGame(registerGames)
        GameSetting:save()
    end
	
	self.m_root:findChildByName("btn_back"):setOnClick(self, function ( self )
		-- body
		--退出房间
		WindowManager:showWindow(WindowTag.menuPopu,{backFunc=function()
    			self:onBack()
    		end,ruleFunc=function ( ... )
    			WindowManager:showWindow(WindowTag.rulePopu,{},WindowStyle.POPUP)
    		end,feedbackFunc = function()
    			WindowManager:showWindow(WindowTag.FeedbackPopu, {feedbackId = 3, chooseGame = false}, WindowStyle.POPUP)
    		end},WindowStyle.NORMAL)
	end);


	----------------------------------------------------------------------
	--游戏UI
	----------------------------------------------------------------------

	UIEx.bind(self, MyUserData, "money", function(value)
		local mySeatId = self:getMySeatId() or 0
		if mySeatId > 0 then
			local player = self.mPlayer[mySeatId]
			if value ~= player:getChip() then
				player:setChip(value)
			end
		end
	end)
	--初始化座位
	for i = 1, #self.mPlayer do
		local playerData = self.mPlayer[i];
		local player 	 = SceneLoader.load(playerLayout);
		local chipWin 	 = self:findChildByName("view_chipwin"):findChildByName(string.format("view_%s", i));

		playerData:setUi(player);
		UIEx.bind(player, playerData, "localSeatId", function(value)
			if value ~= 0 then
				player:setPos(self:getLocalSeatPos(value, false));
				--屏幕左边的4个人，礼物图标要移到右下角
	            if value >= 6 then
	                player:findChildByName("btn_gift"):setAlign(kAlignBottomRight)
                else
                    player:findChildByName("btn_gift"):setAlign(kAlignBottomLeft)
	            end
			end
		end)
		UIEx.bind(player, playerData, "id", function(value)
			local chipBg = player:findChildByName("img_chipbg")
			if value ~= 0 then
				chipBg:setVisible(value ~= MyUserData:getId());
                player:findChildByName("btn_gift"):setVisible(true)
			else
				chipBg:setVisible(false);
				local headView = player:findChildByName("view_phead");
				headView:removeAllChildren();
                player:findChildByName("btn_gift"):setVisible(false)
			end
		end)

		UIEx.bind(player, playerData, "chip", function(value)
			
			if MyUserData:getId() ~= playerData:getId() then
				player:findChildByName('text_chip'):setText(ToolKit.formatMoney(value));
			else
				local chipView = self:findChildByName('view_bottom'):findChildByName("view_chipnumber");
				chipView:removeAllChildren();
				local chipNode = new(require('common.imageNumber'), {
					['0'] = chipNumber_map["0.png"],
					['1'] = chipNumber_map["1.png"],
					['2'] = chipNumber_map["2.png"],
					['3'] = chipNumber_map["3.png"],
					['4'] = chipNumber_map["4.png"],
					['5'] = chipNumber_map["5.png"],
					['6'] = chipNumber_map["6.png"],
					['7'] = chipNumber_map["7.png"],
					['8'] = chipNumber_map["8.png"],
					['9'] = chipNumber_map["9.png"],
					['b'] = chipNumber_map["b.png"],
					['m'] = chipNumber_map["m.png"],
					['k'] = chipNumber_map["k.png"],
					['.'] = chipNumber_map["p.png"],
				});	

				chipNode:setNumber(string.format("%s", ToolKit.formatMoney(value)));
				chipNode:setAlign(kAlignLeft);
				chipView:addChild(chipNode);

				--同步钱
				if value ~= MyUserData:getMoney() then
					MyUserData:setMoney(value);
				end
                
                local curSelectIndex = nil
                for i = 1, #self.mBet do
		            if self.mBet[i]:getIsSelected() then
                        curSelectIndex = i
                        break;
		            end
	            end
                if not curSelectIndex then
                    curSelectIndex = 4
                    self.mBet[curSelectIndex]:setIsSelected(true)
                end
                local ante = self.mRoomInfo:getChip()
                local newSelectIndex = nil
                for i = #self.mBet, 1, -1 do
                    if self.mBet[i]:getAnte() > MyUserData:getMoney() then
                        self.mBet[i]:setAvailable(false)
                        self.mBet[i]:setIsSelected(false)
                        newSelectIndex = i - 1
                    else
                        self.mBet[i]:setAvailable(true)
                    end
                end
                if newSelectIndex and newSelectIndex > 0 and newSelectIndex < curSelectIndex then
                   self.mBet[newSelectIndex]:setIsSelected(true) 
                end

			end
		end)

		UIEx.bind(player, playerData, "win", function(value)
			-- view_chipwin
			--local chip = player:findChildByName("view_win")
			chipWin:removeAllChildren();
			if value and tonumber(value) > 0 then

				local chipNode = new(require('common.imageNumber'), {
					['0'] = dice_room_resultnumber_map["0.png"],
					['1'] = dice_room_resultnumber_map["1.png"],
					['2'] = dice_room_resultnumber_map["2.png"],
					['3'] = dice_room_resultnumber_map["3.png"],
					['4'] = dice_room_resultnumber_map["4.png"],
					['5'] = dice_room_resultnumber_map["5.png"],
					['6'] = dice_room_resultnumber_map["6.png"],
					['7'] = dice_room_resultnumber_map["7.png"],
					['8'] = dice_room_resultnumber_map["8.png"],
					['9'] = dice_room_resultnumber_map["9.png"],
					['b'] = dice_room_resultnumber_map["b.png"],
					['m'] = dice_room_resultnumber_map["m.png"],
					['k'] = dice_room_resultnumber_map["k.png"],
					['.'] = dice_room_resultnumber_map["p.png"],
					['+'] = dice_room_resultnumber_map["+.png"],
				});	

				chipNode:setNumber(string.format("+%s", value));--ToolKit.formatMoney(value)
				--chipNode:setAlign(kAlignLeft);
				local x, y = player:getAbsolutePos();
				chipNode:setPos(x + 61, y + 9);
				if playerData:getLocalSeatId() == 3 or playerData:getLocalSeatId() == 4 then
					local w, h = chipNode:getSize();
					chipNode:setPos(x + 61 -w - 25, y + 9);
				end
				chipWin:addChild(chipNode);
				chipWin:setVisible(true);
				--如果自己赢
			else
				chipWin:setVisible(false);
			end
		end)

		UIEx.bind(player, playerData, "nick", function(value)
			
			if MyUserData:getId() ~= playerData:getId() then
			else
				self:findChildByName('view_bottom'):findChildByName('text_nick'):setText(value);
			end
		end)

		UIEx.bind(player, playerData, "headName", function(value)
			if playerData:getId() ~= 0 then
				local headView = player:findChildByName("view_phead");
				headView:removeAllChildren();
				local width, height = headView:getSize()
				local imgHead 		= new(Mask, value, "dice/head_mask.png");
				imgHead:setSize(width, height)
				headView:addChild(imgHead);
				playerData:checkHeadAndDownload()
			end
		end)
		
		UIEx.bind(player, playerData, "sitdownAnim", function(value)
			if value then
				local viewHead 	= player:findChildByName("view_phead");
				local x, y  	= viewHead:getAbsolutePos();
				local w, h  	= viewHead:getSize();
				local sw, sh 	= System.getScreenScaleWidth(), System.getScreenScaleHeight();
				local dx, dy 	= (sw - w) / 2 - x, (sh - h) / 2 - y;

				local sitdownAnim = viewHead:addPropTranslateWithEasing(1, kAnimNormal, 500, 0, "easeOutSine", "easeOutSine", 0, dx, 0, dy);
				sitdownAnim:setEvent(self, function()
						checkAndRemoveOneProp(viewHead, 1);
						--重新排位
						self:reLocateSeat();
						local x, y  	= viewHead:getAbsolutePos();
						local w, h  	= viewHead:getSize();
						local sw, sh 	= System.getScreenScaleWidth(), System.getScreenScaleHeight();
						local dx, dy 	= (sw - w) / 2 - x, (sh - h) / 2 - y;
						local sitdownAnim = viewHead:addPropTranslateWithEasing(1, kAnimNormal, 500, 0, "easeOutSine", "easeOutSine", dx, -dx, dy, -dy);
						sitdownAnim:setEvent(self, function()
								checkAndRemoveOneProp(viewHead, 1);
								self:playerOperationInAnim();
						end);
				end);
			end
		end)
		--只针对自己有效
		UIEx.bind(player, playerData, "inAnim", function(value)
			if value then
				player:setPos(self:getLocalSeatPos(MY_LOCAL_SEAT_ID, true));
				checkAndRemoveOneProp(player, 1);
				checkAndRemoveOneProp(player, 2);

				local inAnim = player:addPropTranslateWithEasing(1, kAnimNormal, 500, 0, "easeOutSine", "easeOutSine", 0, 0, 120, -120);
				inAnim:setEvent(self, function()
						checkAndRemoveOneProp(player, 1);
                    --动画播放完成后再显示礼物按钮
                    if MyUserData:getId() == playerData:getId() then
                        player:findChildByName("btn_gift"):setVisible(true)
                        playerData:setGiftId(MyUserData:getGiftId() and {MyUserData:getGiftId()[1], MyUserData:getGiftId()[2]})
                    end
				end);
				player:addPropScale(2, kAnimNormal, 0, 0, 1.15, 1.15, 1.15, 1.15, kCenterDrawing);
			end
		end)

        UIEx.bind(player, playerData, "giftId", function(value)
            local giftList = MyUserData:getGiftList()
            if value and value[1] and value[2] and giftList then
                local gift = giftList[value[1]] and giftList[value[1]][value[2]]
                if gift then
                    player:findChildByName("btn_gift"):setFile(gift:getImgName())
                    if playerData:getId() == MyUserData:getId() then
                    	local userInfo = {appid = 0, nick = MyUserData:getNick(), sex = MyUserData:getSex(), micon = MyUserData:getHeadUrl(), giftId = MyUserData:getGiftId()}
						GameSocketMgr:sendMsg(DICE_CLIENT_SEND_USERINFOR, {userInfo = json.encode(userInfo)});
					end
                    return
                end
            end
            player:findChildByName("btn_gift"):setFile(gift_map["gift.icon_gift.png"])
        end)

		player:findChildByName("btn_player"):setOnClick(self, function ( self )
		-- body
			local id = playerData:getId();
            local mySeat = self:getMySeatId()
			if id == 0 and not mySeat then
				local seatId = playerData:getSeatId();
                if self:checkSitDown() then
				    GameSocketMgr:sendMsg(DICE_SITDOWN_IN_GAME_REQ, {u32_seatId = seatId, u64_ante = MyUserData:getMoney()});
                end
			end

			if id ~= 0 and playerData:getLocalSeatId() ~= 0 then
				--弹出个人信息
                local mySeatId = self:getMySeatId() or 0
                local curplayer= mySeatId > 0 and self.mPlayer[mySeatId] or nil;
                --记录登录玩家
	            ---弹出个人信息
                local playerIds = {}
                for i = 1, #self.mPlayer do
                    if self.mPlayer[i]:getId() ~= 0 then
                        table.insert(playerIds, self.mPlayer[i]:getId())
                    end
                end
                --自己没坐下，也就是围观中，广播通知的人包括我自己
                if not mySeat then
                    table.insert(playerIds, MyUserData:getId())
                end
                local windowParam = {player = playerData, present = self.mPresent, gameId = self.mGameId, seatId = self:getMySeatId() or 0, playerIds = playerIds}
                WindowManager:showWindow(WindowTag.RoomPlayerInfoPopu, windowParam, WindowStyle.POPUP);
			end
		end);
        
        --点击弹窗礼物窗口
        player:findChildByName("btn_gift"):setOnClick(self, function(value)
            local playerIds = {}
            for i = 1, #self.mPlayer do
                if self.mPlayer[i]:getId() ~= 0 then
                    table.insert(playerIds, self.mPlayer[i]:getId())
                end
            end
            local mySeat = self:getMySeatId()
            --自己没坐下，也就是围观中，广播通知的人包括我自己
            if not mySeat then
                table.insert(playerIds, MyUserData:getId())
            end
            --点击的是自己的头像上的，弹窗带购买
            if playerData:getId() == MyUserData:getId() then
                WindowManager:showWindow(WindowTag.GiftStorePopu, {from = "myInfoInGame", broadcastId = playerIds}, WindowStyle.POPUP)
            else
                WindowManager:showWindow(WindowTag.GiftStorePopu, {from = "otherInfoInGame", sendId = {playerData:getId()}, broadcastId = playerIds}, WindowStyle.POPUP)
            end
        end)

        self:findChildByName("view_players"):addChild(player);
	end
	
	for i = 1, #self.mPlayer do
		self.mPlayer[i]:clear();
		self.mPlayer[i]:setSeatId(i);
		self.mPlayer[i]:setLocalSeatId(i);
	end

	--初始化下注
	--下注区域

	local betInfo = {
		{name = 'btn_big', 			type = BET_TYPE_BIG, 			param = 0},
		{name = 'btn_small', 		type = BET_TYPE_SMALL, 			param = 0},
		{name = 'btn_three1', 		type = BET_TYPE_SPECIAL_THREE, 	param = 1},
		{name = 'btn_three2', 		type = BET_TYPE_SPECIAL_THREE, 	param = 2},
		{name = 'btn_three3', 		type = BET_TYPE_SPECIAL_THREE, 	param = 3},
		{name = 'btn_three4', 		type = BET_TYPE_SPECIAL_THREE, 	param = 4},
		{name = 'btn_three5', 		type = BET_TYPE_SPECIAL_THREE, 	param = 5},
		{name = 'btn_three6', 		type = BET_TYPE_SPECIAL_THREE, 	param = 6},
		{name = 'btn_three', 		type = BET_TYPE_THREE, 		   	param = 0},
		{name = 'btn_point4', 		type = BET_TYPE_TOTAL_POINTS, 	param = 4},
		{name = 'btn_point5', 		type = BET_TYPE_TOTAL_POINTS, 	param = 5},
		{name = 'btn_point6', 		type = BET_TYPE_TOTAL_POINTS, 	param = 6},
		{name = 'btn_point7', 		type = BET_TYPE_TOTAL_POINTS, 	param = 7},
		{name = 'btn_point8', 		type = BET_TYPE_TOTAL_POINTS, 	param = 8},
		{name = 'btn_point9', 		type = BET_TYPE_TOTAL_POINTS, 	param = 9},
		{name = 'btn_point10', 		type = BET_TYPE_TOTAL_POINTS, 	param = 10},
		{name = 'btn_point11', 		type = BET_TYPE_TOTAL_POINTS, 	param = 11},
		{name = 'btn_point12', 		type = BET_TYPE_TOTAL_POINTS, 	param = 12},
		{name = 'btn_point13', 		type = BET_TYPE_TOTAL_POINTS, 	param = 13},
		{name = 'btn_point14', 		type = BET_TYPE_TOTAL_POINTS, 	param = 14},
		{name = 'btn_point15', 		type = BET_TYPE_TOTAL_POINTS, 	param = 15},
		{name = 'btn_point16', 		type = BET_TYPE_TOTAL_POINTS, 	param = 16},
		{name = 'btn_point17', 		type = BET_TYPE_TOTAL_POINTS, 	param = 17},
		{name = 'btn_singlePoint1', type = BET_TYPE_SINGLE_POINTS, 	param = 1},
		{name = 'btn_singlePoint2', type = BET_TYPE_SINGLE_POINTS, 	param = 2},
		{name = 'btn_singlePoint3', type = BET_TYPE_SINGLE_POINTS, 	param = 3},
		{name = 'btn_singlePoint4', type = BET_TYPE_SINGLE_POINTS, 	param = 4},
		{name = 'btn_singlePoint5', type = BET_TYPE_SINGLE_POINTS, 	param = 5},
		{name = 'btn_singlePoint6', type = BET_TYPE_SINGLE_POINTS, 	param = 6}
	}	
	--下注筹码
	self.mChipBatch = new(require('animation.animBatch'), chip_map, 'chip.png')
	self:findChildByName('view_chiplevel'):addChild(self.mChipBatch);
	-- 下注
	self.mBetArea = {}
	-- 
	--
	for k, v in pairs(betInfo) do
		local betArea = setProxy(new(require('dice.data.betArea')));

		local btn = self:findChildByName(v.name);
		btn:setOnClick(self, function ( self )
		-- body
			GameSocketMgr:sendMsg(DICE_BET_REQ, {u32_betType = v.type, u32_param = v.param, u64_money = self:getCurSelectedAnte()});
		end)
		btn:enableAnim(false);
		btn:enableSound(false);
		btn:enableShow(false);

		UIEx.bind(btn, betArea, "chip", function(value)
			if value == 0 then
				btn:findChildByName("img_chipbg"):setVisible(false);
				btn:findChildByName("view_bet"):removeAllChildren();
			else
				local chip = btn:findChildByName("img_chipbg")
				chip:removeAllChildren();
				local chipNode = new(require('common.imageNumber'), {
					['0'] = dice_room_bet_map["0.png"],
					['1'] = dice_room_bet_map["1.png"],
					['2'] = dice_room_bet_map["2.png"],
					['3'] = dice_room_bet_map["3.png"],
					['4'] = dice_room_bet_map["4.png"],
					['5'] = dice_room_bet_map["5.png"],
					['6'] = dice_room_bet_map["6.png"],
					['7'] = dice_room_bet_map["7.png"],
					['8'] = dice_room_bet_map["8.png"],
					['9'] = dice_room_bet_map["9.png"],
					['b'] = dice_room_bet_map["b.png"],
					['m'] = dice_room_bet_map["m.png"],
					['k'] = dice_room_bet_map["k.png"],
					['.'] = dice_room_bet_map["p.png"],
				});	

				chipNode:setNumber(string.format("%s", ToolKit.formatMoney(value)));
				chipNode:setAlign(kAlignCenter);
				chip:addChild(chipNode);
				chip:setVisible(true);
			end
		end)


		UIEx.bind(btn, betArea, "chipVisible", function(value)
			btn:findChildByName("img_chipbg"):setVisible(value);
		end)

		UIEx.bind(btn, betArea, "allChip", function(value)
			if value == 0 then
				btn:findChildByName("view_allchip"):setVisible(false);
				btn:findChildByName("view_allchip"):removeAllChildren();
			else
				local chip = btn:findChildByName("view_allchip")
				chip:removeAllChildren();
				local chipNode = new(require('common.imageNumber'), {
					['0'] = dice_room_bet_map["s0.png"],
					['1'] = dice_room_bet_map["s1.png"],
					['2'] = dice_room_bet_map["s2.png"],
					['3'] = dice_room_bet_map["s3.png"],
					['4'] = dice_room_bet_map["s4.png"],
					['5'] = dice_room_bet_map["s5.png"],
					['6'] = dice_room_bet_map["s6.png"],
					['7'] = dice_room_bet_map["s7.png"],
					['8'] = dice_room_bet_map["s8.png"],
					['9'] = dice_room_bet_map["s9.png"],
					['b'] = dice_room_bet_map["sb.png"],
					['m'] = dice_room_bet_map["sm.png"],
					['k'] = dice_room_bet_map["sk.png"],
					['.'] = dice_room_bet_map["sp.png"],
				});	

				chipNode:setNumber(string.format("%s", ToolKit.formatMoney(value)));
				chipNode:setAlign(kAlignTop);
				chip:addChild(chipNode);
				chip:setVisible(true);
			end


		end)

		UIEx.bind(btn, betArea, "allChipVisible", function(value)
			btn:findChildByName("view_allchip"):setVisible(value);
		end)

		betArea:setNode(btn);

		self.mBetArea[self:getBetAreaKey(v.type, v.param)]   = betArea;
		--筹码
	end
	--发光层
	local betInfo = {
		{name = 'view_big', 			type = BET_TYPE_BIG, 			param = 0},
		{name = 'view_small', 		type = BET_TYPE_SMALL, 			param = 0},
		{name = 'view_three1', 		type = BET_TYPE_SPECIAL_THREE, 	param = 1},
		{name = 'view_three2', 		type = BET_TYPE_SPECIAL_THREE, 	param = 2},
		{name = 'view_three3', 		type = BET_TYPE_SPECIAL_THREE, 	param = 3},
		{name = 'view_three4', 		type = BET_TYPE_SPECIAL_THREE, 	param = 4},
		{name = 'view_three5', 		type = BET_TYPE_SPECIAL_THREE, 	param = 5},
		{name = 'view_three6', 		type = BET_TYPE_SPECIAL_THREE, 	param = 6},
		{name = 'view_three', 		type = BET_TYPE_THREE, 		   	param = 0},
		{name = 'view_point4', 		type = BET_TYPE_TOTAL_POINTS, 	param = 4},
		{name = 'view_point5', 		type = BET_TYPE_TOTAL_POINTS, 	param = 5},
		{name = 'view_point6', 		type = BET_TYPE_TOTAL_POINTS, 	param = 6},
		{name = 'view_point7', 		type = BET_TYPE_TOTAL_POINTS, 	param = 7},
		{name = 'view_point8', 		type = BET_TYPE_TOTAL_POINTS, 	param = 8},
		{name = 'view_point9', 		type = BET_TYPE_TOTAL_POINTS, 	param = 9},
		{name = 'view_point10', 		type = BET_TYPE_TOTAL_POINTS, 	param = 10},
		{name = 'view_point11', 		type = BET_TYPE_TOTAL_POINTS, 	param = 11},
		{name = 'view_point12', 		type = BET_TYPE_TOTAL_POINTS, 	param = 12},
		{name = 'view_point13', 		type = BET_TYPE_TOTAL_POINTS, 	param = 13},
		{name = 'view_point14', 		type = BET_TYPE_TOTAL_POINTS, 	param = 14},
		{name = 'view_point15', 		type = BET_TYPE_TOTAL_POINTS, 	param = 15},
		{name = 'view_point16', 		type = BET_TYPE_TOTAL_POINTS, 	param = 16},
		{name = 'view_point17', 		type = BET_TYPE_TOTAL_POINTS, 	param = 17},
		{name = 'view_singlePoint1', type = BET_TYPE_SINGLE_POINTS, 	param = 1},
		{name = 'view_singlePoint2', type = BET_TYPE_SINGLE_POINTS, 	param = 2},
		{name = 'view_singlePoint3', type = BET_TYPE_SINGLE_POINTS, 	param = 3},
		{name = 'view_singlePoint4', type = BET_TYPE_SINGLE_POINTS, 	param = 4},
		{name = 'view_singlePoint5', type = BET_TYPE_SINGLE_POINTS, 	param = 5},
		{name = 'view_singlePoint6', type = BET_TYPE_SINGLE_POINTS, 	param = 6}
	}
	for k, v in pairs(betInfo) do
		local betview = self:findChildByName(v.name);
		local betArea = self.mBetArea[self:getBetAreaKey(v.type, v.param)]

		UIEx.bind(betview, betArea, "lighted", function(value)
			imgLight = betview:findChildByName("img_selected");
			if value then
				imgLight:addPropTransparency(1, kAnimLoop, 500, 0, 0.5, 1);
				imgLight:setVisible(true);
			else
				imgLight:setVisible(false);
				checkAndRemoveOneProp(imgLight, 1);
			end
		end)
	end

	--初始化底注
	local ante = self.mRoomInfo:getChip();--{50,250,500,2500}	
    for anteIndex = 1, #self.mBet do
		local bet = self.mBet[anteIndex];
		local btnChip = self:findChildByName("btn_chip"..anteIndex);

		btnChip:setFile(string.format('dice/room/chip_%s.png', ante[anteIndex]))
		
		-- UIEx.bind(btnChip, bet, "ante", function(value)
		-- 	btnChip:findChildByName("text_bet"):setText(ToolKit.formatChip(tostring(value)));
		-- end)
		UIEx.bind(btnChip, bet, "isSelected", function(value)
			btnChip:findChildByName("img_selected"):setVisible(value);
		end)

        UIEx.bind(btnChip, bet, "available", function(value)
            btnChip:findChildByName("img_unSelected"):setVisible(not value)
        end)

		btnChip:setOnClick(self, function ( self )
			-- body
            if MyUserData:getMoney() < ante[anteIndex] then
                AlarmTip.play(STR_LESS_THAN_CHIP)
                return
            end
		    for i = 1, #self.mBet do
				self.mBet[i]:setIsSelected(anteIndex == i);
			end
		end)
		bet:setAnte(ante[anteIndex]);
		bet:setUi(btnChip);
	end
	--默认第一个
	self.mBet[1]:setIsSelected(true);

	self.mClickLast 	= false;
	self.mClickDouble 	= false;

    --商城按钮
    self:findChildByName("view_bottom"):findChildByName("btn_addChip"):setOnClick(self, function(self)
        WindowManager:showWindow(WindowTag.ShopPopu, {}, WindowStyle.POPUP)
    end)
	--双倍
	local btnDouble = self:findChildByName("btn_double");
	btnDouble:findChildByName("btn_title"):setText(STR_DOUBLE);
	btnDouble:setOnClick(self, function ( self )
		-- body
		if self.mClickDouble then
			return
		end
		for k, v in pairs(self.mBetted) do
			self.mClickDouble 	= true;
			GameSocketMgr:sendMsg(DICE_BET_REQ, {u32_betType = v.type, u32_param = v.param, u64_money = v.money});
		end
		
	end)

	local btnLast = self:findChildByName("btn_last");
	btnLast:findChildByName("btn_title"):setText(STR_REPEATLAST);
	btnLast:setOnClick(self, function ( self )
		-- body
		for k, v in pairs(self.mLastBetted) do
			self.mClickLast = true;
			GameSocketMgr:sendMsg(DICE_BET_REQ, {u32_betType = v.type, u32_param = v.param, u64_money = v.money});
		end
		
	end)

	local btnHistory = self:findChildByName("btn_history");
	btnHistory:setOnClick(self, function ( self )
		-- body
		self:showHistoryAnim();
	end)

	local btnTouch = self:findChildByName("btn_touch");
	btnTouch:setOnClick(self, function ( self )
		-- body
		btnTouch:setVisible(false);
		self:hideHistoryAnim();
	end)
	----------------------------------------------------------------------
	--游戏逻辑
	----------------------------------------------------------------------
	--
	UIEx.bind(self.m_root, self.mTableId, "gameId", function(value)
		if self.mTableId:getGameId() ~= self.mGameId then
			-- StateChange.changeState(self.mStateFrom or States.GameLobby);
			local game = app:getGame(self.mTableId:getGameId())
			if game then 
				AlarmTip.play(string.format(STR_GO_BACK_TO_ORIGIN_GAME,game:getName()))
				game:enterRoom() 
			end
			return
		end
		local userInfo = {appid = 0, nick = MyUserData:getNick(), sex = MyUserData:getSex(), micon = MyUserData:getHeadUrl(), giftId = MyUserData:getGiftId()}

		if self.mFollowId then
			GameSocketMgr:sendMsg(DICE_FOLLOW_REQ, {u32_follow 	= self.mFollowId,
													str_key 	= MyUserData:getSessionId(),
													str_info 	= json.encode(userInfo),
													str_followNick = MyUserData:getNick(),
													});
		else
			GameSocketMgr:sendMsg(DICE_LOGIN_GAME_REQ, {u32_tableId = self.mTableId:getId(), 
													u32_uid = MyUserData:getId(),
													str_key = MyUserData:getSessionId(),
													str_info= json.encode(userInfo),
													u32_flag= 0});
		end
	

	end)
	--好友动画
	self.mAnimFriend = new(AnimationFriend, self)
	--开始loading
	self:showLoadingAnim();
	
	--请求桌子ID
	if self.mFollowId then
		self.mTableId:init({tableId = 0, isInTable = 0, gameLevel = 0, gameId = self.mGameId or 0});
	else
		GameSocketMgr:sendMsg(DICE_GET_TABLEID_REQ, {u16_gameLevel = self.mRoomInfo:getLevel(), u32_gameId = self.mGameId});
	end
	self:findChildByName('btn_chat'):setOnClick(self, function(self) 
			local mySeatIdTemp = self:getMySeatId() or 0
			if mySeatIdTemp <= 0 then
				AlarmTip.play(STR_SORRY_FOR_SEND_CHAT_BEFORE_SITDOWN)
			else
				WindowManager:showWindow(WindowTag.RoomChat, {gameid = self.mGameId, chatwords = STRING.SysChatArray}, WindowStyle.POPUP)
			end
		end)
    local btnTask = self:findChildByName('btn_task')
	btnTask:setOnClick(self, function(self) 
		WindowManager:showWindow(WindowTag.TaskPopu, {inRoom = true}, WindowStyle.POPUP)
	end)

	UIEx.bind(self, MyUserData, "taskAward", function(value)    
        btnTask:findChildByName("img_hasAward"):setVisible(value > 0)
        btnTask:findChildByName("img_noAward"):setVisible(value <= 0)
	end)
    MyUserData:setTaskAward(MyUserData:getTaskAward())

	self:findChildByName('btn_send_chips_to_dealer'):setOnClick(self, function(self)
			local mySeatId   = self:getMySeatId() or 0
	        if mySeatId <= 0 then
	            AlarmTip.play(STR_SORRY_FOR_SEND_CHIPS_TO_DEALER_BEFORE_SITDOWN);
	            return 
	        end
            --[[
            local lastTime = MyUserData:getChipTime()
            local curTime   = os.time()
            if curTime - lastTime < 2 then
                AlarmTip.play(string.format(STR_SORRY_FOR_SEND_CHIP_TOO_FREQUEENTLY, 5));
                return 
            end
         	MyUserData:setChipTime(curTime)
			]]
          	if self.mRoomInfo:getChip() and self.mRoomInfo:getChip()[1] and MyUserData:getMoney() >= self.mRoomInfo:getChip()[1] then
            	--HttpModule.getInstance():execute(HttpModule.s_cmds.GET_SEND_GAME_CHIP,{fid = 0, gid=self.mGameId, money = roomInfo:getChip()[1]}, false, true);
            	HttpModule.getInstance():execute(HttpModule.s_cmds.GIVING_TIPS,{fid = 0, mid = MyUserData:getId(), gid=self.mGameId, money = self.mRoomInfo:getChip()[1]}, false, true);
          	else
            	AlarmTip.play(STR_SORRY_FOR_NO_MONEY);
          	end
		end)
	    self:findChildByName('btn_dealer'):setOnClick(self, function(self) 
			WindowManager:showWindow(WindowTag.DealerProsPopu, {controller = self}, WindowStyle.NORMAL)
		end)
end

--[[
	显示loading
]]
function DiceRoomController:showLoadingAnim()
    self.toastShade = new(ToastShade, true, STR_LOAING, true)
    self:addChild(self.toastShade)
    self.toastShade:play()
end

function DiceRoomController:hideLoadingAnim()
    self.toastShade:stop()
end

--[[
	显示历史记录
]]
function DiceRoomController:showHistoryAnim( )
	-- body
	local btnHistory = self:findChildByName("btn_history");
	checkAndRemoveOneProp(btnHistory, 1);
	local w, h = btnHistory:getSize();
	local outAnim = btnHistory:addPropTranslateWithEasing(1, kAnimNormal, 300, 0, "easeOutSine", "easeOutSine", 0, 0, 0, -h);
	outAnim:setEvent(self, function()
			checkAndRemoveOneProp(btnHistory, 1);
			btnHistory:setVisible(false);
			local btnTouch = self:findChildByName("btn_touch");
			local viewHistory = self:findChildByName("view_history");
			local imgHistory = self:findChildByName("img_historybg");
			local w, h = imgHistory:getSize();
			local inAnim = imgHistory:addPropTranslateWithEasing(1, kAnimNormal, 300, 0, "easeOutSine", "easeOutSine", w, -w, 0, 0);
			inAnim:setEvent(self, function()
				checkAndRemoveOneProp(imgHistory, 1);
				-- self:hideHistoryAnim();
			end);
			viewHistory:setVisible(true);
			btnTouch:setVisible(true);

	end);
	btnHistory:setVisible(true);

end
--[[
	收起历史记录
]]
function DiceRoomController:hideHistoryAnim( )
	-- body
	local viewHistory  = self:findChildByName("view_history");
	local imgHistory = self:findChildByName("img_historybg");
	checkAndRemoveOneProp(imgHistory, 1);
	local w, h = imgHistory:getSize();
	local outAnim = imgHistory:addPropTranslateWithEasing(1, kAnimNormal, 300, 0, "easeOutSine", "easeOutSine", 0, w, 0, 0);
	outAnim:setEvent(self, function()
			checkAndRemoveOneProp(imgHistory, 1);
			viewHistory:setVisible(false);

			local btnHistory = self:findChildByName("btn_history");
			local w, h = btnHistory:getSize();
			local inAnim = btnHistory:addPropTranslateWithEasing(1, kAnimNormal, 300, 0, "easeOutSine", "easeOutSine", 0, 0, -h, h);
			inAnim:setEvent(self, function()
				checkAndRemoveOneProp(btnHistory, 1);
			end);
			btnHistory:setVisible(true);

	end);
	viewHistory:setVisible(true);

end

--[[
	坐下动画
]]
function DiceRoomController:playSitdownAnim()
	self:findChildByName("view_bottom"):setVisible(false);
	for i = 1, #self.mPlayer do
		self.mPlayer[i]:setSitdownAnim(true);
	end
	
end
--[[
	玩家操作 弹入
]]
function DiceRoomController:playerOperationInAnim()
	local viewBottom = self:findChildByName("view_bottom");
	checkAndRemoveOneProp(viewBottom, 1);
	local w, h = viewBottom:getSize();
	local inAnim = viewBottom:addPropTranslateWithEasing(1, kAnimNormal, 500, 0, "easeOutSine", "easeOutSine", 0, 0, h, -h);
	inAnim:setEvent(self, function()
			checkAndRemoveOneProp(viewBottom, 1);
	end);
	viewBottom:setVisible(true);
	--玩家头像
	local mySeatId = self:getMySeatId();
	if mySeatId then
		self.mPlayer[mySeatId]:setInAnim(true);
	end
end

--[[
	玩家操作 弹出
]]
function DiceRoomController:playerOperationOutAnim(player, data)
	local viewBottom = self:findChildByName("view_bottom");
	checkAndRemoveOneProp(viewBottom, 1);
	local w, h 		= viewBottom:getSize();
	local outAnim 	= viewBottom:addPropTranslateWithEasing(1, kAnimNormal, 500, 0, "easeOutSine", "easeOutSine", 0, 0, 0, h);
	outAnim:setEvent(self, function()
			checkAndRemoveOneProp(viewBottom, 1);
			viewBottom:setVisible(false);
			local playerUi = player:getUi();
			local toX, toY = self:getLocalSeatPos(MY_LOCAL_SEAT_ID);
			checkAndRemoveOneProp(playerUi, 1);
			checkAndRemoveOneProp(playerUi, 2);
			local x, y = playerUi:getAbsolutePos();
			local inAnim = playerUi:addPropTranslateWithEasing(1, kAnimNormal, 500, 0, "easeOutSine", "easeOutSine", 0, toX - x, 0, toY - y);
			inAnim:setEvent(self, function()
					checkAndRemoveOneProp(playerUi, 1);
					playerUi:setPos(self:getLocalSeatPos(MY_LOCAL_SEAT_ID));
			end);
			playerUi:addPropScale(2, kAnimNormal, 500, 0, 1.15, 1.0, 1.15, 1.0, kCenterDrawing);

	end);
	-- --玩家头像
end

function DiceRoomController:playStandUpAnim(data)
	-- body
	--清除玩家
	local player = self.mPlayer[data.seatId];
	player:clear();
	--弹出动画
	self:playerOperationOutAnim(player, data);

	-- --清除上局的状态
	local needClear = true;
	for i = 1, #self.mPlayer do
		if self.mPlayer[i]:getId() ~= 0 then
			needClear = false;
			break;
		end
	end

	if needClear then
		self:clearGame();
	
		-- table.sort(self.mPlayer, function ( a, b )
		-- 	-- body
		-- 	return a:getLocalSeatId() < b:getLocalSeatId()
		-- end)

		-- for i = 1, #self.mPlayer do
		-- 	self.mPlayer[i]:setSeatId(i);
		-- 	--正在移动中，不能重置
		-- 	if i ~= MY_LOCAL_SEAT_ID then
		-- 		self.mPlayer[i]:setLocalSeatId(i);
		-- 	end
		-- end

	end
end

--[[
	摇骰子动画
]]
function DiceRoomController:loadDiceCup()
	-- body
	if not self.mDiceCup then
		self.mDiceCup = SceneLoader.load(diceCupLayout);
		self:addChild(self.mDiceCup);
		self:setDice({p1 = 1, p2 = 1, p3 = 1});
		self.mDiceCup:setVisible(false);
	end
end
function DiceRoomController:playShakeAnim()
	--杯盖从上往下掉
	--加载骰子
	self:loadDiceCup();

	local viewCup   = self.mDiceCup:findChildByName("view_cup");
	local imgCupTop = self.mDiceCup:findChildByName("img_cuptop");
	local imgCupBottom = self.mDiceCup:findChildByName("img_cupbottom");
	checkAndRemoveOneProp(viewCup, 1);
	checkAndRemoveOneProp(viewCup, 2);
	checkAndRemoveOneProp(imgCupTop, 1);
	checkAndRemoveOneProp(imgCupBottom, 1);

	local imgSpotLight   	= self.mDiceCup:findChildByName("img_spotlight");
	local imgLightBottom    = self.mDiceCup:findChildByName("img_lightbottom");
	imgSpotLight:setVisible(false)
	imgLightBottom:setVisible(false)
	
	local w, h = imgCupTop:getSize();
	checkAndRemoveOneProp(imgCupTop, 1);
	checkAndRemoveOneProp(imgCupTop, 2);
	checkAndRemoveOneProp(imgCupTop, 3);
	local inAnim = imgCupTop:addPropTranslateWithEasing(1, kAnimNormal, 300, 0, "easeInSine", "easeInSine", 0, 0, -1.5 * h, 1.5 * h);
	inAnim:setEvent(self, function()
			checkAndRemoveOneProp(imgCupTop, 1);
			--盖的音效
			kEffectPlayer:play(Effects.AudioDiceClose);
			--整体上挤压
			local scaleAnim = viewCup:addPropScaleYWithEasing(1, kAnimNormal, 300, 0, "easeOutBounce", "easeOutBounce", 0.95, 0.05, kCenterXY, w/2, h);
			scaleAnim:setEvent(self, function()
				checkAndRemoveOneProp(viewCup, 1);
				--开始摇
				kEffectPlayer:play(Effects.AudioDiceShake);
				viewCup:addPropTranslateWithEasing(1, kAnimLoop, 150, 0, "easeInSine", "easeInSine", -75, 150, 0, 0);
				local timerAnim = viewCup:addPropTranslate(2, kAnimNormal, 1575, 0, 0, 0, 0, 0);
				timerAnim:setEvent(self, function()
					checkAndRemoveOneProp(viewCup, 1);
					checkAndRemoveOneProp(viewCup, 2);
					checkAndRemoveOneProp(imgCupTop, 1);
					checkAndRemoveOneProp(imgCupBottom, 1);
					-- 移动
					local dealer = self:findChildByName('btn_dealer');
					local dealerX, dealerY = dealer:getAbsolutePos();
					local dealerW, dealerH = dealer:getSize();
					
					local x, y 			   = viewCup:getAbsolutePos();
					local w, h 			   = viewCup:getSize();
					local srcX, srcY 	   = x + w/2, y + h/2;
					local dstX, dstY 	   = dealerX + dealerW, dealerY + dealerH - h * 0.125;
					viewCup:addPropTranslate(1, kAnimNormal, 300, 500, 0, (dstX - srcX) , 0, (dstY - srcY));
					imgCupTop:addPropScale(1, kAnimNormal, 300, 500, 1, 0.25, 1, 0.25, kCenterXY, w/2, h/2);
					local moveEndAnim = imgCupBottom:addPropScale(1, kAnimNormal, 300, 500, 1, 0.25, 1, 0.25, kCenterXY, w/2, h/2);
					moveEndAnim:setEvent(self, function()
						kEffectPlayer:play(Effects.AudioDiceMove);
					end)
				end);
			end);
	end);
	self.mDiceCup:setVisible(true);
end

--[[
	初始化骰子点数
]]
function DiceRoomController:setDice(data)
	local retData = self:calPoint(data.p1, data.p2, data.p3);
	local bottom = self.mDiceCup:findChildByName("img_cupbottom");
	local p = {data.p1, data.p2, data.p3}
	for i = 1, 3 do
		local image  	= new(Image, dice_room_dicePoint_map[string.format("%s.png", p[i])] or "");
		local imgDice 	= bottom:findChildByName("img_dice"..i);
		imgDice:removeAllChildren();
		imgDice:addChild(image);
	end
end
--[[
	开骰子
]]
function DiceRoomController:playOpenAnim(data)
	self:loadDiceCup();
	--设置点数
	self:setDice(data);

	local viewCup   = self.mDiceCup:findChildByName("view_cup");
	local imgCupTop = self.mDiceCup:findChildByName("img_cuptop");
	local imgCupBottom 	= self.mDiceCup:findChildByName("img_cupbottom");
	local viewLight 	= self.mDiceCup:findChildByName("view_light");
	local imgSpotLight   	= self.mDiceCup:findChildByName("img_spotlight");
	local imgLightBottom    = self.mDiceCup:findChildByName("img_lightbottom");

	checkAndRemoveOneProp(viewCup, 1);
	checkAndRemoveOneProp(viewCup, 2);
	checkAndRemoveOneProp(imgCupTop, 1);
	checkAndRemoveOneProp(imgCupTop, 2);
	checkAndRemoveOneProp(imgCupTop, 3);
	checkAndRemoveOneProp(imgCupBottom, 1);
	checkAndRemoveOneProp(imgSpotLight, 1);
	checkAndRemoveOneProp(imgSpotLight, 2);
	checkAndRemoveOneProp(imgLightBottom, 1);

	imgSpotLight:setVisible(false)
	imgLightBottom:setVisible(false)

	kEffectPlayer:play(Effects.AudioDiceOpen);

	local timerAnim = viewLight:addPropTranslate(1, kAnimLoop, 100, 0, 0, 0, 0, 0);
	local index = 0;
	local total = 10;
	timerAnim:setEvent(self, function()
		if index > 0 then
			local imgLight = viewLight:findChildByName("light_"..index);
			imgLight:setVisible(false);
		end
		index = index + 1;

		if index > total then
			
			checkAndRemoveOneProp(viewLight, 1);
			--盅盖往上移
			checkAndRemoveOneProp(imgCupTop, 1);
			checkAndRemoveOneProp(imgCupTop, 2);
			checkAndRemoveOneProp(imgCupTop, 3);
			local topW, topH = imgCupTop:getSize();
			imgCupTop:addPropTranslate(1, kAnimNormal, 1000, 0, 0, 0, 0, -300);
			imgCupTop:addPropScale(2, kAnimNormal, 1000, 0, 1, 1.2, 1, 1.2, kCenterXY, topW / 2, topH);
			finishAnim = imgCupTop:addPropTransparency(3, kAnimNormal, 2800, 0, 1, 0);
			finishAnim:setEvent(self, function()
				self.mDiceCup:setVisible(false);
				self:setHistorty(data.p1, data.p2, data.p3);
				self:playDealerWinAnim(data);
				
			end)

			imgSpotLight:setVisible(true)
			imgLightBottom:setVisible(true)
			
			local spotW, spotH 		= imgSpotLight:getSize();
			local btmW, btmH 		= imgLightBottom:getSize();
			imgSpotLight:addPropScale(1, kAnimNormal, 800, 200, 0.1, 1, 1, 1, kCenterXY, spotW / 2, 0);
			local betAreaAnim = imgSpotLight:addPropTransparency(2, kAnimNormal, 800, 1000, 1, 1);
			betAreaAnim:setEvent(self, function()
				--高亮投注区
				self:playBetAreaAnim(data);
			end)

			imgLightBottom:addPropScale(1, kAnimNormal, 800, 200, 0.1, 1, 0.1, 1, kCenterXY, btmW / 2, btmH/2);
			

			return ;
		end

		local imgLight = viewLight:findChildByName("light_"..index);
		imgLight:setVisible(true);
		
	end)
	self.mDiceCup:setVisible(true);
end

--[[
	游戏开始提示
]]
function DiceRoomController:playStartTipAnim()
	--加载 游戏开始提示 动画
	if not self.mStartTip then
		self.mStartTip = SceneLoader.load(roomStartTip);
		self:addChild(self.mStartTip);
	end

	local imgBg 	= self.mStartTip:findChildByName("img_bg");
	local imgTitle 	= self.mStartTip:findChildByName("img_title");
	checkAndRemoveOneProp(imgBg, 1);
	checkAndRemoveOneProp(imgBg, 2);
	checkAndRemoveOneProp(imgBg, 3);
	checkAndRemoveOneProp(imgTitle, 1);
	checkAndRemoveOneProp(imgTitle, 2);

	local w, h 	= imgBg:getSize();
	
	local scaleAnim = imgBg:addPropTranslate(3, kAnimNormal, 2000, 0, 0, 0, 0, 0);
					  imgBg:addPropScaleXWithEasing(1, kAnimNormal, 1200, 0, "easeOutBounce", "easeOutBounce", 0.0, 1.0, kCenterDrawing);
					  imgBg:addPropTransparency(2, kAnimNormal, 1200, 0, 0, 1);
	local w, h 	= imgTitle:getSize();
	imgTitle:addPropTransparency(1, kAnimNormal, 1200, 0, 0, 1);
	imgTitle:addPropScaleWithEasing(2, kAnimNormal, 1200, 0, "easeOutBounce", "easeOutBounce", 3.0, -2.0, kCenterDrawing);
	
	scaleAnim:setEvent(self, function()
		checkAndRemoveOneProp(imgBg, 1);
		checkAndRemoveOneProp(imgBg, 2);
		checkAndRemoveOneProp(imgBg, 3);
		checkAndRemoveOneProp(imgTitle, 1);
		checkAndRemoveOneProp(imgTitle, 2);
		self.mStartTip:setVisible(false);
		self:playShakeAnim();
	end)

	if self.mDiceCup then
		self.mDiceCup:setVisible(false);
	end

	if self.mClock then
		self.mClock:setVisible(false);
	end
	if self.mStartTip then
		self.mStartTip:setVisible(true);
	end
end

--[[
]]
function DiceRoomController:playBetAreaAnim(data)

	local retData = self:calPoint(data.p1, data.p2, data.p3);

	for i = 1, #retData do
		self.mBetArea[self:getBetAreaKey(retData[i].type, retData[i].param)]:setLighted(true);
	end

end

--[[
	游戏开始下注提示
]]
function DiceRoomController:playBetStartTipAnim()

	self:loadDiceCup();

	if not self.mBetStartTip then
		self.mBetStartTip = new(Image, "dice/room/anim/bet_start.png");
		self.mBetStartTip:setAlign(kAlignCenter);
		self:addChild(self.mBetStartTip);
	end
	checkAndRemoveOneProp(self.mBetStartTip, 1);
	checkAndRemoveOneProp(self.mBetStartTip, 2);
	checkAndRemoveOneProp(self.mBetStartTip, 3);

	local w, h = self.mBetStartTip:getSize();
	self.mBetStartTip:addPropScaleWithEasing(1, kAnimNormal, 800, 0, "easeOutBounce", "easeOutBounce", 0.7, 0.3, kCenterXY, w/2 * System.getLayoutScale(), h/2 * System.getLayoutScale());
	self.mBetStartTip:addPropTransparency(2, kAnimNormal, 1200, 800, 1, 0);
	local endAnim = self.mBetStartTip:addPropTranslate(3, kAnimNormal, 1200, 800, 0, 0, 0, -60);

	endAnim:setEvent(self, function()
		checkAndRemoveOneProp(self.mBetStartTip, 1);
		checkAndRemoveOneProp(self.mBetStartTip, 2);
		checkAndRemoveOneProp(self.mBetStartTip, 3);
		self.mBetStartTip:setVisible(false);
	end)
	self.mBetStartTip:setVisible(true);
	
end

--[[
	游戏结束提示
]]
function DiceRoomController:playEndTipAnim(data)

	self:loadDiceCup();

	if not self.mEndTip then
		self.mEndTip = new(Image, "dice/room/anim/end.png");
		self.mEndTip:setAlign(kAlignCenter);
		self:addChild(self.mEndTip);
	end
	checkAndRemoveOneProp(self.mEndTip, 1);
	checkAndRemoveOneProp(self.mEndTip, 2);
	checkAndRemoveOneProp(self.mEndTip, 3);

	local w, h = self.mEndTip:getSize();
	scaleAnim = self.mEndTip:addPropScaleWithEasing(1, kAnimNormal, 800, 0, "easeOutBounce", "easeOutBounce", 0.7, 0.3, kCenterXY, w/2 * System.getLayoutScale(), h/2 * System.getLayoutScale());
	scaleAnim:setEvent(self, function()
		self:playOpenAnim(data);
	end)
	self.mEndTip:addPropTransparency(2, kAnimNormal, 1200, 800, 1, 0);
	local endAnim = self.mEndTip:addPropTranslate(3, kAnimNormal, 1200, 800, 0, 0, 0, -60);
	-- endAnim:setEvent(self, function()
	-- 	self:playOpenAnim(data);
	-- end)

	endAnim:setEvent(self, function()
		checkAndRemoveOneProp(self.mEndTip, 1);
		checkAndRemoveOneProp(self.mEndTip, 2);
		checkAndRemoveOneProp(self.mEndTip, 3);
		self.mEndTip:setVisible(false);
		-- self:playOpenAnim(data);
	end)
	self.mEndTip:setVisible(true);
	if self.mClock then
		self.mClock:setVisible(false)
	end
end

--[[
	计时器动画
]]

function DiceRoomController:playClockAnim(totalSeconds)
	--加载 游戏开始提示 动画
	if not self.mClock then
		self.mClock = SceneLoader.load(roomClock);
		self:findChildByName("view_clock"):addChild(self.mClock);
		self.mClock:setPos(800, 135);
	end
	checkAndRemoveOneProp(self.mClock, 1);
	for i = 0, 19 do
		self.mClock:findChildByName("img_number"..i):setVisible(false);
	end
	for i = 0, 4 do
		self.mClock:findChildByName("img_clock"..i):setVisible(false);
	end

	self.mClock:findChildByName("img_clock"..0):setVisible(true);
	local imgCurNumber = self.mClock:findChildByName("img_number"..totalSeconds);
	imgCurNumber:setVisible(true);

	local timerAnim = self.mClock:addPropTranslate(1, kAnimLoop, 50, 0, 0, 0, 0, 0);
	local index = 0;
	local total = 4;
	local seconds = totalSeconds;
	local timesPerSecond = 1000 / 50;
	local timeIndex 	 = 0;

	timerAnim:setEvent(self, function()
		
		--秒
		timeIndex = timeIndex + 1;
		if timeIndex == timesPerSecond then
			timeIndex = 0;
			--timeout
			if seconds > 0 then
				self.mClock:findChildByName("img_number"..seconds):setVisible(false);
				seconds = seconds - 1;
				imgCurNumber = self.mClock:findChildByName("img_number"..seconds);
				imgCurNumber:setVisible(true);
				if seconds > 0 and seconds <= 5 then
					kEffectPlayer:play(Effects.AudioTimeWarning);
				end
			end
			
		end

		--背景
		if seconds <= 5 then
			self.mClock:findChildByName("img_clock"..0):setVisible(false);
			if index > 0 then
				self.mClock:findChildByName("img_clock"..index):setVisible(false);
			end
			index = index + 1;

			if index > total then
				index = 1
			end
			--震动坐标
			local pos = {
				[1] = {x = 0, y = 4},
				[2] = {x = 0, y = 6},
				[3] = {x = 0, y = 4},
				[4] = {x = 0, y = 6},
			}
			imgCurNumber:setPos(pos[index].x, pos[index].y);

			local imgClock = self.mClock:findChildByName("img_clock"..index);
			imgClock:setVisible(true);
		end
	end)

	
	self.mClock:setVisible(true)
end

--[[
	下注动画
]]

function DiceRoomController:playBetAnim(area, nodeFrom, nodeTo, money, seatId, update)

	local chipBatch 	= self.mChipBatch;
	local fromx, fromy 	= nodeFrom:getAbsolutePos();
	local fromw, fromh 	= nodeFrom:getSize();
	local tox, toy 		= nodeTo:getAbsolutePos();
	local tow, toh 		= nodeTo:getSize();

	local chipcount = 0
	for i = 1, chipBatch:count() do
		if chipBatch:getBindData(i).area == area then
			chipcount = chipcount + 1
		end
	end
	printInfo("playBetAnim money "..tostring(money))
	local texName 	= string.format("%s.png", money)
	local texInfo 	= chipBatch:getTexture(texName)
	local chipw, chiph = texInfo.width, texInfo.height
    --local index 	= chipBatch:add(texName, fromx + (fromw - chipw) / 2, fromy + (fromh - chiph) / 2, update);
	fromx = fromx + (fromw - chipw)  / 2;
	fromy = fromy + (fromh - chiph) / 2;
    local index 	= chipBatch:add(texName, fromx, fromy, update)
	tox = tox + (tow - chipw) / 2;
	toy = toy + (toh - chiph) / 2;

	local ratio = math.min(math.max(chipcount / 30, 0.5), 1)
	
	local randomW = (tow - chipw)/2 * ratio;
	local randomH = (toh - chiph)/2 * ratio;

	local a = math.random(math.min(0, randomW), math.max(0, randomW));
	local b = math.random(math.min(0, randomH), math.max(0, randomH));
	local angle = math.random(0, 360)
	local deltax, deltay = a * math.cos(angle), b * math.sin(angle)

	tox = tox + deltax;
	toy = toy + deltay;

	chipBatch:move(index, 300, 0, 'easeOutSine', 'easeOutSine', fromx, fromy, tox, toy, 'easeInSine', angle, angle + 60);

	--绑定必要的信息
	chipBatch:bind(index, {seatId = seatId, area = area})

end
--[[
	庄家赢走筹码动画
]]
function DiceRoomController:playDealerWinAnim(data)
	local dealer 				= self:findChildByName('btn_dealer');
	local dealerx, 	dealery 	= dealer:getAbsolutePos();
	local dealerw, 	dealerh 	= dealer:getSize();
	local todealerx,todealery 	= dealerx + dealerw/2, dealery + dealerh/2

	local finishAnim = nil;

	local chipBatch = self.mChipBatch;

	local areaInfo = {}


	for i = 1, #data.user do
		local user = data.user[i];
		for j = 1, #user.bet do
			local bet = user.bet[j];
			--庄家拿走赢的筹码
			if bet.win <= 0 then
				local area 		= self:getBetAreaKey(bet.type, bet.param);
				local areaNode  = self.mBetArea[area]:getNode():findChildByName("view_bet");
				
				for k = chipBatch:count(), 1, -1 do
					local userdata = chipBatch:getBindData(k)
					if userdata.area == area then
						areaInfo[area] 		 = areaInfo[area] or {}
						areaInfo[area].count = (areaInfo[area].count or 0) + 1
						areaInfo[area].index = 0
					end
				end
			end
		end
	end

	for i = 1, #data.user do
		local user = data.user[i];
		for j = 1, #user.bet do
			local bet = user.bet[j];
			--庄家拿走赢的筹码
			if bet.win <= 0 then
				local area 		= self:getBetAreaKey(bet.type, bet.param);
				local areaNode  = self.mBetArea[area]:getNode():findChildByName("view_bet");

				
				--筹码飞起来
				for k = chipBatch:count(), 1, -1 do
					local userdata = chipBatch:getBindData(k)
					if userdata.area == area then
						local count = areaInfo[area].count or 0
						local index = areaInfo[area].index or 0
						local delay = count > 20 and 600 / count or 30;

						local x, y, w, h = chipBatch:getVertex(k)
						
						chipBatch:move(k, 600, index * delay + 1000, 'easeOutSine', 'easeOutSine', x, y, todealerx - w / 2 + math.random(-(dealerw - w)/3, (dealerw - w)/3), todealery - h / 2 + math.random(-(dealerh - h)/3, (dealerh - h)/3), nil, nil, nil, self, function ()
							-- body
							chipBatch:setVertex(k, -1000, -1000)

						end);

						areaInfo[area].index = index + 1
					end
				end
				

			end
		end
	end

	local effectId = chipBatch:add(nil, 0, 0, true);
	chipBatch:bind(effectId, {seatId = -1, area = ""})
	chipBatch:move(effectId, 1000, 0, 'easeOutSine', 'easeOutSine', 0, 0, 0, 0, nil, nil, nil, self, function() 
			if table.nums(areaInfo) > 0 then
				kEffectPlayer:play(Effects.AudioChipMove);
			end
	end);


	--添加timer
	local timeId = chipBatch:add(nil, 0, 0, true);
	chipBatch:bind(timeId, {seatId = -1, area = ""})
	chipBatch:move(timeId, 2000, 0, 'easeOutSine', 'easeOutSine', 0, 0, 0, 0, nil, nil, nil, self, function() 
			--清除对应的区域筹码数
			for k, v in pairs(areaInfo) do
				self.mBetArea[k]:setChipVisible(false);
				self.mBetArea[k]:setAllChipVisible(false);
			end

			self:playDealerLoseAnim(data);
	end);

end
--[[
	庄家赔钱动画
]]
function DiceRoomController:playDealerLoseAnim(data)
	
	local playCall = false

	for i = 1, #data.user do
		local user = data.user[i];
		for j = 1, #user.bet do
			local bet = user.bet[j];
			if bet.win > 0 then
				local betAreaType 	= self:getBetAreaKey(bet.type, bet.param);
				local toNode 		= self.mBetArea[betAreaType]:getNode():findChildByName("view_bet")

				if user.uid == MyUserData:getId() then
					self.mBetArea[betAreaType]:addChip(bet.win - bet.chip);
				end
				self.mBetArea[betAreaType]:addAllChip(bet.win - bet.chip);

				local bets = self:getBetTypeByMoney(bet.win - bet.chip);

				for i = 1, #bets do
					local bet = bets[i];
					self:playBetAnim(betAreaType, self:findChildByName('btn_dealer'), toNode, bet.chip, user.seatId, false);
					if not playCall then
						kEffectPlayer:play(Effects.AudioChipMove);
						playCall = true
					end
				end

				local betted = self.mBetted[betAreaType] or {};
				betted.type  = bet.type;
				betted.param = bet.param;
				betted.money = (betted.money or 0) + bet.win;
				self.mBetted[betAreaType] = betted;

			end
		end
	end


	

	local chipBatch = self.mChipBatch;

	-- chipBatch:update();

	local timeId = chipBatch:add(nil, 0, 0, true);
	chipBatch:bind(timeId, {seatId = -1, area = ""})
	chipBatch:move(timeId, 1500, 0, 'easeOutSine', 'easeOutSine', 0, 0, 0, 0, nil, nil, nil, self, function() 
			self:playPlayerWinAnim(data);
	end);

end

function DiceRoomController:playPlayerWinAnim(data)
	-- body

	local playCall = false

	local chipBatch = self.mChipBatch;

	local areaInfo = {}

	for i = 1, #data.user do
		local user = data.user[i];
		local player 	= self.mPlayer[user.seatId];
		for j = 1, #user.bet do
			local bet  = user.bet[j];
			local area = self:getBetAreaKey(bet.type, bet.param)
			if bet.win > 0 then
				for k = chipBatch:count(), 1, -1 do
					local userdata = chipBatch:getBindData(k)
					if userdata.area == area and userdata.seatId == user.seatId then
						areaInfo[area] 		 = areaInfo[area] or {}
						areaInfo[area].count = (areaInfo[area].count or 0) + 1
						areaInfo[area].index = 0
					end
				end
			end
		end
	end

	for i = 1, #data.user do
		local user = data.user[i];
		local player 	= self.mPlayer[user.seatId];
		for j = 1, #user.bet do
			local bet  = user.bet[j];
			local area = self:getBetAreaKey(bet.type, bet.param)
			player:addChip(bet.win);--加筹码
			if bet.win > 0 then
				for k = chipBatch:count(), 1, -1 do
					local userdata = chipBatch:getBindData(k)
					if userdata.area == area and userdata.seatId == user.seatId then
						local count = areaInfo[area].count or 0
						local index = areaInfo[area].index or 0
						local delay = count > 20 and 600 / count or 30;

						local x, y, w, h 		= chipBatch:getVertex(k)

						local playerUi 			= player:getUi()
						local playerx, playery 	= playerUi:getAbsolutePos();
						local playerw, playerh 	= playerUi:getSize();
						local tox, toy 			= playerx + playerw/2 + math.random(-(playerw - w)/2, (playerw - w)/2), playery + playerh/2 + math.random(-(playerh - h)/2, (playerh - h)/2);


						-- local vec = new(Vector2, x - tox, y - toy)
						-- vec:normalize()
						-- local radius = math.min(playerw/2, playerh/2);
						-- local total = chipBatch:count() / 2
						-- if k > total then
						-- 	radius = radius * k / 2 / total
						-- else
						-- 	radius = radius * (total - k) / 2 / total
						-- end
						-- local rvec   = vec:rotate(math.pi / chipBatch:count() * k * (k % 2 == 0 and -1 or 1));

						-- local tox, toy = tox + radius * rvec.x, toy + radius * rvec.y 

						-- printInfo('x = '..vec.x)
						-- printInfo('y = '..vec.y)


						chipBatch:move(k, 600, index * delay, 'easeOutSine', 'easeOutSine', x, y, tox - w / 2, toy - h / 2, nil, nil, nil, self, function ()
							-- body
							chipBatch:setVertex(k, -1000, -1000)
						end);
						areaInfo[area].index = index + 1

						if not playCall then
							kEffectPlayer:play(Effects.AudioChipMove);
							playCall = true
						end

					end
				end
			end
		end
	end


	self.mGameResultTemp = nil;

	local timeId = chipBatch:add(nil, 0, 0, true);
	chipBatch:bind(timeId, {seatId = -1, area = ""})
	chipBatch:move(timeId, 1500, 0, 'easeOutSine', 'easeOutSine', 0, 0, 0, 0, nil, nil, nil, self, function() 
            self:showExpResult(data)
			
	end);
    
end

function DiceRoomController:playSendChipAnim(srcPlayer, dstPlayer, money)
	money = money or 0
	local srcx, srcy = srcPlayer:getUi():getPos()
	local srcw, srch = srcPlayer:getUi():getSize()
	local dstx, dsty = dstPlayer.dstx, dstPlayer.dsty
	local dstw, dsth = dstPlayer.dstw, dstPlayer.dsth
	-- local chipIndex = 1
	-- for i = 1, #self.mPresent do
	-- 	if self.mPresent[i] == money then
	-- 		chipIndex = i
	-- 		break
	-- 	end
	-- end


	-- local imgChip 		= new(Image, string.format('dice/room/chip_s%s.png', chipIndex))
	local imgChip 		= new(Image, chip_map[string.format('%s.png', money)])

	local textChip 		= new(Text, ToolKit.formatChip(money), 0, 0, kAlignCenter, "", 24, 0xff, 0xff, 0xff);
	local chipW, chipH 	= imgChip.m_res.m_width, imgChip.m_res.m_height
	local textW, textH 	= textChip.m_res.m_width, textChip.m_res.m_height

	srcx, srcy = srcx + (srcw - math.max(chipW, textW)) / 2, srcy + (srch - chipH - textH) / 2
	dstx, dsty = dstx + (dstw - math.max(chipW, textW)) / 2, dsty + (dsth - chipH - textH) / 2
	imgChip:setPos(srcx, srcy)
	-- imgChip:addChild(textChip)
	textChip:setPos((chipW - textW) / 2, chipH)
	self:addChild(imgChip)
	imgChip:addPropTranslateWithEasing(1, kAnimNormal, 1200, 0, "easeOutSine", "easeOutSine", 0, dstx - srcx, 0, dsty - srcy);
	
	local endAnim = imgChip:addPropTransparency(2, kAnimNormal, 500, 1200, 1.0, 0);
	endAnim:setEvent(self, function()
			self:removeChild(imgChip, true)
	end);
	kEffectPlayer:play(Effects.AudioCall);
end

function DiceRoomController:showExpResult(data)
    if data.userUp then
		for i = 1, #data.userUp do
			if data.userUp[i].uid == MyUserData:getId() then
				--AlarmTip.play('winTimes = ' .. data.userUp[i].WinTimes .. ',loseTimes = ' .. data.userUp[i].LoseTimes .. ',exp = ' .. data.userUp[i].exp)
                if data.userIncExp[i].turn_exp > 0 then
                    local view_addExp = self:findChildByName("view_addExp")

				    local expNode = new(require('common.imageNumber'), {
					    ['0'] = expNumber_map["0.png"],
					    ['1'] = expNumber_map["1.png"],
					    ['2'] = expNumber_map["2.png"],
					    ['3'] = expNumber_map["3.png"],
					    ['4'] = expNumber_map["4.png"],
					    ['5'] = expNumber_map["5.png"],
					    ['6'] = expNumber_map["6.png"],
					    ['7'] = expNumber_map["7.png"],
					    ['8'] = expNumber_map["8.png"],
					    ['9'] = expNumber_map["9.png"],
					    ['+'] = expNumber_map["+.png"],
                        ['e'] = expNumber_map["e.png"],
                        ['x'] = expNumber_map["x.png"],
                        ['p'] = expNumber_map["p.png"],
				    });	

                    view_addExp:addChild(expNode)

				    expNode:setNumber(string.format("+%s", data.userIncExp[i].turn_exp).."exp")
				    expNode:setVisible(true)
				    local x, y = self:getLocalSeatPos(MY_LOCAL_SEAT_ID, true)
				    expNode:setPos(x, y)
                    local animExpMove = expNode:addPropTranslate(1, kAnimNormal, 700, 0, 0, 0, 0, -10)
				    local animExpParency = expNode:addPropTransparency(2, kAnimNormal, 700, 0, 0, 1)
				    animExpParency:setEvent(animAddExp, function ()
                        view_addExp:removeAllChildren()
                        self:showGameResultPopu(data);
			            for k, v in pairs(self.mBetArea) do
				            v:clear();
			            end
                    end)
                    return
                end

			end
		end
	end

    

end

function DiceRoomController:showGameResultPopu( data )

	for i = 1, #data.user do
		local user 	 	= data.user[i];
		local win 	 	= 0;
		local actualWin = 0
		local isThreeSpecial = false
		local isThreeSpecialWin = false
		if data then
			if data.p1 and data.p2 and data.p3 then
				if data.p1 == data.p2 and data.p1 == data.p3 then
					isThreeSpecial = true
				end
			end
		end
		for j = 1, #user.bet do
			win = win + user.bet[j].win;
			actualWin = actualWin + (user.bet[j].win - user.bet[j].chip) 
			--三条
			if (isThreeSpecial == true) and (user.bet[j].param == data.p1) and (user.bet[j].type == BET_TYPE_SPECIAL_THREE) then
				isThreeSpecialWin = true
			end
		end
		self.mPlayer[user.seatId]:setWin(win);
		if user.uid == MyUserData:getId() then
			if #user.bet > 0 then
				if actualWin > tonumber(MyUserData:getMaxwMoney()) then
					HttpModule.getInstance():execute(HttpModule.s_cmds.UPDATE_MEMBER_BEST, {sid = 102, multiValue = {maxwmoney = actualWin}}, false, true)
				end
				if tonumber(MyUserData:getMoney()) > tonumber(MyUserData:getMaxMoney()) then
					HttpModule.getInstance():execute(HttpModule.s_cmds.UPDATE_MEMBER_BEST, {sid = 102, multiValue = {maxmoney = MyUserData:getMoney()}}, false, true)
				end
				WindowManager:showWindow(WindowTag.GameResultPopu, {win = actualWin}, WindowStyle.NORMAL);
				if actualWin > 0 then
					kEffectPlayer:play(Effects.AudioWin);
					AnimationParticles.play(AnimationParticles.DropCoin)
					self.winTimes = self.winTimes + 1
					--分享顺序三条-连胜3局-千万富翁-百万富翁
                    if (MyUserData:getUserType() == UserType.Visitor and GameSetting:getShareTimesGuest() < 3)
                       or (MyUserData:getUserType() ~= UserType.Visitor and GameSetting:getShareTimesFb() < 3) then
					    if isThreeSpecialWin == true then
					        WindowManager:showWindow(WindowTag.SharePopu, {content = STR_SHARE_HIT_150, strParam = {(data.p1*100 + data.p1*10 + data.p1), PhpManager:getGameName()}}, WindowStyle.POPUP)
                        else
						    if self.winTimes == 3 then
                                WindowManager:showWindow(WindowTag.SharePopu, {content = STR_SHARE_THREE_BOARD, strParam = {PhpManager:getGameName()}}, WindowStyle.POPUP)
						    else
                                self:moneyIncreaseShare(MyUserData:getMoney())
						    end
					    end
                    end
				else
					kEffectPlayer:play(Effects.AudioLose);
					self.winTimes = 0
				end
			end
		end
		
	end
	--用户输赢，经验信息
	if data.userUp then
		for i = 1, #data.userUp do
			if data.userUp[i].uid == MyUserData:getId() then
				--AlarmTip.play('winTimes = ' .. data.userUp[i].WinTimes .. ',loseTimes = ' .. data.userUp[i].LoseTimes .. ',exp = ' .. data.userUp[i].exp)

                local level = GameSetting:getLevel()
				local exp = data.userUp[i].exp
                MyUserData:setExp(exp)
				for i = 1,#userLevelExp do
					local x1 = userLevelExp[i].x1 
					local x2 = userLevelExp[i].x2 
					if exp >= x1 and exp < x2 then
						printInfo('levely1 = ' .. i)
						if level == i then --已领取等级奖励
							--AlarmTip.play('已领取等级奖励')
						else --未领取等级奖励
							--请求领取礼包
							--levelTemp = i
                            MyUserData:setExp(exp)
                            MyUserData:setLevel(i)
			                GameSetting:setLevel(i)
			                GameSetting:save()
							HttpModule.getInstance():execute(HttpModule.s_cmds.GET_UPGRADE_REWARD, {sid = 102,level = i}, false, true)
						end
					end 
				end
			end
		end
	end

end

function DiceRoomController:setHistorty(p1, p2, p3)
	-- body
    self:findChildByName("view_history"):findChildByName("img_new"):setVisible(true);
	local btnHistory = self:findChildByName('btn_history');
	local p = {p1, p2, p3};
	table.sort(p, function ( a, b )
		-- body
		return a <= b
	end)
	for i = 1, #p do
		btnHistory:findChildByName('img_'..i):setFile('dice/room/history/'..p[i]..'.png');
	end

	local scHistory = self:findChildByName('sc_history');

	local children 	 = scHistory:getChildren();
	local totalHight = 0;
	for k, v in pairs(children) do
		local x, y = v:getPos();
		local w, h = v:getSize();
		v:setPos(x, y + h);
		totalHight = totalHight + h;
	end

	local item = SceneLoader.load(historyItemLayout);
	--
	for i = 1, #p do
		item:findChildByName('img_p'..i):setFile('dice/room/history/'..p[i]..'.png');
	end
	--set name
	local name = STR_THREE;
	local total = p1 + p2 + p3;
    local numberColor = {0x16, 0xB1, 0xFF}
	if p1 == p2 and p1 == p3 then
		--name = string.format("%s %s", name, total);
        item:findChildByName("text_three"):setText(STR_THREE)
	elseif total >= 4 and total <= 10 then
		--name = string.format("%s %s", STR_SMALL, total);
        item:findChildByName("text_small"):setText(STR_SMALL)
        numberColor = {0xFC, 0xF3, 0x5A}
	elseif total >= 11 and total <= 17 then
		--name = string.format("%s %s", STR_BIG, total);
        item:findChildByName("text_big"):setText(STR_BIG)
        numberColor = {0xDC, 0x2C, 0x03}
	end
    local numberText = item:findChildByName("text_number")
    numberText:setText(tostring(total))	
    numberText:setColor(unpack(numberColor))

	item:setPos(0, 0);
	scHistory:addChild(item);
	local w, h = item:getSize();
	totalHight = totalHight + h;
	scHistory.m_nodeH = totalHight;
	ScrollView.update(scHistory);
    scHistory:setSize(scHistory:getSize())

end
--[[
	--seatId 从1开始
]]
function DiceRoomController:getLocalSeatId(seatId, mySeatId)
	-- body
	-- self.mLastSeatId
	if self.mLastSeatId then
		return (MY_LOCAL_SEAT_ID - 1 + seatId - self.mLastSeatId) % 9 + 1;
	else
		return seatId
	end
end

function DiceRoomController:getLocalSeatPos(localSeatId, sitdown)
	-- body
	local pos = {
		[1] = {x = 786, 	y = 5},
		[2] = {x = 948, 	y = 5},
		[3] = {x = 1177, 	y = 174},
		[4] = {x = 1177, 	y = 334},
		[5] = {x = 40, 		y = 607},
		[6] = {x = 2, 		y = 334},
		[7] = {x = 2, 		y = 174},
		[8] = {x = 232, 	y = 5},
		[9] = {x = 394, 	y = 5},
	}
	if localSeatId == MY_LOCAL_SEAT_ID and not sitdown then
		return 590, 607;
	end
	return pos[localSeatId].x, pos[localSeatId].y;
end
--[[
	获取自己的绝对桌位号
]]
function DiceRoomController:getMySeatId()
	for i = 1, #self.mPlayer do
		local player = self.mPlayer[i];
		if player:getId() == MyUserData:getId() then
			return player:getSeatId();
		end
	end
end
--[[
	重新排位
]]
function DiceRoomController:reLocateSeat()
	-- body
	local mySeatId 	= self:getMySeatId();
	if not mySeatId then return end
	local player 	= self.mPlayer[mySeatId];
	--printInfo('mySeatId = '..mySeatId)
	self.mLastSeatId = mySeatId;

	player:setLocalSeatId(MY_LOCAL_SEAT_ID);

	for i = 1, #self.mPlayer do
		local p 	 = self.mPlayer[i];
		local seatId = p:getSeatId();
		if seatId ~= mySeatId then
			p:setLocalSeatId(self:getLocalSeatId(seatId, mySeatId));
		end
	end
end

--[[
	得到当前选中的底注
]]
function DiceRoomController:getCurSelectedAnte()
	-- body
	for i = 1, #self.mBet do
		if self.mBet[i]:getIsSelected() then
			return self.mBet[i]:getAnte();
		end
	end
    return 0
end
--[[
	得到Bet
]]
function DiceRoomController:getBetIndexByAnte(ante)
	-- body
	for i = 1, #self.mBet do
		if self.mBet[i]:getAnte() == ante then
			return self.mBet[i];
		end
	end
end

function DiceRoomController:getBetTypeByAnte(ante)
	-- body
	for i = 1, #self.mBet do
		if self.mBet[i]:getAnte() == ante then
			return i;
		end
	end
end

function DiceRoomController:getBetTypeByMoney(money)
	-- body
	local bets = {};
	local moneys = {10, 50, 100, 500, 1000, 5000, 10000, 50000, 100000, 500000, 1000000, 5000000, 50000000}
	repeat 
		if money <= 0 then
			break;
		end
		local bet = {};

		for i = #moneys, 1, -1 do
			local ante = moneys[i]
			if money >= ante then
				bet.chip = ante;
				money = money - ante;
				break;
			end
		end

		if not bet.chip then
			break;
		end

		table.insert(bets, bet);

	until false

	return bets;
end

function DiceRoomController:getBetAreaKey(type, param)
	-- body
	return tostring(type)..tostring(param)
end

function DiceRoomController:resume(bundleData)
	--播放音乐
	DiceRoomController.super.resume(self)
	kMusicPlayer:play(Music.AudioGameBack, true)
end

function DiceRoomController:dtor()
	GameSocketMgr:removeCommonSocketReader(self.m_reader);
	GameSocketMgr:removeCommonSocketWriter(self.m_writer);
	GameSocketMgr:removeCommonSocketProcesser(self.m_processer);
	EventDispatcher.getInstance():unregister(HttpModule.s_event, self, self.onHttpRequestsCallBack);
	EventDispatcher.getInstance():unregister(Event.onEventResume,self,self.onEventResume);
	EventDispatcher.getInstance():unregister(Event.onEventPause,self,self.onEventPause);
	delete(self.m_reader);
	delete(self.m_writer);
	delete(self.m_processer);
	self.m_reader 	= nil;
	self.m_writer 	= nil;
	self.m_processer= nil;
	kMusicPlayer:stop(true)
    self:unloadPicture()
end

--HTTP回调
function DiceRoomController:onHttpRequestsCallBack(command, ...)
	if self.s_severCmdEventFuncMap[command] then
     	self.s_severCmdEventFuncMap[command](self,...)
	end 
end

function DiceRoomController:onEventResume()
	GameSocketMgr:tryReconnect()
end

function DiceRoomController:onEventPause()
	GameSocketMgr:closeSocketSync()
end
--[[
	获取桌子ID
]]
function DiceRoomController:onGetTableId( data )
	-- body
	if DEBUG_MODE then
		AlarmTip.play("获取桌子ID")
	end
	self.mTableId:init(data);

end
--[[
	跟随 (只有跟随失败才回包)
]]
function DiceRoomController:onFollow( data )
	-- body
	if DEBUG_MODE then
		AlarmTip.play("追踪")
	end
	self:hideLoadingAnim();
	
	WindowManager:showWindow(WindowTag.LobbyExitPopu, {
		title 	= STR_FRIEND_FOLLOW_TITLE,
		content = STR_FRIEND_FOLLOW_CONTENT,
		confirm = STR_EXIT_GAME_CONFIRM,
		confirmFunc = function ()
			-- body
			self.mFollowId = nil
			--获取tableId
			GameSocketMgr:sendMsg(DICE_GET_TABLEID_REQ, {u16_gameLevel = self.mRoomInfo:getLevel(), u32_gameId = self.mGameId});
		end
	}, WindowStyle.POPUP);

end
function DiceRoomController:onFollowed( data )
    printInfo(string.format(STR_FRIEND_FOLLOWED_TIPS, data.nick or ""))
	if data.code == 0 then
		AlarmTip.play(string.format(STR_FRIEND_FOLLOWED_TIPS, data.nick or ""))
	end
end

--[[
	登录
]]
function DiceRoomController:onLoginGame( data )
	-- body
	if DEBUG_MODE then
		AlarmTip.play("登录")
	end
	for i = 1, #data.user do
		local user 	 		= data.user[i];
		local player 		= self.mPlayer[user.seatId];
		local localSeatId 	= self:getLocalSeatId(user.seatId);
		player:setId(user.uid)
		player:setSeatId(user.seatId);
		local userInfo = json.decode(user.userInfo) or {};
		player:setSex(userInfo.sex or 0);
		player:setNick(userInfo.nick or "");
		player:setHeadUrl(userInfo.micon or "");
        player:setGiftId(userInfo.giftId)
		player:setChip(user.chip);
		player:setLocalSeatId(localSeatId);

		if user.uid == MyUserData:getId() then
			for j = 1, #user.bet  do
				local bet = user.bet[j];
				self:onBet({retcode = 0, money = bet.money, param = bet.param, type = bet.betType});
			end
		else
			for j = 1, #user.bet  do
				local bet = user.bet[j];
				local betData 	= {};
				betData.seatId 	= user.seatId;
				betData.type 	= bet.betType;
				betData.param 	= bet.param;
				betData.money 	= bet.money;
				self:onOtherBet(betData);
			end
		end
	end
    if data.pokerLog then
        for i = 1, #data.pokerLog do
            self:setHistorty(data.pokerLog[i].point1, data.pokerLog[i].point2, data.pokerLog[i].point3)
        end
    end
    local viewHistory = self:findChildByName("view_history")
    viewHistory:findChildByName("text_total"):setText(STR_DICE_SUM)
    viewHistory:findChildByName("text_point"):setText(STR_DICE_POINTS)

	if self:getMySeatId() then
		self:playSitdownAnim();
	else --如果没在座位上，自动坐下
		for i = 1, 9 do
			if self.mPlayer[i]:getId() == 0 and (not self.mFollowId) then
				GameSocketMgr:sendMsg(DICE_SITDOWN_IN_GAME_REQ, {u32_seatId = i, u64_ante = MyUserData:getMoney()});
				break;
			end
		end
	end

	if data.gameStatus == 2 then --正在下注
		self:playClockAnim(data.left_time);
	end

	self:hideLoadingAnim();
end

function DiceRoomController:onLoginErrorGame( data )
	if DEBUG_MODE then
		AlarmTip.play("进入房间失败"..data.retcode)
	end
	WindowManager:showWindow(WindowTag.LobbyExitPopu, {
		title 	= STR_EXIT_ROOM_TITLE,
		content = STR_ENTER_ROOM_FAILED,
		confirm = STR_EXIT_GAME_CONFIRM,
		confirmFunc = function ()
			-- body
			GameSocketMgr:sendMsg(DICE_EXIT_ROOM_REQ);
			StateChange.changeState(self.mStateFrom or States.GameLobby);
		end
	}, WindowStyle.POPUP);

	self:hideLoadingAnim();
end
--[[
	重连
]]
function DiceRoomController:onReLoginGame( data )
	if DEBUG_MODE then
		AlarmTip.play("重连")
	end
	self:clearGame();
	self:onLoginGame(data);
end
--[[
	坐下
]]
function DiceRoomController:onSitDownInGame(data)
    --玩牌上报boyaa广告SDK
    NativeEvent.getInstance():boyaaAd({type = 4, value = MyUserData:getId()})
	if DEBUG_MODE then
		AlarmTip.play("用户坐下")
	end
	MyUserData:setMoney(data.money);
	if data.retcode == 0 then
		local player = self.mPlayer[data.seatId];
		--设置玩家信息
		player:setId(MyUserData:getId());
		player:setSex(MyUserData:getSex());
		player:setNick(MyUserData:getNick());
		player:setHeadUrl(MyUserData:getHeadUrl());
        player:setGiftId(MyUserData:getGiftId() and {MyUserData:getGiftId()[1], MyUserData:getGiftId()[2]})
		player:setChip(data.chip);
		player:setSeatId(data.seatId);
		self:playSitdownAnim();

	else
		if DEBUG_MODE then
			AlarmTip.play("用户坐下失败"..data.retcode)
		end

		if data.retcode == ERROR_WRONG_UID or 
			data.retcode == ERROR_TABLE_NOT_EXIST then
			--重新坐下
				WindowManager:showWindow(WindowTag.LobbyExitPopu, {
				title 	= STR_EXIT_ROOM_TITLE,
				content = STR_SITDOWN_FAILED,
				confirm = STR_EXIT_GAME_CONFIRM,
				confirmFunc = function ()
					-- body
					GameSocketMgr:sendMsg(DICE_EXIT_ROOM_REQ);
					StateChange.changeState(self.mStateFrom or States.GameLobby);
				end
			}, WindowStyle.POPUP);
		elseif data.retcode == ERROR_TABLE_MAX_COUNT then
            local popuContent = ""
            if self.mFollowId then
                popuContent = SIT_NO_SEAT_TO_SIT_DOWN
            else
                popuContent = STR_TABLE_LIMITED
            end
			WindowManager:showWindow(WindowTag.LobbyExitPopu, {
				close 	= true,
				title 	= STR_EXIT_ROOM_TITLE,
				content = popuContent,
				confirm = STR_EXIT_GAME_CONFIRM,
				confirmFunc = function ()
					-- body
					GameSocketMgr:sendMsg(DICE_EXIT_ROOM_REQ);
					StateChange.changeState(self.mStateFrom or States.GameLobby);
				end
			}, WindowStyle.POPUP);
		elseif data.retcode == ERROR_ANTE_MONEY or 
				data.retcode == ERROR_NOT_ENOUGH_MONEY then
				WindowManager:showWindow(WindowTag.LobbyExitPopu, {
				title 	= STR_NOT_ENOUGH_MONEY_TITLE,
				content = STR_MONEY_NOT_ENOUGH,
				confirm = STR_EXIT_GAME_CONFIRM,
				confirmFunc = function ()
					-- body
					GameSocketMgr:sendMsg(DICE_EXIT_ROOM_REQ);
					StateChange.changeState(self.mStateFrom or States.GameLobby);
				end
			}, WindowStyle.POPUP);
		elseif data.retcode == ERROR_TOO_MUCH_MONEY then
			WindowManager:showWindow(WindowTag.LobbyExitPopu, {
				title 	= STR_EXIT_ROOM_TITLE,
				content = STR_TOO_MUCH_MONEY,
				confirm = STR_EXIT_GAME_CONFIRM,
				confirmFunc = function ()
					-- body
					GameSocketMgr:sendMsg(DICE_EXIT_ROOM_REQ);
					StateChange.changeState(self.mStateFrom or States.GameLobby);
				end
			}, WindowStyle.POPUP);
		elseif data.retcode == ERROR_SEAT_HAS_PLAYER then
			WindowManager:showWindow(WindowTag.LobbyExitPopu, {
				close 	= true,
				title 	= STR_EXIT_ROOM_TITLE,
				content = STR_SEAT_ROBBED,
				confirm = STR_EXIT_GAME_CONFIRM,
				confirmFunc = function ()
					-- body
				end
			}, WindowStyle.POPUP);
		end
	end
end
--[[
	广播用户坐下
]]
function DiceRoomController:onBroadcastSitdown(data)
	if DEBUG_MODE then
		AlarmTip.play("广播用户坐下")
	end
	local player = self.mPlayer[data.seatId];
	player:setId(data.uid);
	local userInfo = json.decode(data.userInfo) or {};
	player:setSex(userInfo.sex or 0);
	player:setNick(userInfo.nick or "");
	player:setHeadUrl(userInfo.micon or "");
    player:setGiftId(userInfo.giftId)
	player:setChip(data.chip);


	player:setSeatId(data.seatId);
	player:setLocalSeatId(self:getLocalSeatId(data.seatId, self:getMySeatId()));
end
function DiceRoomController:onStartBet(data)
	if DEBUG_MODE then
		AlarmTip.play("开始下注")
	end
	self:playClockAnim(15);
	self:playBetStartTipAnim();

	-- kEffectPlayer:play(Effects.AudioBeginBet);

end

--[[
	下注
]]

function DiceRoomController:onBet(data)
	if data.retcode == 0 then
		if DEBUG_MODE then
			AlarmTip.play("下注成功")
		end
		local player  		= self.mPlayer[self:getMySeatId()];
		local betAreaType 	= self:getBetAreaKey(data.type, data.param);
		local toNode 		= self.mBetArea[betAreaType]:getNode():findChildByName("view_bet")

		local bets = self:getBetTypeByMoney(data.money);

		for i = 1, #bets do
			local bet = bets[i];
			-- 
			local antes = self.mRoomInfo:getChip();
			for j = 1, #antes do
				if antes[j] == bet.chip or j == #antes then
					self:playBetAnim(betAreaType, self.mBet[j]:getUi(), toNode, bet.chip, self:getMySeatId(), true);
					break
				end
			end
		end

		if player then
			player:addChip(-data.money);
		end

		local betted = self.mBetted[self:getBetAreaKey(data.type, data.param)] or {};
		betted.type  = data.type;
		betted.param = data.param;
		betted.money = (betted.money or 0) + data.money;

		self.mBetted[self:getBetAreaKey(data.type, data.param)] = betted;
		self.mBetArea[self:getBetAreaKey(data.type, data.param)]:addChip(data.money);
		self.mBetArea[self:getBetAreaKey(data.type, data.param)]:addAllChip(data.money);
		--音效
		kEffectPlayer:play(Effects.AudioCall);

		if self.mClickLast then
			local btnLast = self:findChildByName("btn_last");
			checkAndRemoveOneProp(btnLast, 1);
			btnLast:setEnable(false)
			btnLast:addPropTransparency(1, kAnimNormal, 0, 0, 0.5, 0.5);
		end
		if self.mClickDouble then
			local btnDouble = self:findChildByName("btn_double");
			checkAndRemoveOneProp(btnDouble, 1);
			btnDouble:setEnable(false)
			btnDouble:addPropTransparency(1, kAnimNormal, 0, 0, 0.5, 0.5);
		end
	else
		if DEBUG_MODE then
			AlarmTip.play("下注失败")
		end
	end
end
--[[
	下注
]]

function DiceRoomController:onOtherBet(data)
	if self.mTableId:getGameId() ~= self.mGameId then return end
	if DEBUG_MODE then
		-- AlarmTip.play("其他用户下注")
	end
	local player  		= self.mPlayer[data.seatId];
	local betAreaType 	= self:getBetAreaKey(data.type, data.param);
	-- local betType 		= self:getBetTypeByAnte(data.money);
	local fromNode 		= player:getUi();
	local betArea 		= self.mBetArea[betAreaType]
	if not betArea then return end
	local toNode  		= betArea:getNode():findChildByName("view_bet");
	local bets = self:getBetTypeByMoney(data.money);

	betArea:addAllChip(data.money);

	for i = 1, #bets do
		local bet = bets[i];
		self:playBetAnim(betAreaType, fromNode, toNode, bet.chip, data.seatId, true);
	end
		
	player:addChip(-data.money);

	--音效
	kEffectPlayer:play(Effects.AudioCall);
end

--[[
	游戏开始
]]
function DiceRoomController:clearGame()
	-- body
	self.mBetted 	= {}
	--关闭结算
	for k, v in pairs(self.mPlayer) do
		v:setWin(nil);
	end
	WindowManager:closeWindowByTag(WindowTag.GameResultPopu)
	--清除上局的状态
	for k, v in pairs(self.mBetArea) do
		v:clear();
	end

	self.mChipBatch:clear();

	self.mClickLast 	= false;
	self.mClickDouble 	= false;

	checkAndRemoveOneProp(self:findChildByName("btn_double"), 1);
	checkAndRemoveOneProp(self:findChildByName("btn_last"), 1);
	
	self:findChildByName("btn_double"):setEnable(true)
	self:findChildByName("btn_last"):setEnable(true)
end
function DiceRoomController:onGameStart(data)
	if DEBUG_MODE then
		AlarmTip.play("游戏开始")
	end
	
	self:clearGame();

	--如果还没有同步
	if self.mGameResultTemp then
		local result = self.mGameResultTemp;
		for i = 1, #result.user do
			local user = result.user[i];
			for j = 1, #user.bet do
				local bet = user.bet[j];
				local player 	= self.mPlayer[user.seatId];
				if player:getSeatId() > 0 then
					player:addChip(bet.win);--加筹码
				end
			end
		end

		self.mGameResultTemp = nil;
	end

	self:playStartTipAnim();
end

function DiceRoomController:onStandUp(data)
	if DEBUG_MODE then
		AlarmTip.play("站起")
	end
	if data.code == 0 then
		self:playStandUpAnim(data);
        --data.reason == 1 standup on player's own
        --data.reason == 2 standup up as money not enough
        --data.reason == 3 stand up for looking too long
		if data.reason == 2 then
            MyUserData:setMoney(data.money)
			if MyUserData:getMoney() < gBankrupt then
				--破产弹框
				WindowManager:showWindow(WindowTag.BankruptPopu, {state = 1, gameId = self.mGameId, level = self.mRoomInfo:getLevel()}, WindowStyle.POPUP)
			else
				WindowManager:showWindow(WindowTag.LobbyExitPopu, {
					close   = true,
					title 	= STR_NOT_ENOUGH_MONEY_TITLE,
					content = STR_MONEY_NOT_ENOUGH,
					confirm = STR_EXIT_GAME_CONFIRM,
					confirmFunc = function ()
						-- body
						GameSocketMgr:sendMsg(DICE_EXIT_ROOM_REQ);
						StateChange.changeState(self.mStateFrom or States.GameLobby);
					end
				}, WindowStyle.POPUP);
			end
        elseif data.reason == 3 then
            WindowManager:showWindow(WindowTag.LobbyExitPopu, {
		        content = STR_STANDUP_NOTICE,
		        confirm = STR_EXIT_GAME_CONFIRM,
		        close = false,
                autoDisappear = true,
                disappearTime = 30000,
		    }, WindowStyle.POPUP);
		end
	end


end
function DiceRoomController:onBroadcastStandUp(data)
	if DEBUG_MODE then
		AlarmTip.play("广播站起")
	end
	self.mPlayer[data.seatId]:clear();

	local needClear = true;
	for i = 1, #self.mPlayer do
		if self.mPlayer[i]:getId() ~= 0 then
			needClear = false;
			break;
		end
	end

	if needClear then
		self:clearGame();
	end

end
function DiceRoomController:onExitRoom(data)
	if DEBUG_MODE then
		AlarmTip.play("退出房间")
	end
    if self.acceptPrivateRoom then
        --如果私人房邀请来自其他游戏，那么要先切换游戏
        StateChange.changeState(self.mStateFrom or States.GameLobby);
        local game = app:getGame(self.acceptPrivateRoom.gameId)
        if game and game.enterPrivateRoom then
            game:enterPrivateRoom(self.mStateFrom or States.GameLobby, kFalse, self.acceptPrivateRoom.code)
        end
        self.acceptPrivateRoom = nil
    end
	--退回大厅
	--StateChange.changeState(4);
end

--[[
	游戏结束
]]
function DiceRoomController:onGameEnd(data)
	if DEBUG_MODE then
		AlarmTip.play("游戏结束")
	end
    if not data then
        return
    end
	if self.mTableId:getGameId() ~= self.mGameId then return end

	-- kEffectPlayer:play(Effects.AudioEndBet);

	self.mLastBetted 	 = ToolKit.deepcopy(self.mBetted);
	self.mGameResultTemp = data;
	self:playEndTipAnim(data);

	--上报新手牌局数量
	if GameSetting.totalGameCount then
		GameSetting.totalGameCount = GameSetting.totalGameCount + 1
		if GameSetting.totalGameCount == 10 then
			NativeEvent.getInstance():boyaaAd({type = kADCUSTOM or 6, value = MyUserData:getId(), event_name = "newUserGame10"})
		elseif GameSetting.totalGameCount == 15 then
			NativeEvent.getInstance():boyaaAd({type = kADCUSTOM or 6, value = MyUserData:getId(), event_name = "newUserGame15"})
		elseif GameSetting.totalGameCount == 20 then
			NativeEvent.getInstance():boyaaAd({type = kADCUSTOM or 6, value = MyUserData:getId(), event_name = "newUserGame20"})
		elseif GameSetting.totalGameCount == 30 then
			NativeEvent.getInstance():boyaaAd({type = kADCUSTOM or 6, value = MyUserData:getId(), event_name = "newUserGame30"})
		end
	end
end

function DiceRoomController:calPoint(p1, p2, p3)
	-- body
	local data = {}
	
	if p1 == p2 and p1 == p3 then
		--三条
		local d = {}
		d.type = BET_TYPE_THREE;
		d.param= 0;
		table.insert(data, d);
		--特殊三条
		local speciald = {}
		speciald.type = BET_TYPE_SPECIAL_THREE;
		speciald.param= p1;
		table.insert(data, speciald);
	else
		--三条
		local total = p1 + p2 + p3;
		if total >= 11 and total <= 17 then
			local bigd = {}
			bigd.type = BET_TYPE_BIG;
			bigd.param= 0;
			table.insert(data, bigd);
		else
			local bigd = {}
			bigd.type = BET_TYPE_SMALL;
			bigd.param= 0;
			table.insert(data, bigd);
		end
		
	end

	--总点数
	local totalPoint = p1 + p2 + p3;
	if totalPoint > 3 and totalPoint < 18 then
		local totald = {}
		totald.type = BET_TYPE_TOTAL_POINTS;
		totald.param= p1 + p2 + p3;
		table.insert(data, totald);
	end
	--单点
	local singled1 = {}
	singled1.type = BET_TYPE_SINGLE_POINTS;
	singled1.param= p1;
	table.insert(data, singled1);

	local singled2 = {}
	singled2.type = BET_TYPE_SINGLE_POINTS;
	singled2.param= p2;
	table.insert(data, singled2);

	local singled3 = {}
	singled3.type = BET_TYPE_SINGLE_POINTS;
	singled3.param= p3;
	table.insert(data, singled3);

	return data;
end

function DiceRoomController:getAwardName( betType, param, p1, p2, p3)
	-- body
	if betType == BET_TYPE_BIG then
		return STR_BET_BIG
	elseif betType == BET_TYPE_SMALL then
		return STR_BET_SMALL
	elseif betType == BET_TYPE_THREE then
		return STR_BET_THREE
	elseif betType == BET_TYPE_SPECIAL_THREE then
		return STR_BET_SPEC_THREE..param
	elseif betType == BET_TYPE_TOTAL_POINTS then
		return tostring(param)..STR_POINT
	elseif betType == BET_TYPE_SINGLE_POINTS then
		local count = 0;
		local p 	= {p1, p2, p3};
		for i = 1, #p do
			if p[i] == param then
				count = count + 1;
			end
		end
		if count == 1 then
			return string.format("1%s%s%s", STR_DICE, param, STR_POINT)
		elseif count == 2 then
			return string.format("2%s%s%s", STR_DICE, param, STR_POINT)
		elseif count == 3 then
			return string.format("3%s%s%s", STR_DICE, param, STR_POINT)
		else
		end
	else
	end
end
function DiceRoomController:getMultiply( betType, param, p1, p2, p3)
	-- body
	if betType == BET_TYPE_BIG then
		return 1
	elseif betType == BET_TYPE_SMALL then
		return 1
	elseif betType == BET_TYPE_THREE then
		return 24
	elseif betType == BET_TYPE_SPECIAL_THREE then
		return 150
	elseif betType == BET_TYPE_TOTAL_POINTS then
		t = {0, 0, 0, 0, 50, 18, 14, 12, 8, 6, 6, 6, 6, 8, 12, 14, 16, 50}
		return t[tonumber(param)]
	elseif betType == BET_TYPE_SINGLE_POINTS then
		local count = 0;
		local p 	= {p1, p2, p3};
		for i = 1, #p do
			if p[i] == param then
				count = count + 1;
			end
		end
		if count == 1 then
			return 1
		elseif count == 2 then
			return 2
		elseif count == 3 then
			return 3
		else
		end
	else
	end
end

function DiceRoomController:onBack()
	WindowManager:showWindow(WindowTag.LobbyExitPopu, {
		title 	= STR_EXIT_ROOM_TITLE,
		content = STR_EXIT_ROOM_CONTENT,
		cancel 	= STR_EXIT_GAME_CANCEL,
		confirm = STR_EXIT_GAME_CONFIRM,
		confirmFunc = function ()
			-- body
			GameSocketMgr:sendMsg(DICE_EXIT_ROOM_REQ);
			StateChange.changeState(self.mStateFrom or States.GameLobby);
		end
	}, WindowStyle.POPUP);
end

function DiceRoomController:onGetBrokenMoneyResponse(isSuccess, data)
	WindowManager:closeWindowByTag(WindowTag.LobbyExitPopu)
end
--[[
function DiceRoomController:onOtherFromFriend(isSuccess, data)
	if app:checkResponseOk(isSuccess, data) then
		local playerInfo = setProxy(new(require("data.playerInfoData")))
		playerInfo:init(data.data)
        local window = WindowManager:findChildByName(WindowTag.RoomPlayerInfoPopu)
        if window then
            window:initView({playerInfo, self.mRoomInfo, self})
        end
        --WindowManager:showWindow(WindowTag.RoomPlayerInfoPopu, {playerInfo, self.mRoomInfo, self}, WindowStyle.NORMAL);
	else
		AlarmTip.play('无结果');
	end
end
]]
function DiceRoomController:onLoginServer( data )
	-- body
	if data.isInTable > 0 then
		AlarmTip.play(STR_BACK_TO_GAME)
		local roomInfo = data.roomInfo;
		self.mTableId:init({tableId = roomInfo.tid, 
							isInTable = true, 
							gameLevel = roomInfo.serverLvl, 
							gameId = roomInfo.gameID });
	else
		--退出房间
		StateChange.changeState(self.mStateFrom or States.GameLobby);
	end

	if data.isInMatch > 0 then
	end
end


--取得用户 
function DiceRoomController:getUserByUid(uid)
    for k, v in pairs(self.mPlayer) do
		if v:getId() == uid then
			return v
		end
	end
    return nil;
end
--接收server返回道具信息
function DiceRoomController:onSendProp(data)
 --此处处理广播消息,收到发送道具之后
 	--AlarmTip.play('接收到互动道具广播='..data.type..'mid = '..data.mid..'destid='..data.dest_mid)
 	local msgInfo 	 = json.decode(data.msg_info);
   	if data.mid ~= 0 and data.dest_mid == 0 then
        --扔的是荷官
   		local srcPlayer  = self:getUserByUid(data.mid)
   		--local dstPlayer  = self:getUserByUid(data.dest_mid)
   		if not srcPlayer then
	   		return 
	   	end

	    if data.type == 1 then

	    	local dealer = self:findChildByName('btn_dealer');
	   		local srcx, srcy = srcPlayer:getUi():getAbsolutePos()
		   	local dstx, dsty = dealer:getAbsolutePos();
		   	local dstw, dsth = dealer:getSize()
		   	self.mAnimFriend:play(msgInfo.id, {x = srcx , y = srcy}, {x = (dstx + 15), y = dsty + 20})
            if(msgInfo.id ~= 2003 and msgInfo.id ~= 2004 and msgInfo.id ~= 2005) then
		   	    self:showChatWord(STR_DEALER_NEGATIVE[math.random(1, #STR_DEALER_NEGATIVE)], dstx + 30, dsty + 30, dstw, dsth, 10);
            else
                self:showChatWord(STR_DEALER_POSITIVE[math.random(1, #STR_DEALER_POSITIVE)], dstx + 30, dsty + 30, dstw, dsth, 10);
            end
	    elseif data.type == 2 then --处理送筹码
	    	--dstPlayer:addChip(tonumber(msgInfo.addMoney or 0))
	    	local dealer = self:findChildByName('btn_dealer');
			local dstx, dsty = dealer:getAbsolutePos();
			local dstw, dsth = dealer:getSize();
			self:playSendChipAnim(srcPlayer, {dstx = dstx, dsty = dsty, dstw = dstw, dsth = dsth}, tonumber(msgInfo.addMoney or 0))
			local random = math.random(1, 6)
			if MyUserData:getSex() == 0 then
				self:showChatWord(DEALER_MAN[random], dstx - 10, dsty - 10, dstw, dsth, 10)
			else
				self:showChatWord(DEALER_WOMAN[random], dstx - 10, dsty - 10, dstw, dsth, 10)
			end
	    end
   	else
   		local srcPlayer  = self:getUserByUid(data.mid)
   		local dstPlayer  = self:getUserByUid(data.dest_mid)
   		if not srcPlayer or not dstPlayer then
	   		return 
	   	end
	    if data.type == 1 then
	   		local srcx, srcy = srcPlayer:getUi():getAbsolutePos()
		   	local dstx, dsty = dstPlayer:getUi():getAbsolutePos()
		   	self.mAnimFriend:play(msgInfo.id, {x = srcx, y = srcy}, {x = dstx, y = dsty})
	    elseif data.type == 2 then --处理送筹码
	    	local dstx, dsty = dstPlayer:getUi():getAbsolutePos()
			local dstw, dsth = dstPlayer:getUi():getSize()
	    	dstPlayer:addChip(tonumber(msgInfo.addMoney or 0))
			self:playSendChipAnim(srcPlayer, {dstx = dstx, dsty = dsty, dstw = dstw, dsth = dsth}, tonumber(msgInfo.addMoney or 0))
		elseif data.type == 4 then --添加好友
	   		local srcx, srcy = srcPlayer:getUi():getAbsolutePos()
		   	local dstx, dsty = dstPlayer:getUi():getAbsolutePos()
		   	self.mAnimFriend:play(3000, {x = srcx, y = srcy}, {x = dstx, y = dsty})
		end


   	end
end

function DiceRoomController:requestSendFace(faceType)
	if not faceType then return end
	GameSocketMgr:sendMsg(DICE_SEND_FACE, {
		iFaceType = faceType,
		isVipFace = 0,
	})
end

function DiceRoomController:requestSendWord(chatInfo)
	if not chatInfo then return end
	GameSocketMgr:sendMsg(DICE_SEND_CHAT, {
		iChatInfo = chatInfo,
	})
end

--收到礼物的广播，刷新当前房间内在广播里的玩家礼物
function DiceRoomController:onGiftSendBroadcast(data)
    local receiver = data.receiver
    local giftId = data.gift
    --找到送礼人的人
    local sendPlayer = nil
    for i = 1, #self.mPlayer do
        if self.mPlayer[i]:getId() == data.sender then
            sendPlayer = self.mPlayer[i]
            break;
        end
    end 
    --找到礼物对应的图片
    local imgName = ""
    local giftList = MyUserData:getGiftList() or {}
    local gift = giftList[giftId[1]] and giftList[giftId[1]][giftId[2]]
    if gift then
        imgName = gift:getImgName()
    else
    --礼物不存在
        return
    end
    
    --找到需要接收礼物的人
    for i = 1, #receiver do
        for j = 1, #self.mPlayer do
            if self.mPlayer[j]:getId() == receiver[i] then
                --如果送礼物的人和这个收礼物的人是同一个人，那么礼物本来就是从他的位置送出，不需要动画
                if self.mPlayer[j]:getId() ~= data.sender then
                    --找不到赠送人，从屏幕中送出
                    local startX, startY = CONFIG_SCREEN_WIDTH / 2, CONFIG_SCREEN_HEIGHT / 2
                    if sendPlayer then
                        startX, startY = sendPlayer:getUi():findChildByName("btn_gift"):getAbsolutePos()
                    elseif data.sender ~= MyUserData:getId() then
                        AlarmTip.play(string.format(STR_GIFT_GET_FROM_FRIEND, data.name, data.sender, gift:getName()))
                    end
                    --送礼物的动画
                    local endX, endY = self.mPlayer[j]:getUi():findChildByName("btn_gift"):getAbsolutePos()
                    local node = new(Node)
                    local imgGift = new(Image, imgName)
                    imgGift:setAlign(kAlignCenter)
                    node:setSize(imgGift:getSize())
                    node:addChild(imgGift)
                    node:setPos(startX, startY)
                    self:addChild(node)
                    local animGift = node:addPropTranslateWithEasing(1, kAnimNormal, 500, 0, "easeOutSine", "easeOutSine", 0, endX - startX, 0, endY - startY)
                    printInfo("animGift is "..tostring(animGift))
                    animGift:setEvent(self, function()
                        self.mPlayer[j]:setGiftId(giftId)
                        self:removeChild(node, true)
                    end)
                else
                    self.mPlayer[j]:setGiftId(giftId)
                end
                break
            end
        end
    end
end

function DiceRoomController:onAcceptPrivateRoomInvite(data)
    GameSocketMgr:sendMsg(DICE_EXIT_ROOM_REQ);
    self.acceptPrivateRoom = data
--	StateChange.changeState(self.mStateFrom or States.GameLobby);
end

function DiceRoomController:onServerRetire(data)
    StateChange.changeState(States.Lobby);
end

-- 聊天相关
function DiceRoomController:showChatWord(chatInfo, x, y, w, h, seatId)
	local srcx, srcy = x,y
	local srcw, srch = w,h

	local localSeatId = seatId
	local imgFace 		= new(Image, 'animation/roomAnim/word_bg_left.png')
	local imgChipW,imgChipH = imgFace.m_width, imgFace.m_height
	if not self.m_chatWord then
		self.m_chatWord = new(Node)
			:addTo(self:findChildByName("view_players"), 1000)
	end
	self.m_chatWord:removeAllChildren()
	local x, y = self:getChatPosition(localSeatId, srcx, srcy, srcw, srch, imgChipW, imgChipH)
	if not x or not y then return end

	local params = {
		seat = localSeatId,
		--seat = self.m_seat,
		chatInfo = chatInfo,
		pos = ccp(x, y),
		isGame = true
		--isGame = self:getIsInGamePos(),
	}
	new(AnimChat):addTo(self.m_chatWord, 1000)
		:play(params)

	--playChatSound(chatInfo, self.m_userData:getSex())
	self:playChatSound(chatInfo)
end

function DiceRoomController:playChatSound(chatInfo)
	local strWord = STRING.SysChatArray
	if strWord[1] == chatInfo then
		kEffectPlayer:play(Effects.AudioChatCommon1);
	elseif strWord[2] == chatInfo then
		kEffectPlayer:play(Effects.AudioChatCommon2);
	elseif strWord[3] == chatInfo then
		kEffectPlayer:play(Effects.AudioChatCommon3);
	elseif strWord[4] == chatInfo then
		kEffectPlayer:play(Effects.AudioChatCommon4);
	elseif strWord[5] == chatInfo then
		kEffectPlayer:play(Effects.AudioChatCommon5);
	elseif strWord[6] == chatInfo then
		kEffectPlayer:play(Effects.AudioChatCommon6);
	elseif strWord[7] == chatInfo then
		kEffectPlayer:play(Effects.AudioChatCommon7);
	elseif strWord[8] == chatInfo then
		kEffectPlayer:play(Effects.AudioChatCommon8);
	elseif strWord[9] == chatInfo then
		kEffectPlayer:play(Effects.AudioChatCommon9);
	elseif strWord[10] == chatInfo then
		kEffectPlayer:play(Effects.AudioChatCommon10);
	elseif strWord[11] == chatInfo then
		kEffectPlayer:play(Effects.AudioChatCommon11);
	elseif strWord[12] == chatInfo then
		kEffectPlayer:play(Effects.AudioChatCommon12);
	elseif strWord[13] == chatInfo then
		kEffectPlayer:play(Effects.AudioChatCommon13);
	end	
end

function DiceRoomController:getChatPosition(seatId, x, y, w, h, mx, my)
	--AlarmTip.play('seatId = ' .. seatId .. ', x = ' .. x .. ',y = ' .. y .. ',w = ' .. w .. ',h = ' .. h .. ', mx = ' .. mx .. ',my = ' .. my)
	if seatId == 1 then
		return (x - mx), (y - h/2 + my)
	elseif seatId == 2 then
		return (x - mx), (y - h/2 + my)
	elseif seatId == 3 then
		return (x - mx), (y - my/2)
	elseif seatId == 4 then
		return (x - mx), (y - my/2)
	elseif seatId == 5 then
		return (x + w), (y - my/2)
	elseif seatId == 6 then
		return (x + w), (y - my/2)
	elseif seatId == 7 then
		return (x + w), (y - my/2)
	elseif seatId == 8 then
		return (x + w), (y - h/2 + my)
	elseif seatId == 9 then
		return (x + w), (y - h/2 + my)
	elseif seatId == 10 then
		return (x + w), (y - h/2 + my)
	end
end

function DiceRoomController:showChatFace(faceIndex, mid)
	if not self.m_chatFace then
		self.m_chatFace = new(Node)
			:addTo(self, 1001)
	end
	self.m_chatFace:removeAllChildren()
	
	local player  = self:getUserByUid(mid);
	if not player then return end
	local srcx, srcy = player:getUi():getPos();
	local srcw, srch = player:getUi():getSize();
	if not srcx or not srcy then return end

	local faceName = nil;

	local facePrefix = FaceConfig[1].expNamePrefix
	local img = FaceConfig[1].expressInfo .. faceIndex .. "0%02d.png"

	if type(facePrefix) == "table" then
		faceName = facePrefix[img]
	elseif type(facePrefix) == "string" then
		faceName = img
	end
	local imgCount = FaceConfig[1][faceIndex].imgCount
	local playCount = FaceConfig[1][faceIndex].playCount
	local playTime = FaceConfig[1][faceIndex].ms
	local imgFace 		= new(Image, img)
	local imgChipW,imgChipH = imgFace.m_width, imgFace.m_height
	--AlarmTip.play('srcx = ' .. srcx .. ',srcw = ' .. srcw .. ',imgChipW = ' .. imgChipW)
	local x,y = srcx + (srcw - 100)/2, srcy + (srch - 100)/2
	--printInfo("玩家%d发送了表情%d", self.m_seat, faceIndex)
	local params = {
		num = imgCount,
		playCount = playCount,
		faceName = faceName,
		duration = playTime,
		pos = ccp(x, y),
	}
	new(AnimFace):addTo(self:findChildByName("view_players"), 1001)
		:play(params)
end

function DiceRoomController:onSendFace(data)
	if data.mid and data.type and data.isVipFace then
		self:showChatFace(data.type, data.mid);
	else
	end
end

function DiceRoomController:onSendChat(data)
	if data.mid and data.msg then
		local player  = self:getUserByUid(data.mid);
		if not player then return end
		local srcx, srcy = player:getUi():getPos();
		local srcw, srch = player:getUi():getSize();
		local localSeatId = player:getLocalSeatId();
		self:showChatWord(data.msg, srcx, srcy, srcw, srch, localSeatId);
		local player  = self:getUserByUid(data.mid);
		if player then
			G_RoomCfg:addChatRecord(player:getNick(), data.msg)
		end
	end
end

function DiceRoomController:checkSitDown()
    
    local myMoney = MyUserData:getMoney()
    local content = ""
    if myMoney < self.mRoomInfo:getMinchip() then
		content = string.format(STR_NOT_ENOUGH_MONEY_SIT_DOWN, self.mRoomInfo:getMinchip());
	elseif myMoney > self.mRoomInfo:getMaxchip() then
		content = string.format(STR_TOO_MUCH_MONEY_SIT_DOWN, self.mRoomInfo:getMaxchip());
	else
        return true;
    end
    WindowManager:showWindow(WindowTag.LobbyExitPopu, {
			title 	= "",
			close 	= true,
			content = content,
			confirm =STR_EXIT_GAME_CONFIRM,
			}, WindowStyle.POPUP);
   return false 
end

function DiceRoomController:loadPicture()
    self.imgBet = new(Image, "dice/room/bet.png")
    self.imgResultnumber = new(Image, "dice/room/resultnumber.png")
    self.imgResult = new(Image, "dice/room/result.png")
    self.imgDicePoint = new(Image, "dice/room/dicePoint.png")
end

function DiceRoomController:unloadPicture()
    delete(self.imgBet)
    delete(self.imgResultnumber)
    delete(self.imgResult)
    delete(self.imgDicePoint)
end


--HTTP事件
DiceRoomController.s_severCmdEventFuncMap = {
	[HttpModule.s_cmds.GET_BROKEN_MONEY] 		= DiceRoomController.onGetBrokenMoneyResponse,
}

DiceRoomController.messageFunMap = {
	["loginserver"] 			= DiceRoomController.onLoginServer,
	["sendChatFace"] 			= DiceRoomController.requestSendFace,
	["sendChatWord"] 			= DiceRoomController.requestSendWord,
    ["giftSendBroadcast"]       = DiceRoomController.onGiftSendBroadcast,
    ["acceptPrivateRoomInvite"] = DiceRoomController.onAcceptPrivateRoomInvite,
    ["serverRetire"]    = DiceRoomController.onServerRetire,
}


--sokcet事件
DiceRoomController.commandFunMap = {
	[DICE_GET_TABLEID_RSP]		= DiceRoomController.onGetTableId,
	[DICE_FOLLOW_RSP] 			= DiceRoomController.onFollow,	--
	[DICE_FOLLOWED_RSP] 		= DiceRoomController.onFollowed,	--
	[DICE_LOGIN_GAME_RSP]		= DiceRoomController.onLoginGame,
	[DICE_LOGIN_GAME_ERROR_RSP] = DiceRoomController.onLoginErrorGame,
	[DICE_RELOGIN_GAME_RSP] 	= DiceRoomController.onReLoginGame,
	[DICE_SITDOWN_IN_GAME_RSP] 	= DiceRoomController.onSitDownInGame,
	[DICE_BROADCAST_SITDOWN_RSP]= DiceRoomController.onBroadcastSitdown,
	[DICE_START_BET_RSP] 		= DiceRoomController.onStartBet,
	[DICE_BET_RSP] 				= DiceRoomController.onBet,
	[DICE_OTHER_BET_RSP] 		= DiceRoomController.onOtherBet,
	[DICE_GAME_START_RSP] 		= DiceRoomController.onGameStart,
	[DICE_GAME_END_RSP] 		= DiceRoomController.onGameEnd,
	[DICE_STANDUP_RSP] 			= DiceRoomController.onStandUp,
	[DICE_BROADCAST_STANDUP_RSP]= DiceRoomController.onBroadcastStandUp,
	[DICE_EXIT_ROOM_RSP] 		= DiceRoomController.onExitRoom,
    [DICE_SEND_PROP_RSP] 		= DiceRoomController.onSendProp, --接收server返回道具信息
    [DICE_SEND_FACE]			= DiceRoomController.onSendFace,
    [DICE_SEND_CHAT]			= DiceRoomController.onSendChat,
}

return DiceRoomController;