require(ViewPath.."lobbyGameTab")
require(ViewPath.."beginnerGuidLayout")

local GameTab = class(Node)
local kAnimTime = 150
local kIntervalTime = 50

function GameTab:ctor()
    self.diceGameNum = 0
    self.chessGameNum = 0
    self.cardGameNum = 0

    self:initView() 
end

function GameTab:initView()
    self.bg = SceneLoader.load(lobbyGameTab)
    printInfo("zyh self.bg "..self.bg:getPos().." "..self.bg:getSize())
    self.bg:setAlign(kAlignCenter)
    self:addChild(self.bg)
    self:setSize(self.bg:getSize())
    self:setFillParent(1, 1)

    local viewBtnTabs = {self.bg:findChildByName("view_recent"), 
        self.bg:findChildByName("view_card"),
        self.bg:findChildByName("view_dice"),
        -- self.bg:findChildByName("view_chess"),
    }
    local strTitle = {STR_TYPE_RECENT, STR_TYPE_CARD, STR_TYPE_DICE, STR_TYPE_CHESS}
    local tabsBg = self.bg:findChildByName("view_tabs")

    local tabs = {tabsBg:findChildByName("tab_recent"),
        tabsBg:findChildByName("tab_card"),
        tabsBg:findChildByName("tab_dice"),
        -- tabsBg:findChildByName("tab_chess"),
    }
    local tabName = {"tab_recent", "tab_card", "tab_dice", }
    
    local imgSelect = self.bg:findChildByName("img_select")
    local wSelect = imgSelect:getSize()
    local wView = viewBtnTabs[1]:getSize()
    local selectPos = {viewBtnTabs[1]:getPos() + wView / 2 - wSelect / 2,
        viewBtnTabs[2]:getPos() + wView / 2 - wSelect / 2,
        viewBtnTabs[3]:getPos() + wView / 2 - wSelect / 2,
        -- viewBtnTabs[4]:getPos() + wView / 2 - wSelect / 2,
    }
    local lastSelectIndex = 0
    
    for i = 1, #viewBtnTabs do
        local btnTab = viewBtnTabs[i]:findChildByName("btn_unselect")
        btnTab:setOnClick(self, function(self)
            if i == lastSelectIndex then
                return
            end
            --记录下当前显示的页面索引
            GameSetting.lastGameTabPos.typeIndex = i
            for index = 1, #tabs do
--                tabs[index]:setVisible(i == index)
                viewBtnTabs[index]:findChildByName("btn_unselect"):setEnable(false)
            end
            --删除tab的延时不可见动画，防止快速点击切换回来后整个view不可见
            checkAndRemoveOneProp(tabs[i], 1)
            tabs[i]:setVisible(true)            
            checkAndRemoveOneProp(imgSelect, 1)
            local fromx = imgSelect:getPos()
            local tox = selectPos[i]
            --选中按钮移动动画
            local animMove = imgSelect:addPropTranslateWithEasing(1, kAnimNormal, 200, -1, "easeOutBack", "easeOutBack", 0, tox-fromx, 0, 0)
            animMove:setEvent(nil, function()
                for index = 1, #tabs do
                    viewBtnTabs[index]:findChildByName("btn_unselect"):setEnable(true)
                end
                checkAndRemoveOneProp(imgSelect, 1)
                imgSelect:setPos(tox, 0)
            end)
            --加载的游戏列表抖动动画
            local lastTab = self[tabName[lastSelectIndex]]
            if lastTab then
                local gamebtns = {}
                if lastTab:getTab(lastTab:getCurIndex()) then
                    gamebtns = lastTab:getTab(lastTab:getCurIndex()):getChildren()
                    for index = 1, #gamebtns do
                        local btn = gamebtns[index]
                        self:btnHideAnim(btn, index * kIntervalTime)
                    end
                end
                --延时让原先的tabview消失
                local lastTabView = tabs[lastSelectIndex]
                checkAndRemoveOneProp(lastTabView, 1)
                local animTabHide = lastTabView:addPropTranslate(1, kAnimNormal, #gamebtns * kIntervalTime + kAnimTime, 0, 0, 0, 0, 0)
                animTabHide:setEvent(nil, function()
                    lastTabView:setVisible(false)
                    checkAndRemoveOneProp(lastTabView, 1)
                    --tab回滚到第一个tab
                    lastTab:scrollToTabIndex(1)
                    --恢复原先消失了的按钮可见
                    for index = 1, #gamebtns do
                        gamebtns[index]:setVisible(true)
                    end
                end)
            end

            --加载将要显示的抖动动画
            local curTab = self[tabName[i]]
            if curTab and curTab:getTab(1) then
                local gamebtns = curTab:getTab(1):getChildren()
                for index = 1, #gamebtns do
                    local btn = gamebtns[index]
                    btn:setVisible(false)
                    self:btnShowAnim(btn, index * kIntervalTime + kAnimTime)
                end
            end

            lastSelectIndex = i
        end)
        btnTab:findChildByName("text_button"):setText(strTitle[i])
        local w, h = tabs[i]:getSize()
        local newTab = new(require("ui.tabViewEx"), 0, 0, w, h)
        newTab:setScrollAlways(true)
        newTab:setName(tabName[i])
        newTab:setDirection(kHorizontal)
        tabs[i]:addChild(newTab)
        self[tabName[i]] = newTab
        newTab:setArriveStartCallback(nil, function()
            local newIndex = i - 1 < 1 and #viewBtnTabs or i - 1
            local btn = viewBtnTabs[newIndex]:findChildByName("btn_unselect")
            btn.m_eventCallback.func(btn.m_eventCallback.obj)
            --有页面切换时，如果有新手引导，关掉
            if self.beginerHelp then
                self.beginerHelp:setVisible(false)
                self.bg:findChildByName("img_shade"):setVisible(false)
            end
        end)
        newTab:setArriveEndCallback(nil, function()
            local newIndex = i + 1 > #viewBtnTabs and 1 or i + 1
            local btn = viewBtnTabs[newIndex]:findChildByName("btn_unselect")
            btn.m_eventCallback.func(btn.m_eventCallback.obj)
            --有页面切换时，如果有新手引导，关掉
            if self.beginerHelp then
                self.beginerHelp:setVisible(false)
                self.bg:findChildByName("img_shade"):setVisible(false)
            end
        end)
        newTab:setTabChangedCallback(nil, function(obj, curIndex, lastIndex)
            GameSetting.lastGameTabPos.tabIndex = curIndex
            --有页面切换时，如果有新手引导，关掉
            if self.beginerHelp and curIndex ~= lastIndex then
                self.beginerHelp:setVisible(false)
                self.bg:findChildByName("img_shade"):setVisible(false)
            end
        end)
    end

    self:addMainButton()
    self:addRecentGame()
    self:addTabGameButton()

    --记录了上次游戏图标的位置
    if GameSetting.lastGameTabPos then
        local typeIndex = GameSetting.lastGameTabPos.typeIndex
        local tabIndex = GameSetting.lastGameTabPos.tabIndex
        local viewBtn = viewBtnTabs[typeIndex]
        if viewBtn then
            local btn = viewBtn:findChildByName("btn_unselect")
            btn.m_eventCallback.func(btn.m_eventCallback.obj)
            self[tabName[typeIndex]]:scrollToTabIndex(tabIndex)
        end
    else
        GameSetting.lastGameTabPos = {}
        GameSetting.lastGameTabPos.typeIndex = 1
        GameSetting.lastGameTabPos.tabIndex = 1
        --默认点击最近按钮
        local btnRecent = viewBtnTabs[1]:findChildByName("btn_unselect")
        btnRecent.m_eventCallback.func(btnRecent.m_eventCallback.obj)
    end
    
end

function GameTab:addGameButton(tabNode, indexInTab, gameId, hotFlag)
    local pos = {{20, -10}, {350, -10}, {20, 200}, {350, 200}}
    local gameBtn = new(require("lobby.ui.lobbyGameButton"), gameId, false)
    local index = indexInTab % 4
    gameBtn:findChildByName("img_bg"):setFile(lobby_map[string.format("lobby.gamebg%s.png", index)])
    gameBtn:findChildByName("img_hot"):setVisible(hotFlag)
    gameBtn:setPos(unpack(pos[index + 1]))
    self:initButtonClick(gameBtn)

    local pageNum = math.floor(indexInTab / 4) + 1
    local curPage = tabNode:getTab(pageNum)
    if curPage then
        curPage:addChild(gameBtn)
    else
        --新页面，
        local tabPage = new(Node)
        tabPage:setSize(tabNode:getSize())
        tabPage:addChild(gameBtn)
        tabNode:addTab(tabPage)
    end

    return indexInTab + 1
end

function GameTab:addMainButton()
    local gameIds = app:getGameIds()
    local gameId = gameIds[1]
    if gameId then
        local mainGame = new(require("lobby.ui.lobbyGameButton"), gameId, true)
        mainGame:setAlign(kAlignTop)
        mainGame:setName("btn_mainGame")
        self:initButtonClick(mainGame, gameId)
        self.bg:findChildByName("view_main"):addChild(mainGame)
        self.mainGameBtn = mainGame
    end
end

function GameTab:addTabGameButton()
    local gameIds = app:getGameIds()
    local gameList = MyUserData:getGameList() or {}
    --加载游戏列表，只处理dice、card、chess 3个页面的，recent页面另外处理，逻辑不同

    --分类下的第一个游戏自动下载，其他游戏不自动下载
    --目前自动下载的游戏只有马哈，先取消，统一改为点击下载
    local autoDice, autoCard, autoChess
    for i = 1, #gameIds do
        local gameId = gameIds[i]
        local gameType = gameList[tostring(gameId)] or {}
        if gameType.type == "dice" then
            self.diceGameNum  = self:addGameButton(self.tab_dice, self.diceGameNum, gameId)
        elseif gameType.type == "card" then
            self.cardGameNum = self:addGameButton(self.tab_card, self.cardGameNum, gameId)
        elseif gameType.type == "chess" then
            self.chessGameNum = self:addGameButton(self.tab_chess, self.chessGameNum, gameId)
        end
    end
    if self.diceGameNum == 0 then
        local img = new(Image, "lobby/expecting.png")
        local w, h = self.tab_dice:getSize()
        local wImg, hImg = img:getSize()
        self.tab_dice:addChild(img)
        img:setPos(w / 2 - wImg / 2, h / 2 - hImg / 2)
    end
    if self.cardGameNum == 0 then
        local img = new(Image, "lobby/expecting.png")
        local w, h = self.tab_card:getSize()
        local wImg, hImg = img:getSize()
        self.tab_card:addChild(img)
        img:setPos(w / 2 - wImg / 2, h / 2 - hImg / 2)
    end
end

function GameTab:addRecentGame()
    local MAX_RECENT = 2
    local gameRecord = GameSetting:playRecord()
    
    local recentGame = {}
    
    local gameIds = app:getGameIds()
    for i = 2, #gameIds do
        self:addGameButton(self.tab_recent, i - 2, gameIds[i])
    end
--     local firstGame = gameIds[1]
--     if firstGame and firstGame > 0 then
--         recentGame[firstGame] = true
--     end
--     --游戏列表第二个，固定显示hot
--     local hotGameId = gameIds[2]
--     --传2是为了计算位置方便，从0开始计数
--     if hotGameId and hotGameId > 0 then
--         self:addGameButton(self.tab_recent, 2, hotGameId, true)
--         recentGame[hotGameId] = true
--     end
   
--     --首页最多加载两个最近游戏，3、4号位置保留给hot游戏和私人房
--     local recentCount = 0
--     for i = 1, #gameRecord do
--         --首页最多加载两个最近游戏
--         local recentGameId = gameRecord[i]
--         for index = 1,#gameIds do
--             --存在于游戏列表中的最近游戏才需要加载
--             if gameIds[index] == recentGameId then
--                 if recentCount < MAX_RECENT and not recentGame[recentGameId] then
--                     recentCount = self:addGameButton(self.tab_recent, recentCount, recentGameId)
--                     recentGame[recentGameId] = true
--                 end
--                 break
--             end
--         end
        
--     end
--     --最近游戏列表第三格子固定显示游戏列表的第二个游戏，第4格子固定显示私人房，
--     --1、2格有最近游戏显示最近游戏，没有就显示列表的第3、4个游戏。

--     --如果还有空位，从游戏列表第三个游戏开始继续加载
-- printInfo("zyh has load recent game "..recentCount)
--     if recentCount < MAX_RECENT then
--         for i = 3, #gameIds do
--             local gameId = gameIds[i]
--             if gameId and gameId > 0 and not recentGame[gameId] and recentCount < MAX_RECENT then
--                 recentCount = self:addGameButton(self.tab_recent, recentCount, gameId)
--                 recentGame[gameId] = true
--                 printInfo("zyh addListGame  "..recentCount.."   "..gameId)
--             end
--         end
--     else
--         --没有空位了，不加载游戏列表里的游戏,新开一个tab加载最多4个最近游戏,tab内索引从4开始是第二页
--         local tab2Index = 4
--         for i = 1, #gameRecord do
--             if not recentGame[gameRecord[i]] and tab2Index < 8 then
-- printInfo("zyh add extra recent game "..gameRecord[i])
--                 tab2Index = self:addGameButton(self.tab_recent, tab2Index, gameRecord[i])
--                 recentGame[gameRecord[i]] = true
--             end
--         end
--     end
--     local tabNode = self.tab_recent:getTab(1)
--     if tabNode and #tabNode:getChildren() > 0 then

--     else
--         local img = new(Image, "lobby/expecting.png")
--         local w, h = self.tab_recent:getSize()
--         local wImg, hImg = img:getSize()
--         self.tab_recent:addChild(img)
--         img:setPos(w / 2 - wImg / 2, h / 2 - hImg / 2)
--         return
--     end

--     --私人房按钮单独处理
--     local hasPrivateRoom = false
--     for i = 1, #gameIds do
--         if gameIds[i] == kBuddyRoomID then
--             --需要加载私人房
--             hasPrivateRoom = true
--             break
--         end
--     end
--     self:addPrivateRoomButton(hasPrivateRoom)
end

function GameTab:addPrivateRoomButton(hasPrivateRoom)
    local tab = self.tab_recent:getTab(1)
    if not tab then
        return
    end
    require(ViewPath .. "lobbyGameLayout2")
    local button = new(Button, "ui/blank.png");
    local layout = SceneLoader.load(lobbyGameLayout2);
    button:setSize(layout:getSize())
    button:addChild(layout);
    button:findChildByName("text_gameName"):setText(STR_PRIVATE_ROOM_NAME)
    button:findChildByName("btn_chooseroom"):setVisible(false)

    local imgBg = button:findChildByName("img_gamebg")
    if hasPrivateRoom then
        imgBg:setFile(lobby_map["lobby.icon_privateRoom.png"] or "")
    else
        --私人房维护中
        -- imgBg:setFile("lobby/icon_maintain.png")
    end
    imgBg:setSize(imgBg.m_res.m_width, imgBg.m_res.m_height)
    imgBg:setPos(10, 10)
    
    button:findChildByName("img_bg"):setFile(lobby_map["lobby.gamebg3.png"] or "")
    button:setOnClick(self, function(self)
        if hasPrivateRoom then
            WindowManager:showWindow(WindowTag.PrivateRoomPopu, {}, WindowStyle.NORMAL)
            if self.beginerHelp then
                self.beginerHelp:setVisible(false)
                self.bg:findChildByName("img_shade"):setVisible(false)
            end
        end
    end)
    button:setPos(350, 200)
    tab:addChild(button)
end

function GameTab:initButtonClick(gameBtn)
    gameBtn:setOnClickButton(self, function ( self )
		-- body
print_string("zyh initButtonClick")
        --破产了，不让进房间，弹出破产提示框
        if MyUserData:getMoney() < gBankrupt then
            WindowManager:showWindow(WindowTag.LobbyExitPopu, {
			content = STR_NOT_ENOUGH_MONEY,
            confirm = STR_GET_BROKEN,
            confirmFunc = function()
                WindowManager:showWindow(WindowTag.BankruptPopu, {state = 1}, WindowStyle.POPUP)
            end,
			}, WindowStyle.POPUP);

            return
        end
		local gameId = gameBtn:getGameId()
		if gameId > 0 then
			local game = app:getGame(gameId)
			if game then 
                --判断是否低于最低底注，低的话不让进房间
                local roomList = game:getRoomList()
                if roomList then
                    local room = roomList:get(1);
                    --钱少于最低筹码要求，都进不去
                    if room then
                        if (gameId ~= 1004 and room.minchip and room.minchip > MyUserData:getMoney()) 
                            or (gameId == 1004 and room:getAnte() > MyUserData:getMoney()) then
                            WindowManager:showWindow(WindowTag.LobbyExitPopu, {
                                content = STR_MONEY_LESS_THAN_MIN,
                                confirm = STR_EXIT_GAME_CONFIRM,
                            }, WindowStyle.POPUP)
                            return
                        end
                    end
                
                end   
                --未获取房间列表，刷新
                if not game:enterRoom() then
                    app:checkAndDownloadRoomList(gameId, 1, false)
                    AlarmTip.play(STR_NETWORK_TERRIBLE_AND_RETRY)
                end
            end
			app:postFrontStaticstics(string.format('click%sQuickStart', gameId))
		else
			--AlarmTip.play(STR_GAME_CLOSED);
		end
	end)
	--选房
	gameBtn:setOnChooseRoom(self, function ( self )
		-- body
        --破产了，不让进房间，弹出破产提示框
        if MyUserData:getMoney() < gBankrupt then
            WindowManager:showWindow(WindowTag.LobbyExitPopu, {
			content = STR_NOT_ENOUGH_MONEY,
            confirm = STR_GET_BROKEN,
            confirmFunc = function()
                WindowManager:showWindow(WindowTag.BankruptPopu, {state = 1}, WindowStyle.POPUP)
            end,
			}, WindowStyle.POPUP);

            return
        end

		local gameId = gameBtn:getGameId()
		if gameId > 0 then
			local game = app:getGame(gameId)
			if game then 
                game:enterLobby() 
            end
            app:postFrontStaticstics(string.format('click%sChooseGame', gameId))
		else
			AlarmTip.play(STR_GAME_CLOSED);
		end
	end)
end

function GameTab:btnHideAnim(gameBtn, delay)
    checkAndRemoveOneProp(gameBtn, 1)
    checkAndRemoveOneProp(gameBtn, 2)
    local animDisappear = gameBtn:addPropTransparency(1, kAnimNormal, kAnimTime, delay, 1, 0)
    local animShakeLeft = gameBtn:addPropTranslate(2, kAnimNormal, kAnimTime, delay, 0, -10, 0, 0)
    animShakeLeft:setEvent(nil, function()
        checkAndRemoveOneProp(gameBtn, 1)
        checkAndRemoveOneProp(gameBtn, 2)
        gameBtn:setVisible(false)
    end)
end

function GameTab:btnShowAnim(gameBtn, delay)
    checkAndRemoveOneProp(gameBtn, 101)
    checkAndRemoveOneProp(gameBtn, 102)
    checkAndRemoveOneProp(gameBtn, 103)
    checkAndRemoveOneProp(gameBtn, 104)
    gameBtn:addPropTranslateSolid(101, -10, 0)
    local animDelay = gameBtn:addPropTranslate(102, kAnimNormal, 0, delay, 0, 0, 0, 0)
    animDelay:setEvent(nil, function()
        gameBtn:setVisible(true)
        local animShow = gameBtn:addPropTransparency(103, kAnimNormal, kAnimTime, 0, 0, 1)
        local animShakeRight = gameBtn:addPropTranslate(104, kAnimNormal, kAnimTime, 0, 0, 10, 0, 0)
        animShakeRight:setEvent(nil, function()
            checkAndRemoveOneProp(gameBtn, 101)
            checkAndRemoveOneProp(gameBtn, 102)
            checkAndRemoveOneProp(gameBtn, 103)
            checkAndRemoveOneProp(gameBtn, 104)
        end)
    end)
end

function GameTab:enterHallAnim()
printInfo("zyh GameTab:enterHallAnim "..self.bg:getAbsolutePos().." "..self.bg:getPos())
    --用来协调和大厅的动画同步
    local kDelayAll = 350

    local screenWidth 	= CONFIG_SCREEN_WIDTH
  	local screenHeight 	= CONFIG_SCREEN_HEIGHT

    --左边大图进入
    local viewMain = self.bg:findChildByName("view_main")
    local x = viewMain:getAbsolutePos()
    local w = viewMain:getSize()
    local delta = x + w + 10
    checkAndRemoveOneProp(viewMain, 1)
    checkAndRemoveOneProp(viewMain, 2)
    checkAndRemoveOneProp(viewMain, 3)
    --移动距离额外增加，用来做为抖动回来的移动距离
printInfo("zyh x "..x.." delta "..delta.." pos "..viewMain:getPos())
    local animShow = viewMain:addPropTranslateWithEasing(1, kAnimNormal, 300, kDelayAll, "easeOutSine", "easeOutSine", -delta, delta, 0, 0)
    --到达后加个抖动动画
    viewMain:addPropTranslateWithEasing(3, kAnimNormal, 500, 300 + kDelayAll, "easeOutBack", "easeOutBack", 10, -10, 0, 0)
    --大图带点半透明效果
    local imgMain = viewMain:findChildByName("btn_mainGame")
    if imgMain then
        local imgBg = imgMain:findChildByName("img_gamebg")
        checkAndRemoveOneProp(imgBg, 1)
        imgBg:addPropTransparency(1, kAnimNormal, 300, kDelayAll, 0.2, 1.0)
    end
        
    local imgRight = self.bg:findChildByName("img_other")
    local x = imgRight:getAbsolutePos()
    local w = imgRight:getSize()
    local delta = screenWidth - x + 10
printInfo("zyh x "..x.." delta "..delta.." pos "..imgRight:getPos())
    checkAndRemoveOneProp(imgRight, 1)
    checkAndRemoveOneProp(imgRight, 2)
    imgRight:addPropTranslateWithEasing(1, kAnimNormal, 300, kDelayAll, "easeOutSine", "easeOutSine", delta, -delta, 0, 0)
    imgRight:addPropTranslateWithEasing(2, kAnimNormal, 500, 300 + kDelayAll, "easeOutBack", "easeOutBack", -10, 10, 0, 0)

    --游戏图标进入动画
    if self.tab_recent and self.tab_recent:getTab(1) then
        local delay = {50, 200, 100, 250}
        local children = self.tab_recent:getTab(1):getChildren()
        for i = 1, #children do
            local btn = children[i]
            local x = btn:getAbsolutePos()
            local w = btn:getSize()
            local delta = screenWidth - x + 10
            checkAndRemoveOneProp(btn, 1001)
            checkAndRemoveOneProp(btn, 1002)
            btn:addPropTranslateWithEasing(1001, kAnimNormal, 400, kDelayAll + delay[i] or 0, "easeOutSine", "easeOutSine", delta, -delta, 0, 0)
            btn:addPropTranslateWithEasing(1002, kAnimNormal, 500, 400 + kDelayAll + delay[i] or 0, "easeOutBack", "easeOutBack", -10, 10, 0, 0)
        end
    end
end

function GameTab:initBeginnerHelpMain()
printInfo("zyh initBeginnerHelpMain")
    self.bg:findChildByName("view_main"):setLevel(1)
    local shade = self.bg:findChildByName("img_shade")
    shade:setLevel(0)
    shade:setVisible(true)
    shade:setEventTouch(nil, function() end)
    shade:setEventDrag(nil, function() end)
    self.bg:findChildByName("img_other"):setLevel(-1)

    local beginerHelp = SceneLoader.load(beginnerGuidLayout)
    local viewMain = beginerHelp:findChildByName("view_main")
    local viewOther = beginerHelp:findChildByName("view_other")
    viewMain:setVisible(true)
    viewOther:setVisible(false)
    local btnEnter = viewMain:findChildByName("btn_enterMainGame")
    btnEnter:setOnClick(nil, function()
        if self.mainGameBtn then
            self.mainGameBtn.m_eventCallback.func(self.mainGameBtn.m_eventCallback.obj)
        end
    end)
    btnEnter:removeProp(2)
    btnEnter:removeProp(3)
    btnEnter:addPropScaleWithEasing(2, kAnimNormal, 500, 0, "easeInOutSine", "easeInOutSine", 0.9, 0.2, kCenterDrawing)
    btnEnter:addPropScaleWithEasing(3, kAnimNormal, 500, 500, "easeInOutSine", "easeInOutSine", 1.1, -0.2, kCenterDrawing)
    btnEnter:removeProp(1)
    local animRepeat = btnEnter:addPropTranslate(1, kAnimRepeat, 2000, 0, 0, 0, 0, 0)
    animRepeat:setEvent(nil, function()
        btnEnter:removeProp(2)
        btnEnter:removeProp(3)
        btnEnter:addPropScaleWithEasing(2, kAnimNormal, 500, 0, "easeInOutSine", "easeInOutSine", 0.9, 0.2, kCenterDrawing)
        btnEnter:addPropScaleWithEasing(3, kAnimNormal, 500, 500, "easeInOutSine", "easeInOutSine", 1.1, -0.2, kCenterDrawing)
    end)
    local gameIds = app:getGameIds()
    local gameId = gameIds[1]
    if gameId then
        local gameStatus = app:getGameStatus(gameId)
        -- viewMain:findChildByName("text_tip"):setText(string.format(STR_BEGINNER_TIPS1, gameStatus:getName()))
        viewMain:findChildByName("text_tip"):setText(STR_BEGINNER_TIPS1)
    end
    self.beginerHelp = beginerHelp
    self:addChild(beginerHelp)
end

function GameTab:initBeginnerHelpOther()
printInfo("zyh initBeginnerHelpOther")
    self.bg:findChildByName("view_main"):setLevel(-1)
    local shade = self.bg:findChildByName("img_shade")
    shade:setLevel(0)
    shade:setVisible(true)
    shade:setEventTouch(nil, function() end)
    shade:setEventDrag(nil, function() end)
    self.bg:findChildByName("img_other"):setLevel(1)

    local beginerHelp = SceneLoader.load(beginnerGuidLayout)
    local viewMain = beginerHelp:findChildByName("view_main")
    local viewOther = beginerHelp:findChildByName("view_other")
    viewMain:setVisible(false)
    viewOther:setVisible(true)
    local imgHand = viewOther:findChildByName("img_hand")
    imgHand:removeProp(2)
    local animRepeat = imgHand:addPropScale(2, kAnimRepeat, 1500, 0, 1, 1, 1, 1, kCenterDrawing)
    animRepeat:setEvent(nil, function()
        imgHand:removeProp(3)
        imgHand:setVisible(true)
        local moveAnim = imgHand:addPropTranslateWithEasing(3, kAnimNormal, 1000, 0, "easeInOutSine", "easeInOutSine", 0, -200, 0, 0)
        moveAnim:setEvent(nil, function()
            imgHand:setVisible(false)
            imgHand:removeProp(3)
        end)
    end)
    viewOther:findChildByName("text_tip"):setText(STR_BEGINNER_TIPS2)
    self.beginerHelp = beginerHelp
    self:addChild(beginerHelp)
end

function GameTab:dtor()

end

return GameTab