require(ViewPath.."marqueeLayout")
local Marquee = class(Node)

local TIMERID = 1001
local MSG_USER = 1
local MSG_SYS = 2

function Marquee:ctor()
	self.showMsgNum = MyUserData:getShowMsgNum() or 0
	self.lastUserMsgTime = MyUserData:getLastUserMsgTime() or 0
	self.lastSysMsgTime = MyUserData:getLastSysMsgTime() or 0
	self.timerStop = true
	self.msgBuffer = {}

	self.bg = new(BaseLayer, marqueeLayout)
	self.bg:setAlign(kALignCenter)
	self:addChild(self.bg)
	self.bg:findChildByName("img_bg"):setVisible(false)

	EventDispatcher.getInstance():register(HttpModule.s_event, self, self.onPHPRequestsCallBack);
	
end

function Marquee:dtor()
	MyUserData:setShowMsgNum(self.showMsgNum)
	MyUserData:setLastUserMsgTime(self.lastUserMsgTime)
	MyUserData:setLastSysMsgTime(self.lastSysMsgTime)

	EventDispatcher.getInstance():unregister(HttpModule.s_event, self, self.onPHPRequestsCallBack)
end

--加载小喇叭后不直接启动，等动画播放完毕后再添加系统消息请求
function Marquee:getSysMsg()
--4秒一个循环，一直到没有紧急公告和签到弹窗再开始启动小喇叭
    if self:isSysTime() then
        self.bg:removeProp(1001)
        local timer = self.bg:addPropTranslate(1001, kAnimNormal, 5000, 0, 0, 0, 0, 0)
        timer:setEvent(nil, function()
            self.bg:removeProp(1001)
            if WindowManager:containsWindowByTag(WindowTag.UrgentNoticePopu) or WindowManager:containsWindowByTag(WindowTag.AttendencePopu) then
                self:getSysMsg()
            elseif self:isSysTime() then
                HttpModule.getInstance():execute(HttpModule.s_cmds.GET_MARQUEE_NOTICES, {}, false, false)
            end
            
        end)
	end
end

function Marquee:startMarquee()
	--对外提供的接口，收到推送消息后都会调用此函数，
	--当前循环播放的计时器已经停止，判断消息是否强制播放，是否处于可播放期
	if self.timerStop then
		local message = self.msgBuffer[1]
		--队列存在消息才需要显示
		if message then
			--已经过了冷却时间，要么消息强制播放，或者是系统消息，方才启动小喇叭播放
			if self:isUserTime() or message.forceShow or message.type == MSG_SYS then 
				self:startShowAnim()
			end
		end
	else
		--小喇叭已启动，静候下一次循环播放此消息
	end
end

function Marquee:stopMarquee()
	self.bg:removeProp(1)
	self.timerStop = true
	self.msgBuffer = {}
    self.bg:removeProp(101)
	local disappearAnim = self.bg:addPropTranslateWithEasing(101, kAnimNormal, 500, 0, "easeOutSine", "easeOutSine", 0, 0, 0, -300)
	disappearAnim:setEvent(nil, function()
		self.bg:removeProp(101)
		self.bg:findChildByName("img_bg"):setVisible(false)
	end)
end

function Marquee:startShowAnim()
	--已经过了冷却时间，消息数量清0
	if (os.time() - self.lastUserMsgTime >= 600) then
		self.showMsgNum = 0
	end
	self.bg:findChildByName("img_bg"):setVisible(true)

	self.bg:removeProp(1)
	self.timerStop = false
    
    --小喇叭背景从屏幕上滑下来动画
    self.bg:removeProp(101)
    local appearAnim = self.bg:addPropTranslateWithEasing(101, kAnimNormal, 500, 0, "easeOutSine", "easeOutSine", 0, 0, -300, 300)
    appearAnim:setEvent(nil, function()
        self.bg:removeProp(101)
        --启动定时器，4.5秒显示一条小喇叭消息
        self:showContent()
	    local timerAnim = self.bg:addPropTranslate(1, kAnimRepeat, 4500, 0, 0, 0, 0, 0)
	    timerAnim:setEvent(nil, function()
		    self:showContent()
	    end)
    end)
	
end

function Marquee:showContent()
	local message = self.msgBuffer[1]
	if not message then
		--消息已经播放完
		self:stopMarquee()
		return
	end
	--强制播放，用户时间，或者系统消息，都直接播放，不考虑冷却时间
	if message.forceShow or self:isUserTime() or message.type == MSG_SYS then
		table.remove(self.msgBuffer, 1)
		local textContent = self.bg:findChildByName("text_content")
		textContent:setVisible(true)
		textContent:setText(STR_SYS_MESSAGE..": "..message.msg)
		self.bg:removeProp(2)
        --每条消息显示3秒钟
		local textAnim = self.bg:addPropTranslate(2, kAnimNormal, 3000, 0, 0, 0, 0, 0)
		textAnim:setEvent(nil, function()
			textContent:setVisible(false)
		end)
		--用户消息要刷新冷却时间和播放数量
		if message.type == MSG_USER then
			self.showMsgNum = self.showMsgNum + 1
			self.lastUserMsgTime = os.time()
        --刷新最后一条系统消息时间，用于计算一小时的冷却时间
		elseif message.type == MSG_SYS then
			self.lastSysMsgTime = os.time()
		end
	else
		--当前消息不予播放，用户消息处于冷却期间，并且不强制播放，此时停止小喇叭
		self:stopMarquee()
	end
end

function Marquee:isUserTime()
	--已经播放过10条消息，并且距离上一条不到10分钟，处于冷却时间，
	if self.showMsgNum > 10 and (os.time() - self.lastUserMsgTime < 600) then
		return false
	end
	return true
end

--过了一小时的系统消息冷却时间
function Marquee:isSysTime()
    if os.time() - self.lastSysMsgTime > 3600 then
		return true
	end
    return false
end

function Marquee:addUserMsg(str, forceShow)
	--强制插播的消息，直接放入消息队列首位
	if forceShow then
		table.insert(self.msgBuffer, 1, {msg = str, forceShow = true, type = MSG_USER})
	elseif self:isUserTime() then
		--过了冷却时间，放入消息队列
		table.insert(self.msgBuffer, {msg = str, type = MSG_USER})
	end
end

--插入消息队列
function Marquee:addSysMsg(str)
    table.insert(self.msgBuffer, {msg = str, type = MSG_SYS})
end

function Marquee:onPHPRequestsCallBack(command, ...)
	if self.s_severCmdEventFuncMap[command] then
		self.s_severCmdEventFuncMap[command](self, ...)
	end
end

function Marquee:onGetHallUserNotices(isSuccess, data)
    --未到冷却时间，不处理
    if not self:isSysTime() then
        return
    end
	if app:checkResponseOk(isSuccess, data) then
        --收到请求的数据，一个小时内不会再有新的跑马灯系统公告请求
        self.lastSysMsgTime = os.time()
		local messages = data.data;
		for i = 1, #messages do
			--小喇叭公告，上跑马灯
			if messages[i].getway and tonumber(messages[i].getway) == 2 then
				self:addSysMsg(messages[i].content)
		    end
		end
		self:startMarquee()
	end
end

Marquee.s_severCmdEventFuncMap = {
	[HttpModule.s_cmds.GET_MARQUEE_NOTICES] = Marquee.onGetHallUserNotices,
}

return Marquee