require("mjSocket/base/socketTip")
require("gameBase/gameSocket")
require("mjSocket/base/socketConfig")

local CommonProcesser 		= require("mjSocket.common.commonProcesser")
local CommonPhpProcesser	= require("mjSocket.common.commonPhpProcesser")
local CommonReader 			= require("mjSocket.common.commonReader")
local CommonWriter 			= require("mjSocket.common.commonWriter")

--[[
	主要用来管理 套接字的状态
	分发命令字
]]
local SocketMgr = class(GameSocket);
local printInfo, printError = overridePrint("SocketMgr")

-------------------------------------------------------------------------------
-- private
-------------------------------------------------------------------------------
SocketMgr.ctor = function(self, sockName, sockHeader, netEndian)
	self:initSocketTools();
	self:setHeartBeatCmd(Command.HeatBeatReq);

    self.m_maxReconnectTime = 10;
    self.m_tryReconnMaxTime = 60;--尝试重连最大次数
    self.m_maxReconnTime = 5; 		 --最大重连次数
    self.m_reconnTime = self.m_tryReconnMaxTime	--当前重连次数
    self.reconnectTimer = nil       --重连计时器，一旦开始重连就启用这个计时器，8秒后未能连上就直接重新构造socket。
    --如果断线，点击会触发重连
    EventDispatcher.getInstance():register(Event.RawTouch, self, function ( self )
    	-- body
    	self:tryReconnect() 
    end);
    

end

SocketMgr.dtor = function(self)
	self:removeCommonSocketReader(self.m_reader)
	self:removeCommonSocketWriter(self.m_writer)
	self:removeCommonSocketProcesser(self.m_processer)
	self:removeCommonSocketProcesser(self.m_phpProcesser)

	delete(self.m_reader);
    self.m_reader = nil;
	delete(self.m_writer);
    self.m_writer = nil;
	delete(self.m_processer);
    self.m_processer = nil;
    delete(self.m_phpProcesser);
    self.m_phpProcesser = nil;
end

SocketMgr.initSocketTools = function(self)
	self.m_reader = new(CommonReader)
	self.m_writer = new(CommonWriter)
	self.m_processer = new(CommonProcesser)
	self.m_phpProcesser = new(CommonPhpProcesser)

	printInfo("添加通用socket工具")
    self:addCommonSocketReader(self.m_reader);
    self:addCommonSocketWriter(self.m_writer);
    self:addCommonSocketProcesser(self.m_processer);
    self:addCommonSocketProcesser(self.m_phpProcesser);
end

SocketMgr.isSocketOpening = function(self)
	return self.m_isSocketOpening;
end

SocketMgr.createSocket = function(self, sockName, sockHeader, netEndian)
	return new(Socket, sockName, sockHeader, netEndian)
end

SocketMgr.getSocket = function(self)
	return self.m_socket;
end	

SocketMgr.openSocket = function(self)
	--open socket
	local addrList = MyUserData:getAddrList();
	for i = 1, #addrList do
		local addr = addrList[i];
		if addr and addr.ip and addr.port then
			printInfo("ip = %s, port = %s", addr.ip, addr.port)
			if ToolKit.isValidString(addr.ip) and tonumber(addr.port) then
				self.m_isSocketOpening = true;
				GameSocket.openSocket(self, addr.ip, addr.port);
				break;
			end
		end
	end
end

SocketMgr.isSocketClosed = function(self)
	return not self:isSocketOpen() and not self:isSocketOpening()
end
--关闭socket
SocketMgr.closeSocketSync = function(self)
	GameSocket.closeSocketSync(self);
	self.m_isSocketOpening = false;
end

SocketMgr.sendMsg = function(self, cmd, info, anim)
	--send
	if self:isSocketOpen() then
		GameSocket.sendMsg(self, cmd, info)
		return ;
	end
	app:onSocketSendError()	
end

SocketMgr.writeBegin = function(self, socket, cmd)
	return socket:writeBegin3(cmd, kProtocalVersion, 
							kProtocalSubversion, kDeviceTypeANDROID);
	
end

SocketMgr.writePacket = function(self, socket, packetId, cmd, info)
	-- php cmd
	return GameSocket.writePacket(self,socket, packetId, cmd, info);
end

SocketMgr.onTimeout = function(self)
	printInfo("Socket Status onTimeout")
	GameSocket.onTimeout(self);
	app:onSocketTimeOut();
	EventDispatcher.getInstance():dispatch(Event.ConnectTimeout);
	--重连
	self:tryReconnect()
end 

-------------------------------------------------------------------------------
-- private
-- Method: onSocketConnected
-- Action: 当程序打开Socket的时候，如果连接成功会触发该回调函数，此时请求登
-- 陆服务器
SocketMgr.onSocketConnected = function(self)
	printInfo("Socket Status onSocketConnected")
    self:deleteReconnectTimer()
	GameSocket.onSocketConnected(self);
	self.m_isSocketOpening = false;
	app:onSocketConnected()
end

SocketMgr.onSocketReconnecting = function(self)
	printInfo("Socket Status onSocketReconnecting")
    self:deleteReconnectTimer()
	GameSocket.onSocketReconnecting(self);
	self.m_isSocketOpening = true;
end

SocketMgr.onSocketConnectFailed = function(self)
	printInfo("Socket Status onSocketConnectFailed")
    self:deleteReconnectTimer()
    GameSocket.onSocketConnectFailed(self)
    self.m_isSocketOpening = false;
end

SocketMgr.onSocketClosed = function(self)
	printInfo("Socket Status onSocketClosed %s", "大厅连接断开")
    self:deleteReconnectTimer()
    GameSocket.onSocketClosed(self)
    self.m_isSocketOpening = false;
end

SocketMgr.tryReconnect = function(self)
    if System.getPlatform() ~= kPlatformAndroid and not self:isSocketOpen() and not self.reconnectTimer then
        printInfo("zyh create reconnect timer")
        --3秒执行一次，共35次重试，只要socket有任何响应就停止，
        self.repeatCount = 5
        self.reconnectTimer = new(AnimInt,kAnimRepeat,0,1, 3000,0);
        self.reconnectTimer:setEvent(nil, function()
            if self.repeatCount > 0 then
                printInfo("zyh close and reopen socket")
                self:closeSocketSync()
                self:openSocket()
                self.repeatCount = self.repeatCount - 1
            else
                self:deleteReconnectTimer()
            end
        end)
    end
	--打开socket
	if self:isSocketClosed() then
		self:closeSocketSync()
		self:openSocket()
	end
end

SocketMgr.deleteReconnectTimer = function(self)
    printInfo("zyh deleteReconnectTimer "..tostring(self.reconnectTimer))
    if self.reconnectTimer then
        self.repeatCount = 0
        delete(self.reconnectTimer)
        self.reconnectTimer = nil
    end
end

return SocketMgr