-- 各种子平台的基类，子平台需继承该类，覆盖有区别的方法即可
local BasePlatform = class();
local printInfo, printError = overridePrint("BasePlatform")

function BasePlatform:ctor()
	self._name = "BasePlatform"
	self._loginType = UserType.Visitor  -- 默认登录方式
end

function BasePlatform:getName()
	return self._name
end
-- 登录
function BasePlatform:requestLoginPhp(mergeParams)
	--登录PHP
	require('utils.base64')
    local sesskey = UserType.Facebook == MyUserData:getUserType() and GameSetting:getSesskey() or ""
	local param_post = {
							mac 	 		= PhpManager:getMac(),
							sig_sitemid 	= base64.encode(string.format(PhpManager:getDevice_id()..'_gamehallboyaa')),
							machineType 	= PhpManager:getModelName(),
							net 			= PhpManager:getNet(),
							simOperatorName = PhpManager:getNet(),
    						apkVer 	 		= PhpManager:getVersionName(),
    						pixel  			= PhpManager:getRat(),
    						imei	 		= PhpManager:getDevice_id(),
                            sesskey         = sesskey,
						};
    
	table.merge(param_post, mergeParams or {});
    local posStr = NativeEvent.getInstance():getLocation()
    if posStr then
        local pos = json.decode(posStr)
        if pos then
            param_post.latitude = pos.latitude
            param_post.longitude = pos.longitude
        end
    end
    --
	HttpModule.getInstance():execute(HttpModule.s_cmds.LOGIN_PHP, param_post, false, true);
end

-- 手机登录逻辑较多 独立出来
function BasePlatform:requestPhoneLogin()
	local loginData = GameConfig:getLastLoginData()
	if loginData then
		local param = {
	    	['phoneno'] 	= loginData.username,
	    	['pwd'] 		= loginData.password,
	    	['verifycode'] 	= loginData.verifycode,
	    	['act'] 		= loginData.act,
	    };
	    self:requestLoginPhp(param)
	else
		local loginData = ToolKit.getDict("TelLoginPopu", {'remebered', 'username','password'});
		if loginData.username ~= "" and loginData.password ~= "" then
			local param = {
		    	['phoneno'] 	= loginData.username,
		    	['pwd'] 		= loginData.password,
		    };
		    self:requestLoginPhp(param)
		else
			self:requestLoginPhp()
		end
	end
end

function BasePlatform:startSceneInit()

	GameConfig      = GameConfig 		or new(require("data.gameConfig"))
	GameConfig:load()

	MyUserData 		= MyUserData 		or setProxy(new(require("data.userData")))
	MyUpdateData 	= MyUpdateData 		or 	new(require("data.updateData"));

	MyNoticeData 	    = MyNoticeData 		or 	new(require("data.noticeData"));
	MyPayData 		    = MyPayData 		or 	new(require("data.shopData"));
	
    MySignAwardData     = MySignAwardData   or  new(require("data.signAwardData"));
	MyBaseInfoData      = MyBaseInfoData  or  setProxy(new(require("data.baseInfoData")))
    
	--初始化全局socket
	GameSocketMgr = GameSocketMgr or new(require("mjSocket.base.socketMgr"), kGameSocket, PROTOCOL_TYPE_QE, 1)
	
	LoginMethod   = LoginMethod   or new(require("common.loginMethod"))

	WindowManager 	= WindowManager 	or new(require("manager.windowManager"))

	HallConfig 	  	= HallConfig		or new(require("data.hallConfig"))
	HallConfig:load()

	G_RoomCfg 		= G_RoomCfg or setProxy(new(require("room.utils.roomConfig")))
	
	GameSetting		= GameSetting 		or setProxy(new(require("data.gameSetting")))
	GameSetting:load()

   	kMusicPlayer:setVolume(GameSetting:getMusicVolume())
   	kEffectPlayer:setVolume(GameSetting:getSoundVolume())

	--本地更新
	MyUpdate 	= MyUpdate 	or 	new(require("update.update"));
    MyUnzip     = MyUnzip or new(require("update.zip"))

	--支付方式
	MyPayMode 	= MyPayMode or nil;
	MyPay 		= MyPay or new(require('pay.pay'))

	--房间配置
	MyRoomConfig = MyRoomConfig or new(require('data.roomConfig')) 

    --全局开关控制列表
    MySwitchData        = MySwitchData or setProxy(new(require("data.allSwitchData")))

	StateChange.changeState(States.Lobby)
end

function BasePlatform:requestChargeList(isLogin)
	-- 注意先后 先初始化配置 再拉取商品列表
	UnitePay.getInstance():requestChargeConfig(isLogin)
	UnitePay.getInstance():requestChargeList(isLogin)
end

-- isLuoMaFirst 为是否优先选裸码  
-- 由于联运版本的存在 所以默认 [[不开启]]
-- 由主版本覆盖该方法 让isLuoMaFirst生效 
function BasePlatform:selectPayWay(goodsInfo, isLuoMaFirst, sceneChargeData)
	local bundleData = {
		goodsInfo 		= goodsInfo,
		sceneChargeData = sceneChargeData
	}
	UnitePay.getInstance():autoSelectPay(bundleData, isLuoMaFirst)
end

function BasePlatform:autoLoginAdapter(data)
	writeTabToLog({xx = debug.traceback()},"BasePlatform:autoLoginAdapter","ddd.lua")
	MyUserData:setUserType(self._loginType)
	NativeEvent.getInstance():login(data)
end

-- 走第三方的登录方式都会回调到这里来 在进行数据加工 登录php
-- 默认直接登录 如需要修改 则在子类中覆盖
function BasePlatform:loginCallbackAdapter(loginType, data)
	self:requestLoginPhp(data)
end

-- 登录适配
function BasePlatform:loginPopuAdapter(loginViews)
end

-- 登录按钮适配
function BasePlatform:loginBtnClickAdapter(userType)

end

function BasePlatform:addPayTypeAdapter()
end

return BasePlatform