require(ViewPath.."cashRecordItem")
require(ViewPath.."cashTaskItem")
require(ViewPath.."cashAwardTypeItem")
require(ViewPath.."cashAwardListItem")

local CashExchangePopu = class(require("popu.gameWindow"))
new(Image, "cash.png")
require("pintu.cash")

function CashExchangePopu:ctor()
    self.mTask = nil
    self.mAward = nil
    self.mRecord = nil
    self.mBtnAwardType = {}
    self.mSvAwardList = {}
    self.isShopTime = true
    self.successTitle = ""
    self.curBuyGoods = nil
    self.curGoodsData = nil
end

function CashExchangePopu:dtor()
    MyUserData:setCashGoodsList(self.mAward)
    MyUserData:setCashTaskList(self.mTask)
    MyUserData:setHasFriendReward(self.mTask and self.mTask.finished == self.mTask.aim or false)
end

function CashExchangePopu:initView(data)
    self.mData = {}
    self:findChildByName("btn_close"):setOnClick(self, function(self)
        self:dismiss()
    end)
    local viewExchange = self:findChildByName("view_exchange")
    local btnRecord = viewExchange:findChildByName("btn_record")
    local textRecord = new(Text, STR_CASH_BUY_RECORD, 0, 0, kAlignCenter, kFontTextUnderLine, 24, 0xA5, 0x4F, 0x15)    
    textRecord:setAlign(kAlignCenter)
    btnRecord:addChild(textRecord)
    btnRecord:setOnClick(self, function(self)
        if not self.mRecord then
            self.mRecord = {}
            self:execHttpCmd(HttpModule.s_cmds.GET_CASH_EXCHANGE_RECORD, {start = 0, offset = 30}, false, false)
        end
        self:findChildByName("view_shop"):setVisible(false)
        self:findChildByName("view_record"):setVisible(true)
    end)
    local textExchange = new(Text, STR_POINT_REWARD, 0, 0, kAlignCenter, kFontTextUnderLine, 24, 0xA5, 0x4F, 0x15)
    textExchange:setAlign(kAlignCenter)
    local btnExchange = viewExchange:findChildByName("btn_exchange")
    btnExchange:addChild(textExchange)
    btnExchange:setOnClick(self, function(self)
        self:findChildByName("view_shop"):setVisible(true)
        self:findChildByName("view_record"):setVisible(false)
    end)
    
    --点击切换购买页面和任务页面
    --local btnName = {"btn_exchange", "btn_cashTask"}
    --local btnTitle = {STR_POINT_REWARD, STR_GET_POINT_BY_TASK}
    local btnView = {self:findChildByName("view_exchangeBtn"), self:findChildByName("view_cashTaskBtn")}
    local viewName = {self:findChildByName("view_exchange"), self:findChildByName("view_cashTask")}
    local dataName = {"view_exchange", "view_cashTask"}
    local httpCmd = {HttpModule.s_cmds.GET_CASH_AWARDS, HttpModule.s_cmds.GET_CASH_TASK_FRIEND}
    local viewLoading = {self:findChildByName("view_shopLoading")}
    local needUpdate = {true, true}

    for i = 1, #btnView do
        local buttons = btnView[i]
        buttons:findChildByName("btn_unselect"):setOnClick(self, function(self)
            --点击按钮后，切换成被选中的按钮，隐藏不被选中的按钮
            for j = 1, #btnView do
                local curButtons = btnView[j]
                curButtons:findChildByName("btn_select"):setVisible(i == j)
                curButtons:findChildByName("btn_unselect"):setVisible(i ~= j)
                viewName[j]:setVisible(false)
            end
            viewName[i]:setVisible(true)
    --每次点击页面后只重新拉取一次数据
            if needUpdate[i] then
                self:execHttpCmd(httpCmd[i], {mid = MyUserData:getId()}, false, not self.mData[dataName[i]], viewLoading[i])
                needUpdate[i] = false
            end
        end)
    end
    --设置现金币的数量
    local viewCashPoint = self:findChildByName("view_cashPoint")
    local defaultSizeAndColor = {30, 0xF7, 0xAA, 0x82}
    local specialSizeAndColor = {32, 0xF3, 0xBF, 0x1C}
    ToolKit.formatRichText(viewCashPoint, STR_CASH_POINT, 0, defaultSizeAndColor, specialSizeAndColor)
    --self:findChildByName("text_goals"):setText(string.format(STR_CASH_POINT, 0))
    UIEx.bind(self, MyUserData, "cashPoint", function(value)
        viewCashPoint:removeAllChildren()
        ToolKit.formatRichText(viewCashPoint, STR_CASH_POINT, value, defaultSizeAndColor, specialSizeAndColor)
        --self:findChildByName("text_goals"):setText(string.format(STR_CASH_POINT, MyUserData:getCashPoint()))
    end)
    UIEx.bind(self, MyUserData, "hasFriendReward", function(value)
        local viewTip = self:findChildByName("view_friendReward")
        viewTip:removeAllChildren()
        if value then
            local tip = new(Image, 'common/tips.png')
            viewTip:addChild(tip)
        end
    end)
    HttpModule.getInstance():execute(HttpModule.s_cmds.GET_CASH_TOTAL, {}, true, false)

end

function CashExchangePopu:onShowEnd()
    if MyUserData:getCashGoodsList() then
        self.mAward = MyUserData:getCashGoodsList()
        self:initAwards()
        self.mData["view_exchange"] = self.mAward
    end
    if MyUserData:getCashTaskList() then
        self.mTask = MyUserData:getCashTaskList()
        self:initTaskList()
        self.mData["view_cashTask"] = self.mTask
    end
    --默认点击第一个按钮
    local btn = self:findChildByName("view_exchangeBtn"):findChildByName("btn_unselect")
    btn.m_eventCallback.func(btn.m_eventCallback.obj)
    self.super.onShowEnd(self)
end

function CashExchangePopu:initAwards()
    --self.btnAwardTypeLast = nil
    --初始化所有商品
    local svTypeList = self:findChildByName("sv_awardType")
    self:findChildByName("view_goodsList"):removeAllChildren()
    self.mBtnAwardType = {}
    self.mSvAwardList = {}
    svTypeList:removeAllChildren()
    svTypeList:setDirection(kVertical)
    for i = 1, #self.mAward do
        local typName = ""
        local item = SceneLoader.load(cashAwardTypeItem)
        item:findChildByName("text_typeName"):setText(self.mAward[i].name)
        local w, h = item:getSize()
        item:setPos(0, (i - 1) * h)
        item:setAlign(kAlignTopLeft)
        --item:findChildByName("btn_awardType"):enableAnim(false)
        svTypeList:addChild(item)
        table.insert(self.mBtnAwardType, item)
        local goodsList = self:initGoodsList(self.mAward[i])
        table.insert(self.mSvAwardList, goodsList)
    end
    svTypeList:setSize(svTypeList:getSize())

    --local maskBg = self:findChildByName("view_awardType"):findChildByName("img_chooseMask")
    --maskBg:addPropTransparency(1, kAnimNormal, 0, 0, 0.5, 0.5)
    --maskBg:setVisible(false)
    --初始化商品的类别
    local startTime = 0
    local lastPos = 0
    for i = 1, #self.mBtnAwardType do
        self.mBtnAwardType[i]:findChildByName("btn_awardType"):setOnClick(self, function(self)
            if self.btnAwardTypeLast then
                self.mBtnAwardType[self.btnAwardTypeLast]:findChildByName("img_unselected"):setVisible(true)
                self.mBtnAwardType[self.btnAwardTypeLast]:findChildByName("img_selected"):setVisible(false)
                self.mSvAwardList[self.btnAwardTypeLast]:setVisible(false)          
                local typeBtn = self.mBtnAwardType[self.btnAwardTypeLast]
            end
            local item = self.mBtnAwardType[i]:findChildByName("text_typeName")
            local typeName = item:getText()
            item:setText(typeName, 0, 0, 0xff, 0xff, 0xff)
            self.mBtnAwardType[i]:findChildByName("img_unselected"):setVisible(false)
            self.mBtnAwardType[i]:findChildByName("img_selected"):setVisible(true)
            self.mSvAwardList[i]:setVisible(true)
            self.btnAwardTypeLast = i
        end)
    end
    
    local index = self.btnAwardTypeLast and self.btnAwardTypeLast or 1
    if self.mBtnAwardType[index] then
        local btn = self.mBtnAwardType[index]:findChildByName("btn_awardType")
        btn.m_eventCallback.func(btn.m_eventCallback.obj)
        --maskBg:setPos(btn:getPos())
        self.mSvAwardList[index]:setVisible(true)
        self.btnAwardTypeLast = index
    end
end

function CashExchangePopu:initGoodsList(data)
    --加载某个类别下的所有商品
    local viewList = self:findChildByName("view_goodsList")
    local svList = new(ScrollView)
    viewList:addChild(svList)
    svList:setFillParent(true, true)
    local width, height = viewList:getSize()
    svList:setSize(width, height)
    svList:setPos(viewList:getPos())
    svList:setAlign(kAlignCenter)
    
    local curRowNum = 0
    local curColNum = 0
    local wInterval = width / 3
    for i = 1, #data do
        local awardItem = self:initGoodsItem(data[i], data.name)
        local wItem, hItem = awardItem:getSize()
        awardItem:setPos(15 + curColNum * wInterval, curRowNum * (hItem + 10))
        curColNum = curColNum + 1
        if curColNum == 3 then
            curColNum = 0
            curRowNum = curRowNum + 1
        end
        svList:addChild(awardItem)
    end
    svList:setVisible(false)
    svList:setSize(svList:getSize())
    return svList
end

function CashExchangePopu:initGoodsItem(data, goodsType)
    --初始化具体的某个商品
    local item = SceneLoader.load(cashAwardListItem)
    item:findChildByName("text_awardName"):setText(data:getName())
--keep the width/heigh of the original picture
    local imgAward = item:findChildByName("img_award")
    imgAward:setFile(data:getImgName())
    local wGoods = item:findChildByName("view_awardImg"):getSize()
    imgAward:setSize(wGoods, imgAward.m_res.m_height / imgAward.m_res.m_width * wGoods)
    UIEx.bind(item, data, "imgName", function(value)
        imgAward:setFile(data:getImgName())
        local wGoods = item:findChildByName("view_awardImg"):getSize()
        imgAward:setSize(wGoods, imgAward.m_res.m_height / imgAward.m_res.m_width * wGoods)
    end)
    --商品超过50
    local remainStr = ""
    if data.remain > 50 or data.remain == -1 then
        remainStr = STR_CASH_REMAIN_TOO_MUCH
    elseif data.remain > 0 then
        remainStr = string.format(STR_GOODS_REMAIN, data.remain)
    elseif data.multiUse == 1 then
        remainStr = STR_CASH_REMAIN_IS_SHOPPING
    else
        remainStr = STR_CASH_REMAIN_NO_REMAIN
    end
    --item:findChildByName("text_count"):setText(string.format(STR_GOODS_REMAIN, data:getRemain()))
    item:findChildByName("text_count"):setText(remainStr)

    item:findChildByName("text_cost"):setText(string.format(STR_GOODS_PRICE, data:getPrice() or 0))
    item:findChildByName("btn_bg"):setOnClick(self, function(self)
    --积分不足判断
        if data.price > MyUserData:getCashPoint() then
            WindowManager:showWindow(WindowTag.LobbyConfirmPopu, {
                content = STR_EXCHANGE_FAIL_LACK_OF_CASH,
                confirm = STR_EXIT_GAME_CONFIRM,
                cancel = STR_GET_POINT_BY_TASK,
                cancelFunc = function()
                    local btn = self:findChildByName("view_cashTaskBtn"):findChildByName("btn_unselect")
                    btn.m_eventCallback.func(btn.m_eventCallback.obj)
                end, 
            }, WindowStyle.POPUP)
    --余量不足判断
        elseif data.remain == 0 then
            local content = STR_EXCHANGE_FAIL_NO_REMAIN
            --如果会补货，那么提示商品在补货中，不补货提示卖完了
            if data.multiUse == 1 then
                content = STR_EXCHANGE_FAIL_SHOPPING
            end
            WindowManager:showWindow(WindowTag.LobbyConfirmPopu, {
                content = content,
                confirm = STR_EXIT_GAME_CONFIRM,
            }, WindowStyle.POPUP)
        else
    --isShopTime确保上个商品结算期间不能购买第二个
            if self.isShopTime then
                WindowManager:showWindow(WindowTag.LobbyConfirmPopu, {
                content = string.format(STR_BUY_CONFIRM, data.price, data.name),
                cancel = STR_EXIT_GAME_CANCEL,
                confirm = STR_EXIT_GAME_CONFIRM,
                confirmFunc = function()
                    --由于弹窗消失需要时间，所以检查锁，防止连续点击后出现连续购买的情况
                    if self.isShopTime then
                        self:execHttpCmd(HttpModule.s_cmds.EXCHANGE_AWARD_BY_ID, {rid = data:getId(), count = buyNum}, false, true)
                    end
                    self.successTitle = string.format(STR_CONTENT_EXCHANGE_SUCCESS, data.name)
                    self.curBuyGoods = item
                    self.curGoodsData = data
                    self.isShopTime = false
                end
                },WindowStyle.POPUP)    

            end
        end
    end)
    return item
end

function CashExchangePopu:initTaskList()
    --初始化任务列表，暂时只有好友邀请任务
    local svTask = self:findChildByName("sv_cashTask")
    svTask:removeAllChildren()
    svTask:setDirection(kVertical)
    local item = self:initFriendTaskItem(self.mTask)
    local w, h = item:getSize()
    item:setPos(0, 0)
    svTask:addChild(item)

    local attendenceTask = self:initAttendenceItem()
    attendenceTask:setPos(0, h)
    svTask:addChild(attendenceTask)
    svTask:setSize(svTask:getSize())

    svTask:setSize(svTask:getSize())
end

function CashExchangePopu:initFriendTaskItem(data)
    --好友任务初始化
    local item = SceneLoader.load(cashTaskItem)
    item:setName("inviteFriendTask")
    item:findChildByName("text_title"):setText(string.format(STR_SUCCES_INVITE_FRIEND_NUM, data.aim or 0))

    self:initFriendProgress(item, data.finished, data.aim)
    
    item:findChildByName("text_reward"):setText(tostring(data.reward or 0))
    return item
end

function CashExchangePopu:initAttendenceItem()
    local item = SceneLoader.load(cashTaskItem)
    item:setName("attendenceTask")
    item:findChildByName("text_title"):setText(STR_ATTENDENCE_TASK)
    item:findChildByName("img_task_bg"):setVisible(false)
    item:findChildByName("view_award"):setVisible(false)
    item:findChildByName("btn_getAward"):setVisible(false)
    return item
end

function CashExchangePopu:initFriendProgress(item, cur, aim)
    --邀请任务进度条
    cur = tonumber(cur or 0)
	total = tonumber(aim or 0)
    MyUserData:setHasFriendReward(cur == total)
	item:findChildByName("text_progress"):setText(string.format("%s/%s", cur, total));
	local progress = item:findChildByName("img_progress");
	local percent = total ~= 0 and cur/total or 0;
    local minWidth = progress.m_res.m_width
    --122是底下进度条总长度
	local width = (122 - minWidth) * percent;
	progress:setSize(minWidth + width, progress.m_res.m_height);
    if cur == total then    --好友任务已经完成，领奖
        --item:findChildByName("text_getAward"):setText(STR_GET_TASK_AWARD)
        item:findChildByName("img_textGetAward"):setVisible(true)
        item:findChildByName("img_toDoTask"):setVisible(false)
        item:findChildByName("btn_getAward"):setOnClick(self, function(self)
            HttpModule.getInstance():execute(HttpModule.s_cmds.GET_CASH_TASK_FRIEND_REWARD, {}, true, false)
        end)
    else
        --item:findChildByName("text_getAward"):setText(STR_DO_TASK)
        item:findChildByName("img_textGetAward"):setVisible(false)
        item:findChildByName("img_toDoTask"):setVisible(true)
        item:findChildByName("btn_getAward"):setOnClick(self, function(self)
            self:dismiss(true)
            WindowManager:showWindow(WindowTag.FbInvitePopu, {}, WindowStyle.POPUP)
        end)
    end
end

function CashExchangePopu:initRecord()
    --兑换记录
    if #self.mRecord == 0 then
        self:findChildByName("text_no_record"):setText(STR_EXCHANGE_NO_RECORD)
        return
    end
    local svRecord = self:findChildByName("sv_record")
    svRecord:removeAllChildren()
    svRecord:setDirection(kVertical)

    for i = 1, #self.mRecord do
        local item = SceneLoader.load(cashRecordItem)
        item:findChildByName("text_title"):setText(self.mRecord[i].title)
        item:findChildByName("text_time"):setText(self.mRecord[i].time or "")
        local w, h = item:getSize()
        item:setPos(0, (i - 1) * h)
        item:findChildByName("text_content"):setText(self.mRecord[i].content)
        item:findChildByName("btn_total"):setOnClick(self, function(self)
            local contentBg = item:findChildByName("img_bg2")
            local titleBg = item:findChildByName("img_bg")
            if contentBg:getVisible() then
                contentBg:setVisible(false)
                titleBg:setVisible(true)
                item:setSize(titleBg:getSize())
            else
                contentBg:setVisible(true)
                titleBg:setVisible(false)
                item:setSize(contentBg:getSize())
            end
            svRecord:reposition()
            item:findChildByName("btn_total"):setSize(item:getSize())
            svRecord:setSize(svRecord:getSize())
        end)

        if self.mRecord[i].content == nil or self.mRecord[i].content == "" then
            item:findChildByName("btn_total"):setEnable(false)
            item:findChildByName("img_down"):setVisible(false)
        end
        svRecord:addChild(item)
    end
    
    svRecord:setSize(svRecord:getSize())

end

function CashExchangePopu:onGetCashAwards(isSuccess, data)
    if app:checkResponseOk(isSuccess, data) then
        self.mAward = {}
        local allAward = data.data
        for k, v in pairs(allAward) do
            local tAwardList = {}
            tAwardList.name = k
            for i = 1, #v.data do
                local award = setProxy(new(require("data.cashAwardData")))
                award:init(v.data[i])
                table.insert(tAwardList, award)
            end
            table.sort(tAwardList, function(a, b)
                if a.price < b.price then return true end
            end)
            table.insert(self.mAward, tAwardList)
        end

        self.mData["view_exchange"] = self.mAward
        
        self:initAwards()
    end
    
end

function CashExchangePopu:onExchangeAwardById(isSuccess, data)
    if app:checkResponseOk(isSuccess, data) then
        
        if data.data.status == 1 then
            local str = self.successTitle.."\n"
            if data.data.category == 1 then     --兑换的商品是话费
                local card = data.data.card
                str = str..string.format(STR_PHONE_CARD_NUMBER, card.number or "").."\n"
                str = str..string.format(STR_PHONE_CARD_PASSWORD, card.secret or "").."\n"
                str = str..string.format(STR_PHONE_CARD_EXPIRE_TIME, os.date("%Y-%m-%d %H:%M:%S", card.expiry or 0)).."\n"
                str = str..STR_TAKE_CARE_OF_CARD
                WindowManager:showWindow(WindowTag.CashBuyConfirmPopu, {
                    title = STR_TITLE_EXCHANGE_SUCCESS,
                    content = str,
                    confirm = STR_EXIT_GAME_CONFIRM,
                }, WindowStyle.POPUP)
                MyUserData:setCashPoint(tonumber(data.data.diamond) or 0)
                self.curBuyGoods:findChildByName("text_count"):setText(string.format(STR_GOODS_REMAIN, tonumber(data.data.remain) or 0))
            elseif data.data.category == 2 then     --兑换的商品是金币
                local card = data.data.card
                WindowManager:showWindow(WindowTag.LobbyConfirmPopu,{
                    title = STR_TITLE_EXCHANGE_SUCCESS,
                    content = str,
                    confirm = STR_EXIT_GAME_CONFIRM,
                }, WindowStyle.POPUP)
                MyUserData:setCashPoint(tonumber(data.data.diamond) or 0)
                MyUserData:addMoney(tonumber(data.data.money) and tonumber(data.data.money) - MyUserData:getMoney() or 0, true)
                --self.curBuyGoods:findChildByName("text_count"):setText(string.format(STR_GOODS_REMAIN, data.data.remain or 0))
            end
            --MyUserData:setCashPoint(data.data.score or 0)
        elseif data.data.status == 2 then
            --积分不足
            WindowManager:showWindow(WindowTag.LobbyConfirmPopu, {
                content = STR_EXCHANGE_FAIL_LACK_OF_CASH,
                confirm = STR_EXIT_GAME_CONFIRM,
                cancel = STR_GET_POINT_BY_TASK,
                cancelFunc = function()
                    local btn = self:findChildByName("view_cashTaskBtn"):findChildByName("btn_unselect")
                    btn.m_eventCallback.func(btn.m_eventCallback.obj)
                end, 
            }, WindowStyle.POPUP)
        elseif data.data.status == 3 then
            --商品库存不足
            local content = STR_EXCHANGE_FAIL_NO_REMAIN
            --如果会补货，那么提示商品在补货中，不补货提示卖完了
            if self.curGoodsData.multiUse == 1 then
                content = STR_EXCHANGE_FAIL_SHOPPING
            end
            WindowManager:showWindow(WindowTag.LobbyConfirmPopu, {
                content = content,
                confirm = STR_EXIT_GAME_CONFIRM,
            }, WindowStyle.POPUP)
        elseif data.data.status == 4 then
            --达到今日兑换上限
            WindowManager:showWindow(WindowTag.LobbyConfirmPopu, {
                content = STR_EXCHANGE_FAIL_LIMIT_COUNT,
                confirm = STR_EXIT_GAME_CONFIRM,
            }, WindowStyle.POPUP)
        end
    end
    self.isShopTime = true
    self.successTitle = ""
end

function CashExchangePopu:onGetCashRecord(isSuccess, data)
    if app:checkResponseOk(isSuccess, data) then
        self.mRecord = {}
        local records = data.data
        for i = #records, 1, -1 do
            local gift = records[i].gift or {}
            if tonumber(gift.category or 0) == 1 then   -----这条记录是话费购买记录
                local record = {}
                record.id = data.id
                record.title = string.format(STR_CONTENT_EXCHANGE_SUCCESS, gift.name or "")
                local str = string.format(STR_PHONE_CARD_NUMBER, gift.number or "").."\n"
                str = str..string.format(STR_PHONE_CARD_PASSWORD, gift.secret or "").."\n"
                str = str..string.format(STR_PHONE_CARD_EXPIRE_TIME, gift.expiryTime or "")
                record.content = str
                record.time = records[i].extimeTime
                record.recordTime = tonumber(records[i].extime) or 0
                table.insert(self.mRecord, record)
            elseif tonumber(gift.category or 0) == 2 then       --兑换金币记录
                local record = {}
                record.id = data.id
                record.title = string.format(STR_CONTENT_EXCHANGE_SUCCESS, gift.name or "")
                record.content = ""
                record.time = records[i].extimeTime
                record.recordTime = tonumber(records[i].extime) or 0
                table.insert(self.mRecord, record)
            end
        end
        table.sort(self.mRecord, function(a, b)
            return a.recordTime > b.recordTime
        end)
        self:initRecord()
    else
        self:findChildByName("text_no_record"):setText(STR_EXCHANGE_NO_RECORD)
    end
    
end

function CashExchangePopu:onDelCashRecord(isSuccess, data)

end

function CashExchangePopu:onGetCashTaskFriend(isSuccess, data)
    if app:checkResponseOk(isSuccess, data) then
        self.mTask = {}
        --finished是当前已经完成的但是还没领奖的任务，比如已经邀请了9个人，有4个人领奖了，那么finished此时是5
        --aim是此次领奖的目标，如果有奖，那么这个aim是5，当我领到9个人的时候，aim会变成10，finished此时是9。
        self.mTask.finished = data.data.finished and tonumber(data.data.finished) or 0
        self.mTask.aim = data.data.aim and tonumber(data.data.aim) or 0
        self.mTask.reward = data.data.reward and tonumber(data.data.reward) or 0
        self.mData["view_cashTask"] = self.mTask
        self:initTaskList()
    end
    
end

function CashExchangePopu:onGetCashTaskFriendReward(isSuccess, data)
    --刷新积分和任务

    if app:checkResponseOk(isSuccess, data) then
        local schedule = data.data.schedule or {}
        local item = self:findChildByName("sv_cashTask"):findChildByName("inviteFriendTask")
        self:initFriendProgress(item, schedule.finished or 0, schedule.aim or 0)
        item:findChildByName("text_reward"):setText(tostring(schedule.reward or 0))
        item:findChildByName("text_title"):setText(string.format(STR_SUCCES_INVITE_FRIEND_NUM, schedule.aim))
    --刷新积分
        MyUserData:setCashPoint(data.data.diamond or 0)
        local addIcon = new(Image, cash_map["cash.icon_dallor.png"])
        addIcon:setPos(item:findChildByName("img_dollar"):getAbsolutePos())
        self:findChildByName("img_bg"):addChild(addIcon)
        local fromx, fromy = addIcon:getPos()
        local tox, toy = 100, 50
        local iconMove = addIcon:addPropTranslateWithEasing(3, kAnimNormal, 200, 0, "easeOutCirc", "easeOutCirc", 0, (tox-fromx), 0, (toy-fromy))
        iconMove:setEvent(nil, function()
            self:findChildByName("img_bg"):removeChild(addIcon, true)
        end)
        self.mTask.finished = schedule.finished and tonumber(schedule.finished) or 0
        self.mTask.aim = schedule.aim and tonumber(schedule.aim) or 0
        self.mTask.reward = schedule.reward and tonumber(schedule.reward) or 0
    end
end

function CashExchangePopu:onGetCashTotal(isSuccess, data)
    if app:checkResponseOk(isSuccess, data) then
        MyUserData:setCashPoint(data.data.diamond or 0)
        --self:findChildByName("text_goals"):setText(string.format(STR_CASH_POINT, MyUserData:getCashPoint()))
    end
end

CashExchangePopu.s_severCmdEventFuncMap = {
    [HttpModule.s_cmds.GET_CASH_AWARDS] = CashExchangePopu.onGetCashAwards,
    [HttpModule.s_cmds.EXCHANGE_AWARD_BY_ID] = CashExchangePopu.onExchangeAwardById,
    [HttpModule.s_cmds.GET_CASH_EXCHANGE_RECORD] = CashExchangePopu.onGetCashRecord,
    [HttpModule.s_cmds.DEL_CASH_EXCHANGE_RECORD] = CashExchangePopu.onDelCashRecord,
    [HttpModule.s_cmds.GET_CASH_TASK_FRIEND] = CashExchangePopu.onGetCashTaskFriend,
    [HttpModule.s_cmds.GET_CASH_TASK_FRIEND_REWARD] = CashExchangePopu.onGetCashTaskFriendReward,
    [HttpModule.s_cmds.GET_CASH_TOTAL] = CashExchangePopu.onGetCashTotal,
}

return CashExchangePopu