require("ui/ex/ScrollViewEx")
local HelpPopu = class(require("popu.gameWindow"))

local MJ_GUOBIAO    = 1;
local MJ_GUANGDONG  = 2;
local MJ_SHANGHAI   = 3;
local MJ_HEBEI      = 4;

local MJ_RULE    = 1;
local MJ_FAN     = 2;

function HelpPopu:ctor()
    GameSetting:setIsSecondScene(true)
end

function HelpPopu:initView(data)
    
    local backBtn = self:findChildByName("btn_back");
    backBtn:setOnClick(self, function ( self )
        -- body
        self:dismiss();
    end);

    backBtn:enableAnim(false);
    
    --标题按钮
    self.mTitleBtn = {};

    local nodeBtnName = {
            [MJ_GUOBIAO]    = "btn_guobiao",
            [MJ_GUANGDONG]  = "btn_guangdong",
            [MJ_SHANGHAI]   = "btn_shanghai",
            [MJ_HEBEI]      = "btn_hebei",
        }

    for i = MJ_GUOBIAO, MJ_HEBEI do
        self.mTitleBtn[i] = self:findChildByName(nodeBtnName[i]);
        self.mTitleBtn[i]:setDebugName("HelpPopu:"..nodeBtnName[i]);
        self.mTitleBtn[i]:setOnClick(self, function ( self )
            -- body
            self:onClickHelp(i);
        end);
        self.mTitleBtn[i]:enableAnim(false);
    end

    --初始化TAB
    local tabName = {
            [MJ_RULE]       = "btn_rule",
            [MJ_FAN]        = "btn_fan",
        }

    for tabIndex = MJ_RULE, MJ_FAN do
        local tab = self:findChildByName(tabName[tabIndex]);
        tab:setOnClick(self, function ( self )
            -- body
            self:onClickTab(self.mCurHelp, tabIndex, true);
        end);
        tab:enableAnim(false);
    end
    
   
    --初始化帮助文档
    self.mRuleData              = {};
    self.mFanData               = {};
    

    --初始化各种帮助列表
    self.mScrollView = {};

--    ScrollBar.setDefaultImage("rank/scroller.png");

    local viewContent        = self:findChildByName("view_scrollview"):findChildByName("view_content");
    local width, height = viewContent:getSize();

    for i = MJ_GUOBIAO, MJ_HEBEI do
        self.mScrollView[i] = {};
        self.mScrollView[i][MJ_RULE]  = new(ScrollViewEx, 0, 0, width, height, false);
        self.mScrollView[i][MJ_FAN]   = new(ScrollViewEx, 0, 0, width, height, false);
        viewContent:addChild(self.mScrollView[i][MJ_RULE]);
        viewContent:addChild(self.mScrollView[i][MJ_FAN]);

        self.mScrollView[i][MJ_RULE]:setScrollBarWidth(8);
        self.mScrollView[i][MJ_FAN]:setScrollBarWidth(8);
        self.mScrollView[i][MJ_RULE]:setVisible(false);
        self.mScrollView[i][MJ_FAN]:setVisible(false);

        self.mScrollView[i][MJ_RULE]:setScrollBarWidth(8);

        self.mScrollView[i][MJ_RULE]:setOnReachBottom(self, function ( self )
        -- body
            self:onClickTab(i, MJ_RULE);
        end);
        self.mScrollView[i][MJ_FAN]:setOnReachBottom(self, function ( self )
        -- body
            self:onClickTab(i, MJ_FAN);
        end);

    end

    --创建动画
    local rotationX, rotationY = 24 * System.getScreenWidth() / System.getLayoutWidth(), 3 * System.getScreenHeight() / System.getLayoutHeight();
    local decorateNode = self:findChildByName("img_decorate_r2");
    decorateNode:setLevel(2);
    self:findChildByName("img_decorate_r"):setLevel(3);
    decorateNode:addPropRotateWithEasing(1, kAnimLoop, 800, 0, 'easeInOutSine', -10, 20, kCenterXY, rotationX, rotationY)

    --切到相对应的帮助
    self.mCurHelp = nil;

    local lastType = data.gameType or GameType.GBMJ;

    if lastType == GameType.GBMJ then
        self:onClickHelp(MJ_GUOBIAO);
    elseif lastType == GameType.GDMJ then
        self:onClickHelp(MJ_GUANGDONG);
    elseif lastType == GameType.SHMJ then
        self:onClickHelp(MJ_SHANGHAI);
    elseif lastType == GameType.HBMJ then
        self:onClickHelp(MJ_HEBEI);
    end


end

--点击标题
function HelpPopu:onClickHelp(helpType)
    self:onClickTab(helpType, MJ_RULE, true);

    local unselectFile = {
                            [1] = 'common/scenc_title_left_bg1.png',
                            [2] = 'common/scenc_title_mid_bg1.png',
                            [3] = 'common/scenc_title_mid_bg1.png',
                            [4] = 'common/scenc_title_right_bg1.png',
                        }
    local selectFile = {
                            [1] = 'common/scenc_title_left_bg2.png',
                            [2] = 'common/scenc_title_mid_bg2.png',
                            [3] = 'common/scenc_title_mid_bg2.png',
                            [4] = 'common/scenc_title_right_bg2.png',
                        }

    for i = MJ_GUOBIAO, MJ_HEBEI do
        if i ~= helpType then
            self.mTitleBtn[i]:setFile(unselectFile[i]); 
            self.mTitleBtn[i]:findChildByName("text_title"):setColor(0xff, 0xff, 0xff);
        end
    end

    self.mTitleBtn[helpType]:setFile(selectFile[helpType]); 
    self.mTitleBtn[helpType]:findChildByName("text_title"):setColor(0xff, 0xe6, 0x0);
    
    self.mCurHelp = helpType;

    
    
end

--点击副标题
function HelpPopu:onClickTab(helpType, tabType, reset)
    local scView = self.mScrollView[helpType][tabType];
    local width, height = scView:getSize();

    

    if tabType == MJ_RULE then

        local node = self:loadRule(helpType, width, height);
        if node then
            local h = scView:getViewLength();
            node:setPos(0, h);
            scView:addChild(node);
        end
        
    else
        local node = self:loadFan(helpType, width, height);
        if node then
            local h = scView:getViewLength();
            node:setPos(0, h);
            scView:addChild(node);
        end
        
    end

    for k, v in pairs(self.mScrollView) do
        for kk, vv in pairs(v) do
            vv:setVisible(false);
        end
    end

    self.mScrollView[helpType][tabType]:setVisible(true);

    local file = {
                            [1] = 'help/selected.png',
                            [2] = 'help/unselected.png',
                  }

    local btnRule = self:findChildByName("btn_rule");
    local btnFan = self:findChildByName("btn_fan");
   
    if tabType == MJ_RULE then
        btnRule:setFile(file[1]);
        btnRule:findChildByName("text_title"):setColor(0xff, 0xe6, 0x0);
        btnFan:setFile(file[2]);
        btnFan:findChildByName("text_title"):setColor(0xff, 0xff, 0xff);
    else
        btnRule:setFile(file[2]);
        btnRule:findChildByName("text_title"):setColor(0xff, 0xff, 0xff);
        btnFan:setFile(file[1]);
        btnFan:findChildByName("text_title"):setColor(0xff, 0xe6, 0x0);
    end

    --归0
    if reset then
        self.mScrollView[helpType][tabType]:gotoTop();
    end

end


function HelpPopu:loadRule(helpType, width, height)
    -- body

    self:loadRuleData(helpType);

    local ruleData = self.mRuleData[helpType];

    local load = ruleData:getLoad() ;

    if load >= ruleData:count() then
        return ;
    end

    local perLoad = 10;


    local ruleNode = new(Node);

    local x, y = 0, 0;

    --创建内容
    local start = load + 1;
    for i = start, ruleData:count() do

        if i - start >= perLoad and y > height then
            break;
        end

        load = load + 1;

        local data = ruleData:get(i);
        local node = new(Node);

        local title = data:getTitle();

        if title and title ~= "" then
            local nodeTitle = new(Text, title ,0, 0, kAlignTopLeft, "", 32, 0xff, 0xe6, 0x0);
            nodeTitle:setPos(x, y);
            node:addChild(nodeTitle);
            local titleWidth, titleHeight = nodeTitle:getSize();
            y = y + titleHeight + 20;

        else
            local term = new(Text, data:getTerm(), 0, 0, kAlignTopLeft, "", 26, 0xff, 0xff, 0xff);
            node:addChild(term);

            local termWidth, termHeight = term:getSize();

            local description = new(TextView, data:getDescription(), width - termWidth, 0, kAlignTopLeft, "", 26, 0xff, 0xe6, 0);
            description:setPos(termWidth, 0);

            description:setSize(width - termWidth, description.m_res.m_height);

            node:addChild(description);

            local maxHeight = description.m_res.m_height;

            node:setSize(width, maxHeight);
            node:setPos(x, y);

            y = y + maxHeight + 20;
        end

        ruleNode:addChild(node);
        
    end

    ruleNode:setSize(width, y);

    ruleData:setLoad(load);

    return ruleNode;
end

function HelpPopu:loadFan(helpType, width, height)

    self:loadFanData(helpType);

    local fanData = self.mFanData[helpType];

    local load = fanData:getLoad();

    if load >= fanData:count() then
        return ;
    end

    local perLoad = 10;

    local fanNode = new(Node);

    local x, y = 0, 0;

    --创建内容

    start = load + 1;

    for i = start, fanData:count() do

        if i - start >= perLoad and y > height then
            break;
        end

        load = load + 1;


        local data = fanData:get(i);
        local node = new(Node);

        local childY        = 0;
        local totalHeight   = 0;

        local title = data:getTitle();

        if title and title ~= "" then
            local nodeTitle = new(Text, title ,0, 0, kAlignTopLeft, "", 32, 0xff, 0xe6, 0x0);
            node:addChild(nodeTitle);
            totalHeight = nodeTitle.m_res.m_height + 20;
            node:setSize(width, totalHeight);
            node:setPos(x, y);
            y = y + totalHeight;
        else

            local fan = data:getFan();

            --创建番型

            if fan and fan ~= "" then
                local nodeFan = new(Text, fan, 0, 0, kAlignTopLeft, "", 26, 0xff, 0xff, 0xff);
                nodeFan:setPos(0, 0);
                node:addChild(nodeFan);
                childY      = nodeFan.m_res.m_height + 20;
                totalHeight = childY;
            end

            --名词解释
            local term = data:getTerm();

            local termWidth  = 0;
            local termHeight = 0;

            if term and term ~= "" then
                local nodeTerm = new(Text, data:getTerm(), 0, 0, kAlignTopLeft, "", 26, 0xff, 0xff, 0xff);
                nodeTerm:setPos(0, childY);
                node:addChild(nodeTerm);
                termWidth = nodeTerm.m_res.m_width;
                termHeight = nodeTerm.m_res.m_height;

                local description = new(TextView, data:getDescription(), width - termWidth, 0, kAlignTopLeft, "", 26, 0xff, 0xe6, 0);
                description:setPos(termWidth, childY);
                description:setSize(width - termWidth, description.m_res.m_height);
                node:addChild(description);

                local maxHeight = math.max(termHeight, description.m_res.m_height) + 20;
                childY      = childY + maxHeight;
                totalHeight = totalHeight + maxHeight;
            end

            --创建牌
            
            local card = self:createCards(data:getCards());

            if card then
                card:setPos(0, totalHeight);
                node:addChild(card);
                local _, cardHeight = card:getSize();
                totalHeight = totalHeight + cardHeight + 20;
            end

            node:setSize(width, totalHeight);
            node:setPos(x, y);

            y = y + totalHeight;
        end

        fanNode:addChild(node);
        
    end

    fanNode:setSize(width, y);

    fanData:setLoad(load);

    return fanNode;
end


function HelpPopu:createCards(cards)
    -- body
    --创建牌

    if not cards or #cards < 1 then
        return nil;
    end

    local Card                  = require("room/entity/card");
    local CardPanelCoord        = require("room/coord/cardPanelCoord");
    local extraCardBgFile       = CardPanelCoord.extraCardBg[SEAT_1];
    local extraCardImgFileReg   = CardPanelCoord.extraCardImage[SEAT_1];

    local node = new(Node);

    local cardWidth, cardHeight = 0, 0;
    local cardx, cardy          = 0, 0;
    local cardMaxHeight         = 0;

    local has4cards = false;

    for cardIndex = 1, #cards do
        local cardValue = cards[cardIndex];
        if cards[cardIndex] == cardValue and cards[cardIndex + 1] == cardValue and
           cards[cardIndex + 2] == cardValue and cards[cardIndex + 3] == cardValue then
           has4cards = true;
           break;
        end
    end

   local childY = has4cards == true and 10 or 0

    for cardIndex = 1, #cards do

        local cardValue = cards[cardIndex];

        if cards[cardIndex] == cardValue and cards[cardIndex - 1] == cardValue and
           cards[cardIndex - 2] == cardValue and cards[cardIndex - 3] == cardValue then

            card = new(Card, cardValue, extraCardBgFile, extraCardImgFileReg)
                :addTo(node)
                :align(kAlignTopLeft)

            cardWidth, cardHeight = card:getSize();

            card:setPos(cardx - (cardWidth - 5) * 2,  childY - 22);
            card:setBgAlign(kAlignTopLeft)
            card:scaleCardTo(0.8, 0.8)
            card:shiftCardMove(0, -15)

            cardMaxHeight = math.max(cardMaxHeight, cardHeight + childY );

        else
            card = new(Card, cardValue, extraCardBgFile, extraCardImgFileReg)
                :addTo(node)
                :align(kAlignTopLeft)

            cardWidth, cardHeight = card:getSize();

            card:setPos(cardx, childY);
            card:setBgAlign(kAlignTopLeft)
            card:scaleCardTo(0.8, 0.8)
            card:shiftCardMove(0, -15)

            cardx = cardx + (cardWidth - 5);
            cardMaxHeight = math.max(cardMaxHeight, cardHeight);
        end

    end  

    node:setSize(0, cardMaxHeight);

    return node;
    
end

-------------------------------------------
--加载规则
-------------------------------------------



function HelpPopu:loadRuleData(ruleType)
    -- body

    if self.mRuleData[ruleType] then
        return;
    end

    local ruleTable;

    if ruleType == MJ_GUOBIAO then
        require("popu.lobby.help_gb_string");
        ruleTable = GB_RULE_TABLE;
    elseif ruleType == MJ_GUANGDONG then
        require("popu.lobby.help_gd_string");
        ruleTable = GD_RULE_TABLE
    elseif ruleType == MJ_SHANGHAI then
        require("popu.lobby.help_sh_string");
        ruleTable = SH_RULE_TABLE
    elseif ruleType == MJ_HEBEI then
        require("popu.lobby.help_hb_string");
        ruleTable = HB_RULE_TABLE
    end

    if not ruleTable then
        return;
    end

    local ruleData = new(require("data.ruleData"))

    for i = 1, #ruleTable do
        local rule = new(require("data.rule"));
        rule:setTitle(ruleTable[i].title);
        rule:setTerm(ruleTable[i].term);
        rule:setDescription(ruleTable[i].description);
        ruleData:add(rule);
    end
    
    self.mRuleData[ruleType] = ruleData;

end

function HelpPopu:loadFanData(fanType)
    -- body

    if self.mFanData[fanType] then
        return;
    end

    local fanTable;

    if fanType == MJ_GUOBIAO then
        require("popu.lobby.help_gb_string");
        fanTable = GB_FAN_TABLE;
    elseif fanType == MJ_GUANGDONG then
        require("popu.lobby.help_gd_string");
        fanTable = GD_FAN_TABLE
    elseif fanType == MJ_SHANGHAI then
        require("popu.lobby.help_sh_string");
        fanTable = SH_FAN_TABLE
    elseif fanType == MJ_HEBEI then
        require("popu.lobby.help_hb_string");
        fanTable = HB_FAN_TABLE
    end

    if not fanTable then
        return;
    end

    local fanData = new(require("data.fanData"))

    for i = 1, #fanTable do
        local rule = new(require("data.fan"));
        
        rule:setTitle(fanTable[i].title);
        rule:setFan(fanTable[i].fan);
        rule:setTerm(fanTable[i].term);
        rule:setDescription(fanTable[i].description);
        rule:setCards(fanTable[i].cards);
        fanData:add(rule);
    end
    
    self.mFanData[fanType] = fanData;

end

function HelpPopu:dismiss(...)
    GameSetting:setIsSecondScene(false)
    return HelpPopu.super.dismiss(self, ...)
end

return HelpPopu