require(ViewPath.."privateRoomChooseGameItem")
require(ViewPath.."privateRoomListItemLayout")
require(ViewPath.."privateRoomBillBriefItemLayout")

local PrivateRoomPopu = class(require("popu.gameWindow"))

function PrivateRoomPopu:ctor()

end

function PrivateRoomPopu:initView()
    self:findChildByName("btn_back"):setOnClick(self, function(self)
        self:dismiss()
    end)

    --添加反馈弹窗
    self:findChildByName("btn_feedback"):setOnClick(self, function(self)
        WindowManager:showWindow(WindowTag.FeedbackPopu, {feedbackId = 16, chooseGame = false}, WindowStyle.POPUP)
    end)

    self.inputCodeFrame = self:findChildByName("img_inputCodeFrame")
    self.chooseGameFrame = self:findChildByName("img_chooseGameFrame")
    self.billFrame = self:findChildByName("img_billBg")
    --防止点击到背景下的按钮
    self.billFrame:setEventTouch(nil, function() end)

    self:initMainBtn()
    self:initHeadView()
    self:initEnterCodeFrame()
    self:initChooseGameFrame()
    self:initBillFrame()
    HttpModule.getInstance():execute(HttpModule.s_cmds.GET_PRIVATE_ROOM_UNREAD_BILL, {}, false, false)
end

function PrivateRoomPopu:dtor()

end

function PrivateRoomPopu:enterPrivateRoom(gameId, isCreate, codeNum)
    --存在安装路径，游戏存在
    if app:getGameInstallPath(gameId) ~= "" then
        --检查游戏是否已经更新完毕
        if MyGameVerManager:updateGameFromNet(gameId) then
            local game = app:getGame(gameId)
            --校验游戏是否存在私人房
            if game and game.enterPrivateRoom then
                --表示正常进入游戏
                if game:enterPrivateRoom(States.Lobby, isCreate, codeNum) then
                    return true
                else
                    app:checkAndDownloadRoomList(gameId, 1, false)
                    AlarmTip.play(STR_NETWORK_TERRIBLE_AND_RETRY)
                end
            else
                AlarmTip.play(STR_PRIVATE_ROOM_CODE_ERROR1)
            end
        end
    else
        AlarmTip.play(STR_PRIVATE_ROOM_CODE_ERROR1)
    end
end

function PrivateRoomPopu:initMainBtn()
    --进入房间按钮
    local viewBtns = self:findChildByName("view_btns")
    local btnCreate = viewBtns:findChildByName("btn_createRoom")
    btnCreate:setOnClick(self, function(self)
        if MyUserData:getPropNum(kIDPrivateRoomCard) > 0 then
            self.chooseGameFrame:setVisible(true)
            local chooseBg = self.chooseGameFrame:findChildByName("img_chooseBg")
            chooseBg:removeProp(1)
            chooseBg:addPropScaleWithEasing(1, kAnimNormal, 600, -1, 'easeOutBack', 'easeOutBack', 0.0, 1.0)
        else
            --没房卡了，提示去买
            WindowManager:showWindow(WindowTag.LobbyConfirmPopu, {
                content = STR_PRIVATE_ROOM_NO_CARD_TIP,
                cancel = STR_EXIT_GAME_CANCEL,
                confirm = STR_EXIT_GAME_CONFIRM,
                confirmFunc = function()
                    WindowManager:showWindow(WindowTag.ShopPopu, {to = "props"}, WindowStyle.POPUP)
                end,
            }, WindowStyle.POPUP)
        end
    end)
    btnCreate:findChildByName("text_create"):setText(STR_PRIVATE_ROOM_CREATE)
    local viewCreateTip = btnCreate:findChildByName("view_createTip")
    UIEx.bind(self, MyUserData, "privateRoomCardCount", function(value)
        viewCreateTip:removeAllChildren()
        local tipNode = new(Node)
        local w = 0
        tipNode:setAlign(kAlignCenter)
        local textTip = new(Text, STR_PRIVATE_ROOM_CREATE_CONSUME_TIP, 0, 0, kAlignLeft, "", 18, 0xff, 0xff, 0xff)
        textTip:setAlign(kAlignLeft)
        tipNode:addChild(textTip)
        w = textTip:getSize() + 5
        local iconTip = new(Image, "privateRoom/icon_roomCard.png")
        iconTip:setPos(w, 0)
        w = w + iconTip:getSize() + 5
        iconTip:setAlign(kAlignLeft)
        tipNode:addChild(iconTip)
        local textCardNum = new(Text, "x"..(MyUserData:getPropNum(kIDPrivateRoomCard)), 0, 0, kAlignLeft, "", 18, 0xff, 0xff, 0xff)
        textCardNum:setPos(w, 0)
        textCardNum:setAlign(kAlignLeft)
        tipNode:addChild(textCardNum)
        w = w + textCardNum:getSize()
        viewCreateTip:addChild(tipNode)
        tipNode:setSize(w, 0)
    end)
    MyUserData.privateRoomCardCount = MyUserData.privateRoomCardCount

    local btnEnter = viewBtns:findChildByName("btn_enterRoom")
    btnEnter:setOnClick(self, function (self)
        self.inputCodeFrame:setVisible(true)
        local inputBg = self.inputCodeFrame:findChildByName("img_inputBg")
        inputBg:removeProp(1)
        inputBg:addPropScaleWithEasing(1, kAnimNormal, 600, -1, 'easeOutBack', 'easeOutBack', 0.0, 1.0)
    end)
    btnEnter:findChildByName("text_enter"):setText(STR_PRIVATE_ROOM_ENTER)

end

function PrivateRoomPopu:initHeadView()
    local viewHead = self:findChildByName("view_head")
    
    local maxNameLen = viewHead:findChildByName("view_name"):getSize()
    ToolKit.formatTextLength(MyUserData.nick, viewHead:findChildByName("text_name"), maxNameLen)
    
    local lv = tonumber(MyUserData.level) or 0
    viewHead:findChildByName("text_level"):setText("Lv."..lv)
    --初始化经验条
    local exp = tonumber(MyUserData.exp) or 0
    local percent = 0
	if lv then
		local index = tonumber(lv)
		if index > 0 and index < 60 then
			local x1 = userLevelExp[index].x1 
			local x2 = userLevelExp[index].x2
			if exp > x1 then
				percent = math.ceil(((exp - x1)/(x2 - x1))*100)
				if percent >= 100 then
					percent = 99
				end
			end
		elseif index == 60 then
			percent = 99
		end
	end
    
    local progressBg = viewHead:findChildByName("img_progressBg")
    local maxLen = progressBg:getSize()
    local progress = progressBg:findChildByName("img_progress")
    local minLen, h = progress:getSize()
    progress:setSize(minLen + (maxLen - minLen) * percent / 100, h)

    --头像
    local btnHead = viewHead:findChildByName("btn_headBg")
    UIEx.bind(self, MyUserData, "headName", function(value)
		btnHead:removeAllChildren();
		local width, height = btnHead:getSize()
		local imgHead 		= new(Mask, value, "common/head_bg.png");
		imgHead:setSize(width, height)
		btnHead:addChild(imgHead);
		MyUserData:checkHeadAndDownload()
	end)
    MyUserData:setHeadName(MyUserData:getHeadName())
    btnHead:setOnClick(self, function(self)
        WindowManager:showWindow(WindowTag.UserPopu, {}, WindowStyle.POPUP)
    end)
end

function PrivateRoomPopu:initEnterCodeFrame()
    --点击空白处关闭输入框
    local CODE_LEN = 6
    local inputBg = self.inputCodeFrame:findChildByName("img_inputBg")
    inputBg:setEventTouch(nil, function() end)
    inputBg:findChildByName("text_title"):setText(STR_PRIVATE_ROOM_INPUT_AND_ENTER)
    
    --记录当前输入的验证码
    local curInputText = ""
    --记录当前输入的字符串是啥,用来绑定UI
    local imgText = inputBg:findChildByName("img_inputText")
    local textNum = {}
    for i = 1, CODE_LEN do
        textNum[i] = imgText:findChildByName("view_"..i):findChildByName("text_num")
    end 
    local codeData = {}
    addProperty(codeData, "strCode", "")
    local inputCode = setProxy(new(codeData))
    UIEx.bind(self, codeData, "strCode", function(value)
        --字符串长度不会超过6位验证码
        local len = string.len(value)
        for i = 1, CODE_LEN do
            local curIndex = len + 1 - i
            if curIndex > 0 then
                textNum[i]:setText(string.sub(value, curIndex, curIndex))
            else
                textNum[i]:setText("")
            end
            
        end
    end)
    
    local viewBtn = inputBg:findChildByName("view_btn")
    viewBtn:findChildByName("btn_cancel"):setOnClick(self, function()
        curInputText = ""
        inputCode:setStrCode(curInputText)
    end)
    viewBtn:findChildByName("btn_delete"):setOnClick(self, function()
        local len = string.len(curInputText)
        if len > 0 then
            curInputText = string.sub(curInputText, 1, len - 1)
            inputCode:setStrCode(curInputText)
        end
    end)
    for i = 0, 9, 1 do
        viewBtn:findChildByName("btn_"..i):setOnClick(self, function()
            if string.len(curInputText) < CODE_LEN then
                curInputText = curInputText..i
                inputCode:setStrCode(curInputText)
                --输入6位后自动进入游戏
                if string.len(curInputText) >= CODE_LEN then
                    local codeNum = tonumber(curInputText) or 0
                    --游戏id来自验证码的前两位
                    local gameId = math.floor(codeNum / 10000) + 990
                    --存在安装路径，游戏存在
                    if not self:enterPrivateRoom(gameId, kFalse, codeNum) then
                        curInputText = ""
                        inputCode:setStrCode(curInputText)
                        self:hideEnterCodeFrame()
                    end
                end
            end
        end)
    end

    self.inputCodeFrame:setEventTouch(self, function()
        curInputText = ""
        inputCode:setStrCode(curInputText)
        self:hideEnterCodeFrame()
    end)

end

function PrivateRoomPopu:hideEnterCodeFrame()
    local inputBg = self.inputCodeFrame:findChildByName("img_inputBg")
    inputBg:removeProp(1)
    local ainmHide = inputBg:addPropScaleWithEasing(1, kAnimNormal, 300, -1, 'easeInBack', 'easeInBack', 1.0, -0.5)
    ainmHide:setEvent(nil, function()
        self.inputCodeFrame:setVisible(false)
        inputBg:removeProp(1)
    end)
end

function PrivateRoomPopu:initChooseGameFrame()
    local chooseBg = self.chooseGameFrame:findChildByName("img_chooseBg")
    self.chooseGameFrame:setEventTouch(nil, function()
        chooseBg:removeProp(1)
        local animHide = chooseBg:addPropScaleWithEasing(1, kAnimNormal, 300, -1, 'easeInBack', 'easeInBack', 1.0, -0.5)
        animHide:setEvent(nil, function()
            self.chooseGameFrame:setVisible(false)
            chooseBg:removeProp(1)
        end)
    end)
    chooseBg:setEventTouch(nil, function() end)

    local NUM_IN_ONE_LINE = 3
    local imgChooseBg = self.chooseGameFrame:findChildByName("img_chooseBg")
    imgChooseBg:findChildByName("text_title"):setText(STR_PRIVATE_ROOM_CHOOSE_GAME)
    
    local svGameList = imgChooseBg:findChildByName("sv_gameList")
    svGameList:removeAllChildren()
    local gameList = MyUserData.gameList or {}
    local gameIds = app:getGameIds()
    local curRow , curColumn = 0, 0
    local wInterval = svGameList:getSize() / NUM_IN_ONE_LINE
    for i = 1, #gameIds do
        local gameId = gameIds[i]
        local gameInfo = gameList[tostring(gameId)] or {}
        --游戏存在私人房入口
        if gameId > 0 and tonumber(gameInfo.hasPrivate) == 1 then
            local item = SceneLoader.load(privateRoomChooseGameItem)
            item.gameId = gameId
            item:findChildByName("img_game"):setFile(string.format("privateRoom/game_%s.png", gameId))
            item:findChildByName("text_gameName"):setText(gameInfo.localName or "")
            local w, h = item:getSize()
            item:setPos((curColumn + 0.5) * wInterval - w / 2, curRow * h)
            svGameList:addChild(item)
            curColumn = curColumn + 1
            if curColumn >= NUM_IN_ONE_LINE then
                curColumn = 0
                curRow = curRow + 1
            end
            local imgProgressBg = item:findChildByName("img_progressBg")
            local imgProgress = imgProgressBg:findChildByName("img_progress")
            local maxLen = imgProgressBg:getSize()
            local minLen, height = imgProgress:getSize()
            local gameStatus = app:getGameStatus(gameId)
            UIEx.bind(item, gameStatus, "progress", function(value)
                if value < 100 then
                    imgProgressBg:setVisible(true)
                    imgProgress:setSize((maxLen - minLen) * value / 100 + minLen, height)
                else
                    imgProgressBg:setVisible(false)
                end
            end)
        end
    end
    local items = svGameList:getChildren()
    for i = 1, #items do
        local item = items[i]
        item:findChildByName("btn_game"):setOnClick(nil, function()
            for j = 1, #items do
                items[j]:findChildByName("img_select"):setVisible(i == j)
            end
            self.createGameId = item.gameId
            MyGameVerManager:updateGameFromNet(item.gameId)
        end)
    end
    --默认选第一个游戏
    if items[1] then
        local btnGame = items[1]:findChildByName("btn_game")
        btnGame.m_eventCallback.func(btnGame.m_eventCallback.obj)
    end
    
    local btnCreate = imgChooseBg:findChildByName("btn_createRoom")
    btnCreate:setOnClick(self, function(self)
        if self.createGameId then
            self:enterPrivateRoom(self.createGameId, kTrue)
        end
    end)
end

function PrivateRoomPopu:initBillFrame()
    self.billFrame:findChildByName("text_billTime"):setText(STR_PRIVATE_ROOM_BILL_EXPIRETIME)
    local imgTips = self.billFrame:findChildByName("img_tip")
    imgTips:findChildByName("text_tip"):setText(STR_PRIVATE_ROOM_BILL_CLICK_TIP)
    imgTips:setEventTouch(nil, function()
        imgTips:setVisible(false)
    end)
    self.billFrame:findChildByName("btn_tip"):setOnClick(self, function()
        if imgTips:getVisible() then
            imgTips:setVisible(false)
        else
            imgTips:setVisible(true)
            imgTips:removeProp(1)
            imgTips:addPropScaleWithEasing(1, kAnimNormal, 200, 0, "easeOutSine", "easeOutSine", 0, 1.0, kCenterXY, imgTips:getSize())
        end
    end)

    local w = self.billFrame:getSize()
    local btnHide = self.billFrame:findChildByName("btn_hide")
    local btnBill = self:findChildByName("btn_bill")
    btnBill:setOnClick(self, function()
        HttpModule.getInstance():execute(HttpModule.s_cmds.GET_PRIVATE_ROOM_BILL_LIST, {}, false, false)
        self.billFrame:setVisible(true)
        btnHide:setVisible(true)
        self.billFrame:removeProp(1)
        self.billFrame:addPropTranslateWithEasing(1, kAnimNormal, 300, 0, "easeOutSine", "easeOutSine", w, -w, 0, 0)
        --设置账单全部已读
        MyUserData.hasUnReadBill = false
    end)
    local imgUnreadTips = btnBill:findChildByName("img_tips")
    imgUnreadTips:setVisible(MyUserData.hasUnReadBill)
    UIEx.bind(self, MyUserData, "hasUnReadBill", function(value)
        imgUnreadTips:setVisible(value)
    end)

    btnHide:setOnClick(self, function()
        self.billFrame:removeProp(1)
        local hideAnim = self.billFrame:addPropTranslateWithEasing(1, kAnimNormal, 300, 0, "easeOutSine", "easeOutSine", 0, w, 0, 0)
        hideAnim:setEvent(nil, function()
            self.billFrame:setVisible(false)
        end)
        imgTips:setVisible(false)
        btnHide:setVisible(false)
    end)

    local viewRoom = self.billFrame:findChildByName("view_roomAndBill")
    local viewDetail = self.billFrame:findChildByName("view_billDetail")
    viewDetail:findChildByName("btn_back"):setOnClick(self, function()
        viewDetail:setVisible(false)
        viewRoom:setVisible(true)
    end)
    
end

function PrivateRoomPopu:initBillAndRoom(data)
    local gameList = MyUserData:getGameList() or {}
    local viewRoom = self.billFrame:findChildByName("view_roomAndBill")
    local svRoomAndBill = viewRoom:findChildByName("sv_roomAndBill")
    svRoomAndBill:removeAllChildren()
    local viewDetail = self.billFrame:findChildByName("view_billDetail")
    local svBillDetail = viewDetail:findChildByName("sv_billDetail")
    local h = 0

    local roomList = data.roomList
    for i = 1, #roomList do
        local room = roomList[i]
        local gameId = room.gameId
        local item = SceneLoader.load(privateRoomListItemLayout)
        local imgGameName = item:findChildByName("img_gameName")
        imgGameName:setFile(string.format("privateRoom/gameName_%s.png", gameId))
        imgGameName:setSize(imgGameName.m_res.m_width, imgGameName.m_res.m_height)
        item:findChildByName("text_friendNum"):setText(string.format(STR_PRIVATE_ROOM_FRIEND_NUM, room.friendNum or ""))
        local codeNum = tonumber(room.code) or 0
        item:findChildByName("text_code"):setText(STR_PRIVATE_ROOM_UNIQUE_CODE..codeNum)
        local btnEnter = item:findChildByName("btn_enter")
        btnEnter:setOnClick(self, function()
            self:enterPrivateRoom(gameId, kFalse, codeNum)
        end)
        item:setPos(0, h)
        svRoomAndBill:addChild(item)
        local _, itemH = item:getSize()
        h = h + itemH
    end

    --初始化账单名单
    local imgDetailGameName = viewDetail:findChildByName("img_gameName")
    local textDetailGameTime = viewDetail:findChildByName("text_time")
    local billList = data.billList
    for i = 1, #billList do
        local bill = billList[i]
        local gameId = bill.gameId

        local item = SceneLoader.load(privateRoomBillBriefItemLayout)
        local imgGameName = item:findChildByName("img_gameName")
        imgGameName:setFile(string.format("privateRoom/gameName_%s.png", gameId))
        imgGameName:setSize(imgGameName.m_res.m_width, imgGameName.m_res.m_height)
        local gameTime = os.date("%Y-%m-%d %H:%M", bill.time)
        item:findChildByName("text_time"):setText(gameTime)
        for index = 1, #bill.bill do
            if MyUserData:getId() == bill.bill[index].id then
                item:findChildByName("text_nick"):setText(bill.bill[index].name)
                item:findChildByName("text_goal"):setText(bill.bill[index].cost)
            end
        end
        item:findChildByName("btn_detail"):setOnClick(self, function()
            viewRoom:setVisible(false)
            viewDetail:setVisible(true)
            imgDetailGameName:setFile(string.format("privateRoom/gameName_%s.png", gameId))
            imgDetailGameName:setSize(imgDetailGameName.m_res.m_width, imgDetailGameName.m_res.m_height)
            textDetailGameTime:setText(gameTime)
            svBillDetail:removeAllChildren()
            local billDetailHeight = 0
            local billDetail = bill.bill
            for j = 1, #billDetail do
                local name = billDetail[j].name
                local cost = billDetail[j].cost
                local node = new(Node)
                local color = {0xff, 0xff, 0xff}
                if billDetail[j].id == MyUserData:getId() then
                    color = {0x14, 0xAF, 0x65}
                end
                local textNick = new(Text, name, 0, 0, kAlignLeft, "", 26, unpack(color))
                ToolKit.formatTextLength(name, textNick, 150)
                node:addChild(textNick)
                local textMoney = new(Text, cost, 0, 0, kAlignRight, "", 26, unpack(color))
                textMoney:setPos(10, 0)
                textMoney:setAlign(kAlignRight)
                node:addChild(textMoney)
                node:setAlign(kAlignTop)
                --左右保留空隙，免得文字直接贴到最边上
                node:setSize(item:getSize() - 15, 50)
                node:setPos(0, billDetailHeight)
                billDetailHeight = billDetailHeight + 50
                svBillDetail:addChild(node)
            end
            svBillDetail:setSize(svBillDetail:getSize())
        end)
        item:setAlign(kAlignTop)
        item:setPos(0, h)
        svRoomAndBill:addChild(item)
        local _, itemH = item:getSize()
        h = h + itemH
    end
    svRoomAndBill:setSize(svRoomAndBill:getSize())
end

function PrivateRoomPopu:onGetPrivateRoomBillList(isSuccess, data)
dump(data)
    if app:checkResponseOk(isSuccess, data) then
        local roomAndBill = {}
        roomAndBill.roomList = {}
        roomAndBill.billList = {}
        local roomList = data.data.stateList or {}
        for i = 1, #roomList do
            local room = {}
            room.gameId = math.floor(tonumber(roomList[i].code) / 10000) + 990
            room.code = roomList[i].code
            room.friendNum = roomList[i].num
            table.insert(roomAndBill.roomList, room)
        end    
        local billList = data.data.billList or {}
        for i = 1, #billList do
            local bill = {}
            bill.gameId = tonumber(billList[i].roomInfo.gameid)
            bill.time = billList[i].roomInfo.time
            bill.bill = {}
            local moneyList = billList[i].list or {}
            for j = 1, #moneyList do
                table.insert(bill.bill, {id = moneyList[j].uid, name = moneyList[j].name, cost = tonumber(moneyList[j].turnmoney) or 0})
            end
            table.sort(bill.bill, function(a, b)
                return a.cost > b.cost
            end)
            table.insert(roomAndBill.billList, bill)
            table.sort(roomAndBill.billList, function(a, b)
                return a.time > b.time
            end
            )
        end
        self:initBillAndRoom(roomAndBill)
    end
end

function PrivateRoomPopu:onGetPrivateRoomUnreadBill(isSuccess, data)
    if app:checkResponseOk(isSuccess, data) then
        if tonumber(data.data.billFlag) == 1 then
            MyUserData.hasUnReadBill = true
        end
    end
end

PrivateRoomPopu.s_severCmdEventFuncMap = {
	[HttpModule.s_cmds.GET_PRIVATE_ROOM_BILL_LIST]      = PrivateRoomPopu.onGetPrivateRoomBillList,
    [HttpModule.s_cmds.GET_PRIVATE_ROOM_UNREAD_BILL]    = PrivateRoomPopu.onGetPrivateRoomUnreadBill,
}

return PrivateRoomPopu