require("room/chat/faceq_res")

local RoomChatPopu = class(require("popu.gameWindow"))
local PATH = "popu.room.roomChat."

function RoomChatPopu:initView(userInfo)

	self.m_sendBtn  = self.m_root:findChildByName("send_btn")
	self.m_inuptText = self.m_root:findChildByName("input_text")

	self.m_sendBtn:setOnClick(self, self.sendChatWord)

	self.m_faceList = self.m_root:findChildByName("face_list")

	self.m_wordBtn = self.m_root:findChildByName("word_btn")
	self.m_wordBtn:enableAnim(false)
	self.m_wordList = self.m_root:findChildByName("word_list")

	self.m_recordBtn = self.m_root:findChildByName("record_btn")
	self.m_recordBtn:enableAnim(false)
	self.m_recordList = self.m_root:findChildByName("record_list")
	self.m_root:findChildByName('face_title'):setText(STR_CHAT_FACE_TEXT)
	self.m_root:findChildByName('send_text'):setText(STR_CHAT_SEND)

    local viewWord = self.m_root:findChildByName("img_chatBg")
    local wordBg = viewWord:findChildByName("view_word")
    wordBg:findChildByName("record_title"):setText(STR_CHAT_RECORD)
    wordBg:findChildByName("word_title"):setText(STR_CHAT_COMMON)
    local recordBg = viewWord:findChildByName("view_record")
    recordBg:findChildByName("record_title"):setText(STR_CHAT_RECORD)
    recordBg:findChildByName("word_title"):setText(STR_CHAT_COMMON)

	local changeTab = function(index)
		if index == 1 then
            wordBg:setVisible(true)
            recordBg:setVisible(false)
			self:initChatView()
		elseif index == 2 then
            wordBg:setVisible(false)
            recordBg:setVisible(true)
			self:initRecordView()
		end
	end
	self.m_wordBtn:setOnClick(1, changeTab)
	self.m_recordBtn:setOnClick(2, changeTab)

	self.m_inuptText:setOnTextChange(self, function ( self )
		local text = self.m_inuptText:getText()
		text = string.ltrim(text)
		local _, len = ToolKit.utf8len(text)
		if len > 36 then
			AlarmTip.play(STR_CHAT_MAX_TEXT)
			self.m_inuptText:setText(ToolKit.subStr(self.m_inuptText:getText(), 36, nil, true));
		end
	end);

	self:initFaceView()
	self:initChatView(userInfo.gameid, userInfo.chatwords, userInfo.isDealer)
end

function RoomChatPopu:initChatView(gameid, chatwords, isDealer)
	if self.m_recordListView then self.m_recordListView:hide() end
	if self.m_wordListView then self.m_wordListView:show() return end
	local wordList = self.m_wordList
	local width, height = wordList:getSize()

	self.m_wordListView = new(ScrollView, 0, 0, width, 332, false)
	self.m_wordListView:setScrollBarWidth(8);
	self.m_wordListView:setDirection(kVertical)
	wordList:addChild(self.m_wordListView)
    if chatwords then
        self:createWordView(chatwords)
		:addTo(self.m_wordListView)
	elseif gameid == 1001 then
		self:createWordView(chatwords)
		:addTo(self.m_wordListView)
	elseif gameid == 1002 then
		self:createWordView(SysChatArray)
		:addTo(self.m_wordListView)
	elseif gameid == 1003 or gameid == 1010 then
		self:createWordView(SysChatArrayYXX)
		:addTo(self.m_wordListView)
	elseif gameid == 1004 then
		self:createWordView(SysChatArrayMakhos)
		:addTo(self.m_wordListView)
	elseif gameid == 1005 then
		self:createWordView(chatwords)
		:addTo(self.m_wordListView)
	elseif gameid == 1006 then
		self:createWordView(SysChatArrayMixedTen)
		:addTo(self.m_wordListView)
	elseif gameid == 1007 then
		self:createWordView(SysChatArray)
		:addTo(self.m_wordListView)
	elseif gameid == 1008 then
		self:createWordView(chatwords)
		:addTo(self.m_wordListView)
	elseif gameid == 1009 then
		if isDealer then
			self:createWordView(SysChatArray_Dealer)
			:addTo(self.m_wordListView)
		else
			self:createWordView(SysChatArray_Player)
			:addTo(self.m_wordListView)
		end
	elseif gameid == 1000 then
		self:createWordView(chatwords)
		:addTo(self.m_wordListView)
	end
	
end

function RoomChatPopu:createWordView(chatArray)
	local wordNode = new(Node)
	local x,y = 12, 0
	for i=1, #chatArray do
		local chatInfo = chatArray[i]
		if chatInfo then
			local wordBtn = UIFactory.createButton(room_map[PATH .. "record_bg.png"] or "")
				:addTo(wordNode)
				:pos(x, y + 10)
			wordBtn:enableAnim(false)
			wordBtn:setOnClick( nil, function()
	  			EventDispatcher.getInstance():dispatch(Event.Message, "sendChatWord", chatInfo)
				self:dismiss(true)
			end)

			local txt = UIFactory.createText({
					text = chatInfo,
					size = 24,
					color = c3b(24, 84, 154),
				})
				:addTo(wordBtn)
				:align(kAlignLeft, 10, 0)

			y = y + 65
		end
	end
	wordNode:setSize(384, y)
	return wordNode
end

function RoomChatPopu:initRecordView()
	if self.m_wordListView then self.m_wordListView:hide() end
	if self.m_recordListView then self.m_recordListView:show() return end
	if not G_RoomCfg then return end
	local chatHistory = G_RoomCfg:getChatHistory()
	local recordList = self.m_recordList
	local width, height = recordList:getSize()

	self.m_recordListView = new(ScrollView, 0, 0, width, 332, false)
	self.m_recordListView:setScrollBarWidth(8);
	self.m_recordListView:setDirection(kVertical)
	recordList:addChild(self.m_recordListView)

	self:createRecordNode(chatHistory)
end

function RoomChatPopu:createRecordNode(chatHistory)
	local x, y = 12, 0
	local imgWidth
	self.m_records = {}
	for i=#chatHistory, 1, -1 do
		local chatRecord = chatHistory[i]
		local recordImg = UIFactory.createImage(room_map[PATH .. "record_bg.png"] or "")
			:pos(x, y + 10)
			:addTo(self.m_recordListView)
		table.insert(self.m_records, recordImg)

		imgWidth = imgWidth or recordImg:getSize()
		local txt = UIFactory.createTextView({
				text = string.format("[%s]%s", chatRecord.nick, chatRecord.chatInfo),
				size = 24,
				color = c3b(24, 84, 154),
				width = imgWidth - 20,
				height = 0,
			})
			:addTo(recordImg)
			:pos(10, 10)
		local width, txtHeight = txt:getSize()
		txt:setSize(nil, txtHeight)
		txt:setEventTouch(nil, function() end)
		local height = txtHeight + 20
		recordImg:setSize(imgWidth, height)
		
		y = y + height + 20
	end
	return recordNode
end

function RoomChatPopu:initFaceView()
	local qList = self.m_faceList
	local qList_w, qList_h = qList:getSize()

	local qfileArray = {}
	local qfaceName = "expression"
	local faceNum = 21
	for i=1, faceNum do
		if i == 16 then
			qfileArray[i] = faceq_res_map[string.format(qfaceName.."%d.png",25)]
		elseif i == 20 then
			qfileArray[i] = faceq_res_map[string.format(qfaceName.."%d.png",26)]
		else
			qfileArray[i] = faceq_res_map[string.format(qfaceName.."%d.png",i)]
		end 
	end

	self.m_faceListView = qList:findChildByName("sv_faceView")
	self.m_faceListView:setDirection(kVertical)
	self.m_faceListView:addTo(qList)
	self:createFaceNode(qfileArray)
		:addTo(self.m_faceListView)
    self.m_faceListView:setSize(self.m_faceListView:getSize())
end

function RoomChatPopu:createFaceNode(qfileArray)
    local EXP_COUNT_IN_LINE = 4
    local FACE_ITEM_HEIGHT = 100
    local wInterval = self.m_faceListView:getSize() / EXP_COUNT_IN_LINE
	local faceNode = new(Node)
	local x,y = 0, 0
	local sepLines = {}
    --qfileArray定义的表情索引用于和其他玩家交互的表情索引是匹配的，用faceShowIndex来自定义修改表情在输入框的显示顺序
    local faceShowIndex = {1, 11, 3, 4, 17, 15, 14, 19, 6, 7, 16, 2, 21, 9, 20, 5, 13, 10, 12, 18, 8}
    for i = 1, #faceShowIndex do
        local index = faceShowIndex[i]
        local faceBtn = new(Button, qfileArray[index])
        faceBtn:setOnClick( nil, function()
  			EventDispatcher.getInstance():dispatch(Event.Message, "sendChatFace", index)
			self:dismiss(true)
		end)
        faceBtn:setPos(x, y)
		faceNode:addChild(faceBtn)
        if i % EXP_COUNT_IN_LINE == 0 and i ~= #faceShowIndex then
            x = 0
			y = y + FACE_ITEM_HEIGHT
            local lineImg = new(Image, room_map[PATH .. "face_sep_h.png"] or "")
            lineImg:setPos(0, y)
            faceNode:addChild(lineImg)
        else
            x = x + wInterval
			if i ~= EXP_COUNT_IN_LINE then
				local sep_v = new(Image, room_map[PATH .. "face_sep_v.png"] or "")
                sep_v:setPos(x, 0)
				faceNode:addChild(sep_v)
				table.insert(sepLines, sep_v)
			end
        end
    end
    
	for i,v in ipairs(sepLines) do
		v:setSize(2, y)
	end
	faceNode:setSize(384, math.ceil(#faceShowIndex / EXP_COUNT_IN_LINE) * FACE_ITEM_HEIGHT)
	return faceNode
end

function RoomChatPopu:sendChatWord()
	local text = self.m_inuptText:getText()
	text = string.ltrim(text)
	if not ToolKit.isValidString(text) then
		AlarmTip.play(STR_CHAT_INPUT_EMPTAY)
		return
	end
  	EventDispatcher.getInstance():dispatch(Event.Message, "sendChatWord", text)
	self:dismiss(true)
end

return RoomChatPopu