
local HeadProgress = class(Node);

function HeadProgress:ctor(fileName)
    self:init(fileName);
end

function HeadProgress:init(fileName)
    self.circleProgress = new(require('truco.anim.circleProgress'), fileName);
    self.circleProgress:setAlign(kAlignCenter);
    self:addChild(self.circleProgress);

    self.animPic = new(Image, 'truco/anim/head_progress/animPic1.png');
    self.animPic:setAlign(kAlignCenter);
    self:addChild(self.animPic);

    local anim_pics = {
        "truco/anim/head_progress/animPic1.png",
        "truco/anim/head_progress/animPic2.png",
        "truco/anim/head_progress/animPic3.png",
        "truco/anim/head_progress/animPic4.png",
        "truco/anim/head_progress/animPic5.png",
    }

    local animParam = {
        frames = anim_pics,     --动画帧表，file or ResImage
        inteval = 100,          -- 每帧显示的时间
        repeatCount = -1,
        delay = 0,          -- 多长时间后开始播放动画
        DebugName = "HeadCircleAnim.play"
    }

    self.animPic:playAnimation(animParam);

    self.circlePic = new(Image, 'truco/anim/head_progress/circle_light.png');
    self.circlePic:setAlign(kAlignCenter);
    self:addChild(self.circlePic);
    self.circlePic:setVisible(false);--默认不可见
end

function HeadProgress:setProgressPecent(rate) 
    self.circleProgress:setProgressPecent(rate);

    local angle = 360 * rate;
    if self.animPic then
        checkAndRemoveOneProp(self.animPic,11);
        self.animPic:addPropRotateSolid(11, angle, kCenterDrawing);
    end
end

function HeadProgress:initColor()
    if self.circleProgress then
        self.circleProgress:initColor();
    end
end

function HeadProgress:dtor()
   
end

function HeadProgress:setSize(x,y)
    HeadProgress.super.setSize(self,x,y);
    if self.circleProgress then
        self.circleProgress:setSize(x,y);
    end
    if self.animPic then
        local w, h = self.animPic:getSize()
        self.animPic:setSize(w * x / 130, h * y / 130);
    end
    if self.circlePic then
        local w, h = self.circlePic:getSize()
        self.circlePic:setSize(w * x / 130, h * y / 130);
    end
end

function HeadProgress:playTimerAnim(totalSeconds, currSeconds, obj, func)
    self:initColor()
    self:setVisible(true);
    self:setProgressPecent(0.00001);--先清除数据
    totalSeconds = totalSeconds*1000;

    self:playCircleAnim();
    --刷新进度条
    checkAndRemoveOneProp(self,41);
    local seconds = currSeconds == 0 and 0 or (totalSeconds - currSeconds * 1000 );
    local timerAnim = self:addPropTranslate(41, kAnimLoop, 50, 0, 0, 0, 0, 0);
    timerAnim:setEvent(self,function()
        seconds = seconds + 50;
        self:setProgressPecent(seconds/totalSeconds);
        if seconds % 1000 == 0 then
            if func then func(obj, math.floor(seconds/1000)) end
        end
        if seconds == totalSeconds then
            self:playCircleAnim();
            checkAndRemoveOneProp(self,41);
        end
    end);
    
end

function HeadProgress:playCircleAnim()
    self.circlePic:setVisible(true);
    checkAndRemoveOneProp(self.circlePic,1);
    checkAndRemoveOneProp(self.circlePic,2);
    self.circleAnim = self.circlePic:addPropScale(1, kAnimNormal, 600, 0, 1.0, 1.2, 1.0, 1.2, kCenterDrawing);--环缩放
    self.circleAnim:setEvent(self,function()
       self.circlePic:addPropTransparency(2, kAnimNormal, 300, 0, 1, 0);-- 渐消失
    end);
end
function HeadProgress:stopCircleAnim()
    checkAndRemoveOneProp(self.circlePic,1);
    checkAndRemoveOneProp(self.circlePic,2);
    checkAndRemoveOneProp(self,41);
end
return HeadProgress;