require("truco.card.truco_common_card")
CardState =
{
	Normal 			= 1; -- 正常
	NormalSelected 	= 2; -- 正常选中
	Popup 			= 3; -- 弹起
	popupSelected 	= 4; -- 弹起选中
	Dragging 		= 5; -- 拖拽
	Waiting 		= 6; -- 等待出牌
	End 			= 7; -- 结束
};
local Card = class(Node);
addProperty(Card, "type", 0)
addProperty(Card, "value", 0)
addProperty(Card, "byte", 0)
addProperty(Card, "selected", false)
addProperty(Card, "gray", false)
addProperty(Card, "enable", false)
addProperty(Card, "scaleX", 1)
addProperty(Card, "scaleY", 1)
addProperty(Card, "rotation", 0)
addProperty(Card, 'direction', 1)
addProperty(Card,'startime',0)  	--定时器 记录旋转起始时间
addProperty(Card,'currentangle',0)  --旋转角度 记录当前旋转了多少度
addProperty(Card,'perangle',0) 		--旋转速度
addProperty(Card, "delegate", nil)
addProperty(Card, "isOut", false)


----------------------------------- [CLASS] Card--------------------------------------
local SIZE_LIST = {
	{164, 128, 0.6},  -- 1
	{128, 164, 0.6},  -- 2
	{164, 128, 0.6},  -- 3
	{168, 164, 1},  -- 4
}

local COVER_BG_LIST = {
	"bgOver_1.png",
	"bgOver_2.png",
	"bgOver_3.png",
	"bgOver.png"	
}

-- card_type:  花色
-- card_value： 大小
function Card:ctor(card_type, card_value, direction)
	direction = direction or 4
	--direction = 4
	self.lastIsCover_ = false
	self:setDirection(direction)

	local cardLayout = self:getCardLayout(direction)
	self.m_root = SceneLoader.load(cardLayout);
	self:addChild(self.m_root);
	self:setEnable(true)	
	
	local sz = SIZE_LIST[direction]
	self.width_ = sz[1]
	self.height_ = sz[2]
	self.m_root:setSize(self.width_, self.height_)
	-- self:rotationAngle(sz[3])
	if direction == 4 then
		self:scale(1.31,1.23)
	else
		self:scale(sz[3], sz[3])
	end
	
	self:setType(card_type)
	self:setValue(card_value)
	self:setByte(card_value + card_type * 0x10)
	self:render(card_type, card_value,direction)

	self.rotateAnchorx = 0.5
	self.rotateAnchory = 0.5

	local click = self.m_root:findChildByName("btn")
	if click then
		click:setOnClick(self,function ()
			if self.delegate then
				self.delegate:sendCardData(self)
			end
		end)
	end
end

function Card:getCardLayout(dir)
	dir = 4
	local cardLayout

	if dir == 1 or dir == 2 or dir == 3 then
		cardLayout = requireview("view.truco.cardLayout"..dir)
		--JLog.d('card error')
	else
		cardLayout = requireview("view.truco.cardLayout1")
	end
	return cardLayout
end
-- 设置缩放大小
function Card:scale(sx, sy)
	if self.m_children then
		-- body
		self:setScaleX(sx)
		self:setScaleY(sy)
		checkAndRemoveOneProp(self.m_root, 11)
		self.m_root:addPropScaleSolid(11, sx, sy, kCenterXY,0,0)
		local dw, dh = self.m_root:getSize()
		self.m_root:setSize(dw*sx, dh*sy)
		self.width_= self.width_*sx
		self.height_= self.height_*sy
	end
end
function Card:getSize()
	return self.width_,self.height_
end
	-- 0：梅花
	-- 1：方块
	-- 2：红桃
	-- 3：黑桃
function Card:render(card_type, card_value,direction)
	if not self.m_children then return end
	direction = 4
	local img_value = self:findChildByName('img_value')
	local img_stype = self:findChildByName('img_stype')
	
	if img_value and img_stype then
		if card_type >= 0 and  card_value > 0 then 
			img_value:setVisible(true)
			img_stype:setVisible(true)
			self:changeCardRender(card_type, card_value,direction)
		else
			img_value:setVisible(false)
			img_stype:setVisible(false)
			local resId = COVER_BG_LIST[direction]
			--self:findChildByName('img_btype'):setFile(truco_common_card_map[resId]);
			self:findChildByName('img_btype'):setFile('truco/room/yh1/bgOver.png');
		end
	end
end
function Card:changeCardRender(card_type, card_value,direction)
	local numName;
	local smallTypeName;
	local bigTypeName;
	if card_value >= 1 and card_value <= 13 then
		numName 	  = string.format('%d_%d',(card_type == 0 or card_type == 3) and 1 or 2, card_value)
		smallTypeName = "s_"..card_type;		
		bigTypeName   = "b_"..card_type;
		if card_value > 10 then
			bigTypeName = string.format('bg_%d_%d',(card_type == 0 or card_type == 3) and 1 or 2, card_value)
		end
		
	end

	self.imgValue 		= self:findChildByName('img_value');
	self.imgSamllType 	= self:findChildByName('img_stype');
	self.imgBigType 	= self:findChildByName('img_btype');

	if direction == 1 or direction == 2 or direction == 3 then
		numName = numName.."_"..direction
		smallTypeName = smallTypeName.."_"..direction
		bigTypeName = bigTypeName.."_"..direction
	end
	-- 
	self.imgValue:setFile(truco_common_card_map[numName..".png"]);
	self.imgSamllType:setFile(truco_common_card_map[smallTypeName..".png"]);
	self.imgBigType:setFile(truco_common_card_map[bigTypeName..".png"]);
end

function Card:getTransformedPos()
	-- body
	local pos 	= drawing_get_transformed_vertex(self.m_drawingID, 1)
	if pos and pos[1] ~= 0 then
		local px, py = self:getParent():getAbsolutePos()
		return pos[1] / System.getLayoutScale() - px, pos[2] / System.getLayoutScale() - py
	else
		return self:getPos()
	end
end
--旋转到指定角度
function Card:rotateTo(dur,angle,anchorx,anchory,callback)

	self.rotateAnchorx =  anchorx or 0.5--旋转的锚点  x
	self.rotateAnchory =  anchory or 0.5--旋转的锚点  y

	self:stopRotate()

	self.startrotate = sys_get_int("tick_time",-1)  
	self.perangle = (angle - self.currentangle) / dur
	local rotate = self:addPropRotate(16, kAnimNormal,dur, 0, self.currentangle, angle, kCenterXY,self.width_*self.rotateAnchorx,self.height_*self.rotateAnchory);
	if rotate then
		rotate:setEvent(self,function ()
			self.currentangle = angle
			checkAndRemoveOneProp(self,16)
			self:addPropRotateSolid(16, angle,  kCenterXY,self.width_*self.rotateAnchorx,self.height_*self.rotateAnchory)
			self.startrotate = nil 
			if callback then
				callback()
			end
		end)
	end
end
--计算下次旋转前当前旋转的角度
function Card:stopRotate()
	if self.startrotate then
		local stoprotate= sys_get_int("tick_time",-1)
		local offsettime = stoprotate - self.startrotate
		self.currentangle = self.currentangle + offsettime*self.perangle
		checkAndRemoveOneProp(self,16)
		self:addPropRotateSolid(16, self.currentangle,kCenterXY,self.width_*self.rotateAnchorx,self.height_*self.rotateAnchory)
		self.startrotate = nil 
	end
end
--移动到相对父节点的 坐标
function Card:moveTo(dur,x, y,callback)
	-- body
	if self:getEnable() then
		local curX, curY = self:getTransformedPos()
		checkAndRemoveOneProp(self, 15)
		self:setPos(curX, curY)
		local translate = self:addPropTranslateWithEasing(15, kAnimNormal, dur, 0,"easeOutSine", "easeOutSine", 0, x - curX, 0, y - curY)
		if translate then
			translate:setEvent(self,function()
				if callback then
					callback()
				end
			end)
		end
	end
end

function Card:scaleTo(dur,x, y,callback)
	checkAndRemoveOneProp(self,166)
	self:addPropScale(166, kAnimNormal, dur, 0,1.0,0.5,1.0,0.5,kCenterDrawing);
end

function Card:setSelectIcon(value)
	-- if self.m_children then
	-- 	local imgSelect = self:findChildByName("img_select")
	-- 	if imgSelect then
	-- 		checkAndRemoveOneProp(imgSelect, 1)
	-- 		checkAndRemoveOneProp(imgSelect, 2)
	-- 		if value == true then
	-- 			imgSelect:setVisible(true)
				
	-- 			local w, h = imgSelect:getSize()
	-- 			local scaleAnim = imgSelect:addPropScaleWithEasing(1, kAnimNormal, 360, 0, 'easeOutBounce', 'easeOutBounce', 1.8, -0.8, kCenterXY, w/2, h/2)
	-- 			scaleAnim:setEvent(self, function()
	-- 				local cnt = 0
	-- 				local transAnim = imgSelect:addPropTransparency(2, kAnimLoop, 450, 0, 0.0, 1)
	-- 				transAnim:setEvent(self, function()
					
	-- 					checkAndRemoveOneProp(imgSelect, 1)
	-- 					checkAndRemoveOneProp(imgSelect, 2)
						
	-- 				end)
	-- 			end)
	-- 		else
	-- 			imgSelect:setVisible(false)
	-- 		end
	-- 	end
	-- end
	if self.m_children and value then
		local w, h = self:getSize()
		local view_anim = self:findChildByName("view_anim")
		if not view_anim then return end

		checkAndRemoveOneProp(self,111)
		local scaleAnim = self:addPropScaleWithEasing(111, kAnimNormal, 300, 0, 'easeOutBounce', 'easeOutBounce', 1.8, -0.8, kCenterXY, w/2, h/2)
		scaleAnim:setEvent(self, function()
			local animPic = new(Image, 'truco/anim/anim_bp/01.png');
			view_anim:addChild(animPic);
			animPic:setAlign(kAlignCenter);
			local anim_pics = {}
			local pic = ''
			for i=1,16 do
				if i < 10 then
					pic = string.format('truco/anim/anim_bp/0%d.png',i)
				else
					pic = string.format('truco/anim/anim_bp/%d.png',i)
				end
				anim_pics[i] = pic
			end
			local animParam = {
				frames = anim_pics,
				--inteval = 70,
				inteval = 60,
				repeatCount = 1,
				delay = 50,
				onComplete = function()
					view_anim:removeAllChildren();
				end,
				DebugName = "anim_bp.play"
			}
			animPic:playAnimation(animParam);
		end)
	end
end
return Card