--[[
	骰子socket发送协议  2015-03-03
]]
local BaseWriter = require("mjSocket.game.baseWriter")
local RoomWriter = class(BaseWriter)
local printInfo, printError = overridePrint("RoomWriter")

-- 玩家退出房间
function RoomWriter:_logout(packetId, param)
    --CLIENT_CMD_LOGOUT
end

-- 玩家聊天请求
-- chatMsg: 聊天内容(编码： UTF-8)
function RoomWriter:_request_chat(packetId, param)
    --CLIENT_CMD_CHAT_MSG
    self.m_socket:writeString(packetId, param.chatMsg);
end


-- 玩家发送表情
-- faceID: 表情ID号
function RoomWriter:_request_face(packetId, param)
    --CLIENT_CMD_SEND_FACE
    self.m_socket:writeInt(packetId, param.faceID);
end


--玩家触发计时宝箱请求
function RoomWriter:_request_timebox(packetId, param)
    --CLIENT_CMD_ONLINE_TIME
end


-- 玩家资产变更请求
-- money: 变更的资产数 int
function RoomWriter:_money_change(packetId, param)
    --CLIENT_CMD_MONEY_CHANGE
    self.m_socket:writeInt(packetId, param.money);
end


--玩家更新信息
-- infoStr : 待更新到服务器的玩家信息内容
function RoomWriter:_update_userinfo(packetId, param)
    --CLENT_CMD_USER_UPGRADE_INFO
    self.m_socket:writeString(packetId, param.infoStr);
end
--玩家赠送请求
--giftID : 礼物编号ID
--playerCount: 该礼物要送给几个玩家
--userID:  送礼物的玩家ID
function RoomWriter:_user_present(packetId, param)
    --CLENT_CMD_USER_PRESENT
    self.m_socket:writeInt(packetId, param.giftID);
    self.m_socket:writeByte(packetId, param.playerCount);
    self.m_socket:writeInt(packetId, param.userID);
end

-- 发送心跳包
function RoomWriter:_sendHeart(packetId, param)
    --CLIENT_CMD_HEART_BEAT
end
-----------------------------------------------------------------------
--param:{u16_gameLevel = 101, u32_gameId = 1002}
-- 0x118    -- 请求分配桌子ID
function RoomWriter:onGetTableId(packetId, param)
	printInfo("RoomWriter:onGetTableId");
	self.m_socket:writeShort(packetId, param.u16_gameLevel);	-- gamelevel
	self.m_socket:writeInt(packetId, param.u32_gameId); 		-- gameid
	
end
--param:{u32_follow, str_key , str_info}
-- 0x123    -- 跟踪玩家
function RoomWriter:onFollow(packetId, param)
	printInfo("RoomWriter:onFollow");
	self.m_socket:writeInt(packetId, param.u32_follow);			-- u32_follow
	self.m_socket:writeString(packetId, param.str_key); 		--
	self.m_socket:writeString(packetId, param.str_info); 		--
	self.m_socket:writeString(packetId, param.str_followNick); 		--
end
--param:{u32_tableId, u32_uid, str_key, str_info, u32_flag}
-- 0x1001	-- 请求登录某个游戏
function RoomWriter:onLoginGame(packetId, param)
	printInfo("RoomWriter:onLoginGame");
	dump(param)
	self.m_socket:writeInt(packetId, param.u32_tableId);	-- 
	self.m_socket:writeInt(packetId, param.u32_uid); 		--
	self.m_socket:writeString(packetId, param.str_key); 	-- 
	self.m_socket:writeString(packetId, param.str_info); 	--
	self.m_socket:writeInt(packetId, param.u32_flag); 	--
end
-- 0x1031	-- 请求坐下
function RoomWriter:onSitdownInGame(packetId, param)
	printInfo("RoomWriter:onSitdownInGame");
	-- required int8 seat_id = 1; //! 座位号
	local seatId = param.u32_seatId - 1
	self.m_socket:writeByte(packetId, seatId);
end
-- 0x1033	-- 玩家站起
function RoomWriter:onStandUp(packetId, param)
	printInfo("RoomWriter:onStandUp");
end
-- 0x1035	-- 叫分 client -> server
function RoomWriter:onTruco(packetId, param)
	printInfo("RoomWriter:onTruco");
	-- required int seatid     //叫分玩家座位号
	-- required int score      //叫的分数  1，3，6，12
	local seatId = param.seatId - 1
	self.m_socket:writeInt(packetId, seatId);
	self.m_socket:writeInt(packetId, param.score);
	if DEBUG_MODE then
		-- AlarmTip.play("0x1035-->seatId:"..tostring(seatId).."  score:"..tostring(param.score))
		printInfo("0x1035-->seatId:"..tostring(seatId).."  score:"..tostring(param.score))
	end
end
-- 0x1036	-- 出牌 client -> server
function RoomWriter:onDropCard(packetId, param)
	printInfo("RoomWriter:onTruco");
	-- required int seatid;    //出牌的用户座位
	-- required BYTE Card;     //所出牌	
	self.m_socket:writeInt(packetId, param.seatId - 1);
	self.m_socket:writeByte(packetId, param.cardValue);
	self.m_socket:writeByte(packetId, param.cardPos);	
end
-- 0x1037	-- 接受TRUCO 或放弃
function RoomWriter:onAcceptOrRefuseTruco(packetId, param)
	printInfo("RoomWriter:onAcceptOrRefuseTruco");
	-- required int seatid;    //回应叫分用户的座位
	-- required int flag;      //1 接受 0 拒绝
	local seatId = param.seatId - 1
	self.m_socket:writeInt(packetId, seatId);
	self.m_socket:writeByte(packetId, param.flag);
	if DEBUG_MODE then
		-- AlarmTip.play("0x1037-->seatId:"..tostring(seatId).."  flag:"..tostring(param.flag))
		printInfo("0x1037-->seatId:"..tostring(seatId).."  flag:"..tostring(param.flag))
	end
end
-- 0x1038    -- 取消托管  托管类型  1--托管，0--取消托管
function RoomWriter:onCancelDelegate(packetId, param)
	printInfo("RoomWriter:onCancelDelegate");
	-- required int type; 		//0--取消托管
	self.m_socket:writeByte(packetId, param.type);
end
-- param:nil
-- 0x1002    -- 退出房间
function RoomWriter:onExitRoom(packetId, param)
	printInfo("RoomWriter:onExitRoom");
end
-- 0x1003	-- 发送聊天
function RoomWriter:onSendChat(packetId, param)
	self.m_socket:writeString(packetId, param.iChatInfo); 
end
-- 0x1004	-- 发送表情
function RoomWriter:onSendFace(packetId, param)
	self.m_socket:writeInt(packetId, param.iFaceType);
	self.m_socket:writeInt(packetId, param.isVipFace);
end
--创建好友房申请
function RoomWriter:onCreateBuddyRoom(packetId, param)
	printInfo("onCreateBuddyRoom write")
	dump(param)
    self.m_socket:writeShort(packetId, param.i16_gameId)
    self.m_socket:writeInt(packetId, param.i32_baseChips)
    self.m_socket:writeString(packetId, param.str_roomName)
    self.m_socket:writeString(packetId, param.str_password)
    self.m_socket:writeInt(packetId, param.i32_requireChips)
    self.m_socket:writeString(packetId, param.str_guid)
    self.m_socket:writeString(packetId, param.str_userInfo)
    self.m_socket:writeInt(packetId, param.i32_flag)
    self.m_socket:writeInt(packetId, param.i32_escapeflag)
end
--进入好友放申请
function RoomWriter:onEnterBuddyRoom(packetId, param)
	printInfo("onEnterBuddyRoom write")
	dump(param)
    self.m_socket:writeShort(packetId, param.i16_gameId)
    self.m_socket:writeInt(packetId, param.i32_serverId)
    self.m_socket:writeInt(packetId, param.i32_showTid)
    self.m_socket:writeInt(packetId, param.i32_reserved)
    self.m_socket:writeString(packetId, param.str_guid)
    self.m_socket:writeString(packetId, param.str_userInfo)
    self.m_socket:writeInt(packetId, param.i32_flag)
    self.m_socket:writeString(packetId, param.str_password)
end
-- 0X1039 优先到达赛点的玩家对3张手牌的处理
function RoomWriter:onDropThreeCard(packetId, param)
	printInfo("onDropThreeCard write")
	dump(param)
    self.m_socket:writeInt(packetId, param.action)  -- 0： 弃掉 1： 使用
end
-- 0x2001    //请求准备再玩一局   //超时处理先单播玩家站起 广播玩家站起  单播玩家离开
function RoomWriter:onPlayAgain(packetId, param)
	printInfo("0x2001 "..tostring(packetId))
end
-- 0x1042 //玩家请求本回合出暗牌
function RoomWriter:onPutDarkCard(packetId, param)
	printInfo("0x1042 "..tostring(packetId))
end
function RoomWriter:onUpdateUserinfor(packetId, param)
	self.m_socket:writeString(packetId, param.userInfo)
end
--------------
function RoomWriter:initCommandFuncMap()
	RoomWriter.super.initCommandFuncMap(self)
	require('truco.protocol.protocol')
	local s_clientCmdFunMap = {
		[TRUCO_GET_TABLEID_REQ]  	  			= self.onGetTableId,	-- 0x118    -- 请求分配桌子ID
		[TRUCO_FOLLOW_REQ]  					= self.onFollow,	    -- 0x123    -- 跟踪玩家
		[TRUCO_LOGIN_GAME_REQ]					= self.onLoginGame, 	-- 0x1001	-- 请求登录某个游戏
		[TRUCO_SITDOWN_IN_GAME_REQ]				= self.onSitdownInGame,	-- 0x1031	-- 请求坐下
		[TRUCO_STANDUP_REQ]						= self.onStandUp,		-- 0x1033	-- 玩家站起
		[TRUCO_TRUCO_REQ]						= self.onTruco,			-- 0x1035	-- 叫分 client -> server
		[TRUCO_DROP_CARD_REQ]					= self.onDropCard,		-- 0x1036	-- 出牌 client -> server
		[TRUCO_ACCEPT_OR_REFUSE_TRUCO_REQ]		= self.onAcceptOrRefuseTruco,-- 0x1037	-- 接受TRUCO 或放弃
		[TRUCO_CANCEL_DELEGATE_REQ]				= self.onCancelDelegate,-- 0x1038    -- 取消托管  托管类型  1--托管，0--取消托管
		[TRUCO_DROPTHREE_CAR_REQ]				= self.onDropThreeCard,-- 0X1039    -- 优先到达赛点的玩家对3张手牌的处理
		[TRUCO_PLAY_AGAIN_REQ]					= self.onPlayAgain,		-- 0x2001    //请求准备再玩一局   //超时处理先单播玩家站起 广播玩家站起  单播玩家离开
		
		[TRUCO_EXIT_ROOM_REQ] 		  			= self.onExitRoom,
		[TRUCO_SEND_CHAT]			  			= self.onSendChat,
		[TRUCO_SEND_FACE]			  			= self.onSendFace,
        [TRUCO_CREATE_BUDDY_ROOM_REQ]      		= self.onCreateBuddyRoom,
        [TRUCO_ENTER_BUDDY_ROOM_REQ]       		= self.onEnterBuddyRoom,
        [TRUCO_PUTDARKCARD_REQ]					= self.onPutDarkCard,
        [TRUCO_CLIENT_SEND_USERINFOR]           = self.onUpdateUserinfor,
	}
	table.merge(self.s_clientCmdFunMap, s_clientCmdFunMap)
end

return RoomWriter