local playerLayout = requireview("view.truco.trucoPlayerLayout1")

local NumberTimer = require("truco.anim.numberTimer")
local Countdown = require("truco.anim.countdown")
local STRING 	= require('truco.res.config')
local AUDIO 	= require('truco.res.audio.config')
local TrucoBaseController = require('truco.room.trucoBaseController')

require('truco.protocol.protocol')
require('truco.popu.config')
require("pintu.gift")

local AnimationFriend 		= require('animation.friendsAnim.animationFriend')
local RoomController 		= class(TrucoBaseController)
local printInfo, printError = overridePrint("RoomController")

local FrameAnim 	= require("truco.anim.frameAnim")
local TrucoPopuManager = require("truco.popu.trucoPopuManager")
local TrucoGameResult = require("truco.popu.trucoGameResult")
local TrucoFailGameResult = require("truco.popu.trucoFailGameResult")
local TrucoGameMatchElevelTwoPopup = require("truco.popu.trucoGameMatchElevelTwoPopup")
local TrucoGameMatchElevelOnePopup = require("truco.popu.trucoGameMatchElevelOnePopup")
local TrucoNotGameMatchElevelPopup = require("truco.popu.trucoNotGameMatchElevelPopup")
local TableId = require('truco.data.tableId')

local CardManager = require("truco.card.cardManager")
local FOUR_MATCH_CNT = 1
local TWO_MATCH_CNT = 3

local TOTAL_CARD_CNT = 3
local MATCH_POINT = 11

local ERROR_TABLE_MAX_COUNT    	= 2          --桌子人满
local ERROR_NOT_ENOUGH_MONEY   	= 3          --钱太少
local ERROR_TOO_MUCH_MONEY	 	= 4          --钱太多
local ERROR_WRONG_UID			= 5          --无效的用户ID
local ERROR_TABLE_NOT_EXIST	 	= 6          --桌子不存在
local ERROR_ANTE_MONEY         	= 7		   --携带金额错误
local ERROR_SEAT_HAS_PLAYER	 	= 8		   --座位上有人

local BTN_CLICK_INTERVAL = 1000

local DELAY_SERVER_MSG1 = 300
local DELAY_SERVER_MSG2 = 420
local DELAY_SERVER_MSG3 = 240

-- 1为出牌思考状态,2为同意加倍，3为不同意加倍，4叫分标志
local STATUS_ICON_THINK = 1
local STATUS_ICON_ACCEPT = 2
local STATUS_ICON_REJECT = 3
local STATUS_ICON_TRUCO = 4


local SHOW_TASK = true
-- local SELF_DEBUG = true

function RoomController:ctor(viewConfig, param1, param2, param3)
	printInfo("RoomController:ctor()")
	--初始化弹窗
	trucoPopuInit()
	self.mFollowId  = param3[3]
	self.mFollowNick= param3[4]
	self:setRoomCfg(param3[2], param3[5] or GAME_ID.Truco, param3[1], param3[3], param3[4])
	CardManager:getInstance():setDelegate(self)
	self.cardHeap = self:findChildByName("view_dealer")
end

function RoomController:setRoomCfg(roomInfo, gameId, stateFrom, followId, followNick)

	self.mRoomInfo  = roomInfo:clone()
	self.mStateFrom = stateFrom and stateFrom or States.Lobby
	if followId then
		self.mFollowId  = followId
	end
	if followNick then
		self.mFollowNick= followNick or ''
	end
	self.mGameId 	= gameId
	self.isGameOver_ = false
	-- 
	
	self.roomPlayerInfoCache = {}
	-- 用于保存读秒的玩家数据
	self.readTimePlayers_ = {}
	--连胜局数
	self.winTimes = 0
	-- 自动坐下次数
	self.retryAutoSitDownCnt_ = 0
	--送筹码配置
	self.mPresent = self.mRoomInfo:getPresent()

	
	for i = 1, 4 do
		if self.mPlayer and self.mPlayer[i] then
			local ui = self.mPlayer[i]:getUi()
			if ui then
				ui:removeSelf()
			end
		end	
	end
	self.mPlayer  = {};
	for i=1,4 do
		self.mPlayer[i] = setProxy(new(require('truco.data.player')))
	end
	-- 
		--Truco 游戏所需要的数据结构
	self.mTableId = setProxy(new(TableId))
	self.mTableId.ctl = self
	-- self.cardsMgr_ = new(CardsManager, self, self.mTableId)


	self.popupMgr_ = new(TrucoPopuManager, self, self.m_root)
	-- 

    self:initOptBtn()
	self:bindPlayerProxy()
	self:clearPlayData_()
	self:bindTableProxy()
	self:findChildByName("view_dealer"):setVisible(false)
	--好友动画
	self.mAnimFriend = new(AnimationFriend, self)    
	-- 取消托管
	self:findChildByName("btn_canceldelegate"):setOnClick(self, function(self)
		self:cancelDelegateReq()
	end)
	--开始loading
	self:showLoadingAnim();
	if not SELF_DEBUG then
		--请求桌子ID
		self:getTableId()
	end
	-- 
	--self:refreshRoomAnte()
	-- 换肤
	if self.mGameId == 13007 then
		self:findChildByName("img_bg"):setFile("truco/room/13007/truco_tableBg.png")
		local img_table_bg = self:findChildByName("img_table_bg")
		img_table_bg:setFile("truco/room/13007/truco_table.png")
		img_table_bg:setSize(img_table_bg.m_res.m_width, img_table_bg.m_res.m_height)
		self:findChildByName("img_topBar"):setFile("truco/room/13007/board_information.png")
		self:findChildByName("btn_correr"):setFile("truco/room/13007/correr_bg.png")
		self:findChildByName("btn_correr"):findChildByName("Image3"):setFile("truco/room/13007/correr.png")
		self:findChildByName('view_ante'):setPos(72, -5)
	end
end
-- 
function RoomController:refreshRoomAnte()
	local ante = self.mRoomInfo:getAnte()
	self.mTableId:setAnte(ante)
	printInfo("ante:::"..ante)
end
-- 
function RoomController:bindPlayerProxy()
	----------------------------------------------------------------------
	--游戏UI
	----------------------------------------------------------------------
	UIEx.bind(self, MyUserData, "money", function(value)
		local mySeatId = self:getMySeatId() or 0
		if mySeatId > 0 then
			local player = self.mPlayer[mySeatId]
			if value ~= player:getChip() then
				player:setChip(value)
			end
		end
	end)
	-- 
	--初始化座位
	for i = 1, #self.mPlayer do
		local playerData = self.mPlayer[i];
		local player 	 = SceneLoader.load(playerLayout);
		if self.mGameId == 13007 then
			player:findChildByName("relationIcon2"):setFile("truco/room/13007/11.png")
		end
		playerData:setUi(player);
		playerData.index = i
		
		UIEx.bind(player, playerData, "localSeatId", function(value)
			
			local index = playerData.index
			if value > 0 then
				self:resetAIIconPos(value, player)
			end
			if tostring(self.mGameId) == tostring(GAME_ID.Truco) then
				if value== 1  or  value== 3 then
					player:findChildByName("btn_invite"):setVisible(false)
					player:findChildByName("btn_gift"):setVisible(false)
					return 
				end
				if value == 2 or value == 4  then
					player:findChildByName("btn_invite"):setVisible(true) 
				else
					player:findChildByName("btn_invite"):setVisible(false)
					
				end
			
			end
			if playerData:getId() > 0  then
				-- JLog.d("test localSeatId ##"..playerData:getId(),value);
            	player:findChildByName("btn_invite"):setVisible(false) 
            	player:findChildByName("btn_gift"):setVisible(true) 
            	
            else
            	player:findChildByName("btn_invite"):setVisible(true)
            	player:findChildByName("btn_gift"):setVisible(false)
           	end

		end)
		UIEx.bind(player, playerData, "showInfo", function(value)
			player:findChildByName('player_info'):setVisible(value)
		end)
		UIEx.bind(player, playerData, "isWinner", function(value)
			-- if value then
			-- 	local winView = player:findChildByName("view_win");
			-- 	local animPic = new(Image, 'truco/anim/head_win/head_win1.png');
			-- 	winView:addChild(animPic);
			-- 	animPic:setAlign(kAlignCenter);
			-- 	-- 动画图片
			-- 	local anim_pics = {
			-- 		"truco/anim/head_win/head_win1.png",
			-- 		"truco/anim/head_win/head_win2.png",
			-- 		"truco/anim/head_win/head_win3.png",
			-- 		"truco/anim/head_win/head_win4.png",
			-- 		"truco/anim/head_win/head_win5.png",
			-- 		"truco/anim/head_win/head_win6.png",
			-- 		"truco/anim/head_win/head_win7.png",
			-- 		"truco/anim/head_win/head_win8.png",
			-- 		"truco/anim/head_win/head_win9.png",
			-- 		"truco/anim/head_win/head_win10.png",
			-- 		"truco/anim/head_win/head_win11.png",
			-- 		"truco/anim/head_win/head_win12.png",
			-- 		"truco/anim/head_win/head_win13.png",
			-- 	}

			-- 	local animParam = {
			-- 		frames = anim_pics,
			-- 		inteval = 150,
			-- 		repeatCount = 1,
			-- 		delay = 50,
			-- 		onComplete = function()
			-- 			winView:removeAllChildren();
			-- 		end,
			-- 		DebugName = "headWinAnim.play"
			-- 	}
			-- 	animPic:playAnimation(animParam);
			-- end
		end)

		UIEx.bind(player, playerData, "id", function(value)
			local idx = playerData.idx
			local isVisible = false

			if value > 0 then
				isVisible = true
				player:findChildByName("btn_gift"):setVisible(true)
			else
				player:findChildByName('chipView'):removeAllChildren()
				player:findChildByName("btn_gift"):setVisible(false)
			end
			player:findChildByName("relationIcon1"):setVisible(isVisible)
			player:findChildByName("view_style1"):setVisible(isVisible)
			player:findChildByName("view_phead"):removeAllChildren()
            --只有好友房并且我自己已经坐下了，邀请按钮才会可见
            if self.mRoomInfo:getLevel() == TrucoBaseController.BUDDY_ROOM_LEVEL and self.mIsSitDown then 
                if value == MyUserData:getId() then
                    --当我自己坐下的时候，我要让周围的空位置变成邀请按钮
                    for index = 1, #self.mPlayer do
                        local curPlayer = self.mPlayer[index]:getUi()
                        --空位置
                        local isEmpty = (self.mPlayer[index]:getId() == 0)
                        curPlayer:findChildByName("btn_player"):setVisible(not isEmpty)
                    end
                else
                    --别人坐下的时候，这个位置的邀请按钮要消失,别人离开，邀请按钮可见
                    local isVisible = value ~= 0
                    player:findChildByName("btn_player"):setVisible(isVisible)
                end
            else
            	-- if playerData:getId() > 0 and playerData:getLocalSeatId() > 0 then
            	-- 	local uid = playerData:getUi()
            	-- 	uid:findChildByName("btn_invite"):setVisible(false)
            	-- end
                --我还没坐下的时候，邀请按钮都是不可见的
                player:findChildByName("btn_player"):setVisible(true)
            end
		end)
        player:findChildByName("btn_invite"):setOnClick(self, function(self)

        	local idx = playerData.index
			local id = playerData:getId()
            local mySeat = self:getMySeatId()
            local seatId = playerData:getSeatId()
			if id == 0 and not mySeat then
				local seatId = playerData:getSeatId();
                if self:checkSitDown() then
				    GameSocketMgr:sendMsg(TRUCO_SITDOWN_IN_GAME_REQ, {u32_seatId = seatId, u64_ante = MyUserData:getMoney()});
                end
			end

			if id ~= 0 and playerData:getLocalSeatId() ~= 0 then
				--弹出个人信息
                local mySeatId = self:getMySeatId() or 0
                local curplayer= mySeatId > 0 and self.mPlayer[mySeatId] or nil;
                --记录登录玩家
	            ---弹出个人信息
                local playerIds = {}
                for i = 1, #self.mPlayer do
                    if self.mPlayer[i]:getId() ~= 0 then
                        table.insert(playerIds, self.mPlayer[i]:getId())
                    end
                end
                --自己没坐下，也就是围观中，广播通知的人包括我自己
                if not mySeat then
                    table.insert(playerIds, MyUserData:getId())
                end
                local windowParam = {
                	player = playerData, 
                	present = self.mPresent, 
                	gameId = self.mGameId, 
                	seatId = self:getMySeatId() or 0, 
                	room = self.mRoomInfo,
                	cache = self.roomPlayerInfoCache,
                	playerIds = playerIds}
                WindowManager:showWindow(WindowTag.RoomPlayerInfoPopu, windowParam, WindowStyle.POPUP);
			end

            -- local curGame = app:getGame(self.mGameId)
            -- local gameName = ""
            -- if curGame then
            --     gameName = curGame:getName() or ""
            -- end
            -- WindowManager:showWindow(WindowTag.BuddyRoomInvitePopu, {table = self.mTableId}, WindowStyle.POPUP)
        end)
		UIEx.bind(player, playerData, "ready", function(value)
			--如果是本玩家，则需要按制准备按钮
			if playerData:getId() == MyUserData:getId() then
				playerData:setBeReady(not value)
			end
		end)
		
		UIEx.bind(player, playerData, "ai", function(value)

			function opreateBtnsEnable(enable)
				self:findChildByName("btn_dark"):setEnable(enable)
				self:findChildByName("btn_truco"):setEnable(enable)
				self:findChildByName("btn_fold"):setEnable(enable)
				self:findChildByName("btn_doze"):setEnable(enable)
				self:findChildByName("btn_nove"):setEnable(enable)
				self:findChildByName("btn_seis"):setEnable(enable)
				self:findChildByName("btn_correr"):setEnable(enable)
				self:findChildByName("btn_aceitar"):setEnable(enable)
			end

			player:findChildByName('img_ai'):setVisible(value)
			if playerData:getId() == MyUserData:getId() then
				self:findChildByName('btn_canceldelegate'):setVisible(value)
				-- self.cardsMgr_:allOwnCardEnable(not value)
				opreateBtnsEnable(not value)
				if value then
					self:resetAIIconPos(playerData:getLocalSeatId(), player)
				end
			end
		end)

		UIEx.bind(player, playerData, "chip", function(value)
			value = value or 0
			if playerData:getId() < 1 then
				player:findChildByName('chipView'):removeAllChildren()
				return
			end

			if playerData:getId()==MyUserData:getId() and value ~= MyUserData:getMoney() then
				MyUserData:setMoney(value);
			end
			
			if value > 0 then
				local chipNode = new(require('truco.common.imageNumber'), TrucoBaseController.CHIP_RES_CONFIG);	
				chipNode:setNumber(string.format("%s", ToolKit.formatMoney(value)), 0, true)
				--chipNode:setNumber('2.01b', 0, true)
				chipNode:setAlign(KAlignLeft)
				player:findChildByName('chipView'):removeAllChildren()
				player:findChildByName('chipView'):addChild(chipNode)
			end
		end)
		-- 
		UIEx.bind(player, playerData, "nick", function(value)
			value = value or ""
            local length = ToolKit.utf8_isAllEnglish(value) and 7 or 12
			player:findChildByName("text_nick"):setText(ToolKit.formatNick(value, length))
		end)
		-- 
		UIEx.bind(player, playerData, "headName", function(value)
			printInfo("truco headName:::"..tostring(value))
			if playerData:getId() ~= 0 then
				local headView = player:findChildByName("view_phead");
				headView:removeAllChildren();
				local width, height = headView:getSize()
				local imgHead 		= new(Mask, value, "truco/head_mask.png");
				imgHead:setSize(width, height)
				headView:addChild(imgHead);
				playerData:checkHeadAndDownload()

				--player:findChildByName("btn_invite"):setVisible(false)
			end
		end) 
		-- 
		UIEx.bind(player, playerData, "sitdownAnim", function(value)
			if value then
				self.isSitDownAnim_ = true
				local viewHead 	= player:findChildByName("view_phead");
				local x, y  	= viewHead:getAbsolutePos();
				local w, h  	= viewHead:getSize();
				local sw, sh 	= System.getScreenScaleWidth(), System.getScreenScaleHeight() --1280, 720;
				local dx, dy 	= (sw - w) / 2 - x, (sh - h) / 2 - y;
				checkAndRemoveOneProp(viewHead, 1);
				local sitdownAnim = viewHead:addPropTranslateWithEasing(1, kAnimNormal, 500, 0, "easeOutSine", "easeOutSine", 0, dx, 0, dy);
				sitdownAnim:setEvent(self, function()
					checkAndRemoveOneProp(viewHead, 1);
					--重新排位
					self:reLocateSeat();
					local x, y  	= viewHead:getAbsolutePos();
					local w, h  	= viewHead:getSize();
					local sw, sh 	= System.getScreenScaleWidth(), System.getScreenScaleHeight() --1280, 720;
					local dx, dy 	= (sw - w) / 2 - x, (sh - h) / 2 - y;
					local sitdownAnim = viewHead:addPropTranslateWithEasing(2, kAnimNormal, 500, 0, "easeOutSine", "easeOutSine", dx, -dx, dy, -dy);
					sitdownAnim:setEvent(self, function()
						checkAndRemoveOneProp(viewHead, 2)
						--开始准备
                        if MyUserData:getId() == playerData:getId() then
                            playerData:setGiftId(MyUserData:getGiftId() and {MyUserData:getGiftId()[1], MyUserData:getGiftId()[2]})
                            --player:findChildByName("relationIcon1"):setVisible(true)
                            --playerData:setShowInfo(true)
                        end

                        self:sitdownAnimCallback()
                        
					end);

					if self.resetOutCardCallback_ then
						self.resetOutCardCallback_()
						self.resetOutCardCallback_ = nil
						
					end
					self.isSitDownAnim_ = nil

				end)
			end
		end)
		--只针对自己有效
		UIEx.bind(player, playerData, "inAnim", function(value)
			if playerData:getId() == MyUserData:getId() then
				if value then
					player:setPos(self:getLocalSeatPos(TrucoBaseController.MY_LOCAL_SEAT_ID, true))
					checkAndRemoveOneProp(player, 1);
					local inAnim = player:addPropTranslateWithEasing(1, kAnimNormal, 500, 0, "easeOutSine", "easeOutSine", 0, 0, 120, -120);
					inAnim:setEvent(self, function()
						checkAndRemoveOneProp(player, 1);
					end);
				else --弹出
					local toX, toY = self:getLocalSeatPos(TrucoBaseController.MY_LOCAL_SEAT_ID);
					checkAndRemoveOneProp(player, 1);
					checkAndRemoveOneProp(player, 2);
					local x, y = player:getAbsolutePos();
					local inAnim = player:addPropTranslateWithEasing(1, kAnimNormal, 500, 0, "easeOutSine", "easeOutSine", 0, toX - x, 0, toY - y);
					inAnim:setEvent(self, function()
						checkAndRemoveOneProp(player, 1);
						player:setPos(self:getLocalSeatPos(TrucoBaseController.MY_LOCAL_SEAT_ID));
						-- player:findChildByName('img_nickbg'):setPos(2, -35)
					end);
					-- player:addPropScale(2, kAnimNormal, 500, 0, 1.15, 1.0, 1.15, 1.0, kCenterDrawing);
				end
			end
		end)
        -- 显示玩家的礼物图标
        UIEx.bind(player, playerData, "giftId", function(value)
            local giftList = MyUserData:getGiftList()
            if value and value[1] and value[2] and giftList then
                local gift = giftList[value[1]] and giftList[value[1]][value[2]]
                if gift then
                    player:findChildByName("btn_gift"):setFile(gift:getImgName())
                    if playerData:getId() == MyUserData:getId() then
                    	local userInfo = {appid = 0, nick = MyUserData:getNick(), sex = MyUserData:getSex(), micon = MyUserData:getHeadUrl(), giftId = MyUserData:getGiftId()}
						GameSocketMgr:sendMsg(TRUCO_CLIENT_SEND_USERINFOR, {userInfo = json.encode(userInfo)});
					end
                    return
                end
            end
            player:findChildByName("btn_gift"):setFile(gift_map["gift.icon_gift.png"])
        end)
		--玩家准备时间
		UIEx.bind(player, playerData, "readyTime", function(value, oldData)
			--JLog.d('hehe readytime',value,self.mTableId:getTotalCardTime())
			local circleProgress = player:findChildByName('circleProgress')

			local isShow = false
			if value > 0 then
				isShow = true
				local total_time = self.mTableId:getTotalCardTime()
				if (not total_time) or (total_time < value) then total_time = 15 end

				circleProgress:playTimerAnim(total_time, value, self, function (self, second)

					if value and self:getMyPlayer() and self:getMyPlayer():getId() == playerData:getId() and  self.mTableId:getTotalCardTime() - second <= 3 then  -- 倒数3秒
						kEffectPlayer:play(AUDIO.TIME_WARNING)
					end
				end)
				table.insert(self.readTimePlayers_, playerData)
			else
				-- self:hideAllOprBtn()
				circleProgress:stopCircleAnim()
			end
			circleProgress:setVisible(isShow)

			--倒计时
			local img_clock = player:findChildByName('img_clock')
			img_clock:setVisible(value > 0)
			if value > 0 then
				local viewClockNum_ = player:findChildByName('view_clockNum')
				checkAndRemoveOneProp(viewClockNum_, 11)
				viewClockNum_:addPropScaleSolid(11, 0.6, 0.6, kNotCenter)
				local nodeClock = new(require('truco.common.imageNumber', -5), {
					    ['0'] = 'truco/room/truco_downTime/0.png',
					    ['1'] = 'truco/room/truco_downTime/1.png',
					    ['2'] = 'truco/room/truco_downTime/2.png',
					    ['3'] = 'truco/room/truco_downTime/3.png',
					    ['4'] = 'truco/room/truco_downTime/4.png',
					    ['5'] = 'truco/room/truco_downTime/5.png',
					    ['6'] = 'truco/room/truco_downTime/6.png',
					    ['7'] = 'truco/room/truco_downTime/7.png',
					    ['8'] = 'truco/room/truco_downTime/8.png',
					    ['9'] = 'truco/room/truco_downTime/9.png',
				    })
				viewClockNum_:removeAllChildren()
			    viewClockNum_:addChild(nodeClock)
			    nodeClock:setNumber(string.format("%s", tostring(value)));
			    nodeClock:setVisible(true)
			    nodeClock:setAlign(kAlignCenter)
			    nodeClock:setPos(18, 20)

			    local downClockNum_ = value
			    local animEvent = img_clock:addPropTranslate(10, kAnimLoop, 1000, 0, 0, 0, 0, 0)
				animEvent:setEvent(self, function()
					downClockNum_ = downClockNum_ - 1
					if downClockNum_ >= 0 then
						nodeClock:setNumber(string.format("%s", tostring(downClockNum_)))
					else
						checkAndRemoveOneProp(img_clock, 10)
					end
				end)
			end
		end)
		--玩家身份
		UIEx.bind(player, playerData, "status", function(value, oldData)
			
		end)
		--出牌提示
		UIEx.bind(player, playerData, "playCard", function(value)
			
		end)
		--玩家头像样式
		UIEx.bind(player, playerData, "headStyleIdx", function(value)
			player:findChildByName("relationIcon1"):setVisible(false)
			player:findChildByName("relationIcon2"):setVisible(false)
			player:findChildByName("view_style1"):setVisible(false)
			player:findChildByName("view_style2"):setVisible(false)
			-- if playerData:getSeatId() < 1 then
			-- 	return
			-- end

			if value and value > 0 then
				if value == 1 then
					player:findChildByName("relationIcon1"):setVisible(true)
					player:findChildByName("view_style1"):setVisible(true)
				elseif value == 2 then
					player:findChildByName("relationIcon2"):setVisible(true)
					player:findChildByName("view_style2"):setVisible(true)
				end
			end
		end)
		-- 玩家状态图标 1为出牌思考状态,2为同意加倍，3为不同意加倍，4叫分标志
		UIEx.bind(player, playerData, "statusIcon", function(value)
			local cleanViewStatus = function()
				local viewStatus 	= player:findChildByName("view_status");
				viewStatus:removeAllChildren()
			end

			if player:findChildByName("img_ai"):getVisible() then
				cleanViewStatus()
				return;
			end

			local fileName = nil
			local txt_Icon = nil
			local repeat_Icon = nil
			local px, py = 10, -80
			--local offx, offy = 30, -30
			local isDelayDel = false
			local ts = 280
			local startScaleX, endScaleX = 0.1, 1

			if value > 0 then
				--cleanViewStatus()

				if value == STATUS_ICON_THINK then
					--fileName = "truco/room/status_icon/icon1.png"
					--isDelayDel = true
					
				elseif value == STATUS_ICON_ACCEPT then
					--fileName = "truco/room/status_icon/icon2.png"
					--repeat_Icon = "truco/room/status_icon/accept_repeat_icon.png"
					self:play_fa(player,playerData:getLocalSeatId(),6)
					kEffectPlayer:play(AUDIO.TRUCO_SCORE_ACCEPT)
					return
				elseif value == STATUS_ICON_REJECT then
					--fileName = "truco/room/status_icon/icon3.png"
					--repeat_Icon = "truco/room/status_icon/refuse_repeat_icon.png"
					self:play_fa(player,playerData:getLocalSeatId(),8)
					kEffectPlayer:play(AUDIO.TRUCO_SCORE_CORRER)
					return
				elseif value == STATUS_ICON_TRUCO then
					kEffectPlayer:play(AUDIO.TRUCO_SCORE)
					local statusScore = playerData:getStatusScore()
					-- fileName = "truco/room/status_icon/icon4.png"
					-- if statusScore == 3 or statusScore == 6 or statusScore == 9 or statusScore == 12 then
					-- 	txt_Icon = "truco/room/status_icon/truco_trucoTxt_"..statusScore..".png"
					-- else
					-- 	txt_Icon = "truco/room/status_icon/truco_trucoTxt_3.png"
					-- end
					-- px = 0
					local flag = nil
					if self.mGameId == 13007 then
						local status_map = {[0] = 2, [1] = 2,[3] = 3,[6] = 4,[9] = 5}
						flag = status_map[self.mTableId:getBetScore()]
					else
						local status_map = {[3] = 2,[6] = 3,[9] = 4,[12] = 5}
						flag = status_map[statusScore]
					end
					if flag then
						self:play_fa(player,playerData:getLocalSeatId(),flag)
					end
					return
				end
			else
				cleanViewStatus()
				local imgIcon = playerData:getImgIcon()
				-- if imgIcon and imgIcon.m_children then
				-- 	local w, h = imgIcon:getSize()
				-- 	local animMove = imgIcon:addPropTransparency(1, kAnimRepeat, 320, 0, 1, 0)
				-- 	local scaleAnim = imgIcon:addPropScale(3, kAnimNormal, 320, delayTs, 1, 0.1, 1, 0.1,kCenterXY, w/2, h/2)
				-- 	scaleAnim:setEvent(self, function()
				-- 		cleanViewStatus()
				-- 	end)					
				-- end
				playerData:setImgIcon(nil)
			end
		end)

		UIEx.bind(player, playerData, "newTurn", function(value)

		end)

		UIEx.bind(player, playerData, "enableCards", function(value)
			if playerData:getId() == MyUserData:getId() then

			end
		end)

		UIEx.bind(player, playerData, "cards", function(value)
			if playerData:getId() == MyUserData:getId() then
				if value then
					-- self.cardsMgr_:renderSelfCards(value)
				else

				end
			else
				if value then
					local localSeatId = playerData:getLocalSeatId()
					-- self.cardsMgr_:renderOtherCards(localSeatId, value)
				else
					
				end		
			end			
		end)
		--调置牌数量
		UIEx.bind(player, playerData, "cardNumber", function(value)
			
		end)
		-- 
		UIEx.bind(player, playerData, "exchangeChip", function(value)
			local chipNode
			local chipStr = "+"..tostring(math.abs(value))
			if value > 0 then
				chipNode = new(require('truco.common.imageNumber'), TrucoBaseController.EXCHANGE_CHIP_RES_CONFIG_ADD);
			else
				chipNode = new(require('truco.common.imageNumber'), TrucoBaseController.EXCHANGE_CHIP_RES_CONFIG_SUB);
				chipStr = "-"..tostring(math.abs(value))
			end
			chipNode:setNumber(chipStr)
			chipNode:setAlign(kAlignCenter)
			-- 
			local viewChipAnim = player:findChildByName('view_chipAnim')
			viewChipAnim:removeAllChildren()
			viewChipAnim:addChild(chipNode)

			local localSeatId = playerData:getLocalSeatId()
			local px, py = 0, 0
			if localSeatId == 1 then
				--px = -200
				chipNode:setAlign(kAlignRight)
			elseif localSeatId == 3 or localSeatId == 4 then
				--px = 200
				chipNode:setAlign(kAlignLeft)
			end
			chipNode:setPos(px, py)
			-- 
	        local animExpMove = chipNode:addPropTranslate(1, kAnimNormal, 500, 0, 0, 0, 0, -30)
		    local animExpParency = chipNode:addPropTransparency(2, kAnimNormal, 500, 0, 0, 1)
		    animExpParency:setEvent(animAddExp, function ()
		    	checkAndRemoveOneProp(chipNode, 1)
		    	checkAndRemoveOneProp(chipNode, 2)
		    	-- chipNode:setPos(0, 0)
		    	-- local animExpMove2 = chipNode:addPropTranslate(3, kAnimNormal, 1000, 1000, 0, 0, 0, -60)
		    	local animExpParency2 = chipNode:addPropTransparency(4, kAnimNormal, 1000, 1000, 1, 0)
		    	animExpParency2:setEvent(animAddExp, function ()
		    		-- checkAndRemoveOneProp(chipNode, 3)
		    		checkAndRemoveOneProp(chipNode, 4)
		    		viewChipAnim:removeAllChildren()
		    	end)
	        end)
		end)
		--等待倒计时
		UIEx.bind(player, playerData, "downNum", function(value)
			local imgDownNum = player:findChildByName("img_DownNum")
			local viewDownNum = player:findChildByName("view_DownNum")
			local imgNoReady = player:findChildByName("img_noReady")
			if value <= 0 then
				checkAndRemoveOneProp(viewDownNum, 100)
				checkAndRemoveOneProp(viewDownNum, 101)
				imgDownNum:setVisible(false)
				imgNoReady:setVisible(false)
			else
				local localSeatId = playerData:getLocalSeatId()
				if localSeatId == 1 then
					-- imgDownNum:setPos(80, 30)
				else
					-- imgDownNum:setPos(-45, 30)
				end
				local downNum = value
				imgDownNum:setVisible(true)
				if MyUserData:getId() ~= playerData:getId() then
					imgNoReady:setVisible(true)
				end

				viewDownNum:setPos(8, 8)
				checkAndRemoveOneProp(viewDownNum, 100)
				checkAndRemoveOneProp(viewDownNum, 101)
				viewDownNum:addPropScaleSolid(101, 0.7, 0.7, kNotCenter)
				local refreshDownNumFunc = function()
					if downNum >= 0 then
						local nodeClock = new(require('truco.common.imageNumber', -8), TrucoBaseController.TRUCO_DOWNTIME_RES_CONFIG)
					    viewDownNum:removeAllChildren()
					    viewDownNum:addChild(nodeClock)
					    local downNumStr = tostring(downNum)
					    if string.len(downNumStr) == 1 then
					    	viewDownNum:setPos(8, 8)
					    else
					    	viewDownNum:setPos(6, 8)
					    end
					    nodeClock:setNumber(downNumStr);
					    nodeClock:setAlign(kAlignCenter)
				    	downNum = downNum - 1
				    else
				    	checkAndRemoveOneProp(viewDownNum, 100)
				    	viewDownNum:removeAllChildren()
				    	imgDownNum:setVisible(false)
				    end
				end
				
				refreshDownNumFunc()
				local animEvent = viewDownNum:addPropTranslate(100, kAnimLoop, 1000, 0, 0, 0, 0, 0)
				animEvent:setEvent(self, refreshDownNumFunc)
			end
		end)
		-- 
		player:findChildByName("btn_player"):setOnClick(self, function ( self )
			local idx = playerData.index
			local id = playerData:getId()
            local mySeat = self:getMySeatId()
            local seatId = playerData:getSeatId()
			if id == 0 and not mySeat then
				local seatId = playerData:getSeatId();
                if self:checkSitDown() then
				    GameSocketMgr:sendMsg(TRUCO_SITDOWN_IN_GAME_REQ, {u32_seatId = seatId, u64_ante = MyUserData:getMoney()});
                end
			end

			if id ~= 0 and playerData:getLocalSeatId() ~= 0 then
				--弹出个人信息
                local mySeatId = self:getMySeatId() or 0
                local curplayer= mySeatId > 0 and self.mPlayer[mySeatId] or nil;
                --记录登录玩家
	            ---弹出个人信息
                local playerIds = {}
                for i = 1, #self.mPlayer do
                    if self.mPlayer[i]:getId() ~= 0 then
                        table.insert(playerIds, self.mPlayer[i]:getId())
                    end
                end
                --自己没坐下，也就是围观中，广播通知的人包括我自己
                if not mySeat then
                    table.insert(playerIds, MyUserData:getId())
                end
                local windowParam = {
                	player = playerData, 
                	present = self.mPresent, 
                	gameId = self.mGameId, 
                	seatId = self:getMySeatId() or 0, 
                	room = self.mRoomInfo,
                	cache = self.roomPlayerInfoCache,
                	playerIds = playerIds}
                WindowManager:showWindow(WindowTag.RoomPlayerInfoPopu, windowParam, WindowStyle.POPUP);
			end
		end)
        --点击弹窗礼物窗口
        player:findChildByName("btn_gift"):setOnClick(self, function(value)
            local playerIds = {}
            for i = 1, #self.mPlayer do
                if self.mPlayer[i]:getId() ~= 0 then
                    table.insert(playerIds, self.mPlayer[i]:getId())
                end
            end
            local mySeat = self:getMySeatId()
            --自己没坐下，也就是围观中，广播通知的人包括我自己
            if not mySeat then
                table.insert(playerIds, MyUserData:getId())
            end
            --点击的是自己的头像上的，弹窗带购买
            if playerData:getId() == MyUserData:getId() then
                WindowManager:showWindow(WindowTag.GiftStorePopu, {from = "myInfoInGame", broadcastId = playerIds}, WindowStyle.POPUP)
            else
                WindowManager:showWindow(WindowTag.GiftStorePopu, {from = "otherInfoInGame", sendId = {playerData:getId()}, broadcastId = playerIds}, WindowStyle.POPUP)
            end
        end)
        self:findChildByName("view_players"):addChild(player);
		--进度条
		local circleProgress = new(require('truco.anim.headProgress'), "truco/anim/head_progress/circle_bg.png");
		--circleProgress:setSize(84, 84)
		circleProgress:setSize(98, 98)
		circleProgress:setName('circleProgress')
		circleProgress:setVisible(false)
		circleProgress:setAlign(kAlignCenter);
		--circleProgress:setPos(-1, -2)
		player:findChildByName('view_progress'):addChild(circleProgress)
		self:findChildByName("view_players"):addChild(player);
		player:findChildByName('view_progress'):setLevel(99)
		-- if self.mTableId:isTwoMatch() then
		-- if tostring(self.mGameId) == tostring(GAME_ID.Truco) then
		-- 	if i == 1 or i == 3 then
		-- 		player:findChildByName("btn_player"):setVisible(false)
		-- 	end
		-- end
		-- AlarmTip.play("self.mGameId:::"..tostring(self.mGameId))
	end
end
-- 
function RoomController:bindTableProxy()
	local redPoint_pics = {}
	local bluePoint_pics = {}
	for i=2,16 do
		-- "truco/room/truco_point/hong0002.png",
		table.insert(redPoint_pics, string.format("truco/room/truco_point/hong%04d.png", i))
		-- "truco/room/truco_point/lan0002.png",
		if self.mGameId == 13007 then
			table.insert(bluePoint_pics, string.format("truco/room/13007/truco_point/lv%04d.png", i))
		else
			table.insert(bluePoint_pics, string.format("truco/room/truco_point/lan%04d.png", i))
		end
	end
	self.redPointAnims_ = {}
	self.bluePointAnims_ = {}
	for i=1,3 do
		local redSpr = self:findChildByName("Img_redPoint"..i)
		local redAnim = new(FrameAnim, redPoint_pics, 0, 1, 50, redSpr)
		self.redPointAnims_[i] = {redSpr, redAnim}
		-- 
		local blueSpr = self:findChildByName("Img_bluePoint"..i)
		local blueAnim = new(FrameAnim, bluePoint_pics, 0, 1, 50, blueSpr)
		if self.mGameId == 13007 then
			blueSpr:setFile("truco/room/13007/truco_point/lv0016.png")
			blueSpr:setSize(32,32)
			blueAnim:setSize(32,32)
			blueAnim:setAlign(kAlignCenter)
		end
		self.bluePointAnims_[i] = {blueSpr, blueAnim}
	end
	----------------------------------------------------------------------
	--游戏逻辑
	----------------------------------------------------------------------
	UIEx.bind(self.m_root, self.mTableId, "gameId", function(value)
		local tmpGameId = self.mTableId:getGameId()
		if tostring(tmpGameId) ~= tostring(self.mGameId) then
			if tmpGameId ~= 13004 and tmpGameId ~= 13006 then
				-- StateChange.changeState(self.mStateFrom or States.GameLobby);
				local game = app:getGame(self.mTableId:getGameId())
				if game then 
					AlarmTip.play(string.format(STR_GO_BACK_TO_ORIGIN_GAME, game:getName()))
					game:enterRoom() 
				end
				return
			end
			local game = app:getGame(value)
			AlarmTip.play(string.format(STR_GO_BACK_TO_ORIGIN_GAME, game:getName()))
			local newRoomInfo = game:getRoomFromLevel(self.mTableId.gameLevel)
			self:setRoomCfg(newRoomInfo, value)
			self:sitdownAnimCallback()
		end

		local userInfo = {appid = 0, nick = MyUserData:getNick(), sex = MyUserData:getSex(), micon = MyUserData:getHeadUrl(), giftId = MyUserData:getGiftId()}
		--好友房不走登陆流程，在收到进入好友房的请求之后，自动走了一遍登陆流程，好友跟随如果是好友房也走得进入好友房的流程
        if self.mRoomInfo:getLevel() ~= TrucoBaseController.BUDDY_ROOM_LEVEL then
			if self.mFollowId then
				GameSocketMgr:sendMsg(TRUCO_FOLLOW_REQ, {
					u32_follow 	= self.mFollowId,
					str_key 	= MyUserData:getSessionId(),
					str_info 	= json.encode(userInfo),
					str_followNick = MyUserData:getNick(),
				});
			else
				GameSocketMgr:sendMsg(TRUCO_LOGIN_GAME_REQ, {
					u32_tableId = self.mTableId:getId(), 
					u32_uid = MyUserData:getId(),
					str_key = MyUserData:getSessionId(),
					str_info= json.encode(userInfo),
					u32_flag= 0
				});
			end
		--好友房在正常进入好友房的流程里，系统自动登录，不需要走登陆流程
        --如果重启后断线重连，申请进入好友房返回onGetTableId的报文，需要手动登陆
        elseif self.mTableId:getIsInTable() == 1 then
            GameSocketMgr:sendMsg(TRUCO_LOGIN_GAME_REQ, {
            	u32_tableId = self.mTableId:getId(), 
			    u32_uid = MyUserData:getId(),
			    str_key = MyUserData:getSessionId(),
			    str_info= json.encode(userInfo),
			    u32_flag= 0
			});
	    end
	end)
	--桌子人数
	UIEx.bind(self.m_root, self.mTableId, "count", function(value)
		local myPlayer = self:getMyPlayer()
		if not myPlayer then
			self:cleanAllStatus()		
		else
			local playerStatus = myPlayer:getStatus()
			-- if value < 2 then
			-- 	self:showOrhideReadView(true)
			-- else
			-- 	self:showOrhideReadView(false)
			-- end			
		end
		-- self:showOrhideReadView()
	end)
	-- 0x1054    -- 发3张手牌 server -> client
	UIEx.bind(self.m_root, self.mTableId, "handCardValues", function(value)
		if not value then
			printInfo("handCardValues:::value is nil  "..tostring(value))
			return
		end
		-- 
		if self.isReloginInitHandCardValues_ then
			self.isReloginInitHandCardValues_ = nil
			printInfo("handCardValues:::self.isReloginInitHandCardValues_")
			return 
		end

		local seatId = self.mTableId:getOprSeatId()
		local localSeatId = self:getLocalSeatId(seatId)
		local isCardOver = 1
		if MATCH_POINT == self.lastSelfScore_ and self.lastRivalScore_ == MATCH_POINT then
			isCardOver = 0
		end

		
	
		
	end)
	-- 0x4010          //！开始游戏 玩家同伴关系 server -> client
	UIEx.bind(self.m_root, self.mTableId, "startGamePlayerList", function(d)
		--JLog.d('set startGamePlayerList',d)
		local teamNicks = {"", ""}
		for i=1,#d do
			local user = d[i]
			local player = self.mPlayer[user.seatId]
			local headStyleIdx = user.headStyleIdx
			if headStyleIdx and headStyleIdx > 0 then
				player:setHeadStyleIdx(headStyleIdx)
				-- 
				local nick = player:getNick()
				if self.mTableId:isTwoMatch() then
					local length = ToolKit.utf8_isAllEnglish(nick) and 16 or 14
					nick = ToolKit.formatNick(nick, length)
				else
					local length = ToolKit.utf8_isAllEnglish(nick) and 8 or 7
					nick = ToolKit.formatNick(nick, length)
				end
				-- 
				if teamNicks[headStyleIdx] == "" then
					teamNicks[headStyleIdx] = nick
				else
					teamNicks[headStyleIdx] = teamNicks[headStyleIdx] .. ","..nick
				end
			end
		end
		-- 
		for i=1,#teamNicks do
			-- local length = ToolKit.utf8_isAllEnglish(value) and 15 or 16
			-- self:findChildByName('lbl_teamNick'..i):setText(ToolKit.formatNick(value, length))

			--self:findChildByName('lbl_teamNick'..i):setText(teamNicks[i])
		end
	end)
	-- 被翻过来的那张牌
	UIEx.bind(self.m_root, self.mTableId, "viraCardValue", function(value)
		
		if value and value > 0 then
		print_string("111111111111111111")
			CardManager:getInstance():showOrHideUnderCard(self.cardHeap,value)
		else
		print_string("22222222222222222")
			CardManager:getInstance():showOrHideUnderCard(self.cardHeap,0)	
		end
	end)
	-- 更新桌面双方积分
	UIEx.bind(self.m_root, self.mTableId, "score", function(value)
		--JLog.d('self.mTableId_score',self.lastSelfScore_,self.lastRivalScore_)
		local gamePlayerList = self.mTableId:getStartGamePlayerList()
		if gamePlayerList then
			for i=1,#gamePlayerList do
				local player = gamePlayerList[i]
				if player.headStyleIdx == 1 then
				    if not self.lastSelfScore_ or self.lastSelfScore_ ~= player.score then
				    	self.lastSelfScore_ = player.score
				    	if value then
				    		self:delayCall(function()
					    		self:refreshScore(self.lastSelfScore_<0 and 0 or self.lastSelfScore_, "view_score1")
					    	end, 800)
				    	else
				    		self:refreshScore(self.lastSelfScore_<0 and 0 or self.lastSelfScore_, "view_score1")
				    	end				    	
				    end
				elseif player.headStyleIdx == 2 then
				 	if not self.lastRivalScore_ or self.lastRivalScore_ ~= player.score then
				    	self.lastRivalScore_ = player.score 
						if value then
				    		self:delayCall(function()
					    		self:refreshScore(self.lastRivalScore_, "view_score2")
					    	end, 800)
				    	else
				    		self:refreshScore(self.lastRivalScore_, "view_score2")
				    	end
				    end
				else
					printInfo('truco Error:: UIEx.bind(self.m_root, self.mTableId, "score", function(value)')
				end
			end
		end
	end)
	-- 当前是第几局（一共打三局）
	UIEx.bind(self.m_root, self.mTableId, "match", function(value)
		
	end)	
	-- 
	UIEx.bind(self.m_root, self.mTableId, "name", function(value)
		
	end)
	-- 当前是第几回合（一轮三回合）
	UIEx.bind(self.m_root, self.mTableId, "round", function(value)
		if nil == value or value < 1 then
			value = 0
		end
	end)
	-- 自己每一回合状态
	UIEx.bind(self.m_root, self.mTableId, "selfRoundStatus", function(value)
		if value then
			for i=1,3 do
				local arr = self.redPointAnims_[i]
				local pointSpr = arr[1]
				local pointAnim = arr[2]
				local lastVisible = pointSpr:getVisible()
				if value[i]~=nil then
					local w, h = pointSpr:getSize()
					if value[i] == 1 then
						if not lastVisible then
							pointSpr:setVisible(true)
							pointAnim:play()
						else
							pointSpr:setVisible(true)
							pointAnim:stop()
						end
					else
						if lastVisible then
							pointAnim:stop()
							checkAndRemoveOneProp(pointSpr, 1)
							local animHide1 = pointSpr:addPropScale(1, kAnimNormal, 320, 0, 1, 0.01, 1, 0.01,kCenterDrawing, w/2, h/2)
							animHide1:setEvent(self, function()
								checkAndRemoveOneProp(pointSpr, 1)
								pointSpr:setVisible(false)
							end)
						else
							pointAnim:stop()
							pointSpr:setVisible(false)
						end						
					end
				else
					pointAnim:stop()
					pointSpr:setVisible(false)
				end
			end
		end
	end)
	-- 对手每一回合状态
	UIEx.bind(self.m_root, self.mTableId, "rivalRoundStatus", function(value)
		if value then
			for i=1,3 do
				local arr = self.bluePointAnims_[i]
				local pointSpr = arr[1]
				local pointAnim = arr[2]
				local lastVisible = pointSpr:getVisible()
				if value[i]~=nil then
					local w, h = pointSpr:getSize()
					if value[i] == 1 then
						if not lastVisible then
							pointSpr:setVisible(true)
							pointAnim:play()
						else
							pointSpr:setVisible(true)
							pointAnim:stop()
						end
					else
						if lastVisible then
							pointAnim:stop()
							checkAndRemoveOneProp(pointSpr, 1)
							local animHide1 = pointSpr:addPropScale(1, kAnimNormal, 320, 0, 1, 0.01, 1, 0.01,kCenterDrawing, w/2, h/2)
							animHide1:setEvent(self, function()
								checkAndRemoveOneProp(pointSpr, 1)
								pointSpr:setVisible(false)
							end)
						else
							pointAnim:stop()
							pointSpr:setVisible(false)
						end						
					end
				else
					pointAnim:stop()
					pointSpr:setVisible(false)
				end
			end
		end
	end)
	-- 自己赢了多少场
	UIEx.bind(self.m_root, self.mTableId, "selfWinMathCnt", function(value)
	    value = value or 0
		if not self.lastSelfMatchCnt_ or self.lastSelfMatchCnt_ ~= value then
	    	self.lastSelfMatchCnt_ = value
	    	--self:refreshMatchCnt(self.lastSelfMatchCnt_, "view_matchCnt1")
	    end
	    -- 
	    if self.mTableId:getStatus() == 0 then
	    	--self:findChildByName("view_matchCnt1"):setVisible(false)
	    else
	    	--self:findChildByName("view_matchCnt1"):setVisible(true)
	    end
	end)
	-- 对手赢了多少场
	UIEx.bind(self.m_root, self.mTableId, "rivalWinMathCnt", function(value)
		value = value or 0
		if not self.lastRivalMatchCnt_ or self.lastRivalMatchCnt_ ~= value then
	    	self.lastRivalMatchCnt_ = value
	    	--self:refreshMatchCnt(self.lastRivalMatchCnt_, "view_matchCnt2")
	    end
	    -- 
	    if self.mTableId:getStatus() == 0 then
	    	--self:findChildByName("view_matchCnt2"):setVisible(false)
	    else
	    	--self:findChildByName("view_matchCnt2"):setVisible(true)
	    end
	end)
	-- 
	UIEx.bind(self.m_root, self.mTableId, "ante", function(value)
		if value and value > 0 then
			local viewAnte = self:findChildByName('view_ante')
			local nodeAnte = new(require('truco.common.imageNumber'), self.mGameId == 13007 and TrucoBaseController.ANTE_RES_CONFIG_13007 or TrucoBaseController.ANTE_RES_CONFIG)
			--local scale = 1

			--nodeAnte:addPropScaleSolid(1,scale,scale,kCenterDrawing)

			viewAnte:removeAllChildren()
	        viewAnte:addChild(nodeAnte)
	        nodeAnte:setNumber(string.format("%s", value), 1);
		    nodeAnte:setVisible(true)
		    -- 
		    local w1, h1 = viewAnte:getSize()
		    local w2, h2 = nodeAnte:getSize()
		    --w2 = w2*scale
		    --w1 = w1*scale


		    if self:findChildByName("view_dealer"):getVisible() then
		    	nodeAnte:setPos(5, (h1 - h2)/2)
		    	--nodeAnte:setPos((scale-1.0)*w2/2+5, (h1 - h2)/2)
		    else
		    	nodeAnte:setPos((w1-w2)/2, (h1 - h2)/2)
		    end
		end
	end)	
	--游戏开始
	-- UIEx.bind(self.m_root, self.mTableId, "start", function(value)
	-- 	--设置玩家状态可手牌数量
	-- 	for k,user in pairs(self.mPlayer) do
	-- 		local status = user:getStatus()
	-- 		local uid = user:getId()
	-- 		if user and uid ~= 0 and status >= TrucoBaseController.PLAYER_STATUS.PLAYER_STATUS_SIT then
	-- 			user:setStatus(TrucoBaseController.PLAYER_STATUS.PLAYER_STATUS_PLAYING)

	-- 		end
	-- 	end
	-- end)
	UIEx.bind(self.m_root, self.mTableId, "status", function(value)
		if value > 0 then
			self:updateRoomStatus(value)
		end
	end)
	UIEx.bind(self.m_root, self.mTableId, "baseScore", function(value)
		local chipNode = new(require('truco.common.imageNumber'), self.mGameId == 13007 and TrucoBaseController.TRUCO_DOWNTIME_RES_CONFIG_13007 or TrucoBaseController.TRUCO_DOWNTIME_RES_CONFIG);	
		chipNode:setNumber(value or 1)
		chipNode:setAlign(kAlignCenter)
		self:findChildByName('view_round'):removeAllChildren()
		self:findChildByName('view_round'):addChild(chipNode)
	end)
end
-- 
function RoomController:refreshMatchCnt(matchCnt, viewKey)
	local viewMatchCnt = self:findChildByName(viewKey)
	local nodeMatchCnt = new(require('truco.common.imageNumber'), TrucoBaseController.SCORE_RES_CONFIG)
	viewMatchCnt:removeAllChildren()
    viewMatchCnt:addChild(nodeMatchCnt)
    nodeMatchCnt:setNumber(string.format("%s", ToolKit.formatMoney(matchCnt)), 1);
    nodeMatchCnt:setVisible(true)
    nodeMatchCnt:setAlign(kAlignCenter)

    checkAndRemoveOneProp(viewMatchCnt, 1)
    local scaleAnim = viewMatchCnt:addPropScaleWithEasing(1, kAnimNormal, 300, 200, "easeOutBounce", "easeOutBounce", 3.0, -2.0, kCenterDrawing);
	scaleAnim:setEvent(self, function()
		checkAndRemoveOneProp(viewMatchCnt, 1)
	end)
end
-- 呈现积分
function RoomController:refreshScore(score, viewKey)
	local viewScore = self:findChildByName(viewKey)
	local nodeScore = new(require('truco.common.imageNumber'), TrucoBaseController.SCORE_RES_CONFIG2)
	viewScore:removeAllChildren()
    viewScore:addChild(nodeScore)
    nodeScore:setNumber(string.format("%s", ToolKit.formatMoney(score)), 1);
    nodeScore:setVisible(true)
    nodeScore:setAlign(kAlignCenter)

    checkAndRemoveOneProp(viewScore, 1)
    local scaleAnim = viewScore:addPropScaleWithEasing(1, kAnimNormal, 300, 0, "easeOutBounce", "easeOutBounce", 3.0, -2.0, kCenterDrawing);
	scaleAnim:setEvent(self, function()
		checkAndRemoveOneProp(viewScore, 1)
	end)
end
-- 
function RoomController:initOptBtn()
	self.viewReady_ = self:findChildByName("view_ready")
	self.viewReady_:setVisible(false)
	-- 叫分(3分)
	self.btnTruco_ = self:findChildByName("btn_truco")
	self.btnTruco_:setOnClick(self, function()
		local seatId = self:getMySeatId()
		if seatId then
			GameSocketMgr:sendMsg(TRUCO_TRUCO_REQ, {seatId = seatId, score = 3});
			-- kEffectPlayer:play(AUDIO.TRUCO_SCORE_03)
		end
	end)
	-- 6分
	self.btnSeis_ = self:findChildByName("btn_seis")
	self.btnSeis_:setOnClick(self, function()
		local seatId = self:getMySeatId()
		if seatId then
			self:hideAllOprBtn()
			-- self.isSelfOptStatus_ = true  -- true为自己叫分， nil为被动叫分
			if self.isSelfOptStatus_ then
				GameSocketMgr:sendMsg(TRUCO_TRUCO_REQ, {seatId = seatId, score = 6});
			else
				GameSocketMgr:sendMsg(TRUCO_ACCEPT_OR_REFUSE_TRUCO_REQ, {seatId = seatId, flag = 2});
			end
			-- kEffectPlayer:play(AUDIO.TRUCO_SCORE_06)
			if DEBUG_MODE then
				printInfo("self.isSelfOptStatus_::::"..tostring(self.isSelfOptStatus_).."   flag::"..6)
			end
		end
	end)
	-- 9分
	self.btnNove_ = self:findChildByName("btn_nove")
	self.btnNove_:setOnClick(self, function()
		local seatId = self:getMySeatId()
		if seatId then
			self:hideAllOprBtn()
			-- self.isSelfOptStatus_ = true  -- true为自己叫分， nil为被动叫分
			if self.isSelfOptStatus_ then
				GameSocketMgr:sendMsg(TRUCO_TRUCO_REQ, {seatId = seatId, score = 9});
			else
				GameSocketMgr:sendMsg(TRUCO_ACCEPT_OR_REFUSE_TRUCO_REQ, {seatId = seatId, flag = 2});
			end
			-- kEffectPlayer:play(AUDIO.TRUCO_SCORE_09)

			if DEBUG_MODE then
				printInfo("self.isSelfOptStatus_::::"..tostring(self.isSelfOptStatus_).."   flag::"..9)
			end
		end
	end)
	-- 12分
	self.btnDoze_ = self:findChildByName("btn_doze")
	self.btnDoze_:setOnClick(self, function()
		local seatId = self:getMySeatId()
		if seatId then
			self:hideAllOprBtn()
			-- 自己叫分1035， 被动加分要走1037
			-- self.isSelfOptStatus_ = true  -- true为自己叫分， nil为被动叫分
			if self.isSelfOptStatus_ then
				GameSocketMgr:sendMsg(TRUCO_TRUCO_REQ, {seatId = seatId, score = 12});
			else
				GameSocketMgr:sendMsg(TRUCO_ACCEPT_OR_REFUSE_TRUCO_REQ, {seatId = seatId, flag = 2});
			end
			-- kEffectPlayer:play(AUDIO.TRUCO_SCORE_12)

			if DEBUG_MODE then
				printInfo("self.isSelfOptStatus_::::"..tostring(self.isSelfOptStatus_).."   flag::"..12)
			end
		end
	end)
	-- 弃牌du
	self.btnFold_ = self:findChildByName("btn_fold")
	self.btnFold_:setOnClick(self, function()
		self:hideAllOprBtn()
		-- self.cardsMgr_:waitPlayDealerCardSetCurrentUser(handler(self, self.abandonCardReq))
	end)
	-- 接受
	self.btnAceitar_ = self:findChildByName("btn_aceitar")
	self.btnAceitar_:setOnClick(self, function()
		local seatId = self:getMySeatId()
		if seatId then
			self:hideAllOprBtn()
			GameSocketMgr:sendMsg(TRUCO_ACCEPT_OR_REFUSE_TRUCO_REQ, {seatId = seatId, flag = 1});
			-- kEffectPlayer:play(AUDIO.TRUCO_SCORE_ACCEPT)
			if DEBUG_MODE then
				printInfo("接受 TRUCO_ACCEPT_OR_REFUSE_TRUCO_REQ")
			end
		end
	end)
	-- 拒绝
	self.btnCorrer_ = self:findChildByName("btn_correr")
	self.btnCorrer_:setOnClick(self, function()
		local seatId = self:getMySeatId()
		if seatId then
			self:hideAllOprBtn()
			GameSocketMgr:sendMsg(TRUCO_ACCEPT_OR_REFUSE_TRUCO_REQ, {seatId = seatId, flag = 0});
			-- kEffectPlayer:play(AUDIO.TRUCO_SCORE_CORRER)
			if DEBUG_MODE then
				printInfo("拒绝 TRUCO_ACCEPT_OR_REFUSE_TRUCO_REQ")
			end
		end
	end)
	-- 暗牌
	self.btnDark_ = self:findChildByName("btn_dark")
	self.btnDark_:setOnClick(self, function()
		local seatId = self:getMySeatId()
		if seatId then
			-- self:renderDarkStyle(false)
			GameSocketMgr:sendMsg(TRUCO_PUTDARKCARD_REQ, {seatId = seatId, flag = 0});
			kEffectPlayer:play(AUDIO.TRUCO_SCORE_DARK)
			if DEBUG_MODE then
				printInfo("暗牌")
			end
		end
	end)
	-- 准备按钮
	self.btnReady_ = self:findChildByName("btn_ready")
	self.btnReady_:setOnClick(self, function()
		-- 向Server发送准备操作
		GameSocketMgr:sendMsg(TRUCO_PLAY_AGAIN_REQ);
	end)

	--聊天按钮位置
	local btn_chat = self:findChildByName('btn_chat')
	self.btn_chat = btn_chat
	
	if tostring(self.mGameId) == tostring(GAME_ID.Truco) then
		btn_chat:setPos(10,250)
	else
		btn_chat:setPos(10,180)
	end
	

	self.viewReadyNum_ = self:findChildByName("view_readyNum")
	self.btnReady_:setVisible(false)
	-- self:findChildByName("btn_canceldelegate"):setVisible(false)
	self.btnTruco_:setVisible(false)
	self.btnSeis_:setVisible(false)
	self.btnNove_:setVisible(false)
	self.btnDoze_:setVisible(false)
	self.btnFold_:setVisible(false)
	self.btnAceitar_:setVisible(false)
	self.btnCorrer_:setVisible(false)
	self.btnDark_:setVisible(false)

	self.originTrucox_, self.originTrucoy_ = self.btnTruco_:getPos()
	self.originSeisx_, self.originSeisy_ = self.btnSeis_:getPos()
	-- 
	self:hideAllOprBtn()
end

function RoomController:abandonCardReq()
	local seatId = self:getMySeatId()
	if seatId then
		GameSocketMgr:sendMsg(TRUCO_ABANDON_CARD_REQ, {seatId = seatId, flag = 0});
		-- kEffectPlayer:play(AUDIO.TRUCO_SCORE_FOLD)
		if DEBUG_MODE then
			printInfo("拒绝 TRUCO_ACCEPT_OR_REFUSE_TRUCO_REQ")
		end
	end
end
-- 
function RoomController:onCleanup()
	
end
--坐下动画
function RoomController:playSitdownAnim()
	for i = 1, #self.mPlayer do
		-- if self.mPlayer[i]:getId() == MyUserData:getId() then
			self.mPlayer[i]:setSitdownAnim(true)
		-- else
			-- self.mPlayer[i]:setSitdownAnim(false)
		-- end
	end
end
--经验功画
function RoomController:showExpResult(user)
	if user.turnExp > 0 then
    	isShowGameResultWindow = true
        local view_addExp = self:findChildByName("view_exp")
	    local expNode = new(require('truco.common.imageNumber'), {
		    ['0'] = expNumber_map["0.png"],
		    ['1'] = expNumber_map["1.png"],
		    ['2'] = expNumber_map["2.png"],
		    ['3'] = expNumber_map["3.png"],
		    ['4'] = expNumber_map["4.png"],
		    ['5'] = expNumber_map["5.png"],
		    ['6'] = expNumber_map["6.png"],
		    ['7'] = expNumber_map["7.png"],
		    ['8'] = expNumber_map["8.png"],
		    ['9'] = expNumber_map["9.png"],
		    ['+'] = expNumber_map["+.png"],
            ['e'] = expNumber_map["e.png"],
            ['x'] = expNumber_map["x.png"],
            ['p'] = expNumber_map["p.png"],
	    });	
        view_addExp:addChild(expNode)
        expNode:setNumber(string.format("+%s", user.turnExp).."exp")
	    expNode:setVisible(true)
	    local localSeatId = self:getLocalSeatId(user.seatId)
	    local x, y = self:getLocalSeatPos(localSeatId, true)
	    expNode:setPos(x, y+50)
        local animExpMove = expNode:addPropTranslate(1, kAnimNormal, 500, 0, 0, 0, 0, -20)
	    local animExpParency = expNode:addPropTransparency(2, kAnimNormal, 500, 0, 0, 1)
	    animExpParency:setEvent(animAddExp, function ()
	    	checkAndRemoveOneProp(expNode, 2)
	    	local animExpMove2 = expNode:addPropTranslate(3, kAnimNormal, 1000, 2800, 0, 0, 0, -40)
	    	local animExpParency2 = expNode:addPropTransparency(2, kAnimNormal, 1000, 2800, 1, 0)
	    	animExpParency2:setEvent(animAddExp, function ()
	    		checkAndRemoveOneProp(expNode, 1)
	    		checkAndRemoveOneProp(expNode, 2)
	    		checkAndRemoveOneProp(expNode, 3)
	    		view_addExp:removeAllChildren()
	    	end)
            -- WindowManager:showWindow(WindowTag.GameResultPopu, {players = self.mPlayer, times = math.min(self.mTableId:getMaxTimes(), data.times), fee = data.fee, ante = self.mTableId:getAnte(), resultPlayers = resultPlayers})
        end)
    end
end
-- 
function RoomController:checkSitDown()
    local myMoney = MyUserData:getMoney()
    local content = ""
    if myMoney < self.mRoomInfo:getMinchip() then
		content = string.format(STR_NOT_ENOUGH_MONEY_SIT_DOWN, self.mRoomInfo:getMinchip());
    	--好友房里不检查金币上限，
    elseif self.mRoomInfo:getLevel() ~= TrucoBaseController.BUDDY_ROOM_LEVEL and myMoney > self.mRoomInfo:getMaxchip() then
		content = string.format(STR_TOO_MUCH_MONEY_SIT_DOWN, self.mRoomInfo:getMaxchip());
	else
        return true;
    end
    WindowManager:showWindow(WindowTag.LobbyExitPopu, {
			title 	= "",
			close 	= true,
			content = content,
			confirm =STR_EXIT_GAME_CONFIRM,
			}, WindowStyle.POPUP);
   return false 
end
-- 
function RoomController:resume(bundleData)
	--播放音乐
	RoomController.super.resume(self)
	--播放音乐
	kMusicPlayer:play(AUDIO.BACKGROUND, true)
end
-- 
function RoomController:dtor()
	printInfo("RoomController:dtor()")
	kMusicPlayer:stop(true)
end
-- 
function RoomController:dctor()
	-- if self.cardsMgr_ then
		-- self.cardsMgr_:cleanAllCards()
	-- end
end
--[[
	清除游戏状态
]]
function RoomController:clearGame()
	-- body
	for k, v in pairs(self.mPlayer) do
		v:clearTurnData()
	end
	--关闭结算
	-- WindowManager:closeWindowByTag(WindowTag.GameResultPopu)
end
--背景单击事件
function RoomController:onBgTouch()
	--[[
	if self.m_roomToolbar then 
		self.m_roomToolbar:closeToolBar();
	end

	self:hitShowPlayerInfo();

	if self.m_rankList then
		self.m_rankList:swapStates("invisible");
	end
	
	if self.m_gemsTask then
		self.m_gemsTask:hideDescription();
	end

	if self.m_checkOutWnd then
		self.m_checkOutWnd:hide();u
	end
	]]
end

-- 
function RoomController:resetLocalVars()
	self.lastRivalScore_ = -1
	self.lastSelfScore_ = -1
	self.lastRivalMatchCnt_ = -1
	self.lastSelfMatchCnt_ = -1
end
--重连
function RoomController:onReLoginGame( data )
	self:clearGame();
	self:onLoginGame(data, true);
end
-- 0x1007    -- 登录成功
function RoomController:onLoginGame(data, isReLoginGame)
	-- if DEBUG_MODE then
	 	JLog.d('function RoomController:onLoginGame(data, isReLoginGame)',data)
	-- 	JLog.d('loginfollowid',self.mFollowId)
	-- end
	-- 
	self.mTableId:setMaxCard(data.max_card)
	self.mTableId:setAnte(data.ante)
	self.mRoomInfo:setAnte(data.ante)
	self.resetOutCardCallback_ = nil
	self:cleanAllDelayProp()
	self:hideAllOprBtn()
	self:resetLocalVars()
	self:closeAllPopup()
	--清桌子
	-- self.cardsMgr_:cleanAllCards()
	self.mTableId:clear()
	--清玩家
	for k, v in pairs(self.mPlayer) do
		v:clearPlayer()
		v:getUi():findChildByName("view_cards"):removeAllChildren()
	end
	--桌子满了,并且不是好友房
	if not data.isSitDown and data.userList and #data.userList == 4 and self.mRoomInfo:getLevel() ~= TrucoBaseController.BUDDY_ROOM_LEVEL then
		--房间满了，需要换桌
		if not self.mFollowId then
			--先退出房间
			self.mSendExitRoom = true
			GameSocketMgr:sendMsg(TRUCO_EXIT_ROOM_REQ)
			--请求桌子ID
			self:getTableId()
			return 
		end
	end
	if data.leftTime > 0 then
		self.mTableId:setLeftTime(data.leftTime)
	else
		self.mTableId:setLeftTime(0)
	end
	-- 
	for i=1,#data.userList do
		local user 	 		= data.userList[i];
		local player 		= self.mPlayer[user.seatId];
		if player then
			local localSeatId 	= self:getLocalSeatId(user.seatId);
			player:init(user)	-- 用户id
			-- 设置本地座位号
			player:setLocalSeatId(localSeatId)

		end
	end
	self:playSitdownAnim() 			--坐下动画
	-- 玩牌阶段
	if data.status == 1 then

		self:findChildByName("view_dealer"):setVisible(true)
		self:refreshRoomAnte()
		self.mTableId:initPlaying(data)
		self.mTableId:initRoundStatus(data.roundStatus)
		self.mTableId:onBroadCastStartGame(data)
		self.mTableId:onBroadCastOneRound(data.playerList)
		self.mTableId:setTotalCardTime(data.outCardTm)
		-- 
		if data.viraCardValue > 0 then
			self.resetOutCardCallback_ = function()
				if data.isTruco == 1 then
					local trucoPlayer = self:getUserByUid(data.lastTrucoUid)
					if trucoPlayer then
						trucoPlayer:setStatusScore(data.lastTrucoScore or 3)
						trucoPlayer:setStatusIcon(STATUS_ICON_TRUCO)
					end
					-- Server发送断开重连后，紧接会发送0x4013,延迟1s是规避0x4013设置Button状态
					if data.trucoOtherPlayers then
						-- self:delayCall(function()
							local dataObj = {}
							dataObj.uid = data.lastTrucoUid
							dataObj.action = 1
							dataObj.leftTime = data.leftTime
							dataObj.score = data.lastTrucoScore
							dataObj.playerList = data.trucoOtherPlayers
							self:onBroadCastSpecialOpt(dataObj)
						
						-- end, 1000)
					end				
				else
					local oprPlayer = self.mPlayer[data.oprSeatId];
					if oprPlayer then
						oprPlayer:setReadyTime(data.leftTime)
					end
				end
				-- 清理当前桌面牌数据
				-- self.cardsMgr_:cleanAllCards()
				-- 断开重连呈现玩家手牌
				self.isReloginInitHandCardValues_ = true
				if not self.isSitDownAnim_ then
				
					CardManager:getInstance():showOrHideUnderCard(self.cardHeap,data.viraCardValue)
				else
					self:delayCall(function()
						
						CardManager:getInstance():showOrHideUnderCard(self.cardHeap,data.viraCardValue)
					end, 600)
				end
				
				local isCardOver = false --赛点手牌盖着
				if MATCH_POINT == data.selfScore and data.rivalScore == MATCH_POINT then
					isCardOver = true
				end
				local isSelfOpt = false --当前是否自己操作
				if data.oprUid == MyUserData:getId() then
					isSelfOpt = true
					if self.mGameId == 13007 then
						self.mTableId:setBetScore(data.lastTrucoScore)
					end
					self:showSelfOptBtn()
				end
				if data.cardValues and #data.cardValues > 0 then
					local player = self:getUserByUid(MyUserData:getId())
					CardManager:getInstance():setHandCards(player,isCardOver,isSelfOpt,data.cardValues,#data.cardValues) -- 断开重连呈现当前玩家手牌	
				end
				-- 
				local maxCnt = 0
				for i=1,data.playerCnt do
					local playerItem = data.playerList[i]
					local dropCardValues = playerItem.dropCardValues
					local cnt = #dropCardValues
					maxCnt = math.max(cnt, maxCnt)
				end
				for i=1,data.playerCnt do
					local playerItem = data.playerList[i]
					local dropCardValues = playerItem.dropCardValues
					local localSeatId = self:getLocalSeatId(playerItem.seatId)

					local cardCnt = TOTAL_CARD_CNT - playerItem.dropCardCnt
					if playerItem.uid ~= MyUserData:getId() and data.viraCardValue > 0 then
						-- 呈现其他玩家手牌数
						if (#dropCardValues>0 and dropCardValues[1] <100) or dropCardValues[1] == 0 then
							cardCnt = cardCnt + 1
						end
						-- self.cardsMgr_:renderOtherPlayerCards(localSeatId, cardCnt,playerItem.uid)
						 local player = self:getUserByUid( playerItem.uid)
						CardManager:getInstance():setHandCards(player,false,false,{},cardCnt)
					end
					-- 呈现玩家出牌的牌

					  for i=1,#dropCardValues do
						if i == 1 then
							if dropCardValues[1] > 0 and dropCardValues[1]<100 then
								 CardManager:getInstance():outCard(dropCardValues[1],self.cardHeap,self.mPlayer[playerItem.seatId],1)
								 CardManager:getInstance():closeCardPanel(self.mPlayer[playerItem.seatId])
							elseif dropCardValues[1] == 0 then
								 CardManager:getInstance():outCard(0,self.cardHeap,self.mPlayer[playerItem.seatId],0)
							end
						end
					 end            
				end			
			end
		end
	end

	for i=1,#data.readys do
		local player = self:getUserByUid(data.readys[i].uid)
		if player then
			player:setDownNum(data.readys[i].readylefttime)

		end
	end
	--2 播放结算动画是重来你回来
	if data.status == 2 then
		self:onPopupVictoryAnimation(data)
		self:resetCircleProgress()
	end
	--3 桌子处于玩牌 但此时是等待发牌阶段 当前无操作玩家
	if data.status == 3 then
		self:findChildByName("view_dealer"):setVisible(true)
		self:refreshRoomAnte()
		self.mTableId:initPlaying(data)
		self.mTableId:initRoundStatus(data.roundStatus)
		-- self.mTableId:onBroadCastStartGame(data.playerList)
		-- self.mTableId:onBroadCastOneRound(data.playerList)
		--self.mTableId:setTotalCardTime(self.mTableId:getPlayCardTime())
	end

	self.mTableId:onMatchDataList(data.matchDataList)

	if data.mySelfUser then
		self.mTableId:setCount(self:getPlayerCount())
		
	else
		if not isReLoginGame then
			self:autoSitDownAction()
		end
		--更新人数
		self.mTableId:setCount(self:getPlayerCount())
	end
	-- 
	self:hideLoadingAnim()
    self.mTableId:init1007(data)
end
-- 
function RoomController:cleanAllPlayerHeadStatus()
	for i = 1, #self.mPlayer do
		local player = self.mPlayer[i]
		local playerUI = player:getUi()
		if playerUI then
			local viewStatus 	= playerUI:findChildByName("view_status");
			viewStatus:removeAllChildren()
		end
	end
end

-- 取消托管请求
function RoomController:cancelDelegateReq()
	-- 检测按钮点击间隔
	if true == self:findChildByName('btn_canceldelegate'):getVisible() then
		GameSocketMgr:sendMsg(TRUCO_CANCEL_DELEGATE_REQ, {type=0}) -- self:getMyPlayer():setAi(false)
		if DEBUG_MODE then
			printInfo("cancelDelegateReq true!!!")
		end
	else
		printInfo("cancelDelegateReq false")
	end
end
-- 准备开始游戏
function RoomController:onReadyGame()
	for k, v in pairs(self.mPlayer) do
		local playerUI = v:getUi()
	end
	-- 
	-- self.cardsMgr_:cleanAllCards()
end
-- 初始化玩家数据
function RoomController:clearPlayData_()
	for i = 1, #self.mPlayer do
		-- local seat = 1;
		 -- if tostring(self.mGameId) == tostring(GAME_ID.truco) then
			-- if i == 4 then
			-- 	seat = 1
			-- elseif i == 1 then
			-- 	seat = 4
			-- else
			-- 	seat = i
			--  end
		 -- else
		 -- 	seat = i
		 -- end
		self.mPlayer[i]:setSeatId(i);
		self.mPlayer[i]:setLocalSeatId(i);
		self.mPlayer[i]:clear(i);
	end
end
-- 
function RoomController:onServerCmdDealCard(data)
	
end
-- 获取当前玩家状态
function RoomController:getCurrentMyStatus()
	local selfPlayerData = self:getMyPlayer()
	local isCurrentMy = false
	-- 获取当前头像进度条
	if selfPlayerData and selfPlayerData:getUi():findChildByName('circleProgress'):getVisible() then
		isCurrentMy = true
	end
	return isCurrentMy
end
-- 
function RoomController:showOrhideReadView()
	local myPlayer = self:getUserByUid(MyUserData:getId())
	-- 自己准备，且其他没有其他玩家的时候显示"view_ready"提示文字 “正在等待其他玩家准备”
	self.viewReady_:setVisible(false)
	-- local playerCnt = self:getPlayerCount()
	local totalCnt = 0
	local cnt = 0
	-- if myPlayer and myPlayer:getIsReady() then
		-- if playerCnt > 1 then
			-- local isNotReady = false
			totalCnt = self.mTableId:isTwoMatch() and 2 or 4			
			for i,v in ipairs(self.mPlayer) do
				if v:getId() > 0 and v:getIsReady() then
					cnt = cnt + 1
				end
			end
			--
			if cnt<totalCnt and myPlayer and myPlayer:getIsReady() then
				self.viewReady_:setVisible(true)
			else
				self.viewReady_:setVisible(false)
			end
		-- 	if cnt > 0 then
		-- 		self.viewReady_:setVisible(true)
		-- 	else
		-- 		if playerCnt == totalCnt then
		-- 			self.viewReady_:setVisible(false)
		-- 		else
		-- 			self.viewReady_:setVisible(true)
		-- 		end
		-- 	end
		-- else
		-- 	self.viewReady_:setVisible(false)
		-- end
	-- else
		-- self.viewReady_:setVisible(false)
	-- end
end
--[[
	--seatId 从1开始
]]
function RoomController:getLocalSeatId(seatId, mySeatId)
	if self.mLastSeatId then
		local localSeatId = TrucoBaseController.MY_LOCAL_SEAT_ID + seatId - self.mLastSeatId
		-- self.mGameId 	= param3[5] or GAME_ID.Truco
		if tostring(self.mGameId)==tostring(GAME_ID.Truco) then
			if localSeatId ~= TrucoBaseController.MY_LOCAL_SEAT_ID  then
				if seatId<=2 then
					localSeatId = 2 
				elseif seatId == 3 then
					localSeatId = 3
				elseif seatId == 4 then
					localSeatId = 1
				end
				
			end
			return localSeatId
		else
			if localSeatId > TrucoBaseController.MY_LOCAL_SEAT_ID then
				localSeatId = localSeatId % TrucoBaseController.MY_LOCAL_SEAT_ID
			end
			return localSeatId
		end

	else

		if tostring(self.mGameId)==tostring(GAME_ID.Truco) then
			local player = self.mPlayer[4]
			player:setLocalSeatId(1)
			 player = self.mPlayer[2]
			player:setLocalSeatId(4)
			return seatId*2
		else
			return seatId
		end
		
	end
end
--[[
	重新排位
]]-- 
function RoomController:reLocateSeat()
	local mySeatId 	= self:getMySeatId();
	if not mySeatId then 
		mySeatId = self:getFollowSeatId()
		if not mySeatId then return end
		--return
	end
	local player 	= self.mPlayer[mySeatId]
	self.mLastSeatId = mySeatId
	player:setLocalSeatId(TrucoBaseController.MY_LOCAL_SEAT_ID)
	for i = 1, #self.mPlayer do

		local p 	 = self.mPlayer[i];		
		
		local seatId = p:getSeatId();
		local uid 	 = p:getId()
		if seatId ~= mySeatId then
			local localSeatId = self:getLocalSeatId(seatId)
			
			p:setLocalSeatId(localSeatId);
			if self.mTableId:isTwoMatch() then
				if uid > 0 then
					-- p:getUi():setVisible(true)
					p:getUi():findChildByName("btn_player"):setVisible(true)
				else
					-- p:getUi():setVisible(true)
					p:getUi():findChildByName("btn_player"):setVisible(false)
				end
			end
		end
	end
end

-- 清理所有的玩家倒计时
function RoomController:cleanAllPlayerReadTime()
	for _,v in pairs(self.readTimePlayers_) do
		v:setReadyTime(0)
	end
end

----------------------------------------------------------------------------------
-- 0x4010	-- 广播开始游戏，发牌结束之后的一个广播(只在整个游戏开始的时候发)
function RoomController:onBroadCastStartGame(data)
	--JLog.d('function RoomController:onBroadCastStartGame(data)',data)
	self:cleanViewChips()
	self.viewReady_:setVisible(false)
	self.mTableId:onBroadCastStartGame(data)
	-- 
	local downTimes = 3
	self:addDownNumber_(downTimes+1)
	self:closeAllPopup()
	for k,user in pairs(self.mPlayer) do
		local status = user:getStatus()
		local uid = user:getId()
		user:setIsReady(false)
		if user and uid ~= 0 and status >= TrucoBaseController.PLAYER_STATUS.PLAYER_STATUS_SIT then
			user:setStatus(TrucoBaseController.PLAYER_STATUS.PLAYER_STATUS_PLAYING)

		end
	end
end
function RoomController:getAllPlayers()
	local players = {}
	for k,player in pairs(self.mPlayer) do
		if player:getId()>0 then
			table.insert(players,player)
		end
	end
	return players
end
-- 0x4011	-- 广播开始发牌
function RoomController:onBroadCastDealCard(data)
	--JLog.d("0x4011	-- 广播开始发牌")
	
	for i = 1, #self.mPlayer do
		local playerData = self.mPlayer[i]
		if playerData:getId() > 0 then
			playerData:setStatus(TrucoBaseController.PLAYER_STATUS.PLAYER_STATUS_PLAYING)
		end
	end
	local uid = self:getMyPlayer()
	if uid == nil then
		--self.mTableId:onDealerCard({cardValues={}})
		--CardManager:getInstance():showOrHideUnderCard(self.cardHeap,0)
	end
	self.mTableId:setCurrentRound(0)
	self.mTableId:onBroadCastDealCard(data)
	self.mTableId:resetRoundStatus()        -- 重置回合状态
	if self.mFollowId and tonumber(self.mFollowId)>0 then  --旁观人发牌动画
		local players = self:getAllPlayers()
		CardManager:getInstance():dealCards(data.cardValues,self.cardHeap,players,0)
	end
end
-- 0x1054    -- 发3张手牌 server -> client
function RoomController:onDealerCard(data)
	-- 停止之前的收牌动画
	checkAndRemoveOneProp(self,1020)
	CardManager:getInstance():finishMoveLastCardToHeap(self.mPlayer,self.cardHeap)
	CardManager:getInstance():finishHideUnderCard(self.cardHeap)
	checkAndRemoveOneProp(self,1024)
	CardManager:getInstance():finishMoveOutCardToHeap(self.cardHeap)
	--

	--JLog.d("0x1054    -- 发3张手牌 server -> client",data)
	self.mTableId:setBaseScore(data.baseScore)
	self.mTableId:setAcceptTrucoSeatId(TableId.DEFAULT_TRUCO_STATUS)
	self.mTableId:setAcceptTrucoRound(TableId.INVALIDE_ROUND_ID) -- 清空接收叫分的回合
	if self.mGameId == 13007 then
		self.mTableId:setBetScore(data.betScore)
		-- self:showSelfOptBtn()
	end
	local players = self:getAllPlayers()
	local isCardOver = 1
	if MATCH_POINT == self.lastSelfScore_ and self.lastRivalScore_ == MATCH_POINT then
		isCardOver = 0
	end
	CardManager:getInstance():dealCards(data.cardValues,self.cardHeap,players,isCardOver)
	self:stop_play_fa()
end
-- 0x4012    -- 广播vira牌和底分 server -> client
function RoomController:onBroadCastViraCard(data)
	--printInfo("0x4012    -- 广播vira牌和底分 server -> client")
	self.mTableId:onBroadCastViraCard(data)
	CardManager:getInstance():showOrHideUnderCard(self.cardHeap,data.viraCardValue)
	-- if self.mGameId == 13007 then
	-- 	self:showSelfOptBtn()
	-- end
end
-- 0x4013	-- 广播当前轮到操作用户 server -> client
function RoomController:onBroadCastCurrentUser(data,isRelogin)
	--JLog.d('function RoomController:onBroadCastCurrentUser(data,isRelogin)',data)
	if self.isRounOver_ then
		-- self.cardsMgr_:hideAllLastOutCards()
	end
	-- 
	-- self:cleanAllPlayerStatusIcon()
	self:cleanAllPlayerReadTime()	-- 
	self:closeAllPopup()	-- 
	self.mTableId:onBroadCastCurrentUser(data)
	local oprPlayer = self.mPlayer[data.seatId]
	if oprPlayer then
		-- local func = function()
			oprPlayer:setReadyTime(data.leftTime)
			-- 如果是当前玩家，呈现可操作按钮状态，同时设置手牌出牌准备效果
			--local delayAnim = self:addPropTranslate(1024,kAnimNormal,0,1000,0,0,0,0);
			--if delayAnim then
			--	delayAnim:setEvent(self,function()
			--		checkAndRemoveOneProp(self,1024)
					  -- self.cardsMgr_:resetOutCardLayout()
					 if data.uid == MyUserData:getId() then
						self:showSelfOptBtn()
						CardManager:getInstance():openCardPanel(oprPlayer)
						kEffectPlayer:play(AUDIO.TURN)
					 else
						self:hideAllOprBtn()
					 end
				--end)
			--end
				
			-- checkAndRemoveOneProp(self,1021)
			-- local delayAnim = self:addPropTranslate(1021,kAnimNormal,0,3000,0,0,0,0);
			-- if delayAnim then
			-- 	delayAnim:setEvent(self,function()
			-- 		checkAndRemoveOneProp(self,1021)
			-- 		if oprPlayer:getUi():findChildByName('circleProgress'):getVisible() then
			-- 			oprPlayer:setStatusIcon(STATUS_ICON_THINK)
			-- 		end
			-- 	end)
			-- end

				-- 
				if self.mTableId:isTwoMatch() and self.mTableId:getBaseScore() == 1 and not self.mTableId:getIsMatchPoint() then
					-- local val = self.cardsMgr_:isTableOutCard()
					-- self.btnFold_:setVisible(not val)
				end
			
		-- end
		-- self.cardsMgr_:waitPlayDealerCardSetCurrentUser(func)
	end
end
-- 0x4004 	-- 广播用户出牌
function RoomController:onBroadCastDropCard(data)
	--JLog.d('function RoomController:onBroadCastDropCard(data)',data)
	if not data then
		return
	end
	-- 
	if self.isRounOver_ then
		-- self.cardsMgr_:clearOutCardSelectStatus()
		self.isRounOver_ = nil
	end

	local localSeatId = self:getLocalSeatId(data.seatId)
	local result = self.mTableId:outCardOptAnsyPlayerCardInfo(data.seatId, data.cardValue)
	local oprPlayer = self.mPlayer[data.seatId]
	if oprPlayer then
		--local anim =self:addPropTranslate(data.seatId+1024,kAnimNormal,0,1000,0,0,0,0)
	--	if anim then
		--	anim:setEvent(self,function()
				--local isCardOver = data.flag == 0 and true or false
				CardManager:getInstance():outCard(data.cardValue,self.cardHeap,oprPlayer,data.flag)
				kEffectPlayer:play(AUDIO.ChuPai)
				checkAndRemoveOneProp(self,data.seatId+1024)
			--end)
		--end
	end
	-- 
	if data.uid == MyUserData:getId() then
		self:hideAllOprBtn()
	end
	CardManager:getInstance():closeCardPanel(oprPlayer)
	self:stop_play_fa()
end
-- 0x4016 	-- 广播托管结果
function RoomController:onBroadCastUserAI(data)
	printInfo("pickPlayerData:onBroadcastUserAI():"..data.uid)
	if DEBUG_MODE then
		dump(data)
	end
	local player = self:getUserByUid(data.uid)
	if player and data then
		if data.aiType == 1 then
			player:setAi(true)
		else
			player:setAi(false)
		end
	end
end
-- 0x4003 	-- 广播玩家叫分 
function RoomController:onBroadCastTruco(data)
	self:resetCircleProgress()
	local player = self.mPlayer[data.seatId]
	if player then
		self.mTableId:setAcceptTrucoSeatId(TableId.DEFAULT_TRUCO_STATUS)
		self.mTableId:setAcceptTrucoRound(TableId.INVALIDE_ROUND_ID) -- 清空接收叫分的回合

		player:setStatusScore(data.score or 3)
		player:setStatusIcon(STATUS_ICON_TRUCO)
	end
end
-- 0x4005 	-- 广播玩家回应叫分
function RoomController:onBroadCastTrucoReply(data)
	-- self:cleanAllPlayerReadTime()
	if data.uid == MyUserData:getId() then
		-- self.cardsMgr_:recoverOutCardLayout()
	end
	-- 
	local currentRnd = self.mTableId:getCurrentRound()
	writeTabToLog({currentRnd=currentRnd}, "RoomController 0x4005", "1dddkk_mk"..MyUserData:getId()..".lua")
	local player = self.mPlayer[data.seatId]
	if player then
		-- //回应 （接受 1 拒绝 0 加倍 2）
		if data.flag == 1 then  -- 接受
			player:setStatusIcon(STATUS_ICON_ACCEPT)
			self.mTableId:setBaseScore(data.score)   		-- 同步桌子上的分值
			self.mTableId:setAcceptTrucoSeatId(data.seatId) -- 接收叫分的玩家SeatId
			self.mTableId:setAcceptTrucoRound(currentRnd) -- 接收叫分的回合
		elseif data.flag == 0 then  -- 拒绝
			player:setStatusIcon(STATUS_ICON_REJECT)
		elseif data.flag == 2 then  -- 加倍
			self.mTableId:setAcceptTrucoSeatId(TableId.DEFAULT_TRUCO_STATUS) -- 接收叫分的玩家SeatId
			self.mTableId:setBaseScore(data.score)   		-- 同步桌子上的分值
			if self.mGameId == 13007 then
				self.mTableId:setBetScore(data.betScore)
			end
			local score = (data.score or 3) + 3
			player:setStatusScore(score)
			player:setStatusIcon(STATUS_ICON_TRUCO)
		end			
	end
end
-- 0x4006 	-- 广播一局结束
function RoomController:onBroadCastOneRound(d)
	--JLog.d('function RoomController:onBroadCastOneRound(d)',d)
	self:hideAllOprBtn()
	self:closeAllPopup()
	if d.flag == 1 then
		-- 一局结束
	elseif d.flag == 2 then
		-- 一场结束
	end
	self.mTableId:onBroadCastOneRound(d.playerList,d.flag)
	-- 延迟1秒做收牌动作
	checkAndRemoveOneProp(self,1020)
	local anim = self:addPropTranslate(1020,kAnimNormal,0,1800,0,0,0,0)
	if anim then
		anim:setEvent(self,function ( ... )
			CardManager:getInstance():moveLastCardToHeap(self.mPlayer,self.cardHeap)
			checkAndRemoveOneProp(self,1020)
			CardManager:getInstance():showOrHideUnderCard(self.cardHeap,0)
		end)
	end 
	-- TODO 清理玩家头像倒计时
	self:resetCircleProgress()
	-- self:stop_play_fa()
end
-- 
function RoomController:resetCircleProgress()
	for i,v in ipairs(self.mPlayer) do
		v:setReadyTime(0)
	end
end

-- 0x4007 	-- 广播游戏结束（有一个玩家最先两次拿到12分）
function RoomController:onBroadCastGameOver(data)
	self:cloneAllPlayers()
	CardManager:getInstance():showOrHideUnderCard(self.cardHeap,0)
	self.mTableId:setIsMatchPoint(false)
	for i=1,#self.mPlayer do
		self.mPlayer[i]:setAi(false)
	end
	-- 重置玩家状态为准备状态
	-- if user then
	-- 	user:setStatus(TrucoBaseController.PLAYER_STATUS.PLAYER_STATUS_TRUCO)
	-- end
	self:findChildByName('btn_canceldelegate'):setVisible(false)
	self:onPopupVictoryAnimation(data)
	self:resetCircleProgress()
	checkAndRemoveOneProp(self,1022)
	local anim = self:addPropTranslate(1022,kAnimNormal,0,2800,0,0,0,0)
	if anim then
		anim:setEvent(self,function ( ... )
			
			checkAndRemoveOneProp(self,1022)
			self:findChildByName("view_dealer"):setVisible(false)
		end)
	end
	self:delayCall(function()
		-- self.cardsMgr_:takeBackAllOutCards()
		
	end, 1000)
end
-- 0x4008     //广播玩家赛点选择
function RoomController:onBroadCastUserDropCard(data)
	--JLog.d('function RoomController:onBroadCastUserDropCard(data)',data)
	if data.action == 1 then
		self.mTableId:setIsMatchPoint(true)
	end
	if self.popupMgr_.gameElevenTwoView_ then
		if data.uid == MyUserData:getId() then
			
			self.popupMgr_.gameElevenTwoView_:setButtonEnable(false)
			self.popupMgr_.gameElevenTwoView_:setTipsType(TrucoGameMatchElevelTwoPopup.WAITING_TYPE)
		else
			if data.action == 0 then
				
				self.popupMgr_.gameElevenTwoView_:setTipsType(TrucoGameMatchElevelTwoPopup.REFUSE_TYPE)
			elseif data.action == 1 then
				
				self.popupMgr_.gameElevenTwoView_:setTipsType(TrucoGameMatchElevelTwoPopup.ACCEPT_TYPE)
			else
				
				self.popupMgr_.gameElevenTwoView_:setTipsType(TrucoGameMatchElevelTwoPopup.WAITING_TYPE)
			end
		end
	elseif self.popupMgr_.gameElevenOneView_ then
		self.popupMgr_.gameElevenOneView_:setButtonEnable(false)
	end
end
-- 0x4009	-- 广播下一场开始
function RoomController:onBroadCastNestMathStart(data)
	self:resetScoreZero()
	self.mTableId:setCurrentMatch(data.currentMatch)			  -- 当前第几场
	self.mTableId:onMatchDataList(data.matchDataList)
end
-- 0x4014	-- 广播当前应该 (回应叫分 或 赛点选择是否要牌) 的玩家
function RoomController:onBroadCastSpecialOpt(data)
	self:cleanAllPlayerReadTime()
	-- action 操作动作    （1 回应叫分  2 赛点操作）
	if data.action == 1 then
		local isSelf = false
		for _,u in pairs(data.playerList) do
			local player = self.mPlayer[u.seatId]
			if player then
				player:setReadyTime(data.leftTime)
				if u.uid == MyUserData:getId() then
					isSelf = true
				end

				-- local func = function()
					--player:setReadyTime(data.leftTime)
					-- 如果是当前玩家，呈现可操作按钮状态，同时设置手牌出牌准备效果
					if data.uid == MyUserData:getId() then
						self:showSelfOptBtn()
						kEffectPlayer:play(AUDIO.TURN)
					end
					checkAndRemoveOneProp(self,1000)
					local delayAnim = self:addPropTranslate(1000,kAnimNormal,0,2800,0,0,0,0);
					if delayAnim then
						delayAnim:setEvent(self,function()
							checkAndRemoveOneProp(self,1000)
							if player:getUi():findChildByName('circleProgress'):getVisible() then
								 player:setStatusIcon(STATUS_ICON_THINK)
							end
						end)
					end
			end
		end
		if isSelf then
			self:showSelfReplyTrucoOptBtn(data.score)
		else
			self:hideAllOprBtn()
		end
	elseif data.action == 2 then
		
	end
end
-- 0x4015	-- 广播一回合的结束
function RoomController:onBroadCastOneRoundOver(data)
	--JLog.d('function RoomController:onBroadCastOneRoundOver(data)',data)
	self.isRounOver_ = true
	-- TODO 次处可以添加动画，判断是输赢一方后，做一个飞上去光效
	self.mTableId:setCurrentRound(data.round)
	--local anim = self:addPropTranslate(1023,kAnimNormal,0,800,0,0,0,0)
	local anim = self:addPropTranslate(1023,kAnimNormal,0,300,0,0,0,0)
	if anim then
		anim:setEvent(self,function ( ... )
			if data.uid > 0 then
				CardManager:getInstance():showMaxCard(data.seatId)
			else
				CardManager:getInstance():showMaxCard()
			end
			self.mTableId:onBroadCastOneRoundOver(data)
			checkAndRemoveOneProp(self,1023)
		end)
	end 
	
	checkAndRemoveOneProp(self,1024)
	--local anim = self:addPropTranslate(1024,kAnimNormal,0,2500,0,0,0,0)
	local anim = self:addPropTranslate(1024,kAnimNormal,0,1600,0,0,0,0)
	if anim then
		anim:setEvent(self,function ( ... )
			CardManager:getInstance():moveOutCardToHeap(self.cardHeap)
			checkAndRemoveOneProp(self,1024)
		end)
	end 

	self:cleanAllPlayerReadTime()
	self:stop_play_fa()
end
-- 
function RoomController:onBroadcastUserLogin(data)

end
-- 
function RoomController:cleanAllStatus()
	self:hideAllOprBtn()
end

function RoomController:update_btn_pos(btnList)
	-- btn_truco
	-- btn_seis
	-- btn_nove
	-- btn_doze
	-- btn_fold
	-- btn_aceitar
	-- btn_correr
	-- btn_dark
	--a~h

	--case
	--a,b,c,d,ha,hb,hc,hd,af,bf,cf,df,gbf,gcf,gdf

	local btn_map = {[self.btnTruco_] = 'a',[self.btnSeis_] = 'b',
		[self.btnNove_] = 'c',[self.btnDoze_] = 'd',
		[self.btnFold_] = 'e',[self.btnAceitar_] = 'f',
		[self.btnCorrer_] = 'g',[self.btnDark_] = 'h',}

	local btn_map2 = {}
	for k,v in pairs(btn_map) do
		btn_map2[v] = k
	end

	local start_x = 560
	local dx = 150
	local case_pos = {['a'] = {start_x},['b'] = {start_x},['c'] = {start_x},['d'] = {start_x},['h'] = {start_x},
	['ah'] = {start_x,start_x-dx},['bh'] = {start_x,start_x-dx},['ch'] = {start_x,start_x-dx},['dh'] = {start_x,start_x-dx},
	['af'] = {start_x-dx,start_x},['bf'] = {start_x-dx,start_x},['cf'] = {start_x-dx,start_x},['df'] = {start_x-dx,start_x},['fg'] = {start_x,start_x-dx},
	['bfg'] = {start_x-dx,start_x,start_x-dx*2},['cfg'] = {start_x-dx,start_x,start_x-dx*2},['dfg'] = {start_x-dx,start_x,start_x-dx*2}}

	local btn_list_str = ''
	local btn_list_str_t = {}

	for _,v in ipairs(btnList) do
		table.insert(btn_list_str_t,btn_map[v])
	end
	table.sort(btn_list_str_t)

	for _,v in ipairs(btn_list_str_t) do
		btn_list_str = btn_list_str..v
	end

	local pos = case_pos[btn_list_str]

	if pos then
		table.sort(btnList,function (a,b)
			return btn_map[a] < btn_map[b]
		end)

		for i=1,#pos do
			btnList[i]:setPos(pos[i],10)
		end
	end
end

-- 隐藏所有自己操作的按钮
function RoomController:hideAllOprBtn(btnList)
	btnList = btnList or {}
	-- 接受
	if not self:findBtnsExists(btnList, self.btnAceitar_) then
		self:hideBtnAnim(self.btnAceitar_)	-- self.btnAceitar_:hide()
	end
	-- 拒绝
	if not self:findBtnsExists(btnList, self.btnCorrer_) then
		self:hideBtnAnim(self.btnCorrer_)	-- self.btnCorrer_:hide()
	end
	-- 叫分(3分)
	if not self:findBtnsExists(btnList, self.btnTruco_) then
		self:hideBtnAnim(self.btnTruco_)	-- self.btnTruco_:hide()
	end
	-- 6分
	if not self:findBtnsExists(btnList, self.btnSeis_) then
		self:hideBtnAnim(self.btnSeis_)	-- self.btnSeis_:hide()
	end
	-- 9分
	if not self:findBtnsExists(btnList, self.btnNove_) then
		self:hideBtnAnim(self.btnNove_)	-- self.btnNove_:hide()
	end
	-- 12分
	if not self:findBtnsExists(btnList, self.btnDoze_) then
		self:hideBtnAnim(self.btnDoze_)	-- self.btnDoze_:hide()
	end
	-- 弃牌
	if not self:findBtnsExists(btnList, self.btnFold_) then
		self:hideBtnAnim(self.btnFold_)	-- self.btnFold_:hide()
	end
	-- 暗牌
	if not self:findBtnsExists(btnList, self.btnDark_) then
		self:hideBtnAnim(self.btnDark_)	-- self.btnDark_:hide()
	end
end
-- 
function RoomController:findBtnsExists(btnList, key)
	local result = false
	for i=1,#btnList do
		if btnList[i] == key then
			result = true
			break
		end
	end
	return result
end
-- 
function RoomController:renderDarkStyle(value)
	self:findChildByName("Img_normalCard"):setVisible(not value)
	-- self:findChildByName("Img_darkCard"):setVisible(not value)
end
-- 
function RoomController:isTwoElevelPoint()
	self.lastSelfScore_  = self.lastSelfScore_  and self.lastSelfScore_  or -1
	self.lastRivalScore_ = self.lastRivalScore_ and self.lastRivalScore_ or -1
	   
	if self.lastSelfScore_ >= 11 and self.lastRivalScore_ >= 11 then
		return true
	end
	return false
end
-- 显示自己当前可以操作的按钮
function RoomController:showSelfOptBtn()
	self.isSelfOptStatus_ = true  -- true为自己叫分， nil为被动叫分
	-- 
	local isValidate = self.mTableId:isValidateTrucoRule(self:getMySeatId())
	local acceptTrucoRnd = self.mTableId:getAcceptTrucoRound()
	local currentRnd = self.mTableId:getCurrentRound()
	local acceptTrucoSeatId = self.mTableId:getAcceptTrucoSeatId()
	local baseScore = self.mTableId:getBaseScore()
	local betScore = baseScore
	if self.mGameId == 13007 then
		betScore = self.mTableId:getBetScore()
	end
	local isFriend = self.mTableId:checkSeatIdIsFriend(acceptTrucoSeatId)
	local isMatchPoint = self.mTableId:getIsMatchPoint()
	local btnList = {}
	if not self.mTableId:getIsMatchPoint() and not self:isTwoElevelPoint() then
		if (isValidate and acceptTrucoRnd ~= currentRnd) or acceptTrucoSeatId == self:getMySeatId() 
			or isFriend then
			if 1 == betScore or 0 == betScore then
				if MATCH_POINT ~= self.lastSelfScore_ and self.lastRivalScore_ ~= MATCH_POINT then
					table.insert(btnList, self.btnTruco_)
					self:showBtnAnim(self.btnTruco_)	-- self.btnTruco_:show()
				end
			elseif 3 == betScore then
				table.insert(btnList, self.btnSeis_)
				self.btnSeis_:setPos(self.originTrucox_, self.originTrucoy_)
				self:showBtnAnim(self.btnSeis_)	-- self.btnSeis_:show()
			elseif 6 == betScore then
				table.insert(btnList, self.btnNove_)
				self.btnNove_:setPos(self.originTrucox_, self.originTrucoy_)
				self:showBtnAnim(self.btnNove_)	-- self.btnNove_:show()
			elseif 9 == betScore then
				table.insert(btnList, self.btnDoze_)
				self.btnDoze_:setPos(self.originTrucox_, self.originTrucoy_)
				self:showBtnAnim(self.btnDoze_)	-- self.btnDoze_:show()
			end
		end
	end
	-- 
	local round = self.mTableId:getCurrentRound()  -- 弃牌只在第一回合，并且不是赛点，才显示
	if self.mTableId:isTwoMatch() and round == 0 and self.mTableId:getBaseScore() == 1 and not self.mTableId:getIsMatchPoint() then
		-- local val = self.cardsMgr_:isTableOutCard()
		if round == 0 and not val then
			-- self:showBtnAnim(self.btnFold_) -- self.btnFold_:show()
			-- table.insert(btnList, self.btnFold_)
		end
	end
	-- 
	if round > 0 then
		if MATCH_POINT ~= self.lastSelfScore_ or self.lastRivalScore_ ~= MATCH_POINT then
			self:showBtnAnim(self.btnDark_) -- self.btnFold_:show()
			table.insert(btnList, self.btnDark_)
			self:renderDarkStyle(false)
		end
	end

	self:hideAllOprBtn(btnList)
	self:update_btn_pos(btnList)
end
-- 显示自己回应叫分操作
function RoomController:showSelfReplyTrucoOptBtn(baseScore)
	self.isSelfOptStatus_ = nil
	-- self:hideAllOprBtn()
	local isMatchPoint = self.mTableId:getIsMatchPoint()
	local isRule = self.mTableId:isValidateTrucoRule(self:getMySeatId())
	local btnList = {}
	if not self.mTableId:getIsMatchPoint() and not self:isTwoElevelPoint() then
		if self.mTableId:isValidateTrucoRule(self:getMySeatId()) then
			if 1 == baseScore then
				if MATCH_POINT ~= self.lastSelfScore_ and self.lastRivalScore_ ~= MATCH_POINT then
					table.insert(btnList, self.btnSeis_)
					self.btnSeis_:setPos(self.originSeisx_, self.originSeisy_)
					self:showBtnAnim(self.btnSeis_) -- self.btnSeis_:show()
				end
			elseif 3 == baseScore then
				table.insert(btnList, self.btnSeis_)	
				self.btnSeis_:setPos(self.originSeisx_, self.originSeisy_)
				self:showBtnAnim(self.btnSeis_) --self.btnSeis_:show()
			elseif 6 == baseScore then
				table.insert(btnList, self.btnNove_)
				self.btnNove_:setPos(self.originSeisx_, self.originSeisy_)
				self:showBtnAnim(self.btnNove_)	-- self.btnNove_:show()
			elseif 9 == baseScore then
				table.insert(btnList, self.btnDoze_)
				self.btnDoze_:setPos(self.originSeisx_, self.originSeisy_)
				self:showBtnAnim(self.btnDoze_)	-- self.btnDoze_:show()
			end
		end
	end
	-- 接受
	table.insert(btnList, self.btnAceitar_)
	self:showBtnAnim(self.btnAceitar_)	-- self.btnAceitar_:show()
	-- 拒绝
	table.insert(btnList, self.btnCorrer_)
	self:showBtnAnim(self.btnCorrer_)	-- self.btnCorrer_:show()
	-- 
	self:hideAllOprBtn(btnList)
	self:update_btn_pos(btnList)
end
-- 
function RoomController:showBtnAnim(btn)
	if btn:getVisible() then
		return
	end
	checkAndRemoveOneProp(btn, 2)
	btn:setVisible(true)
	local ts = 120
	local scaleAnim = btn:addPropScale(2, kAnimNormal, ts, 0, 0.01, 1.0, 0.01, 1.0, kCenterDrawing)
	scaleAnim:setEvent(self, function()
		checkAndRemoveOneProp(btn, 2)
	end)
	-- btn:setVisible(true)
end
-- 
function RoomController:hideBtnAnim(btn)
	if not btn:getVisible() then
		return
	end	
	checkAndRemoveOneProp(btn, 1)
	local ts = 120
	local scaleAnim = btn:addPropScale(1, kAnimNormal, ts, 0, 1, 0.01, 1, 0.01, kCenterDrawing)
	scaleAnim:setEvent(self, function()
		checkAndRemoveOneProp(btn, 1)
		btn:setVisible(false)
	end)
	-- btn:setVisible(false)
end
-- 判断当前玩家是否为AI状态
function RoomController:isSelfAi()
	local selfPlayerData = self:getMyPlayer()
	if selfPlayerData then
		return selfPlayerData:getAi()
	end
	return false
end
-- 再来一局
function RoomController:onContinue()
	self:resetScoreZero()

	self.mTableId:resetRoundStatus()
	self.mTableId:resetMatchCnt()

	GameSocketMgr:sendMsg(TRUCO_PLAY_AGAIN_REQ);
	self:closeAllPopup()
	self:cancelDelegateReq()
	-- self:cleanAllUserAIStatus()
	--JLog.d('function RoomController:onContinue()')
end
-- 
function RoomController:resetScoreZero()
	self:refreshScore(0, "view_score1")
	self:refreshScore(0, "view_score2")
end
-- “接收”当前牌型
function RoomController:onAcceptCurrentCards()
	GameSocketMgr:sendMsg(TRUCO_DROPTHREE_CAR_REQ, {action=1, seatId=1});
	-- self.popupMgr_:reomveVictoryAnimation()
end
-- “拒绝”当前牌型
function RoomController:onRefuseCurrentCards()
	GameSocketMgr:sendMsg(TRUCO_DROPTHREE_CAR_REQ, {action=0, seatId=1});
	-- self.popupMgr_:reomveVictoryAnimation()
end
-- 0x1057	 --发送给11分玩家桌面的信息
function RoomController:onGamePoint(data)
	self:delayCall(function()
		if #data.friendCardValues > 0 then
			self.popupMgr_:openGameMatchElevenTwoPopup(data)
		else
			self.popupMgr_:openGameMatchElevenOnePopup(data)
		end
	end, 1000)
end
-- 0x1058    --发送给非11分玩家的等待时间
function RoomController:onNotPointOptTime(data)
	self:delayCall(function()
		self.popupMgr_:openNotGameMatchElevelPopup(data)
	end, 1000)
end
-- 0x4017 	-- 广播玩家弃牌
function RoomController:onBroadCastAbandonCard(data)
	local localSeatId = self:getLocalSeatId(data.seatId)
	-- self.cardsMgr_:abandonHandCards(localSeatId)
	kEffectPlayer:play(AUDIO.DEALER_CARD_OUT)
end
-- 0x4018  广播玩家准备 TRUCO_BROADCAST_USER_READY
function RoomController:onBroadCastUserReady(data)
	-- local seatId = data.seatId
	local player = self:getUserByUid(data.uid)
	if not player then
		return
	end
	player:setIsReady(true)
	if data.uid == MyUserData:getId() then
		self:onCleanReadClock()
	else
		player:setDownNum(-1)
	end
	self:showOrhideReadView()
end
-- 0x1059 //玩家出牌返回
function RoomController:onPutCard(data)
	if data.uid == MyUserData:getId() then
		self:renderDarkStyle(data.flag == 0 and true or false)
		-- self:findChildByName("Img_normalCard"):setVisible(not value)
		if DEBUG_MODE then
			printinfo("RoomController:onPutCard()::"..data.flag)
		end
	end
end
-- 1060 玩家请求本回合出暗牌返回
function RoomController:onPutDarkCard(data)
	self:renderDarkStyle(data.flag == 0 and true or false)
end
-- 
function RoomController:onCleanReadClock()
	checkAndRemoveOneProp(self, 100)
	self.btnReady_:setVisible(false)
	local player = self:getUserByUid(MyUserData:getId())
	if player and player:getUi() then
		player:getUi():findChildByName("img_noReady"):setVisible(false)
	end
	self.viewReadyNum_:removeAllChildren()
end
-- 显示倒计时
function RoomController:showReadyClockNum(value)
	self.downClockNum_ = value or 15
	self.btnReady_:setVisible(true)
	local player = self:getUserByUid(MyUserData:getId())
	if player and player:getUi() then
		-- player:getUi():findChildByName("img_noReady"):setVisible(true)
	end
	self:renderReadyClockNum(self.downClockNum_)
	-- 
	checkAndRemoveOneProp(self, 100)
	local animEvent = self:addPropTranslate(100, kAnimLoop, 1000, 0, 0, 0, 0, 0)
	animEvent:setEvent(self, function()
		self.downClockNum_ = self.downClockNum_ - 1
		if self.downClockNum_ >= 0 then
			self:renderReadyClockNum(self.downClockNum_)
		else
			checkAndRemoveOneProp(self, 100)
		end
	end)
end
-- 呈现倒计时
function RoomController:renderReadyClockNum(value)
	local nodeClock = new(require('truco.common.imageNumber', -8), TrucoBaseController.TRUCO_DOWNTIME_RES_CONFIG)
	self.viewReadyNum_:removeAllChildren()
    self.viewReadyNum_:addChild(nodeClock)
    nodeClock:setNumber(string.format("%s", tostring(value)));
    nodeClock:setVisible(true)
    nodeClock:setAlign(kAlignCenter)
    
end
-------------------------------------------------------------------------
function RoomController:checkScoresWin2(player_list,scores)
	for i=1,#player_list do
		local s = player_list[i].scores
		if scores[1] < s[1] then
			return false
		end
	end
	return true
end

function RoomController:checkScoresWin(scores, totalCnt)
	totalCnt = totalCnt or 1
	if 3 > #scores then
		return false
	end
	local cnt = 0
	for i=1,3 do
		if scores[i] > 11 then
			cnt = cnt + 1
		end
	end

	if cnt >= totalCnt then
		return true
	else
		return false
	end
end

function RoomController:checkSelfSide()--检查自己是否旁观
	for i,v in ipairs(self.mPlayer) do
		if v and v:getId() == MyUserData:getId() then
			return false
		end
	end
	return true
end
function RoomController:followFriendWin( winlist )
	for i,v in ipairs(winlist) do
		if tonumber(v.uid )== tonumber(self.mFollowId) then
			return true
		end
	end
	return false
end
-- 结算弹出框
function RoomController:onPopupVictoryAnimation(data)
	--JLog.d('function RoomController:onPopupVictoryAnimation(data)',data)
	self:hideAllOprBtn()
	local winPlayerList = {}
	local losePlayerList = {}
	local isSelfWin = nil
	local winScore = 12
	local totalWinTeamMoney = 0
	local totalFailTeamMoney = 0
	
	local sendChipData = {}
	sendChipData.winSeatIds = {}
	sendChipData.winChips = {}
	sendChipData.loseSeatIds = {}
	sendChipData.loseChips = {}

	if not data.playerList then
		return
	end

	local totalCnt = 1
	for i=1,1 do
		for j=1,#data.playerList do
			local scores = data.playerList[j].scores
			if scores[i] > 0 then
				winScore = math.max(scores[i], winScore)
			end

			if i > 1 and scores[i] > 0 then
				totalCnt = 2
				winScore = scores[i]
				break
			end
		end
	end

	for i=1,#data.playerList do
		local player = data.playerList[i]
		local seatId = player.seatId
		local uid = player.uid
		-- player.playerData = self.mPlayer[seatId]
		if self.clonePlayers_ then
			player.playerData = self.clonePlayers_[uid]
		end
		-- 
		--if self:checkScoresWin(player.scores, totalCnt) then
		if self:checkScoresWin2(data.playerList, player.scores) then
			table.insert(winPlayerList, player)
			-- 检测uid是否为队友或自己
			if self.mTableId:checkUidIsFriend(player.uid) then
				isSelfWin = true
			end
			totalWinTeamMoney = totalWinTeamMoney + player.turnmoney
			table.insert(sendChipData.winSeatIds, seatId)
			table.insert(sendChipData.winChips, {player.turnmoney, player.money})        
		else
			table.insert(losePlayerList, player)
			totalFailTeamMoney = totalFailTeamMoney + player.turnmoney
			table.insert(sendChipData.loseSeatIds, seatId)
			table.insert(sendChipData.loseChips, {player.turnmoney, player.money})
		end
	end
	-- 
	data.winPlayerList = winPlayerList
	data.losePlayerList = losePlayerList
	data.totalWinTeamMoney = totalWinTeamMoney
	data.totalFailTeamMoney = totalFailTeamMoney	
    self.mTableId:resetMatchCnt()
	-- 
	local currentMatch = self.mTableId:getCurrentMatch()
	if currentMatch < data.selfScore then
		data.selfScore = currentMatch
	end
	self.mTableId:setSelfWinMathCnt(data.selfScore)
	if currentMatch < data.rivalScore then
		data.rivalScore = currentMatch
	end
	self.mTableId:setRivalWinMathCnt(data.rivalScore)
	-- 
	self:delayCall(function()
		for i=1,#sendChipData.winSeatIds do
			local loseSeatId = sendChipData.loseSeatIds[i] or 0
			local loseChip = 0
			local loseMoney = nil
			local winChip = 0
			local winMoney = nil
			local winseatId = sendChipData.winSeatIds[i] or 0
			if sendChipData.loseChips[i] then
				loseChip = sendChipData.loseChips[i][1]
				loseMoney = sendChipData.loseChips[i][2]
			end

			if sendChipData.winChips[i] then
				winChip = sendChipData.winChips[i][1]
				winMoney = sendChipData.winChips[i][2]
			end
			-- 
			self:rewardChipAnim(loseSeatId, loseChip, loseMoney, winseatId, winChip, winMoney, i)
		end
	end, 800)
	self.mTableId:setBaseScore(1)
	if self:checkSelfSide()  then
		return
	end	

	if isSelfWin  then
		--self:refreshScore(winScore, "view_score1")
		-- 获胜结算框
		data.totalMoney = totalWinTeamMoney
		self:delayCall(function()
			CardManager:getInstance():moveOutCardToHeap(self.cardHeap)
			CardManager:getInstance():moveLastCardToHeap(self.mPlayer,self.cardHeap)
			self.popupMgr_:openTrucoGameResultPopup(data)
		end, 2600)
	else
		--self:refreshScore(winScore, "view_score2")
		-- 失败结算框
		data.totalMoney = totalFailTeamMoney
		self:delayCall(function()
			CardManager:getInstance():moveOutCardToHeap(self.cardHeap)
			CardManager:getInstance():moveLastCardToHeap(self.mPlayer,self.cardHeap)
			self.popupMgr_:openFailTrucoGameResultPopup(data)
		end, 2600)
	end
	-- 显示经验
	for i,user in ipairs(data.playerList) do
		--self:showExpResult(user)
		if user.uid == MyUserData:getId() then
			MyUserData:setExp(user.exp)
			-- 
			local level = GameSetting:getLevel()
			local exp = user.exp
			for i = 1,#userLevelExp do
				local x1 = userLevelExp[i].x1 
				local x2 = userLevelExp[i].x2 
				if exp >= x1 and exp < x2 then
					printInfo('levely1 = ' .. i)
					if level == i then --已领取等级奖励
						--AlarmTip.play('已领取等级奖励')
					else --未领取等级奖励
						--请求领取礼包
						--levelTemp = i
                        MyUserData:setExp(exp)
                        MyUserData:setLevel(i)
		                GameSetting:setLevel(i)
		                GameSetting:save()
						HttpModule.getInstance():execute(HttpModule.s_cmds.GET_UPGRADE_REWARD, {level = i}, false, true)
					end
				end 
			end
		end
	end
	
	-- 
	
end
-- 添加倒计时
-- totalSeconds:为倒计总时长
function RoomController:addDownNumber_(totalSeconds, startTS, startCallback, endCallback)
	startTS = startTS or totalSeconds
	if not self.chipDownNode_ then
		self.chipDownNode_ = new(require("truco.anim.countdown"), nil, self)
		self:findChildByName("view_downNumber"):addChild(self.chipDownNode_)
		-- self:addChild(self.chipDownNode_)
		self.chipDownNode_:setAlign(kAlignCenter)
	end
	self.chipDownNode_:setCallback(function(self, times)
		if times <= 0 then
			if startCallback then
				startCallback()
				startCallback = nil
			end
			-- 
			if endCallback then
				endCallback()
			end
			self.chipDownNode_:setVisible(false)
			self:findChildByName("view_dealer"):setVisible(true)
			self:refreshRoomAnte()
		else
			if times <= startTS then
				if startCallback then
					startCallback()
					startCallback = nil
					self.chipDownNode_:setVisible(true)
				end
			end
		end
	end)
	
	self.chipDownNode_:setVisible(true)
	self.chipDownNode_:playGo(totalSeconds)
end
-- 
function RoomController:closeAllPopup()
	self.popupMgr_:cleanAllPopup()
end
-- 
function RoomController:cleanAllUserAIStatus()
	for k, v in pairs(self.mPlayer) do
		if v:getId() > 0 then
			v:setAi(false)
		end
	end
end
-- 
function RoomController:cleanViewChips()
	local viewChips = self:findChildByName("view_chips")
	viewChips:removeAllChildren()
end
-- 给赢家筹码动画
function RoomController:rewardChipAnim(loseSeatId, loseChip, loseMoney, winseatId, winChip, winMoney, idx)
	local viewChips = self:findChildByName("view_chips")
	-- viewChips:removeAllChildren()
	-- 
	local userPlayer = self.mPlayer[loseSeatId]
	if not userPlayer then
		return
	end
	kEffectPlayer:play(AUDIO.TRUCO_WINCHIP)

	userPlayer:setExchangeChip(loseChip)
	if loseMoney then
		userPlayer:setChip(loseMoney)
	end

	math.randomseed(os.time())
	local chipNode = new(Node):addTo(viewChips)
	local winLocalSeatId = self:getLocalSeatId(loseSeatId);
	local sx, sy,dw, dh = self:getLocalSeatPos(winLocalSeatId)
	local ax, ay = dw*0.5 - 350 + (idx-1)*60, dh*0.5
	local cnt = 10
	local chips = {}
	local animEvent = nil
	local headOffx = 20
	local headOffy = 20
	local ts = 260
	local unitTs = 55
	for i=1,cnt do
		local chipImg = new(Image, "truco/room/truco_chip.png"):addTo(chipNode)
		chipImg:setPos(sx+headOffx, sy+headOffy)
		local dx = ax - sx + math.random(-30, 30)
		local dy = ay - sy - i*2 + math.random(-30, 30)
		animEvent = chipImg:addPropTranslate(1, kAnimNormal, ts, (i - 1) * unitTs, 0, dx, 0, dy);
		-- chipImg:addPropTransparency(11, kAnimNormal, 0, (i - 1) * unitTs, 1.0, (cnt-i+1)*0.1);
		chips[i] = chipImg
	end
	-- 
	if animEvent then
		animEvent:setEvent(self, function()
			kEffectPlayer:play(AUDIO.TRUCO_WINCHIP)

			local winLocalSeatId = self:getLocalSeatId(winseatId);
			local sx, sy,dw, dh = self:getLocalSeatPos(winLocalSeatId)
			local animEvent2 = nil
			local len = #chips
			for i=len,1,-1 do
				local chipImg2 = chips[i]
				local dx = sx - ax
				local dy = sy - ax - i*2
				animEvent2 = chipImg2:addPropTranslate(2, kAnimNormal, ts, (len - i) * unitTs + ts + unitTs*2, 0, dx-0, 0, dy-headOffy)
			end
			if animEvent2 then
				animEvent2:setEvent(self, function()
					for k,v in pairs(chips) do
						checkAndRemoveOneProp(v, 1)
						checkAndRemoveOneProp(v, 11)
						checkAndRemoveOneProp(v, 2)
					end
					viewChips:removeChild(chipNode, true)
				end)
				-- 
				local userPlayer2 = self.mPlayer[winseatId]
				if userPlayer2 then
					userPlayer2:setExchangeChip(winChip)
					userPlayer2:setIsWinner(true)
					if winMoney then
						userPlayer2:setChip(winMoney)
					end
				end
			else
				for k,v in pairs(chips) do
					checkAndRemoveOneProp(v, 1)
					checkAndRemoveOneProp(v, 2)
				end
				checkAndRemoveOneProp(chipNode, 1)
				viewChips:removeChild(chipNode, true)
			end
		end)
	else
		checkAndRemoveOneProp(chipNode, 1)
		viewChips:removeChild(chipNode, true)
	end
end
-- 
function RoomController:testStatusIcon(uid, statusIcon)
	local user = self:getUserByUid(uid)
	if user then
		user:setStatusIcon(statusIcon)
	end
end
-- 
function RoomController:cleanAllPlayerStatusIcon(uid)
	for k, v in pairs(self.mPlayer) do
		if not uid or v:getId() ~= uid then
			v:setStatusIcon(-1)
		end
	end
end
-- 
function RoomController:cloneAllPlayers()
	self.clonePlayers_ = {}
	for i,player in ipairs(self.mPlayer) do
		if player:getId() > 0 then
			local uid = player:getId()
			local newPlayer = setProxy(new(require('truco.data.player')))
			newPlayer:cloneOtherPlayer(player)
			self.clonePlayers_[uid] = newPlayer
		end
	end
end
-- 更新房间积分信息
function RoomController:updateRoomStatus(value)
	if value == 0 then
		--self:findChildByName("view_matchCnt1"):setVisible(false)
		--self:findChildByName("view_matchCnt2"):setVisible(false)
		self:findChildByName("view_score1"):setVisible(false)
		self:findChildByName("view_score2"):setVisible(false)			
	else
		--self:findChildByName("view_matchCnt1"):setVisible(true)
		--self:findChildByName("view_matchCnt2"):setVisible(true)
		self:findChildByName("view_score1"):setVisible(true)
		self:findChildByName("view_score2"):setVisible(true)
	end
end

function RoomController:resetAIIconPos(localSeatId, player)
	--重新整理位置
	local px, py = self:getLocalSeatPos(localSeatId, false)
	player:setPos(px, py)	-- 设置玩家头像位置
	local nickPy = 100
	local offx = 55
	local offy = 40
	local ipx = 75
	local imgDownNum = player:findChildByName("img_DownNum")
	local player_info = player:findChildByName('player_info')
	local card1_x = 60
	local card2_x = 68
	local card3_x = 76
	local chip_view_y = 20

	if localSeatId == 1 then
		player_info:setPos(10, 80)
		player_info:findChildByName('card1'):setPos(-card1_x,0)
		player_info:findChildByName('card2'):setPos(-card2_x,0)
		player_info:findChildByName('card3'):setPos(-card3_x,0)
		player_info:findChildByName('text_nick'):setPos(85,-13)
		player_info:findChildByName('chipView'):setPos(80,chip_view_y)
		player:findChildByName('img_ai'):setPos(40, 10)	-- 设置ai图标的位置
		player:findChildByName('btn_gift'):setPos(-40, 10)	--礼物
		player:findChildByName('img_clock'):setPos(-85, 0)
	elseif localSeatId == 2 then
		--player_info:setPos(120, -15)
		player_info:setPos(95, -15)
		player_info:findChildByName('card1'):setPos(card1_x,0)
		player_info:findChildByName('card2'):setPos(card2_x,0)
		player_info:findChildByName('card3'):setPos(card3_x,0)
		player_info:findChildByName('text_nick'):setPos(80,-13)
		player_info:findChildByName('chipView'):setPos(80,chip_view_y)
		player:findChildByName('img_ai'):setPos(40, 10)	-- 设置ai图标的位置
		player:findChildByName('btn_gift'):setPos(-40, 10)	--礼物	
		--player:findChildByName('img_clock'):setPos(-85, 0)
		player:findChildByName('img_clock'):setPos(-77, -65)
	elseif localSeatId == 3 then
		player_info:setPos(-10, 80)
		player_info:findChildByName('card1'):setPos(card1_x,0)
		player_info:findChildByName('card2'):setPos(card2_x,0)
		player_info:findChildByName('card3'):setPos(card3_x,0)
		player_info:findChildByName('text_nick'):setPos(70,-13)
		player_info:findChildByName('chipView'):setPos(60,chip_view_y)
		player:findChildByName('img_ai'):setPos(-40, 10)	-- 设置ai图标的位置
		player:findChildByName('btn_gift'):setPos(40, 10)	--礼物	
		player:findChildByName('img_clock'):setPos(85, 0)
	elseif localSeatId == 4 then
		--player_info:setPos(120, -15)
		player_info:setPos(70, -15)
		player_info:findChildByName('text_nick'):setPos(110,-13)
		player_info:findChildByName('chipView'):setPos(110,chip_view_y)
		player:findChildByName('img_ai'):setPos(-40, 10)	-- 设置ai图标的位置
		player:findChildByName('btn_gift'):setPos(40, 10)	--礼物	
		--player:findChildByName('img_clock'):setPos(85, -85)
		player:findChildByName('img_clock'):setPos(105, -85)
		imgDownNum:setVisible(false)
	end
end

function RoomController:sitdownAnimCallback()
	if self.mTableId:isTwoMatch() then
		for i,v in ipairs(self.mPlayer) do
			if v and v:getUi()  then
			--	v:getUi():findChildByName("btn_invite"):setVisible(false)
			end
		end
	else
     
	end
end
function RoomController:onBroadcastChangeBaseScore(data)
	self.mTableId:setBaseScore(data.score)
end
function RoomController:sendDropCmd(card)
	if card then
		local value = card:getByte()
		GameSocketMgr:sendMsg(TRUCO_DROP_CARD_REQ, {seatId=0, cardValue=value, cardPos=0})
		
	end
end
function RoomController:opreateBtnsEnable(enable)
	--操作按钮是否可点击
	self:findChildByName("btn_dark"):setEnable(enable)
	self:findChildByName("btn_truco"):setEnable(enable)
	self:findChildByName("btn_fold"):setEnable(enable)
	self:findChildByName("btn_doze"):setEnable(enable)
	self:findChildByName("btn_nove"):setEnable(enable)
	self:findChildByName("btn_seis"):setEnable(enable)
	self:findChildByName("btn_correr"):setEnable(enable)
	self:findChildByName("btn_aceitar"):setEnable(enable)	
	--代理按钮显示和隐藏
	self:findChildByName('btn_canceldelegate'):setVisible(not value)	
end

function RoomController:get_frames(flag)
	--flag 1 气泡
	--2 truco ,3 seis, 4 nove, 5 doze,6 joga, 7 aceito, 8 corri, 9 nao posso

	-- if flag == 6 then
	-- 	flag = math.random(6,7)
	-- elseif flag == 8 then
	-- 	flag = math.random(8,9)
	-- end

	local m = {}
	local fa_file = ''

	local all_num = {11,9,9,9,9,9,9,9,9}
	local num = all_num[flag]

	for i=1,num do
		if i < 10 then 
			fa_file = string.format('truco/anim/anim%d/0%d.png',flag,i) 
		else
			fa_file = string.format('truco/anim/anim%d/%d.png',flag,i)
		end
		m[i] = fa_file
	end

	local pic = new(Image, m[1])

	return m,pic
end

function RoomController:stop_play_fa()
	for i=1,4 do
		local p = self.mPlayer[i]
		if p and p:getId() > 0 then
			local ui = p:getUi()
			ui:findChildByName("view_status"):removeAllChildren()
		end
	end
end

function RoomController:play_fa(player,localSeatId,flag)
	self:stop_play_fa()
	JLog.d("GHJ_TEST", flag)
    --2 truco ,3 seis, 4 nove, 5 doze,6 joga, 7 aceito, 8 corri
	if self.mGameId == 13007 and flag ~= 7 then
		local viewStatus1 	= player:findChildByName("view_status")
		viewStatus1:removeAllChildren()

		local px, py
		if localSeatId == 1 then
			px = -10
			py = -95
		elseif localSeatId == 2 then
			px = -123
			py = -11
		elseif localSeatId == 3 then
			px = 15
			py = -95
		else
			px = 15
			py = -95
		end

		local imgPath = "truco/room/13007/bubble_text/"
		if flag == 2 then
			imgPath = imgPath.."truco.png"
		elseif flag == 3 then
			imgPath = imgPath.."seis.png"
		elseif flag == 4 then
			imgPath = imgPath.."nove.png"
		elseif flag == 5 then
			imgPath = imgPath.."doze.png"
		elseif flag == 6 then
			imgPath = imgPath.."joga.png"
		-- elseif flag == 7 then
		-- 	imgPath = imgPath.."truco.png"
		-- 	str = "ACEITO!"
		elseif flag == 8 then
			imgPath = imgPath.."corri.png"
		end

		local image = nil
		local text = nil
		if localSeatId == 2 then
			image = new(Image, "truco/room/13007/bubble2.png")
			-- text = new(Text, str, 117, 50, kAlignCenter, kFontTextBold, 30, 153, 78, 17)
			-- image:addChild(text)
			-- text:setAlign(kAlignCenter)
			-- text:setPos(-9, -1)
			imageBubble = new(Image, imgPath)
			image:addChild(imageBubble)
			imageBubble:setAlign(kAlignCenter)
			imageBubble:setPos(-9, -1)
			imageBubble:setSize(imageBubble.m_res.width, imageBubble.m_res.heigth)
		else
			image = new(Image, "truco/room/13007/bubble1.png")
			-- text = new(Text, str, 117, 50, kAlignCenter, kFontTextBold, 30, 153, 78, 17)
			-- image:addChild(text)
			-- text:setAlign(kAlignCenter)
			-- text:setPos(-2, -9)
			imageBubble = new(Image, imgPath)
			image:addChild(imageBubble)
			imageBubble:setAlign(kAlignCenter)
			imageBubble:setPos(-2, -9)
			imageBubble:setSize(imageBubble.m_res.width, imageBubble.m_res.heigth)
		end
		local viewStatus1 = player:findChildByName("view_status")
		viewStatus1:removeAllChildren()
		viewStatus1:addChild(image)
		image:setAlign(kAlignCenter)
		image:setPos(px, py)
		return
	end

	local px, py = 10, -100
	if localSeatId == 1 then
		px = px-10
	elseif localSeatId == 2 then
		px = -140
		py = -5
	elseif localSeatId == 3 then
		px = px+10
	end

	local viewStatus1 	= player:findChildByName("view_status")
	viewStatus1:removeAllChildren()

	local m1,pic1 = self:get_frames(1)
	local fa_bg = new (FrameAnim,m1,0,1,nil,pic1)
	pic1:setAlign(kAlignCenter)
	pic1:addTo(viewStatus1)
	pic1:setPos(px,py)
	fa_bg:play()


	local m2,pic2 = self:get_frames(flag)
	local fa = new (FrameAnim,m2,0,-1,nil,pic2)
	self:delayCall(function ()

		
		pic2:setAlign(kAlignCenter)
		pic2:addTo(viewStatus1)
		--pic2:addTo(pic1)
		pic2:setPos(px,py-7)
		--pic2:setPos(0,0-7)
		fa:play()
		
	end,500)

	-- self:delayCall(function()
	-- 	if pic1.m_children then
	-- 		viewStatus1:removeChild(pic1, true)
	-- 	end
	-- 	if pic2.m_children then
	-- 		viewStatus1:removeChild(pic2, true)
	-- 	end
	-- 	fa_bg:stop()
	-- 	fa:stop()
	-- end, 3000)
end

function RoomController:test111()
	--self.popupMgr_:openGameMatchElevenOnePopup(data)
	--self.popupMgr_:openGameMatchElevenTwoPopup()
	--WindowManager:showWindow(WindowTag.TrucoGameResult_2, {}, WindowStyle.POPUP)
	self:play_fa(self.mPlayer[1]:getUi(),4,8)
	--CardManager:getInstance():dealCards_({1,2,3},self.cardHeap,self.mPlayer,0)
end

return RoomController