local TrucoLobbyController 	= class(BaseController)
local STRING = require('truco.res.config')
local roomListItemLayout = requireview("view.truco.roomListItemLayout")
local printInfo, printError = overridePrint("TrucoLobbyController")

function TrucoLobbyController:ctor(viewConfig, param1, param2, param3)
	--加入骰子SERVER协议
    self.mGameId    = param3[5] or GAME_ID.Truco
	self.m_reader		= new(require('truco.protocol.lobby.lobbyReader'))
	self.m_writer		= new(require('truco.protocol.lobby.lobbyWriter'))
	GameSocketMgr:addCommonSocketReader(self.m_reader);
	GameSocketMgr:addCommonSocketWriter(self.m_writer);
	--监听HTTP请求
	EventDispatcher.getInstance():register(HttpModule.s_event, self, self.onHttpRequestsCallBack);
	--返回
	self.m_root:findChildByName("btn_back"):setOnClick(self, function ( self )
		-- body
		StateChange.changeState(States.Lobby, self.mGameId);
	end);
    local viewPlayer = self:findChildByName("img_player")
    viewPlayer:findChildByName("btn_headbg"):setOnClick(self, function(self)
        if not MyUserData:getPropCount() then
		      HttpModule.getInstance():execute(HttpModule.s_cmds.GET_PROP_LIST,{pcid = 2, sid = 102}, false, false)
		end
        WindowManager:showWindow(WindowTag.UserPopu, {}, WindowStyle.POPUP)
    end)
	viewPlayer:findChildByName("btn_add"):setOnClick(self, function ( self )
		-- body
		WindowManager:showWindow(WindowTag.ShopPopu, {}, WindowStyle.POPUP)
		app:postFrontStaticstics('clickShoppingMall')
	end);
	--头像
	UIEx.bind(self, MyUserData, "headName", function(value)
		local headView 		= viewPlayer:findChildByName("view_head")
		headView:removeAllChildren();
		local width, height = headView:getSize()
		local imgHead 		= new(Mask, value, "common/head_bg.png");
		imgHead:setSize(width, height)
		headView:addChild(imgHead);
		MyUserData:checkHeadAndDownload()
	end)
	--昵称
	UIEx.bind(self, MyUserData, "nick", function(value)
		viewPlayer:findChildByName("text_nick"):setText(value);
	end)
	--金币
	UIEx.bind(self, MyUserData, "money", function(value)
		chipView = viewPlayer:findChildByName("view_chipnumber");
		chipView:removeAllChildren();
				local chipNode = new(require('common.imageNumber'), {
                    ['0'] = chipNumber_map["0.png"],
		            ['1'] = chipNumber_map["1.png"],
		            ['2'] = chipNumber_map["2.png"],
		            ['3'] = chipNumber_map["3.png"],
		            ['4'] = chipNumber_map["4.png"],
		            ['5'] = chipNumber_map["5.png"],
		            ['6'] = chipNumber_map["6.png"],
		            ['7'] = chipNumber_map["7.png"],
		            ['8'] = chipNumber_map["8.png"],
		            ['9'] = chipNumber_map["9.png"],
		            [','] = chipNumber_map["c.png"],
		            ['k'] = chipNumber_map["k.png"],
		            ["m"] = chipNumber_map["m.png"],
		            ["b"] = chipNumber_map["b.png"],
		            ["."] = chipNumber_map["p.png"],
				});	

				chipNode:setNumber(string.format("%s", ToolKit.formatMoney(value)));
				chipNode:setAlign(kAlignLeft);
				chipView:addChild(chipNode);
	end)
    self:initLevel()
	--性别
	MyUserData:setSex(MyUserData:getSex());
	MyUserData:setNick(MyUserData:getNick());
	MyUserData:setMoney(MyUserData:getMoney());
	--快速开始
	local btnStart = self.m_root:findChildByName("btn_quickStart");
	btnStart:setOnClick(self, handler(self, self.onQuickStart));
	--加载房间列表
	self:loadRoomList();
	--登录
	HttpModule.s_config[HttpModule.s_cmds.GET_GAME_LOGIN][HttpConfigContants.URL] = tostring(self.mGameId)
	HttpModule.getInstance():execute(HttpModule.s_cmds.GET_GAME_LOGIN, {sid=self.mGameId}, MyRoomConfig:get(self.mGameId) == nil, true);
end

function TrucoLobbyController:onQuickStart()
	local game = app:getGame(self.mGameId) --truco
	if game then
		local room = game:getRoomFromMoney(MyUserData:getMoney())
		if room then
			game:enterRoom(States.GameLobby, room)
		else
			AlarmTip.play(STR_MONEY_LESS_THAN_MIN)
		end
	end
end

function TrucoLobbyController:dtor()
	printInfo("TrucoLobbyController:dtor")
	EventDispatcher.getInstance():unregister(HttpModule.s_event, self, self.onHttpRequestsCallBack);

	GameSocketMgr:removeCommonSocketReader(self.m_reader);
	GameSocketMgr:removeCommonSocketWriter(self.m_writer);
	delete(self.m_reader);
	delete(self.m_writer);
	self.m_reader = nil;
	self.m_writer = nil;
end

function TrucoLobbyController:initLevel()
    local lv = tonumber(MyUserData:getLevel())
    local exp = tonumber(MyUserData:getExp()) 
    local percent = 0
	if lv then
		local index = tonumber(lv)
		if index > 0 and index < 60 then
			local x1 = userLevelExp[index].x1 
			local x2 = userLevelExp[index].x2
			if exp > x1 then
				percent = math.ceil(((exp - x1)/(x2 - x1))*100)
				if percent >= 100 then
					percent = 99
				end
			end
		elseif index == 60 then
			percent = 99
		end
	end
    local progress = self:findChildByName("img_progress")
    --经验的底图和背景一起，写死长度
    local maxLen = 164
    local minLen, h = progress:getSize()
    progress:setSize(minLen + (maxLen - minLen) * percent / 100, h)

end
--[[
最小携带  Mínimo
底注  antes
]]
function TrucoLobbyController:loadRoomList()
	local WINTERVAL = 45
	local scrollView 	= self:findChildByName("sv_roomList");
    scrollView:setDirection(kHorizontal)
    scrollView:removeAllChildren()

	local game 			= app:getGame(self.mGameId)
	local roomList 		= game:getRoomList()
    local w = WINTERVAL
    local properItem = nil
    local properRoom = game:getRoomFromMoney(MyUserData:getMoney())
	for i = 1, roomList and roomList:count() or 0 do
		local room 	 = roomList:get(i);
		local item 	 = SceneLoader.load(roomListItemLayout);
        if properRoom == room then
            properItem = item
        end
		local lw, lh = item:getSize();

		local node = new(Node);
		local imgPlayer = new(Image, 'lobbyRoom/card/online_icon.png');
		local imgPlayerW, imgPlayerH = imgPlayer:getSize();
		local textOnline= new(Text, room:getOnline(), 0, 0, kAlignCenter, "", 24, 104, 97, 96);
		local textOnlineW, textOnlineH = textOnline:getSize();
		textOnline:setPos(imgPlayerW + 5, (imgPlayerH - textOnlineH) /2 );
		node:addChild(imgPlayer);
		node:addChild(textOnline);
		node:setSize(imgPlayerW + 5 + textOnlineW, imgPlayerH);
		node:setAlign(kAlignCenter);
		item:findChildByName("view_online"):addChild(node);
		item:findChildByName("text_ante"):setText(string.format(STRING.TRUCO_ANTE, ToolKit.formatAnteWithoutFloor(room:getAnte())))
		item:findChildByName("text_minlimit"):setText(string.format(STRING.TRUCO_LIMIT, ToolKit.formatAnteWithoutFloor(room:getMinchip())));
		local btnBg = item:findChildByName("btn_bg")
        btnBg:setFile(string.format("lobbyRoom/card/level_%s.png", i) or "")
		btnBg:setOnClick(self, function ( self )
		-- body
			local myMoney = MyUserData:getMoney()
			local room 	  = roomList:get(i)			
			local content = nil
			local minChip = room:getMinchip()
			local maxChip = room:getMaxchip()
			if myMoney < minChip then
				content = string.format(STR_MONEY_LESS_THAN_ANTE, minChip);
			elseif myMoney > maxChip then
				content = string.format(STR_MONEY_GREATER_THAN_ANTE, maxChip);
			else
				game:enterRoom(States.GameLobby, room)
				return ;
			end

			WindowManager:showWindow(WindowTag.LobbyExitPopu, {
			title 	= "",
			close 	= true,
			content = content,
			confirm = STR_QUICK_CHOOSE_ROOM,
			confirmFunc = function ()
				-- body
				local game = app:getGame(self.mGameId)
				if game then
					local room = game:getRoomFromMoney(MyUserData:getMoney())
					if room then
						game:enterRoom(States.GameLobby, room)
					else
						AlarmTip.play(STR_MONEY_NOT_ENOUGH)
					end
				end
			end
			}, WindowStyle.POPUP);			
		end);

		UIEx.bind(textOnline, room, "online", function(value)
			textOnline:setText(tostring(value));
		end)

		item:setAlign(kAlignLeft)
        item:setPos(w, 0)
        w = w + item:getSize() + WINTERVAL
		scrollView:addChild(item);
	end

	scrollView:setSize(scrollView:getSize());
    if properItem then
        scrollView:scrollItemToView(properItem)
    end
end

--HTTP回调
function TrucoLobbyController:onHttpRequestsCallBack(command, ...)
	if self.s_severCmdEventFuncMap[command] then
     	self.s_severCmdEventFuncMap[command](self,...)
	end 
end
function TrucoLobbyController:onGetRoomList(isSuccess, data)
	if app:checkResponseOk(isSuccess, data) then
		self:loadRoomList();
	end
end
--[[
	登录
]]
function TrucoLobbyController:onGameLogin(isSuccess, data)

	if app:checkResponseOk(isSuccess, data) then
		local gameInfo = data.data
		if gameInfo then
			--下载房间列表
			app:checkAndDownloadRoomList(self.mGameId, gameInfo.configVersion.room or 1, true);
			--更新在线人数
			local roomList = app:getGame(self.mGameId):getRoomList()
			if roomList then
				local online = gameInfo.onlinenumber;
				for i = 1, roomList:count() do
					local count = online[i] and online[i][2] or 0
					roomList:get(i):setOnline(count)
				end
			end

		end
	end
end

function TrucoLobbyController:onBack()
    StateChange.changeState(States.Lobby, self.mGameId);
end

--HTTP事件
TrucoLobbyController.s_severCmdEventFuncMap = {
	[HttpModule.s_cmds.GET_ROOM_LIST] 	= TrucoLobbyController.onGetRoomList,
	[HttpModule.s_cmds.GET_GAME_LOGIN] 	= TrucoLobbyController.onGameLogin,
}

return TrucoLobbyController;